@@ -13,16 +13,15 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "city.h"
 #include "game.h"
 #include "government.h"
 #include "log.h"
 #include "map.h"
 #include "unit.h"
 
-#include "cityhand.h"
 #include "citytools.h"
 #include "settlers.h"
-#include "unitfunc.h"
 #include "unittools.h"
 
 #include "advmilitary.h"
@@ -32,16 +31,10 @@
 
 #include "advdomestic.h"
 
-/********************************************************************** 
-... this function should assign a value to choice and want and type, where 
-    want is a value between 1 and 100.
-    if want is 0 this advisor doesn't want anything
-    type = 1 means unit, type = 0 means building
-***********************************************************************/
 
 static int ai_best_tile_value(struct city *pcity)
 {
-  int x, y, bx, by, food, best, cur;
+  int bx, by, food, best, cur;
 
 /* food = (pcity->size *2 -get_food_tile(2,2, pcity)) + settler_eats(pcity); */
   food = 0; /* simply works better as far as I can tell */
@@ -57,7 +50,7 @@ static int ai_best_tile_value(struct city *pcity)
           best = cur;
         }
       }
-    }
+    } city_map_iterate_end;
   } while(0);
   if (bx || by)
      return(best);
@@ -96,49 +89,49 @@ static int building_value(int max, struct city *pcity, int val)
 
 static int ocean_workers(struct city *pcity)
 {
-  int x, y, i = 0;
+  int i = 0;
   city_map_iterate(x, y) {
     if (map_get_tile(pcity->x+x-2, pcity->y+y-2)->terrain == T_OCEAN) {
       i++; /* this is a kluge; wasn't getting enough harbors because
 often everyone was stuck farming grassland. */
       if (is_worker_here(pcity, x, y)) i++;
     }
-  }
+  } city_map_iterate_end;
   return(i/2);
 }
 
 static int road_trade(struct city *pcity)
 {
-  int x, y, i = 0; 
+  int i = 0; 
   city_map_iterate(x, y) {
     if (is_worker_here(pcity, x, y)) {
       if (map_get_special(pcity->x+x-2, pcity->y+y-2) & S_ROAD) i++;
     }
-  }  
+  } city_map_iterate_end;
   return(i); 
 }
 
 static int farmland_food(struct city *pcity)
 {
-  int x, y, i = 0; 
+  int i = 0; 
   city_map_iterate(x, y) {
     if (is_worker_here(pcity, x, y)) {
       if (map_get_special(pcity->x+x-2, pcity->y+y-2) & S_FARMLAND) i++;
     }
-  }  
+  } city_map_iterate_end;
   return(i); 
 }
 
 static int pollution_cost(struct player *pplayer, struct city *pcity,
 			  Impr_Type_id id)
 {
-  int p, mod = 0, poppul = 0, a, b, c, x, y, tmp = 0;
+  int p, a, b, c, tmp = 0;
   p = 0;
   city_map_iterate(x, y) {
     if(get_worker_city(pcity, x, y)==C_TILE_WORKER) {
-      p += get_shields_tile(x, y, pcity);
+      p += city_get_shields_tile(x, y, pcity);
     }
-  }
+  } city_map_iterate_end;
   if (city_got_building(pcity, B_FACTORY) || id == B_FACTORY) {
     if (city_got_building(pcity, B_MFG) || id == B_MFG) tmp = 100;
     else tmp = 50;
@@ -156,9 +149,8 @@ static int pollution_cost(struct player *pplayer, struct city *pcity,
            id == B_HYDRO || id == B_HOOVER || id == B_NUCLEAR) p /= 2;
 
   if (!city_got_building(pcity, B_MASS) && id != B_MASS) {
-    mod = player_knows_techs_with_flag(pplayer, TF_POPULATION_POLLUTION_INC);
-    poppul=(pcity->size*mod)/4;
-    p += poppul;
+    p += (player_knows_techs_with_flag(pplayer, TF_POPULATION_POLLUTION_INC)*
+    pcity->size)/4;
   }
   p -= 20;
   if (p < 0) p = 0;
@@ -179,14 +171,14 @@ static int pollution_cost(struct player *pplayer, struct city *pcity,
 void ai_eval_buildings(struct city *pcity)
 {
   struct government *g = get_gov_pcity(pcity);
-  int i, val, a, t, food, j, k, hunger, bar, grana;
+  int i, val, t, food, j, k, hunger, bar, grana;
+  Impr_Type_id id, id2;
   int tax, prod, sci, values[B_LAST];
   int est_food = pcity->food_surplus + 2 * pcity->ppl_scientist + 2 * pcity->ppl_taxman; 
   struct player *pplayer = city_owner(pcity);
   int needpower;
   int wwtv = worst_worker_tile_value(pcity);
   
-  a = get_nation_by_plr(city_owner(pcity))->attack;
   t = pcity->ai.trade_want; /* trade_weighting */
   sci = (pcity->trade_prod * pplayer->economic.science + 50) / 100;
   tax = pcity->trade_prod - sci;
@@ -213,15 +205,16 @@ void ai_eval_buildings(struct city *pcity)
   if (i > 0 && !pcity->ppl_scientist && !pcity->ppl_taxman) hunger = i + 1;
   else hunger = 1;
 
-  for (i=0;i<B_LAST;i++) {
-    values[i]=0;
-  } /* rewrite by Syela - old values seemed very random */
+  memset(values, 0, sizeof(values)); /* clear to initialize */
 
+  /* I don't know how the change in granary size calculation would affect 
+     this because I don't really understand this piece of code.  Exact same 
+     thing with B_SEWER later in this function.  --Jing   12/30/2000 */
   if (could_build_improvement(pcity, B_AQUEDUCT)) {
     int asz = game.aqueduct_size;
     if (city_happy(pcity) && pcity->size > asz-1 && est_food > 0)
       values[B_AQUEDUCT] = ((((city_got_effect(pcity, B_GRANARY) ? 3 : 2) *
-         (pcity->size+1) * game.foodbox)/2) - pcity->food_stock) * food;
+         city_granary_size(pcity->size))/2) - pcity->food_stock) * food;
     else {
       int tmp = ((asz+1 - MIN(asz, pcity->size)) * (MAX(asz, pcity->size)+1) *
 	       game.foodbox - pcity->food_stock);
@@ -346,12 +339,13 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
 
   if (could_build_improvement(pcity, B_SDI))
     values[B_SDI] = 50; /* WAG */
-  
+
+  /* see comments above on B_AQUADUCT (wrt. granary size).  --Jing */
   if (could_build_improvement(pcity, B_SEWER)) {
     int ssz = game.sewer_size;
     if (city_happy(pcity) && pcity->size > ssz-1 && est_food > 0)
       values[B_SEWER] = ((((city_got_effect(pcity, B_GRANARY) ? 3 : 2) *
-         (pcity->size+1) * game.foodbox)/2) - pcity->food_stock) * food;
+         city_granary_size(pcity->size))/2) - pcity->food_stock) * food;
     else {
       int tmp = ((ssz+1 - MIN(ssz, pcity->size)) * (MAX(ssz, pcity->size)+1) *
 	       game.foodbox - pcity->food_stock);
@@ -402,135 +396,145 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
 /* ignored: AIRPORT, PALACE, and POLICE. -- Syela*/
 /* military advisor will deal with CITY and PORT */
 
-  for (i = 0; i < B_LAST; i++) {
-    if (is_wonder(i) && could_build_improvement(pcity, i)
-	&& !wonder_obsolete(i)&& is_wonder_useful(i)) {
-      if (i == B_ASMITHS)
-        for (j = 0; j < B_LAST; j++)
-          if (city_got_building(pcity, j) && improvement_upkeep(pcity, j) == 1)
-            values[i] += t;
-      if (i == B_COLLOSSUS)
-        values[i] = (pcity->size + 1) * t; /* probably underestimates the value */
-      if (i == B_COPERNICUS)
-        values[i] = sci/2;
-      if (i == B_CURE)
-        values[i] = building_value(1, pcity, val);
-      if (i == B_DARWIN) /* this is a one-time boost, not constant */
-        values[i] = ((research_time(pplayer) * 2 + game.techlevel) * t -
+  for (id = 0; id < game.num_impr_types; id++) {
+    if (is_wonder(id) && could_build_improvement(pcity, id)
+	&& !wonder_obsolete(id)&& is_wonder_useful(id)) {
+      if (id == B_ASMITHS)
+        for (id2 = 0; id2 < game.num_impr_types; id2++)
+          if (city_got_building(pcity, id2) &&
+              improvement_upkeep(pcity, id2) == 1)
+            values[id] += t;
+      if (id == B_COLLOSSUS)
+        values[id] = (pcity->size + 1) * t; /* probably underestimates the value */
+      if (id == B_COPERNICUS)
+        values[id] = sci/2;
+      if (id == B_CURE)
+        values[id] = building_value(1, pcity, val);
+      if (id == B_DARWIN) /* this is a one-time boost, not constant */
+        values[id] = ((research_time(pplayer) * 2 + game.researchcost) * t -
                     pplayer->research.researched * t) / MORT;
-      if (i == B_GREAT) /* basically (100 - freecost)% of a free tech per turn */
-        values[i] = (research_time(pplayer) * (100 - game.freecost)) * t *
+      if (id == B_GREAT) /* basically (100 - freecost)% of a free tech per turn */
+        values[id] = (research_time(pplayer) * (100 - game.freecost)) * t *
                     (game.nplayers - 2) / (game.nplayers * 100); /* guessing */
 
-      if (i == B_WALL && !city_got_citywalls(pcity))
+      if (id == B_WALL && !city_got_citywalls(pcity))
 /* allowing B_CITY when B_WALL exists, don't like B_WALL when B_CITY exists. */
         values[B_WALL] = 1; /* WAG */
  /* was 40, which led to the AI building WALL, not being able to build CITY,
 someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela */
 
-      if (i == B_HANGING) /* will add the global effect to this. */
-        values[i] = building_value(3, pcity, val) -
+      if (id == B_HANGING) /* will add the global effect to this. */
+        values[id] = building_value(3, pcity, val) -
                     building_value(1, pcity, val);
-      if (i == B_HOOVER && !city_got_building(pcity, B_HYDRO) &&
+      if (id == B_HOOVER && !city_got_building(pcity, B_HYDRO) &&
                            !city_got_building(pcity, B_NUCLEAR))
-        values[i] = (city_got_building(pcity, B_POWER) ? 0 :
+        values[id] = (city_got_building(pcity, B_POWER) ? 0 :
                (needpower * prod)/4) + pollution_cost(pplayer, pcity, B_HOOVER);
-      if (i == B_ISAAC)
-        values[i] = sci;
-      if (i == B_LEONARDO) {
-        unit_list_iterate(pcity->units_supported, punit)
-          j = can_upgrade_unittype(pplayer, punit->type);
-  	  if (j >= 0) {
+      if (id == B_ISAAC)
+        values[id] = sci;
+      if (id == B_LEONARDO) {
+        unit_list_iterate(pcity->units_supported, punit) {
+          Unit_Type_id unit_id;
+
+          unit_id = can_upgrade_unittype(pplayer, punit->type);
+  	  if (unit_id >= 0) {
 	     /* this is probably wrong -- Syela */
-	    int tmp = 8 * unit_upgrade_price(pplayer, punit->type, j);
-	    values[i] = MAX(values[i], tmp);
+	    int tmp = 8 * unit_upgrade_price(pplayer, punit->type, unit_id);
+	    values[id] = MAX(values[id], tmp);
 	  }
-        unit_list_iterate_end;
+        } unit_list_iterate_end;
       }
-      if (i == B_BACH)
-        values[i] = building_value(2, pcity, val);
-      if (i == B_RICHARDS) /* ignoring pollu, I don't think it matters here -- Syela */
-        values[i] = (pcity->size + 1) * SHIELD_WEIGHTING;
-      if (i == B_MICHELANGELO) {
+      if (id == B_BACH)
+        values[id] = building_value(2, pcity, val);
+      if (id == B_RICHARDS) /* ignoring pollu, I don't think it matters here -- Syela */
+        values[id] = (pcity->size + 1) * SHIELD_WEIGHTING;
+      if (id == B_MICHELANGELO) {
 	/* Note: Mich not built, so get_cathedral_power() doesn't include its effect. */
         if (improvement_variant(B_MICHELANGELO)==0 &&
 	    !city_got_building(pcity, B_CATHEDRAL)) {
 	  /* Assumes Mich will act as the Cath that is not in this city. */
-          values[i] = building_value(get_cathedral_power(pcity), pcity, val);
+          values[id] = building_value(get_cathedral_power(pcity), pcity, val);
 	} else if (improvement_variant(B_MICHELANGELO)==1 &&
 		   city_got_building(pcity, B_CATHEDRAL)) {
 	  /* Assumes Mich will double the power of the Cath that is in this city. */
-          values[i] = building_value(get_cathedral_power(pcity), pcity, val);
+          values[id] = building_value(get_cathedral_power(pcity), pcity, val);
 	}
       }
 
       /* The following is probably wrong if B_ORACLE req is
 	 not the same as game.rtech.temple_plus (was A_MYSTICISM)
 	 --dwp */
-      if (i == B_ORACLE) {
+      if (id == B_ORACLE) {
         if (city_got_building(pcity, B_TEMPLE)) {
           if (get_invention(pplayer, game.rtech.temple_plus) == TECH_KNOWN)
-            values[i] = building_value(2, pcity, val);
+            values[id] = building_value(2, pcity, val);
           else {
-            values[i] = building_value(4, pcity, val) - building_value(1, pcity, val);
-            values[i] += building_value(2, pcity, val) - building_value(1, pcity, val);
+            values[id] = building_value(4, pcity, val) - building_value(1, pcity, val);
+            values[id] += building_value(2, pcity, val) - building_value(1, pcity, val);
 /* The += has nothing to do with oracle, just the tech_Want of mysticism! */
           }
         } else {
           if (get_invention(pplayer, game.rtech.temple_plus) != TECH_KNOWN) {
-            values[i] = building_value(2, pcity, val) - building_value(1, pcity, val);
-            values[i] += building_value(2, pcity, val) - building_value(1, pcity, val);
+            values[id] = building_value(2, pcity, val) - building_value(1, pcity, val);
+            values[id] += building_value(2, pcity, val) - building_value(1, pcity, val);
           }
         }
       }
           
-      if (i == B_PYRAMIDS && improvement_variant(B_PYRAMIDS)==0
+      if (id == B_PYRAMIDS && improvement_variant(B_PYRAMIDS)==0
 	  && !city_got_building(pcity, B_GRANARY))
-        values[i] = food * pcity->food_surplus; /* different tech req's */
-      if (i == B_SETI && !city_got_building(pcity, B_RESEARCH))
-        values[i] = sci/2;
-      if (i == B_SHAKESPEARE)
-        values[i] = building_value(pcity->size, pcity, val);
-      if (i == B_SUNTZU)
-        values[i] = bar;
-      if (i == B_WOMENS) {
+        values[id] = food * pcity->food_surplus; /* different tech req's */
+      if (id == B_SETI && !city_got_building(pcity, B_RESEARCH))
+        values[id] = sci/2;
+      if (id == B_SHAKESPEARE)
+        values[id] = building_value(pcity->size, pcity, val);
+      if (id == B_SUNTZU)
+        values[id] = bar;
+      if (id == B_WOMENS) {
         unit_list_iterate(pcity->units_supported, punit)
-          if (punit->unhappiness) values[i] += t * 2;
+          if (punit->unhappiness) values[id] += t * 2;
         unit_list_iterate_end;
       }
 
-      if ((i == B_APOLLO) && game.spacerace) {
-        values[i] = values[B_CAPITAL]+1;
+      if ((id == B_APOLLO) && game.spacerace) {
+        values[id] = values[B_CAPITAL]+1;
       }
 
       /* ignoring APOLLO, LIGHTHOUSE, MAGELLAN, MANHATTEN, STATUE, UNITED */
     }
   }
 
-  for (i=0;i<B_LAST;i++) {
-    if (values[i]) {
+  for (id=0;id<game.num_impr_types;id++) {
+    if (values[id]) {
       freelog(LOG_DEBUG, "%s wants %s with desire %d.",
-	      pcity->name, get_improvement_name(i), values[i]);
+	      pcity->name, get_improvement_name(id), values[id]);
     }
-    if (!is_wonder(i)) values[i] -= improvement_upkeep(pcity, i) * t;
-    values[i] *= 100;
-    if (!is_wonder(i)) { /* trying to buy fewer improvements */
-      values[i] *= SHIELD_WEIGHTING;
-      values[i] /= MORT;
+    if (!is_wonder(id)) values[id] -= improvement_upkeep(pcity, id) * t;
+    values[id] *= 100;
+    if (!is_wonder(id)) { /* trying to buy fewer improvements */
+      values[id] *= SHIELD_WEIGHTING;
+      values[id] /= MORT;
     }
-    j = improvement_value(i);
+    j = improvement_value(id);
 /* handle H_PROD here? -- Syela */
-    pcity->ai.building_want[i] = values[i] / j;
+    pcity->ai.building_want[id] = values[id] / j;
   }
 
   return;
-}  
+}
+
+/********************************************************************** 
+... this function should assign a value to choice and want and type, where 
+    want is a value between 1 and 100.
+    if want is 0 this advisor doesn't want anything
+***********************************************************************/
 
 void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				   struct ai_choice *choice)
 {
   struct government *g = get_gov_pplayer(pplayer);
-  int set, con, utid, want, iunit, dw;
+  int con, want, dw;
+  Unit_Type_id utid, iunit;
   struct ai_choice cur;
   int est_food = pcity->food_surplus + 2 * pcity->ppl_scientist + 2 * pcity->ppl_taxman; 
   int vans = 0;
@@ -538,8 +542,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 
   choice->choice = 0;
   choice->want   = 0;
-  choice->type   = 0;
-  set = city_get_settlers(pcity);
+  choice->type   = CT_NONE;
   con = map_get_continent(pcity->x, pcity->y);
 
   utid = best_role_unit(pcity, F_SETTLERS);
@@ -554,11 +557,11 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
       freelog(LOG_DEBUG, "%s (%d, %d) desires settlers with passion %d",
 	      pcity->name, pcity->x, pcity->y, want);
       choice->want = want;
-      choice->type = 1;
+      choice->type = CT_NONMIL;
       ai_choose_role_unit(pplayer, pcity, choice, F_SETTLERS, want);
     } else if (want < 0) { /* need boats to colonize! */
       choice->want = 0 - want;
-      choice->type = 1;
+      choice->type = CT_NONMIL;
       choice->choice = best_role_unit(pcity, F_SETTLERS); /* default */
       ai_choose_ferryboat(pplayer, pcity, choice);
     }
@@ -577,11 +580,11 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 	freelog(LOG_DEBUG, "%s (%d, %d) desires founders with passion %d",
 		pcity->name, pcity->x, pcity->y, want);
 	choice->want = want;
-	choice->type = 1;
+	choice->type = CT_NONMIL;
 	ai_choose_role_unit(pplayer, pcity, choice, F_CITIES, want);
       } else if (want < -choice->want) { /* need boats to colonize! */
 	choice->want = 0 - want;
-	choice->type = 1;
+	choice->type = CT_NONMIL;
 	choice->choice = best_role_unit(pcity, F_CITIES); /* default */
 	ai_choose_ferryboat(pplayer, pcity, choice);
       }
@@ -610,7 +613,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 /* is a lot less warmap generation than there would be otherwise. -- Syela */
       iunit = best_role_unit(pcity, F_CARAVAN);
       dw = pcity->ai.distance_to_wonder_city * 8 /
-	((iunit==U_LAST) ? 3 : get_unit_type(iunit)->move_rate);
+	((iunit==U_LAST) ? SINGLE_MOVE : get_unit_type(iunit)->move_rate);
       want -= dw;
       /* value of 8 is a total guess and could be wrong,
        * but it's better than 0 -- Syela */
@@ -618,7 +621,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
       if (can_build_unit_direct(pcity, iunit)) {
         if (want > choice->want) {
           choice->want = want;
-          choice->type = 1;
+          choice->type = CT_NONMIL;
 	  ai_choose_role_unit(pplayer, pcity, choice, F_CARAVAN, dw/2);
         }
       } else
@@ -632,7 +635,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
     choice->want = cur.want;
 /*    if (choice->want > 100) choice->want = 100; */
 /* want > 100 means BUY RIGHT NOW */
-    choice->type = 0;
+    choice->type = CT_BUILDING;
   }
 /* allowing buy of peaceful units after much testing -- Syela */
 
@@ -644,7 +647,7 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
     }
     if (iunit != U_LAST) {
       choice->want = 1;
-      choice->type = 1;
+      choice->type = CT_NONMIL;
       choice->choice = iunit;
     }
   }
