@@ -13,6 +13,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "combat.h"
 #include "game.h"
 #include "government.h"
 #include "log.h"
@@ -22,8 +23,6 @@
 #include "cityturn.h"
 #include "gotohand.h"		/* warmap has been redeployed */
 #include "settlers.h"
-#include "unitfunc.h"
-#include "unittools.h"		/* for get_defender, amazingly */
 
 #include "aicity.h"
 #include "aitools.h"
@@ -31,15 +30,12 @@
 
 #include "advmilitary.h"
 
-extern struct move_cost_map warmap;
-
 /********************************************************************** 
-... this function should assign a value to choice and want, where 
-    want is a value between 1 and 100.
-    if choice is A_NONE this advisor doesn't want any tech researched at
-    the moment
+ This function should assign a value to choice and want, where 
+ want is a value between 1 and 100.
+ if choice is A_NONE this advisor doesn't want any tech researched at
+ the moment
 ***********************************************************************/
-
 void military_advisor_choose_tech(struct player *pplayer,
 				  struct ai_choice *choice)
 {
@@ -48,8 +44,11 @@ void military_advisor_choose_tech(struct player *pplayer,
   /* this function haven't been implemented yet */
 }
 
+/********************************************************************** 
+Need positive feedback in m_a_c_b and bodyguard routines. -- Syela
+***********************************************************************/
 int assess_defense_quadratic(struct city *pcity)
-{ /* Need positive feedback in m_a_c_b and bodyguard routines. -- Syela */
+{
   int v, def, l;
   int igwall = 0; /* this can be an arg if needed, but seems unneeded */
   def = 0;
@@ -73,8 +72,11 @@ int assess_defense_quadratic(struct city *pcity)
   return(def * def);
 }
 
+/**************************************************************************
+one unit only, mostly for findjob; handling boats correctly 980803 -- Syela
+**************************************************************************/
 int assess_defense_unit(struct city *pcity, struct unit *punit, int igwall)
-{ /* one unit only, mostly for findjob; handling boats correctly 980803 -- Syela */
+{
   int v;
   v = get_defense_power(punit) * punit->hp *
       (is_sailing_unit(punit) ? 1 : get_unit_type(punit->type)->firepower);
@@ -88,8 +90,11 @@ int assess_defense_unit(struct city *pcity, struct unit *punit, int igwall)
   return(0);
 }
 
+/********************************************************************** 
+ Most of the time we don't need/want positive feedback. -- Syela
+***********************************************************************/
 static int assess_defense_backend(struct city *pcity, int igwall)
-{ /* Most of the time we don't need/want positive feedback. -- Syela */
+{
   int def;
   def = 0;
   unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit)
@@ -101,16 +106,25 @@ static int assess_defense_backend(struct city *pcity, int igwall)
 /* unclear whether this should treat settlers/caravans as defense -- Syela */
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 int assess_defense(struct city *pcity)
 {
   return(assess_defense_backend(pcity, 0));
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 static int assess_defense_igwall(struct city *pcity)
 {
   return(assess_defense_backend(pcity, 1));
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 static int dangerfunct(int v, int m, int dist)
 {
 #ifdef OLDCODE
@@ -125,7 +139,7 @@ static int dangerfunct(int v, int m, int dist)
     v /= 2;
     dist -= m;
   }
-  m /= 3;
+  m /= SINGLE_MOVE;
   while (dist && dist >= m) {
     num *= 4;
     denom *= 5;
@@ -140,6 +154,9 @@ static int dangerfunct(int v, int m, int dist)
 #endif
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 static int assess_danger_unit(struct city *pcity, struct unit *punit)
 {
   int v, sailing;
@@ -159,30 +176,37 @@ static int assess_danger_unit(struct city *pcity, struct unit *punit)
   return(v);
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 static int assess_distance(struct city *pcity, struct unit *punit, int m,
 			   int boatid, int boatdist, int boatspeed)
 {
   int x, y, dist;
-  if (is_tiles_adjacent(punit->x, punit->y, pcity->x, pcity->y)) dist = 3;
+
+  if (same_pos(punit->x, punit->y, pcity->x, pcity->y))
+    return 0;
+
+  if (is_tiles_adjacent(punit->x, punit->y, pcity->x, pcity->y)) dist = SINGLE_MOVE;
   else if (is_sailing_unit(punit)) dist = warmap.seacost[punit->x][punit->y];
   else if (!is_ground_unit(punit))
-    dist = real_map_distance(punit->x, punit->y, pcity->x, pcity->y) * 3;
+    dist = real_map_distance(punit->x, punit->y, pcity->x, pcity->y) * SINGLE_MOVE;
   else if (unit_flag(punit->type, F_IGTER))
     dist = real_map_distance(punit->x, punit->y, pcity->x, pcity->y);
   else dist = warmap.cost[punit->x][punit->y];
 /* if dist = 9, a chariot is 1.5 turns away.  NOT 2 turns away. */
 /* Samarkand bug should be obsoleted by re-ordering of events */
-  if (dist < 3) dist = 3;
+  if (dist < SINGLE_MOVE) dist = SINGLE_MOVE;
 
   if (is_ground_unit(punit) && boatid &&
       find_beachhead(punit, pcity->x, pcity->y, &x, &y)) {
 /* this bug is so obvious I can't believe it wasn't discovered sooner. -- Syela */
     y = warmap.seacost[punit->x][punit->y];
     if (y >= 6 * THRESHOLD)
-      y = real_map_distance(pcity->x, pcity->y, punit->x, punit->y) * 3;
+      y = real_map_distance(pcity->x, pcity->y, punit->x, punit->y) * SINGLE_MOVE;
     x = MAX(y, boatdist) * m / boatspeed;
     if (dist > x) dist = x;
-    if (dist < 3) dist = 3;
+    if (dist < SINGLE_MOVE) dist = SINGLE_MOVE;
   }
   return(dist);
 }
@@ -204,7 +228,8 @@ void assess_danger_player(struct player *pplayer)
 ***********************************************************************/
 int assess_danger(struct city *pcity)
 {
-  int i, danger = 0, v, dist, con, m;
+  int i, danger = 0, v, dist, m;
+  Unit_Type_id utype;
   int danger2 = 0; /* linear for walls */
   int danger3 = 0; /* linear for coastal */
   int danger4 = 0; /* linear for SAM */
@@ -224,7 +249,6 @@ int assess_danger(struct city *pcity)
 
   memset(&virtualunit, 0, sizeof(struct unit));
   pplayer = &game.players[pcity->owner];
-  con = map_get_continent(pcity->x, pcity->y); /* Not using because of boats */
 
   generate_warmap(pcity, 0); /* generates both land and sea maps */
 
@@ -253,10 +277,10 @@ int assess_danger(struct city *pcity)
           virtualunit.owner = i;
           virtualunit.x = acity->x;
           virtualunit.y = acity->y;
-          v = acity->currently_building;
-          virtualunit.type = v;
-          virtualunit.veteran = do_make_unit_veteran(acity, v);
-          virtualunit.hp = unit_types[v].hp;
+          utype = acity->currently_building;
+          virtualunit.type = utype;
+          virtualunit.veteran = do_make_unit_veteran(acity, utype);
+          virtualunit.hp = unit_types[utype].hp;
 /* yes, I know cloning all this code is bad form.  I don't really
 want to write a funct that takes nine ints by reference. -- Syela */
           m = get_unit_type(funit->type)->move_rate;
@@ -347,19 +371,19 @@ content to let it remain that way for now. -- Syela 980805 */
 /* afraid *100 would create integer overflow, but *10 surely won't */
 
   if (danger < 0 || danger > 1<<24) /* I hope never to see this! */
-    freelog(LOG_NORMAL, "Dangerous danger (%d) in %s.  Beware of overflow.",
+    freelog(LOG_ERROR, "Dangerous danger (%d) in %s.  Beware of overflow.",
 	    danger, pcity->name);
   if (danger2 < 0 || danger2 > 1<<24) /* I hope never to see this! */
-    freelog(LOG_NORMAL, "Dangerous danger2 (%d) in %s.  Beware of overflow.",
+    freelog(LOG_ERROR, "Dangerous danger2 (%d) in %s.  Beware of overflow.",
 	    danger2, pcity->name);
   if (danger3 < 0 || danger3 > 1<<24) /* I hope never to see this! */
-    freelog(LOG_NORMAL, "Dangerous danger3 (%d) in %s.  Beware of overflow.",
+    freelog(LOG_ERROR, "Dangerous danger3 (%d) in %s.  Beware of overflow.",
 	    danger3, pcity->name);
   if (danger4 < 0 || danger4 > 1<<24) /* I hope never to see this! */
-    freelog(LOG_NORMAL, "Dangerous danger4 (%d) in %s.  Beware of overflow.",
+    freelog(LOG_ERROR, "Dangerous danger4 (%d) in %s.  Beware of overflow.",
 	    danger4, pcity->name);
   if (danger5 < 0 || danger5 > 1<<24) /* I hope never to see this! */
-    freelog(LOG_NORMAL, "Dangerous danger5 (%d) in %s.  Beware of overflow.",
+    freelog(LOG_ERROR, "Dangerous danger5 (%d) in %s.  Beware of overflow.",
 	    danger5, pcity->name);
   if (pcity->ai.grave_danger)
     urgency += 10; /* really, REALLY urgent to defend */
@@ -409,9 +433,12 @@ trying again, but this will require yet more tedious observation -- Syela */
   return(urgency);
 }
 
-int unit_desirability(int i, int def)
+/************************************************************************** 
+...
+**************************************************************************/
+int unit_desirability(Unit_Type_id i, int def)
 {
-  int cur, a, d;   
+  int cur, a, d;
   cur = get_unit_type(i)->hp;
   if (unit_types[i].move_type != SEA_MOVING || !def)
     cur *= get_unit_type(i)->firepower;
@@ -430,12 +457,22 @@ int unit_desirability(int i, int def)
   return(cur);  
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
+int unit_attack_desirability(Unit_Type_id i)
+{
+  return(unit_desirability(i, 0));
+} 
+
 static void process_defender_want(struct player *pplayer, struct city *pcity,
 				  int danger, struct ai_choice *choice)
 {
-  int i, j, k, l, m, n;
+  Unit_Type_id i;
+  Unit_Type_id bestid = 0; /* ??? Zero is legal value! (Settlers by default) */
+  Tech_Type_id tech_req;
+  int j, k, l, m, n;
   int best= 0;
-  int bestid = 0;
   int walls = city_got_citywalls(pcity);
   int desire[U_LAST]; /* what you get is what you seek */
   int shore = is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN);
@@ -462,7 +499,7 @@ static void process_defender_want(struct player *pplayer, struct city *pcity,
       } else if (k > 0 && (shore || m == LAND_MOVING) &&
                 unit_types[i].tech_requirement != A_LAST) {
         if (m == LAND_MOVING) { j *= pcity->ai.wallvalue; j /= 10; }
-        l = k * (k + pplayer->research.researchpoints) * game.techlevel /
+        l = k * (k + pplayer->research.researchpoints) * game.researchcost /
          (game.year > 0 ? 2 : 4); /* cost (shield equiv) of gaining these techs */
         l /= city_list_size(&pplayer->cities);
 /* Katvrr advises that with danger high, l should be weighted more heavily */
@@ -481,30 +518,35 @@ static void process_defender_want(struct player *pplayer, struct city *pcity,
 /* multiply by unit_types[bestid].build_cost / best */
   for (i = 0; i < game.num_unit_types; i++) {
     if (desire[i] && is_ai_simple_military(i)) {
-      j = unit_types[i].tech_requirement;
+      tech_req = unit_types[i].tech_requirement;
       n = desire[i] * unit_types[bestid].build_cost / best;
-      pplayer->ai.tech_want[j] += n; /* not the totally idiotic
-      pplayer->ai.tech_want[j] += n * pplayer->ai.tech_turns[j];  I amaze myself. -- Syela */
+      pplayer->ai.tech_want[tech_req] += n; /* not the totally idiotic
+      pplayer->ai.tech_want[tech_req] += n * pplayer->ai.tech_turns[tech_req];
+                                               I amaze myself. -- Syela */
       freelog(LOG_DEBUG, "%s wants %s for defense with desire %d <%d>",
-		    pcity->name, advances[j].name, n, desire[i]);
+		    pcity->name, advances[tech_req].name, n, desire[i]);
     }
   }
   choice->choice = bestid;
   choice->want = danger;
-  choice->type = 3;
+  choice->type = CT_DEFENDER;
   return;
 }
 
+/************************************************************************** 
+n is type of defender, vet is vet status, x,y is location of target
+I decided this funct wasn't confusing enough, so I made k_s_w send
+it some more variables for it to meddle with -- Syela
+**************************************************************************/
 static void process_attacker_want(struct player *pplayer,
-			    struct city *pcity, int b, int n,
+			    struct city *pcity, int b, Unit_Type_id n,
                             int vet, int x, int y, int unhap, int *e0, int *v,
                             int bx, int by, int boatspeed, int needferry)
 { 
-/* n is type of defender, vet is vet status, x,y is location of target */
-/* I decided this funct wasn't confusing enough, so I made k_s_w send
-it some more variables for it to meddle with -- Syela */
-  int a, c, d, e, i, a0, b0, f, g, fprime;
-  int j, k, l, m, q;
+  Unit_Type_id i;
+  Tech_Type_id j;
+  int a, c, d, e, a0, b0, f, g, fprime;
+  int k, l, m, q;
   int shore = is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN);
   struct city *acity = map_get_city(x, y);
   int movetype = unit_types[*v].move_type;
@@ -519,7 +561,7 @@ it some more variables for it to meddle with -- Syela */
          (k || !can_build_unit_direct(pcity, unit_types[i].obsoleted_by)) &&
          unit_types[i].attack_strength && /* otherwise we get SIGFPE's */
          m == movetype) { /* I don't think I want the duplication otherwise -- Syela */
-      l = k * (k + pplayer->research.researchpoints) * game.techlevel;
+      l = k * (k + pplayer->research.researchpoints) * game.researchcost;
       if (game.year > 0) l /= 2;
       else l /= 4; /* cost (shield equiv) of gaining these techs */
       l /= city_list_size(&pplayer->cities);
@@ -535,7 +577,7 @@ it some more variables for it to meddle with -- Syela */
 
       m = unit_types[i].move_rate;
       q = (acity ? 1 : unit_types[n].move_rate * (unit_flag(n, F_IGTER) ? 3 : 1));
-      if (unit_flag(i, F_IGTER)) m *= 3; /* not quite right */
+      if (unit_flag(i, F_IGTER)) m *= SINGLE_MOVE; /* not quite right */
       if (unit_types[i].move_type == LAND_MOVING) {
         if (boatspeed) { /* has to be a city, so don't bother with q */
           c = (warmap.cost[bx][by] + m - 1) / m + 1 +
@@ -546,8 +588,8 @@ it some more variables for it to meddle with -- Syela */
       } else if (unit_types[i].move_type == SEA_MOVING) {
         if (warmap.seacost[x][y] <= m) c = 1;
         else c = (warmap.seacost[x][y] * q + m - 1) / m;
-      } else if (real_map_distance(pcity->x, pcity->y, x, y) * 3 <= m) c = 1;
-      else c = real_map_distance(pcity->x, pcity->y, x, y) * 3 * q / m;
+      } else if (real_map_distance(pcity->x, pcity->y, x, y) * SINGLE_MOVE <= m) c = 1;
+      else c = real_map_distance(pcity->x, pcity->y, x, y) * SINGLE_MOVE * q / m;
 
       m = get_virtual_defense_power(i, n, x, y);
       m *= unit_types[n].hp * unit_types[n].firepower;
@@ -617,28 +659,29 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
 				struct unit *myunit, struct ai_choice *choice)
 {
   int a, b, c, d, e, f, g; /* variables in the attacker-want equation */
-  int m, n, vet, dist, v, a0, b0, fprime;
+  Unit_Type_id v, n;
+  int m, vet, dist, a0, b0, fprime;
   int x, y, unhap = 0;
   struct unit *pdef, *aunit, *ferryboat;
   struct city *acity;
-  int boatid = 0, harborcity = 0, bx, by;
+  int boatid = 0, bx = 0, by = 0;
   int needferry = 0, fstk, boatspeed, sanity;
 
   if (pcity->ai.danger && !assess_defense(pcity)) return;
 
   if (is_ground_unit(myunit)) boatid = find_boat(pplayer, &bx, &by, 2);
-  if (is_ground_unit(myunit) && !myunit->id &&
-      is_terrain_near_tile(myunit->x, myunit->y, T_OCEAN))
-    harborcity++;
 
   ferryboat = player_find_unit_by_id(pplayer, boatid);
-  if (ferryboat) boatspeed = (unit_flag(ferryboat->type, F_TRIREME) ? 6 : 12);
-  else boatspeed = (get_invention(pplayer, game.rtech.nav) != TECH_KNOWN ? 6 : 12);
+  /* FIXME: hardcoded boat speed */
+ if (ferryboat) boatspeed = (unit_flag(ferryboat->type, F_TRIREME)
+			     ? 2*SINGLE_MOVE : 4*SINGLE_MOVE);
+  else boatspeed = (get_invention(pplayer, game.rtech.nav) != TECH_KNOWN
+		    ? 2*SINGLE_MOVE : 4*SINGLE_MOVE);
 
   fstk = find_something_to_kill(pplayer, myunit, &x, &y);
 
   acity = map_get_city(x, y);
-  if (!acity) aunit = get_defender(pplayer, myunit, x, y);
+  if (!acity) aunit = get_defender(myunit, x, y);
   else aunit = 0;
   if (acity && acity->owner == pplayer->player_no) acity = 0;
 
@@ -658,12 +701,12 @@ before the 1.7.0 release so I'm letting this stay ugly. -- Syela */
     a *= a;
 
     if (acity) {
-      pdef = get_defender(pplayer, myunit, x, y);
+      pdef = get_defender(myunit, x, y);
 
       m = unit_types[v].move_rate;
       if (unit_flag(v, F_IGTER)) m *= 3; /* not quite right */
 
-      sanity = (goto_is_sane(pplayer, myunit, acity->x, acity->y, 1) &&
+      sanity = (goto_is_sane(myunit, acity->x, acity->y, 1) &&
               warmap.cost[x][y] <= (MIN(6, m) * THRESHOLD));
 
       if (is_ground_unit(myunit)) {
@@ -677,7 +720,7 @@ before the 1.7.0 release so I'm letting this stay ugly. -- Syela */
         } else c = (warmap.cost[acity->x][acity->y] + m - 1) / m;
       } else if (is_sailing_unit(myunit))
         c = (warmap.seacost[acity->x][acity->y] + m - 1) / m;
-      else c = real_map_distance(myunit->x, myunit->y, acity->x, acity->y) * 3 / m;
+      else c = real_map_distance(myunit->x, myunit->y, acity->x, acity->y) * SINGLE_MOVE / m;
 
       n = ai_choose_defender_versus(acity, v);
       m = get_virtual_defense_power(v, n, x, y);
@@ -720,7 +763,6 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
     } /* end dealing with cities */
 
     else {
-      m = 0;
       pdef = aunit; /* we KNOW this is the get_defender -- Syela */
 
       m = unit_types[pdef->type].build_cost;
@@ -729,7 +771,7 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
 
       if (is_ground_unit(myunit)) dist = warmap.cost[x][y];
       else if (is_sailing_unit(myunit)) dist = warmap.seacost[x][y];
-      else dist = real_map_distance(pcity->x, pcity->y, x, y) * 3;
+      else dist = real_map_distance(pcity->x, pcity->y, x, y) * SINGLE_MOVE;
       if (dist > m) {
         dist *= unit_types[pdef->type].move_rate;
         if (unit_flag(pdef->type, F_IGTER)) dist *= 3;
@@ -821,11 +863,11 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
           choice->choice = B_BARRACKS2;
         else choice->choice = B_BARRACKS;
         choice->want = e;
-        choice->type = 0;
+        choice->type = CT_BUILDING;
       } else {
         if (!myunit->id) {
           choice->choice = v;
-          choice->type = 2; /* type == 2 identifies attackers */
+          choice->type = CT_ATTACKER;
           choice->want = e;
           if (needferry) ai_choose_ferryboat(pplayer, pcity, choice);
 	  freelog(LOG_DEBUG, "%s has chosen attacker, %s",
@@ -834,16 +876,16 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
           choice->choice = ai_choose_defender(pcity);
 	  freelog(LOG_DEBUG, "%s has chosen defender, %s",
 			pcity->name, unit_types[choice->choice].name);
-          choice->type = 1;
+          choice->type = CT_NONMIL;
           choice->want = e;
         }
         if (is_sailing_unit(myunit) && improvement_exists(B_PORT)
 	    && !city_got_building(pcity, B_PORT)) {
-	  int tech = get_improvement_type(B_PORT)->tech_req;
+	  Tech_Type_id tech = get_improvement_type(B_PORT)->tech_req;
           if (get_invention(pplayer, tech) == TECH_KNOWN) {
             choice->choice = B_PORT;
             choice->want = e;
-            choice->type = 0;
+            choice->type = CT_BUILDING;
           } else pplayer->ai.tech_want[tech] += e;
         }
       }
@@ -881,12 +923,12 @@ static int port_is_within(struct player *pplayer, int d)
 ... this function should assign a value to choice and want and type, 
     where want is a value between 1 and 100.
     if want is 0 this advisor doesn't want anything
-    type = 1 means unit, type = 0 means building
 ***********************************************************************/
 void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				    struct ai_choice *choice)
 {
-  int def, danger, v, urgency, want;
+  Unit_Type_id v;
+  int def, danger, urgency, want;
   struct unit *myunit = 0;
   struct tile *ptile = map_get_tile(pcity->x, pcity->y);
   struct unit virtualunit;
@@ -894,7 +936,7 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
   struct unit *aunit = 0;
   choice->choice = 0;
   choice->want   = 0;
-  choice->type   = 0;
+  choice->type   = CT_NONE;
 
 /* TODO: recognize units that can DEFEND_HOME but are in the field. -- Syela */
 
@@ -905,24 +947,15 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
   freelog(LOG_DEBUG, "Assessed danger for %s = %d, Def = %d",
 	  pcity->name, danger, def);
 
-  if (pcity->ai.diplomat_threat) {
-  /* if I put a && (danger < def), if def = 0 and there is a little enemy unit
-     near, ai will ignore the diplomat threat.
-     On the other hand, if there is a joined attach of diplomats and
-     classic military units , ai will build a diplomat as last defender.
-     With the actual functions asses_danger and assess_danger_unit,
-     I can't know in which case I am.
-     The confusion is between the actual threat and the virtual threat. 
-     How many units may I loose at this turn and
-     how many units may I loose during next turns? (17/12/98) (--NB)
-  */
+  if ((pcity->ai.diplomat_threat) && (def)){
+  /* It's useless to build a diplomat as the last defender of a town. --nb */ 
 
-    int u = best_role_unit(pcity, F_DIPLOMAT);
+    Unit_Type_id u = best_role_unit(pcity, F_DIPLOMAT);
     if (u<U_LAST) {
        freelog(LOG_DEBUG, "A diplomat will be built in city %s.", pcity->name);
        choice->want = 16000; /* diplomat more important than soldiers */ 
        pcity->ai.urgency = 1;
-       choice->type = 3; /* defender */
+       choice->type = CT_DEFENDER;
        choice->choice = u;
        return;
     } else if (num_role_units(F_DIPLOMAT)>0) {
@@ -955,7 +988,7 @@ creating any other problems that are worse. -- Syela */
       choice->choice = B_CITY; /* great wall is under domestic */
       choice->want = pcity->ai.building_want[B_CITY]; /* hacked by assess_danger */
       if (!urgency && choice->want > 100) choice->want = 100;
-      choice->type = 0;
+      choice->type = CT_BUILDING;
     } else if (pcity->ai.building_want[B_COASTAL] && def &&
         can_build_improvement(pcity, B_COASTAL) &&
         (danger < 101 || unit_list_size(&ptile->units) > 1) &&
@@ -963,7 +996,7 @@ creating any other problems that are worse. -- Syela */
       choice->choice = B_COASTAL; /* great wall is under domestic */
       choice->want = pcity->ai.building_want[B_COASTAL]; /* hacked by assess_danger */
       if (!urgency && choice->want > 100) choice->want = 100;
-      choice->type = 0;
+      choice->type = CT_BUILDING;
     } else if (pcity->ai.building_want[B_SAM] && def &&
         can_build_improvement(pcity, B_SAM) &&
         (danger < 101 || unit_list_size(&ptile->units) > 1) &&
@@ -971,7 +1004,7 @@ creating any other problems that are worse. -- Syela */
       choice->choice = B_SAM; /* great wall is under domestic */
       choice->want = pcity->ai.building_want[B_SAM]; /* hacked by assess_danger */
       if (!urgency && choice->want > 100) choice->want = 100;
-      choice->type = 0;
+      choice->type = CT_BUILDING;
     } else if (danger > 0 && unit_list_size(&ptile->units) <= urgency) {
       process_defender_want(pplayer, pcity, danger, choice);
       if (!urgency && unit_types[choice->choice].defense_strength == 1) {
@@ -1005,7 +1038,7 @@ the intrepid David Pfitzner discovered was in error. -- Syela */
     if (want > choice->want) {
       choice->want = want;
       choice->choice = v;
-      choice->type = 1;
+      choice->type = CT_NONMIL; /* Why not CT_DEFENDER? -- Caz */
     }
   }
 
@@ -1043,9 +1076,10 @@ the intrepid David Pfitzner discovered was in error. -- Syela */
 
 void establish_city_distances(struct player *pplayer, struct city *pcity)
 {
-  int dist = 0;
+  int dist;
   int wondercity;
-  int freight;
+  Unit_Type_id freight;
+  int moverate;
 /* at this moment, our warmap is intact.  we need to do two things: */
 /* establish faraway for THIS city, and establish d_t_w_c for ALL cities */
 
@@ -1053,14 +1087,14 @@ void establish_city_distances(struct player *pplayer, struct city *pcity)
     wondercity = map_get_continent(pcity->x, pcity->y);
   else wondercity = 0;
   freight = best_role_unit(pcity, F_CARAVAN);
-  freight = (freight==U_LAST) ? 3 : get_unit_type(freight)->move_rate;
+  moverate = (freight==U_LAST) ? SINGLE_MOVE : get_unit_type(freight)->move_rate;
 
   pcity->ai.downtown = 0;
   city_list_iterate(pplayer->cities, othercity)
     dist = warmap.cost[othercity->x][othercity->y];
     if (wondercity && map_get_continent(othercity->x, othercity->y) == wondercity)
       othercity->ai.distance_to_wonder_city = dist;
-    dist += freight - 1; dist /= freight;
+    dist += moverate - 1; dist /= moverate;
     pcity->ai.downtown += MAX(0, 5 - dist); /* four three two one fire */
   city_list_iterate_end;
   return;
