@@ -14,11 +14,13 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "city.h"
+#include "combat.h"
 #include "events.h"
 #include "fcintl.h"
 #include "game.h"
@@ -36,7 +38,6 @@
 #include "gotohand.h"
 #include "plrhand.h"
 #include "settlers.h"
-#include "unitfunc.h"
 #include "unithand.h"
 #include "unittools.h"
 
@@ -50,11 +51,13 @@
 
 #include "aicity.h"
 
+static void ai_manage_city(struct player *pplayer, struct city *pcity);
+     
 #ifdef UNUSED
 /************************************************************************** 
 ...
 **************************************************************************/
-void ai_do_build_unit(struct city *pcity, int unit_type)
+void ai_do_build_unit(struct city *pcity, Unit_Type_id unit_type)
 {
   pcity->is_building_unit = 1;
   pcity->currently_building = unit_type;
@@ -78,7 +81,8 @@ void ai_city_build_settler(struct city *pcity)
 **************************************************************************/
 int ai_city_build_peaceful_unit(struct city *pcity)
 {
-  int i;
+  Unit_Type_id i;
+
   if (is_building_other_wonder(pcity)) {
     i = best_role_unit(pcity, F_CARAVAN);
     if (i < U_LAST) {
@@ -102,7 +106,9 @@ int ai_city_build_peaceful_unit(struct city *pcity)
 static void ai_manage_buildings(struct player *pplayer)
 { /* we have just managed all our cities but not chosen build for them yet */
   struct government *g = get_gov_pplayer(pplayer);
-  int i, j, values[B_LAST], leon = 0, palace = 0, corr = 0;
+  Impr_Type_id i;
+  Tech_Type_id j;
+  int values[B_LAST], leon = 0, palace = 0, corr = 0;
   memset(values, 0, sizeof(values));
   memset(pplayer->ai.tech_want, 0, sizeof(pplayer->ai.tech_want));
 
@@ -110,7 +116,7 @@ static void ai_manage_buildings(struct player *pplayer)
   city_list_iterate(pplayer->cities, pcity)
     ai_eval_buildings(pcity);
     if (!palace) corr += pcity->corruption * 8;
-    for (i = 0; i < B_LAST; i++) 
+    for (i = 0; i < game.num_impr_types; i++) 
       if (pcity->ai.building_want[i] > 0) values[i] += pcity->ai.building_want[i];
     if (pcity->ai.building_want[B_LEONARDO] > leon)
       leon = pcity->ai.building_want[B_LEONARDO];
@@ -119,7 +125,7 @@ static void ai_manage_buildings(struct player *pplayer)
 /* this is a weird place to iterate a units list! */
   unit_list_iterate(pplayer->units, punit)
     if (is_sailing_unit(punit))
-      values[B_MAGELLAN] += unit_types[punit->type].build_cost * 6 /
+      values[B_MAGELLAN] += unit_types[punit->type].build_cost * 2 * SINGLE_MOVE /
                             unit_types[punit->type].move_rate;
   unit_list_iterate_end;
   values[B_MAGELLAN] *= 100 * SHIELD_WEIGHTING;
@@ -131,7 +137,7 @@ static void ai_manage_buildings(struct player *pplayer)
    * regression testing --dwp */
   if (g->index != game.default_government
       && g->index != game.government_when_anarchy) {
-    for (i = 0; i < B_LAST; i++) {
+    for (i = 0; i < game.num_impr_types; i++) {
       j = improvement_types[i].tech_req;
       if (get_invention(pplayer, j) != TECH_KNOWN)
         pplayer->ai.tech_want[j] += values[i];
@@ -163,7 +169,7 @@ static void ai_manage_buildings(struct player *pplayer)
   city_list_iterate_end;
 
   city_list_iterate(pplayer->cities, pcity) /* wonder-kluge */
-    for (i = 0; i < B_LAST; i++) {
+    for (i = 0; i < game.num_impr_types; i++) {
       if (!pcity->is_building_unit && is_wonder(i) &&
           is_wonder(pcity->currently_building))
         pcity->ai.building_want[i] += pcity->shield_stock / 2;
@@ -180,10 +186,11 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
 
   bestchoice.choice = A_NONE;      
   bestchoice.want   = 0;
-  bestchoice.type   = 0;
+  bestchoice.type   = CT_NONE;
 
   if( is_barbarian(pplayer) ) {    /* always build best attack unit */
-    int i, iunit, bestunit = -1, bestattack = 0;
+    Unit_Type_id i, iunit, bestunit = -1;
+    int bestattack = 0;
     for(i = 0; i < num_role_units(L_BARBARIAN_BUILD); i++) {
       iunit = get_role_unit(L_BARBARIAN_BUILD, i);
       if (get_unit_type(iunit)->attack_strength > bestattack) {
@@ -202,7 +209,7 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
     if (bestunit != -1) {
       bestchoice.choice = bestunit;
       bestchoice.want   = 101;
-      bestchoice.type   = 2;
+      bestchoice.type   = CT_ATTACKER;
     }
     else {
       freelog(LOG_VERBOSE, "Barbarians don't know what to build!\n");
@@ -227,22 +234,25 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
     pcity->ai.choice.type = bestchoice.type; /* instead of the one atop the list */
   }
 
-/* type 0 = building, 1 = non-mil unit, 2 = attacker, 3 = defender */
-
   if (bestchoice.want) { /* Note - on fallbacks, will NOT get stopped building msg */
+
+    ASSERT_REAL_CHOICE_TYPE(bestchoice.type);
+
     freelog(LOG_DEBUG, "%s wants %s with desire %d.", pcity->name,
-		  (bestchoice.type ? unit_name(bestchoice.choice)
-		   : get_improvement_name(bestchoice.choice)),
+		  (is_unit_choice_type(bestchoice.type) ?
+                   unit_name(bestchoice.choice) :
+		   get_improvement_name(bestchoice.choice)),
 		  bestchoice.want);
     if(!pcity->is_building_unit && is_wonder(pcity->currently_building) &&
-      (bestchoice.type || bestchoice.choice != pcity->currently_building))
+       (is_unit_choice_type(bestchoice.type) ||
+        bestchoice.choice != pcity->currently_building))
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STOPPED,
                    _("Game: The %s have stopped building The %s in %s."),
                    get_nation_name_plural(pplayer->nation),
                    get_impr_name_ex(pcity, pcity->currently_building),
                    pcity->name);
 
-    if (!bestchoice.type && (pcity->is_building_unit ||
+    if (bestchoice.type == CT_BUILDING && (pcity->is_building_unit ||
                  pcity->currently_building != bestchoice.choice) &&
                  is_wonder(bestchoice.choice)) {
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STARTED,
@@ -250,12 +260,12 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
                     get_nation_name_plural(city_owner(pcity)->nation),
                     get_impr_name_ex(pcity, bestchoice.choice), pcity->name);
       pcity->currently_building = bestchoice.choice;
-      pcity->is_building_unit    = (bestchoice.type > 0);
+      pcity->is_building_unit    = is_unit_choice_type(bestchoice.type);
       generate_warmap(pcity, 0); /* need to establish distance to wondercity */
       establish_city_distances(pplayer, pcity); /* for caravans in other cities */
     } else {
       pcity->currently_building = bestchoice.choice;
-      pcity->is_building_unit    = (bestchoice.type > 0);
+      pcity->is_building_unit   = is_unit_choice_type(bestchoice.type);
     }
 
     return;
@@ -280,7 +290,8 @@ I haven't seen them, but I want to somewhat prepare for them anyway. -- Syela */
 ...
 **************************************************************************/
 #ifdef GRAVEDANGERWORKS
-static int ai_city_defender_value(struct city *pcity, int a_type, int d_type)
+static int ai_city_defender_value(struct city *pcity, Unit_Type_id a_type,
+                                  Unit_Type_id d_type)
 {
   int m;
   m = get_virtual_defense_power(a_type, d_type, pcity->x, pcity->y);
@@ -296,8 +307,8 @@ static int ai_city_defender_value(struct city *pcity, int a_type, int d_type)
 **************************************************************************/
 static void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
 {
-  int id;
-  for (id = 0; id < B_LAST; id++) {
+  Impr_Type_id id;
+  for (id = 0; id < game.num_impr_types; id++) {
     if (can_sell_building(pcity, id) && id != B_CITY) {
 /* selling walls to buy defenders is counterproductive -- Syela */
       really_handle_city_sell(pplayer, pcity, id);
@@ -311,7 +322,8 @@ static void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
 **************************************************************************/
 static void ai_new_spend_gold(struct player *pplayer)
 {
-  int buycost, id, cost, frugal = 0, trireme=0;
+  Unit_Type_id id;
+  int buycost, cost, frugal = 0, trireme=0;
   int total, build, did_upgrade, reserve;
   struct ai_choice bestchoice;
   struct city *pcity = NULL;
@@ -323,7 +335,7 @@ static void ai_new_spend_gold(struct player *pplayer)
       break;
 
     bestchoice.want = 0;
-    bestchoice.type = 0;
+    bestchoice.type = CT_NONE;
     bestchoice.choice = 0;
     city_list_iterate(pplayer->cities, acity)
       if (acity->anarchy) continue;
@@ -336,8 +348,12 @@ static void ai_new_spend_gold(struct player *pplayer)
     city_list_iterate_end;
 
     if (!bestchoice.want) break;
+
+    ASSERT_REAL_CHOICE_TYPE(bestchoice.type);
  
-    if (bestchoice.type && (!frugal || bestchoice.want > 200)) { /* if we want a unit */
+    if (is_unit_choice_type(bestchoice.type) &&
+        (!frugal || bestchoice.want > 200)) { /* if we want a unit */
+
       buycost = city_buy_cost(pcity);
       unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit)
         id = can_upgrade_unittype(pplayer, punit->type);
@@ -380,10 +396,8 @@ static void ai_new_spend_gold(struct player *pplayer)
 		   unit_types[bestchoice.choice].name);
               notify_player(pplayer, _("Game: %s disbanded in %s."),
 			    unit_types[punit->type].name, pcity->name);
-	      /* FIXME: should be handled in server... */
-              pcity->shield_stock+=(get_unit_type(punit->type)->build_cost/2);
-              send_city_info(pplayer, pcity);
-              wipe_unit_safe(punit, &myiter);
+
+              do_unit_disband_safe(pcity, punit, &myiter);
             }
           }
         }
@@ -398,7 +412,7 @@ static void ai_new_spend_gold(struct player *pplayer)
                buycost >= 200) ; /* wait for more vans */
       else if (bestchoice.type && unit_flag(bestchoice.choice, F_SETTLERS) &&
           !city_got_effect(pcity, B_GRANARY) && (pcity->size < 2 ||
-         pcity->food_stock < pcity->size * game.foodbox)) ;
+         pcity->food_stock < city_granary_size(pcity->size-1))) ;
       else if (bestchoice.type && bestchoice.type < 3 && /* not a defender */
         buycost > unit_types[bestchoice.choice].build_cost * 2) { /* too expensive */
         if (unit_flag(bestchoice.choice, F_CARAVAN) &&
@@ -438,10 +452,14 @@ static void ai_new_spend_gold(struct player *pplayer)
           } else if (buycost > pplayer->ai.maxbuycost) pplayer->ai.maxbuycost = buycost;
 /* possibly upgrade units here */
         } /* end panic subroutine */
-        if (bestchoice.type && unit_flag(bestchoice.choice, F_CARAVAN))
-            if (buycost > pplayer->ai.maxbuycost) pplayer->ai.maxbuycost = buycost;
+        if (is_unit_choice_type(bestchoice.type) &&
+            unit_flag(bestchoice.choice, F_CARAVAN)) {
+          if (buycost > pplayer->ai.maxbuycost)
+            pplayer->ai.maxbuycost = buycost;
 /* Gudy reminded me AI was slow to build wonders, I thought of the above -- Syela */
-        if (!bestchoice.type) switch (bestchoice.choice) {
+        }
+        if (bestchoice.type == CT_BUILDING) {
+          switch (bestchoice.choice) {
           case B_AQUEDUCT:
             if (city_happy(pcity)) break;
         /* PASS THROUGH */
@@ -456,7 +474,8 @@ static void ai_new_spend_gold(struct player *pplayer)
             if (buycost > pplayer->ai.maxbuycost) pplayer->ai.maxbuycost = buycost;
             break;
 /* granaries/harbors/wonders not worth the tax increase */
-        } /* end switch */
+          } /* end switch */
+        }
 /* I just saw Alexander save up for walls then buy a granary.  Never again! -- Syela */
 /*        if (bestchoice.want > 200 && pplayer->ai.maxbuycost) frugal++; */
         if (pplayer->ai.maxbuycost) frugal++; /* much more frugal -- Syela */
@@ -471,7 +490,7 @@ static void ai_new_spend_gold(struct player *pplayer)
     trireme = get_role_unit(F_TRIREME, 0);
     id = can_upgrade_unittype(pplayer, trireme);
   }
-  if (id > 0 && !frugal) {
+  if (id >= 0 && !frugal) {
     unit_list_iterate(pplayer->units, punit)
       if (punit->type != trireme) continue;
       pcity = map_get_city(punit->x, punit->y);
@@ -548,8 +567,9 @@ we don't rely on the seamap being current since we will recalculate. -- Syela */
 int city_get_buildings(struct city *pcity)
 {
   int b=0;
-  int i;
-  for (i=0; i<B_LAST; i++) {
+  Impr_Type_id i;
+
+  for (i=0; i<game.num_impr_types; i++) {
     if (is_wonder(i)) continue;
     if (i==B_PALACE)  continue;
     if (city_got_building(pcity, i))
@@ -566,18 +586,18 @@ static int worst_elvis_tile(struct city *pcity, int x, int y, int bx, int by,
 {
   int a, b;
   a = city_tile_value(pcity, x, y,
-      foodneed + get_food_tile(x, y, pcity),
-      prodneed + get_shields_tile(x, y, pcity));
+		      foodneed + city_get_food_tile(x, y, pcity),
+		      prodneed + city_get_shields_tile(x, y, pcity));
   b = city_tile_value(pcity, bx, by,
-      foodneed + get_food_tile(bx, by, pcity),
-      prodneed + get_shields_tile(bx, by, pcity));
+		      foodneed + city_get_food_tile(bx, by, pcity),
+		      prodneed + city_get_shields_tile(bx, by, pcity));
   return (a < b);
 }
 
 /************************************************************************** 
 ...
 **************************************************************************/
-static int is_defender_unit(int unit_type) 
+static int is_defender_unit(Unit_Type_id unit_type) 
 {
   return unit_has_role(unit_type, L_DEFEND_GOOD)
       || unit_has_role(unit_type, L_DEFEND_OK);
@@ -602,6 +622,7 @@ int city_get_defenders(struct city *pcity)
   return def;
 }
 
+#ifdef UNUSED
 /************************************************************************** 
 ...
 **************************************************************************/
@@ -615,23 +636,18 @@ int city_get_settlers(struct city *pcity)
   unit_list_iterate_end;
   return set;
 }
+#endif
 
+#ifdef UNUSED
 /************************************************************************** 
-...
+... (unused)
 **************************************************************************/
 int ai_in_initial_expand(struct player *pplayer)
 {
   int expand_cities [3] = {3, 5, 7};
   return (pplayer->score.cities < expand_cities[get_nation_by_plr(pplayer)->expand]);  
 }
-
-/************************************************************************** 
-...
-**************************************************************************/
-int unit_attack_desirability(int i)
-{
-  return(unit_desirability(i, 0));
-} 
+#endif
 
 void ai_choose_ferryboat(struct player *pplayer, struct city *pcity, struct ai_choice *choice)
 {
@@ -639,12 +655,14 @@ void ai_choose_ferryboat(struct player *pplayer, struct city *pcity, struct ai_c
 }
 
 /* don't ask me why this is in aicity, I can't even remember -- Syela */
-static int ai_choose_attacker(struct city *pcity, enum unit_move_type which)
-{ 
-  int i;
+static Unit_Type_id ai_choose_attacker(struct city *pcity,
+                                       enum unit_move_type which)
+{
+  Unit_Type_id i;
+  Unit_Type_id bestid = 0; /* ??? Zero is legal value! (Settlers by default) */
   int best = 0;
-  int bestid = 0;
   int cur;
+
   for (i = 0; i < game.num_unit_types; i++) {
     /* not dealing with planes yet */
     if (!is_ai_simple_military(i)) continue;
@@ -660,21 +678,22 @@ static int ai_choose_attacker(struct city *pcity, enum unit_move_type which)
   return bestid;
 }
 
-int ai_choose_attacker_ground(struct city *pcity)
+Unit_Type_id ai_choose_attacker_ground(struct city *pcity)
 {
   return(ai_choose_attacker(pcity, LAND_MOVING));
 }
 
-int ai_choose_attacker_sailing(struct city *pcity)
+Unit_Type_id ai_choose_attacker_sailing(struct city *pcity)
 {
   return(ai_choose_attacker(pcity, SEA_MOVING));
 }
 
-int ai_choose_defender_versus(struct city *pcity, int v)
+Unit_Type_id ai_choose_defender_versus(struct city *pcity, Unit_Type_id v)
 {
-  int i, j, m;
+  Unit_Type_id i;
+  Unit_Type_id bestid = 0; /* ??? Zero is legal value! (Settlers by default) */
+  int j, m;
   int best= 0;
-  int bestid = 0;
 
   for (i = 0; i < game.num_unit_types; i++) {
     if (!is_ai_simple_military(i)) continue;
@@ -722,11 +741,13 @@ int has_a_normal_defender(struct city *pcity)
   return 0;
 }
 
-int ai_choose_defender_limited(struct city *pcity, int n, enum unit_move_type which)
+Unit_Type_id ai_choose_defender_limited(struct city *pcity, int n,
+                                        enum unit_move_type which)
 {
-  int i, j, m;
+  Unit_Type_id i;
+  Unit_Type_id bestid = 0; /* ??? Zero is legal value! (Settlers by default) */
+  int j, m;
   int best= 0;
-  int bestid = 0;
   int walls = 1; /* just assume city_got_citywalls(pcity); in the long run -- Syela */
   int isdef = has_a_normal_defender(pcity);
 
@@ -750,12 +771,13 @@ int ai_choose_defender_limited(struct city *pcity, int n, enum unit_move_type wh
   return bestid;
 }
 
-int ai_choose_defender_by_type(struct city *pcity, enum unit_move_type which)
+Unit_Type_id ai_choose_defender_by_type(struct city *pcity,
+                                        enum unit_move_type which)
 {
   return (ai_choose_defender_limited(pcity, pcity->shield_stock + 40, which));
 }
 
-int ai_choose_defender(struct city *pcity)
+Unit_Type_id ai_choose_defender(struct city *pcity)
 {
   return (ai_choose_defender_limited(pcity, pcity->shield_stock + 40, 0));
 }
@@ -777,7 +799,7 @@ void adjust_build_choice(struct player *pplayer, struct ai_choice *cur,
 ... 
 **************************************************************************/
 
-static int building_unwanted(struct player *plr, int i)
+static int building_unwanted(struct player *plr, Impr_Type_id i)
 {
   if (plr->research.researching != A_NONE)
     return 0;
@@ -790,9 +812,10 @@ static int building_unwanted(struct player *plr, int i)
 
 static void ai_sell_obsolete_buildings(struct city *pcity)
 {
-  int i;
+  Impr_Type_id i;
+
   struct player *pplayer = city_owner(pcity);
-  for (i=0;i<B_LAST;i++) {
+  for (i=0;i<game.num_impr_types;i++) {
     if(city_got_building(pcity, i) 
        && !is_wonder(i) 
        && i != B_CITY /* selling city walls is really, really dumb -- Syela */
@@ -810,13 +833,13 @@ static void ai_sell_obsolete_buildings(struct city *pcity)
 /**************************************************************************
  cities, build order and worker allocation stuff here..
 **************************************************************************/
-void ai_manage_city(struct player *pplayer, struct city *pcity)
+static void ai_manage_city(struct player *pplayer, struct city *pcity)
 {
-  city_check_workers(pplayer, pcity); /* no reason not to, many reasons to do so! */
   auto_arrange_workers(pcity);
   if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer,1))
     ai_scientists_taxmen(pcity);
   ai_sell_obsolete_buildings(pcity);
+  sync_cities();
 /* ai_city_choose_build(pplayer, pcity); -- moved by Syela */
 }
 
@@ -827,7 +850,7 @@ void ai_manage_city(struct player *pplayer, struct city *pcity)
 static int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
 {
   struct government *g = get_gov_pcity(pcity);
-  int x,y, foodneed, prodneed;
+  int foodneed, prodneed;
   int luxneed, pwr, e;
 
   foodneed=(pcity->size *2) + settler_eats(pcity);
@@ -864,10 +887,10 @@ static int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
 	}
       }
     }
-  }
+  } city_map_iterate_end;
   if (*xp == 0 && *yp == 0) return 0;
-  foodneed += get_food_tile(*xp, *yp, pcity);
-  prodneed += get_shields_tile(*xp, *yp, pcity);
+  foodneed += city_get_food_tile(*xp, *yp, pcity);
+  prodneed += city_get_shields_tile(*xp, *yp, pcity);
   if (e > 1) {
     foodneed -= (e - 1) * 2; /* forgetting these two lines */
     prodneed -= (e - 1); /* led to remarkable idiocy -- Syela */
@@ -887,17 +910,18 @@ static int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
 /**************************************************************************
 ...
 **************************************************************************/
-
 int ai_make_elvis(struct city *pcity)
 {
   int xp, yp, val;
   if ((val = ai_find_elvis_pos(pcity, &xp, &yp))) {
-    set_worker_city(pcity, xp, yp, C_TILE_EMPTY);
+    server_remove_worker_city(pcity, xp, yp);
     pcity->ppl_elvis++;
     city_refresh(pcity); /* this lets us call ai_make_elvis in luxury routine */
     return val; /* much more useful! */
   } else
     return 0;
+
+  sync_cities();
 }
 
 /**************************************************************************
@@ -913,13 +937,13 @@ static void make_elvises(struct city *pcity)
  
   while (1) {
     if ((elviscost = ai_find_elvis_pos(pcity, &xp, &yp))) {
-      if (get_food_tile(xp, yp, pcity) > pcity->food_surplus)
+      if (city_get_food_tile(xp, yp, pcity) > pcity->food_surplus)
 	break;
-      if (get_food_tile(xp, yp, pcity) == pcity->food_surplus && city_happy(pcity))
+      if (city_get_food_tile(xp, yp, pcity) == pcity->food_surplus && city_happy(pcity))
 	break; /* scientists don't party */
       if (elviscost >= 24) /* doesn't matter if we wtbb or not! */
         break; /* no benefit here! */
-      set_worker_city(pcity, xp, yp, C_TILE_EMPTY);
+      server_remove_worker_city(pcity, xp, yp);
       pcity->ppl_elvis++;
       city_refresh(pcity);
     } else
@@ -973,6 +997,8 @@ void ai_scientists_taxmen(struct city *pcity)
     make_taxmen(pcity);
   else
     make_scientists(pcity);
+
+  sync_cities();
 }
 
 /**************************************************************************
@@ -989,31 +1015,32 @@ int ai_fix_unhappy(struct city *pcity)
   return (!city_unhappy(pcity));
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
 { /* I don't care how slow this is; it will very rarely be used. -- Syela */
 /* this would be a lot easier if I made ->worked a city id. */
   struct city_list minilist;
   struct packet_unit_request pack;
-  int i, j;
-   
+
   freelog(LOG_VERBOSE, "Emergency in %s! (%d unhap, %d hap, %d food, %d prod)",
        pcity->name, pcity->ppl_unhappy[4], pcity->ppl_happy[4],
        pcity->food_surplus, pcity->shield_surplus);
   city_list_init(&minilist);
-  city_map_iterate(i, j) {
-    int x=map_adjust_x(pcity->x+i-2), y=map_adjust_y(pcity->y+j-2);
+  map_city_radius_iterate(pcity->x, pcity->y, x, y) {
     struct city *acity=map_get_tile(x,y)->worked;
     if(acity!=NULL && acity!=pcity && acity->owner==pcity->owner)  {
       if(acity->x==x && acity->y==y) /* can't stop working city center */
 	continue;  
       freelog(LOG_DEBUG, "Availing square in %s", acity->name);
-      set_worker_city(acity, map_to_city_x(acity, x), 
-                      map_to_city_y(acity, y), C_TILE_EMPTY);
+      server_remove_worker_city(acity, map_to_city_x(acity, x),
+				map_to_city_y(acity, y));
+      acity->ppl_elvis++;
       if (!city_list_find_id(&minilist, acity->id))
 	city_list_insert(&minilist, acity);
     }
-  }
-  city_check_workers(pplayer, pcity);
+  } map_city_radius_iterate_end;
   auto_arrange_workers(pcity);
   if (ai_fix_unhappy(pcity) && ai_fuzzy(pplayer,1))
     ai_scientists_taxmen(pcity);
@@ -1038,15 +1065,13 @@ void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
      }
   unit_list_iterate_end;       
 
-  city_list_iterate(minilist, acity)
+  city_list_iterate(minilist, acity) {
     city_refresh(acity); /* otherwise food total and stuff was wrong. -- Syela */
-    city_check_workers(pplayer, acity);
-    add_adjust_workers(acity);
-    city_refresh(acity);
+    auto_arrange_workers(pcity);
     if (ai_fix_unhappy(acity) && ai_fuzzy(pplayer,1))
       ai_scientists_taxmen(acity);
     freelog(LOG_DEBUG, "Readjusting workers in %s", acity->name);
-    /* FIXME: should be handled seperately, and in server... */
-    send_city_info(city_owner(acity), acity);
-  city_list_iterate_end;
+  } city_list_iterate_end;
+
+  sync_cities();
 }
