@@ -24,7 +24,6 @@
 #include "shared.h"
 #include "unit.h"
 
-#include "cityhand.h"
 #include "citytools.h"
 #include "cityturn.h"
 #include "maphand.h"
@@ -124,9 +123,9 @@ void copy_if_better_choice(struct ai_choice *cur, struct ai_choice *best)
 
 void ai_advisor_choose_building(struct city *pcity, struct ai_choice *choice)
 { /* I prefer the ai_choice as a return value; gcc prefers it as an arg -- Syela */
-  int i;
-  int id = B_LAST;
-  int prod = 0, danger = 0, downtown = 0, cities = 0;
+  Impr_Type_id i;
+  Impr_Type_id id = B_LAST;
+  int danger = 0, downtown = 0, cities = 0;
   int want=0;
   struct player *plr;
         
@@ -134,25 +133,20 @@ void ai_advisor_choose_building(struct city *pcity, struct ai_choice *choice)
      
   /* too bad plr->score isn't kept up to date. */
   city_list_iterate(plr->cities, acity)
-    prod += acity->shield_prod;
     danger += acity->ai.danger;
     downtown += acity->ai.downtown;
     cities++;
   city_list_iterate_end;
  
-  for(i=0; i<B_LAST; i++) {
+  for(i=0; i<game.num_impr_types; i++) {
     if (!is_wonder(i) ||
        (!pcity->is_building_unit && is_wonder(pcity->currently_building) &&
        pcity->shield_stock >= improvement_value(i) / 2) ||
        (!is_building_other_wonder(pcity) &&
-/* built_elsewhere removed; it was very, very bad with multi-continent empires */
-/* city_got_building(pcity, B_TEMPLE) && - too much to ask for, I think */
         !pcity->ai.grave_danger && /* otherwise caravans will be killed! */
-/*        pcity->shield_prod * cities >= prod &&         this shouldn't matter much */
         pcity->ai.downtown * cities >= downtown &&
         pcity->ai.danger * cities <= danger)) { /* too many restrictions? */
 /* trying to keep wonders in safe places with easy caravan access -- Syela */
-/* new code 980620; old code proved grossly inadequate after extensive testing -- Syela */
       if(pcity->ai.building_want[i]>want) {
 /* we have to do the can_build check to avoid Built Granary.  Now Building Granary. */
         if (can_build_improvement(pcity, i)) {
@@ -174,7 +168,7 @@ void ai_advisor_choose_building(struct city *pcity, struct ai_choice *choice)
   }
   choice->want = want;
   choice->choice = id;
-  choice->type = 0;
+  choice->type = CT_BUILDING;
 }
 
 
@@ -266,62 +260,52 @@ int ai_evaluate_government (struct player *pplayer, struct government *g)
   int trade_prod       = 0;
   int shield_need      = 0;
   int food_need        = 0;
-  /*
-  int happy            = 0;
-  int unhappy          = 0;
-  */
-  int unhappy_cities   = 0;
   int gov_overthrown   = 0;
-  int score            = 0;
+  int score;
 
   pplayer->government = g->index;
 
   city_list_iterate(pplayer->cities, pcity) {
-    struct city tmp_city = *pcity; 
-    int x, y;
+    city_refresh(pcity);
 
-    city_refresh (&tmp_city);
-     
-    /* the 4 lines that follow are copied from ai_manage_city -
+    /* the lines that follow are copied from ai_manage_city -
        we don't need the sell_obsolete_buildings */
-    city_check_workers (pplayer, &tmp_city);
-    auto_arrange_workers (&tmp_city);
-    if (ai_fix_unhappy (&tmp_city))
-       ai_scientists_taxmen (&tmp_city);
+    auto_arrange_workers(pcity);
+    if (ai_fix_unhappy (pcity))
+      ai_scientists_taxmen(pcity);
 
-    trade_prod     += tmp_city.trade_prod;
-    if (tmp_city.shield_prod > 0)
-      shield_surplus += tmp_city.shield_surplus;
+    trade_prod     += pcity->trade_prod;
+    if (pcity->shield_prod > 0)
+      shield_surplus += pcity->shield_surplus;
     else
-      shield_need    += tmp_city.shield_surplus;
-    if (tmp_city.food_surplus > 0)
-      food_surplus   += tmp_city.food_surplus;
+      shield_need    += pcity->shield_surplus;
+    if (pcity->food_surplus > 0)
+      food_surplus   += pcity->food_surplus;
     else
-      food_need      += tmp_city.food_surplus;
-    /*
-    happy            += tmp_city.happy;
-    unhappy          += tmp_city.unhappy;
-    */
+      food_need      += pcity->food_surplus;
 
-    if (city_unhappy (&tmp_city)) {
-      unhappy_cities++;
+    if (city_unhappy(pcity)) {
       /* the following is essential to prevent falling into anarchy */
-      if (tmp_city.anarchy > 0
+      if (pcity->anarchy > 0
 	  && government_has_flag(g, G_REVOLUTION_WHEN_UNHAPPY)) 
         gov_overthrown = 1;
     }
-
-    /* the following is essential because the above may change the ptile->worked */
-    city_map_iterate(x, y) {
-       struct tile *ptile = map_get_tile(pcity->x+x-2, pcity->y+y-2);
-       if (ptile->worked == &tmp_city)
-           ptile->worked = NULL;
-       set_worker_city(pcity, x, y, get_worker_city(pcity, x, y));
-    }
   } city_list_iterate_end;
 
   pplayer->government = current_gov;
 
+  /* Restore all cities. */
+  city_list_iterate(pplayer->cities, pcity) {
+    city_refresh(pcity);
+
+    /* the lines that follow are copied from ai_manage_city -
+       we don't need the sell_obsolete_buildings */
+    auto_arrange_workers(pcity);
+    if (ai_fix_unhappy (pcity))
+      ai_scientists_taxmen(pcity);
+  } city_list_iterate_end;
+  sync_cities();
+
   score =
     3 * trade_prod
   + 2 * shield_surplus + 2 * food_surplus
