@@ -18,40 +18,28 @@ struct city;
 struct unit;
 struct ai_choice;
 
-enum ai_unit_task { AIUNIT_NONE, AIUNIT_AUTO_SETTLER, AIUNIT_BUILD_CITY,
-                    AIUNIT_DEFEND_HOME, AIUNIT_ATTACK, AIUNIT_FORTIFY,
-                    AIUNIT_RUNAWAY, AIUNIT_ESCORT, AIUNIT_EXPLORE,
-                    AIUNIT_PILLAGE };
-
 void ai_manage_units(struct player *pplayer); 
 int could_unit_move_to_tile(struct unit *punit, int x0, int y0, int x, int y);
 int look_for_charge(struct player *pplayer, struct unit *punit,
                     struct unit **aunit, struct city **acity);
 
-int ai_manage_explorer(struct player *pplayer,struct unit *punit);
-void ai_manage_military(struct player *pplayer,struct unit *punit);
-void ai_military_findjob(struct player *pplayer,struct unit *punit);
-void ai_military_gohome(struct player *pplayer,struct unit *punit);
-void ai_military_attack(struct player *pplayer,struct unit *punit);
+int ai_manage_explorer(struct unit *punit);
 
-void ai_manage_caravan(struct player *pplayer, struct unit *punit);
 int find_something_to_kill(struct player *pplayer, struct unit *punit, 
                             int *x, int *y);
 int find_beachhead(struct unit *punit, int dest_x, int dest_y, int *x, int *y);
 
-int unit_move_turns(struct unit *punit, int x, int y);
 int unit_belligerence_basic(struct unit *punit);
 int unit_belligerence(struct unit *punit);
 int unit_vulnerability_basic(struct unit *punit, struct unit *pdef);
 int unit_vulnerability_virtual(struct unit *punit);
 int unit_vulnerability(struct unit *punit, struct unit *pdef);
-int unit_can_defend(int type);
 
-int is_on_unit_upgrade_path(int test, int base);
-int is_ai_simple_military(int type);
+int is_on_unit_upgrade_path(Unit_Type_id test, Unit_Type_id base);
+int is_ai_simple_military(Unit_Type_id type);
 
-int ai_wants_role_unit(struct player *pplayer, struct city *pcity,
-		       int role, int want);
+Unit_Type_id ai_wants_role_unit(struct player *pplayer, struct city *pcity,
+                                int role, int want);
 void ai_choose_role_unit(struct player *pplayer, struct city *pcity,
 			 struct ai_choice *choice, int role, int want);
 
