@@ -65,8 +65,9 @@ static char *cr_entry_hstate_verbose(struct city *pcity)
 {
   static char buf[16];
   my_snprintf(buf, sizeof(buf), "%s",
-	      (city_celebrating(pcity) ? _("Rapture") :
-	       (city_unhappy(pcity) ? _("Disorder") : _("Peace"))));
+	      (city_celebrating(pcity) ? Q_("?city_state:Rapture") :
+	       (city_unhappy(pcity) ? Q_("?city_state:Disorder") :
+		Q_("?city_state:Peace"))));
   return buf;
 }
 
@@ -119,7 +120,7 @@ static char *cr_entry_food(struct city *pcity)
   static char buf[32];
   my_snprintf(buf, sizeof(buf), "%d/%d",
 	      pcity->food_stock,
-	      (pcity->size+1) * game.foodbox);
+	      city_granary_size(pcity->size) );
   return buf;
 }
 
@@ -191,13 +192,13 @@ static char *cr_entry_corruption(struct city *pcity)
 #define FUNC_TAG(var)  cr_entry_##var, #var 
 
 struct city_report_spec city_report_specs[] = {
-  { 1,-15, 0, "",  N_("Name"),        N_("City Name"),
+  { 1,-15, 0, NULL,  N_("Name"),      N_("City Name"),
                                       FUNC_TAG(cityname) },
-  { 0,  2, 1, "",  N_("Sz"),          N_("Size"),
+  { 0,  2, 1, NULL,  N_("Sz"),        N_("Size"),
                                       FUNC_TAG(size) },
-  { 1, -8, 1, "",  N_("State"),       N_("Rapture/Peace/Disorder"),
+  { 1, -8, 1, NULL,  N_("State"),     N_("Rapture/Peace/Disorder"),
                                       FUNC_TAG(hstate_verbose) },
-  { 0,  1, 1, "",  "",                N_("Concise *=Rapture, X=Disorder"),
+  { 0,  1, 1, NULL,  NULL,            N_("Concise *=Rapture, X=Disorder"),
                                       FUNC_TAG(hstate_concise) },
   { 1,  8, 1, N_("Workers"), N_("H/C/U"),
                                       N_("Workers: Happy, Content, Unhappy"),
@@ -211,13 +212,13 @@ struct city_report_spec city_report_specs[] = {
   { 1, 10, 1, N_("Economy"), N_("G/L/S"),
                                       N_("Economy: Gold, Luxuries, Science"),
                                       FUNC_TAG(output) },
-  { 0,  1, 1, "n", "T",               N_("Number of Trade Routes"),
+  { 0,  1, 1, N_("n"), N_("T"),       N_("Number of Trade Routes"),
                                       FUNC_TAG(num_trade) },
   { 1,  7, 1, N_("Food"), N_("Stock"), N_("Food Stock"),
                                       FUNC_TAG(food) },
-  { 0,  3, 1, "", N_("Pol"),          N_("Pollution"),
+  { 0,  3, 1, NULL, N_("Pol"),        N_("Pollution"),
                                       FUNC_TAG(pollution) },
-  { 0,  3, 1, "", N_("Cor"),          N_("Corruption"),
+  { 0,  3, 1, NULL, N_("Cor"),        N_("Corruption"),
                                       FUNC_TAG(corruption) },
   { 1,  0, 1, N_("Currently Building"), N_("(Stock,Target,Turns,Buy)"),
                                       N_("Currently Building"),
