@@ -52,6 +52,7 @@
 #include "mapview_g.h"
 #include "menu_g.h"
 #include "messagewin_g.h"
+#include "netintf.h"
 #include "options.h"
 #include "packhand.h"
 #include "plrdlg_g.h"
@@ -60,12 +61,15 @@
 
 #include "civclient.h"
 
-char metaserver[256]="";
-char server_host[512];
-char name[512];
-int server_port;
+/* this is used in strange places, and is 'extern'd where
+   needed (hence, it is not 'extern'd in civclient.h) */
 int is_server = 0;
 
+char metaserver[256];
+char server_host[512];
+char player_name[512];
+int server_port;
+
 enum client_states client_state=CLIENT_BOOT_STATE;
 
 static char *tile_set_name = NULL;
@@ -77,10 +81,11 @@ int turn_gold_difference;
 
 int did_advance_tech_this_turn;
 
+static void client_remove_all_cli_conn(void);
+
 /**************************************************************************
 ...
 **************************************************************************/
-
 int main(int argc, char *argv[])
 {
   int i;
@@ -94,9 +99,9 @@ int main(int argc, char *argv[])
   /* set default argument values */
   loglevel=LOG_NORMAL;
   server_port=DEFAULT_SOCK_PORT;
-  mystrlcpy(server_host, "localhost", 512);
-  mystrlcpy(metaserver, METALIST_ADDR, 256);
-  name[0] = '\0';
+  sz_strlcpy(server_host, "localhost");
+  sz_strlcpy(metaserver, METALIST_ADDR);
+  player_name[0] = '\0';
 
   i = 1;
 
@@ -114,25 +119,25 @@ int main(int argc, char *argv[])
 #endif
     fprintf(stderr, _("  -m, --meta HOST\tConnect to the metaserver at HOST\n"));
 #ifdef DEBUG
-    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0,1,2,3,"
-                                  "or 3:file1,min,max:...)\n"));
+    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 4,"
+                                  " or 4:file1,min,max:...)\n"));
 #else
-    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0,1,2)\n"));
+    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 3)\n"));
 #endif
     fprintf(stderr, _("  -v, --version\t\tPrint the version number\n"));
    } else if (is_option("--version",argv[i])) {
-    fprintf(stderr, "%s\n", FREECIV_NAME_VERSION);
+    fprintf(stderr, "%s\n", freeciv_name_version());
     exit(0);
    } else if ((option = get_option("--log",argv,&i,argc)) != NULL)
       logfile = mystrdup(option); /* never free()d */
    else if ((option = get_option("--name",argv,&i,argc)) != NULL)
-      mystrlcpy(name, option, 512);
-   else if ((option = get_option("--metaserver",argv,&i,argc)) != NULL)
-      mystrlcpy(metaserver, option, 256);
+     sz_strlcpy(player_name, option);
+   else if ((option = get_option("--meta",argv,&i,argc)) != NULL)
+      sz_strlcpy(metaserver, option);
    else if ((option = get_option("--port",argv,&i,argc)) != NULL)
      server_port=atoi(option);
    else if ((option = get_option("--server",argv,&i,argc)) != NULL)
-      mystrlcpy(server_host, option, 512);
+      sz_strlcpy(server_host, option);
    else if ((option = get_option("--debug",argv,&i,argc)) != NULL) {
       loglevel=log_parse_level_str(option);
       if (loglevel==-1) {
@@ -150,10 +155,13 @@ int main(int argc, char *argv[])
   log_init(logfile, loglevel, NULL);
 
   /* after log_init: */
-  if (name[0] == '\0') {
-    mystrlcpy(name, user_username(), 512);
+  if (player_name[0] == '\0') {
+    sz_strlcpy(player_name, user_username());
   }
 
+  /* initialization */
+
+  my_init_network();
   init_messages_where();
   init_our_capability();
   game_init();
@@ -165,7 +173,14 @@ int main(int argc, char *argv[])
   boot_help_texts();
   tilespec_read_toplevel(tile_set_name); /* get tile sizes etc */
 
+  /* run gui-specific client */
+
   ui_main(argc, argv);
+
+  /* termination */
+
+  my_shutdown_network();
+
   return 0;
 }
 
@@ -232,6 +247,10 @@ void handle_packet_input(char *packet, int type)
     handle_city_info((struct packet_city_info *)packet);
     break;
 
+  case PACKET_SHORT_CITY:
+    handle_short_city((struct packet_short_city *)packet);
+    break;
+
   case PACKET_REMOVE_UNIT:
     handle_remove_unit((struct packet_generic_integer *)packet);
     break;
@@ -348,9 +367,19 @@ void handle_packet_input(char *packet, int type)
     handle_advance_focus((struct packet_generic_integer *)packet);
     break;
     
+  case PACKET_CONN_INFO:
+    handle_conn_info((struct packet_conn_info *)packet);
+    break;
+    
+  case PACKET_CONN_PING:
+    send_packet_generic_empty(&aconnection, PACKET_CONN_PONG);
+    break;
+
   default:
-    freelog(LOG_FATAL, _("Received unknown packet from server!"));
-    exit(1);
+    freelog(LOG_ERROR, "Received unknown packet (type %d) from server!", type);
+    /* Old clients (<= some 1.11.5-devel, capstr +1.11) used to exit()
+     * here, so server should not rely on client surviving.
+     */
     break;
   }
 
@@ -407,6 +436,20 @@ void send_move_unit(struct unit *punit)
   send_packet_move_unit(&aconnection, &move);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void send_goto_unit(struct unit *punit, int dest_x, int dest_y)
+{
+  struct packet_unit_request req;
+
+  req.unit_id = punit->id;
+  req.name[0] = '\0';
+  req.x = dest_x;
+  req.y = dest_y;
+  send_packet_unit_request(&aconnection, &req, PACKET_UNIT_GOTO_TILE);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -447,6 +490,8 @@ void set_client_state(enum client_states newstate)
     }
     else if(client_state==CLIENT_PRE_GAME_STATE) {
       popdown_all_city_dialogs();
+      close_all_diplomacy_dialogs();
+      client_remove_all_cli_conn();
       game_remove_all_players();
       set_unit_focus_no_center(0);
       clear_notify_window();
@@ -467,5 +512,31 @@ enum client_states get_client_state(void)
   return client_state;
 }
 
+/**************************************************************************
+  Remove pconn from all connection lists in client, then free it.
+**************************************************************************/
+void client_remove_cli_conn(struct connection *pconn)
+{
+  if (pconn->player) {
+    conn_list_unlink(&pconn->player->connections, pconn);
+  }
+  conn_list_unlink(&game.all_connections, pconn);
+  conn_list_unlink(&game.est_connections, pconn);
+  conn_list_unlink(&game.game_connections, pconn);
+  free(pconn);
+}
+
+/**************************************************************************
+  Remove (and free) all connections from all connection lists in client.
+  Assumes game.all_connections is properly maintained with all connections.
+**************************************************************************/
+static void client_remove_all_cli_conn(void)
+{
+  while (conn_list_size(&game.all_connections)) {
+    struct connection *pconn = conn_list_get(&game.all_connections, 0);
+    client_remove_cli_conn(pconn);
+  }
+}
+
 void dealloc_id(int id); /* double kludge (suppress a possible warning) */
 void dealloc_id(int id) { }/* kludge */
