@@ -13,23 +13,31 @@
 #ifndef FC__CIVCLIENT_H
 #define FC__CIVCLIENT_H
 
-#include "packets.h"
-#include "game.h"
+#include "packets.h"		/* enum report_type */
+#include "game.h"		/* enum client_states */
 
 void handle_packet_input(char *packet, int type);
-void user_ended_turn(void);
-void handle_tile_info(struct packet_tile_info *packet);
-void handle_player_info(struct packet_player_info *packet);
-void handle_game_info(struct packet_game_info *packet);
-void handle_map_info(struct packet_map_info *pinfo);
-void handle_select_nation(struct packet_generic_values *packet);
-void handle_unit_info(struct packet_unit_info *packet);
-void handle_chat_msg(struct packet_generic_message *packet);
+
 void send_unit_info(struct unit *punit);
+void send_move_unit(struct unit *punit);
+void send_goto_unit(struct unit *punit, int dest_x, int dest_y);
 void send_report_request(enum report_type type);
+
+void user_ended_turn(void);
+
 void set_client_state(enum client_states newstate);
 enum client_states get_client_state(void);
+
+void client_remove_cli_conn(struct connection *pconn);
+
 extern int turn_gold_difference;
-void send_move_unit(struct unit *punit);
+extern int seconds_to_turndone;
+extern int last_turn_gold_amount;
+extern int did_advance_tech_this_turn;
+
+extern char metaserver[256];
+extern char server_host[512];
+extern char player_name[512];
+extern int server_port;
 
 #endif  /* FC__CIVCLIENT_H */
