@@ -39,6 +39,7 @@ used throughout the client.
 #include "packets.h"
 #include "support.h"
 
+#include "clinet.h"
 #include "chatline_g.h"
 #include "citydlg_g.h"
 #include "cityrep_g.h"
@@ -48,8 +49,6 @@ used throughout the client.
 
 #include "climisc.h"
 
-extern struct connection aconnection;
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -322,7 +321,7 @@ void client_change_all(int x, int y)
 /**************************************************************************
 Format a duration, in seconds, so it comes up in minutes or hours if
 that would be more meaningful.
-(Eleven characters, maximum.  Enough for, e.g., "99h 59m 59s".)
+(7 characters, maximum.  Enough for, e.g., "99h 59m".)
 **************************************************************************/
 void format_duration(char *buffer, int buffer_size, int duration)
 {
@@ -335,21 +334,55 @@ void format_duration(char *buffer, int buffer_size, int duration)
     my_snprintf(buffer, buffer_size, Q_("?mins/secs:%02dm %02ds"),
 		duration/60, duration%60);
   else if (duration < 360000)	/* < 100 hours */
-    my_snprintf(buffer, buffer_size, Q_("?hrs/mns/scs:%02dh %02dm %02ds"),
-		duration/3600, (duration/60)%60, duration%60);
+    my_snprintf(buffer, buffer_size, Q_("?hrs/mns:%02dh %02dm"),
+		duration/3600, (duration/60)%60);
   else if (duration < 8640000)	/* < 100 days */
-    my_snprintf(buffer, buffer_size, Q_("?dys/hrs/mns:%02dd %02dh %02dm"),
-		duration/86400, (duration/3600)%24, (duration/60)%60);
+    my_snprintf(buffer, buffer_size, Q_("?dys/hrs:%02dd %02dh"),
+		duration/86400, (duration/3600)%24);
   else
     my_snprintf(buffer, buffer_size, Q_("?duration:overflow"));
 }
 
+/***************************************************************************
+Return a string indicating one nation's embassy status with another
+***************************************************************************/
+char *get_embassy_status(struct player *me, struct player *them)
+{
+  if (player_has_embassy(me, them)) {
+    if (player_has_embassy(them, me))
+      return Q_("?embassy:Both");
+    else
+      return Q_("?embassy:Yes");
+  } else if (player_has_embassy(them, me))
+    return Q_("?embassy:With Us");
+  else
+    return "";
+}
+
+/***************************************************************************
+Return a string indicating one nation's shaed vision status with another
+***************************************************************************/
+char *get_vision_status(struct player *me, struct player *them)
+{
+  if (me->gives_shared_vision & (1<<them->player_no)) {
+    if (them->gives_shared_vision & (1<<me->player_no))
+      return Q_("?vision:Both");
+    else
+      return Q_("?vision:To Them");
+  } else if (them->gives_shared_vision & (1<<me->player_no))
+    return Q_("?vision:To Us");
+  else
+    return "";
+}
+
 /**************************************************************************
 Copy a string that describes the given clause into the return buffer.
 **************************************************************************/
 void client_diplomacy_clause_string(char *buf, int bufsiz,
 				    struct Clause *pclause)
 {
+  struct city *pcity;
+
   switch(pclause->type) {
   case CLAUSE_ADVANCE:
     my_snprintf(buf, bufsiz, _("The %s give %s"),
@@ -357,9 +390,15 @@ void client_diplomacy_clause_string(char *buf, int bufsiz,
 		advances[pclause->value].name);
     break;
   case CLAUSE_CITY:
-    my_snprintf(buf, bufsiz, _("The %s give %s"),
-		get_nation_name_plural(pclause->from->nation),
-		find_city_by_id(pclause->value)->name);
+    pcity = find_city_by_id(pclause->value);
+    if (pcity) {
+      my_snprintf(buf, bufsiz, _("The %s give %s"),
+                  get_nation_name_plural(pclause->from->nation),
+		  pcity->name);
+    } else {
+      my_snprintf(buf, bufsiz,_("The %s give unknown city."),
+                  get_nation_name_plural(pclause->from->nation));
+    }
     break;
   case CLAUSE_GOLD:
     my_snprintf(buf, bufsiz, _("The %s give %d gold"),
@@ -383,6 +422,10 @@ void client_diplomacy_clause_string(char *buf, int bufsiz,
   case CLAUSE_ALLIANCE:
     my_snprintf(buf, bufsiz, _("The parties create an alliance"));
     break;
+  case CLAUSE_VISION:
+    my_snprintf(buf, bufsiz, _("The %s gives shared vision"),
+		get_nation_name_plural(pclause->from->nation));
+    break;
   default:
     if (bufsiz > 0) *buf = '\0';
     break;
