@@ -26,6 +26,8 @@ void client_change_all(int x, int y);
 
 void format_duration(char *buffer, int buffer_size, int duration);
 
+char *get_embassy_status(struct player *me, struct player *them);
+char *get_vision_status(struct player *me, struct player *them);
 void client_diplomacy_clause_string(char *buf, int bufsiz,
 				    struct Clause *pclause);
 
