@@ -48,14 +48,8 @@
 #include <sys/select.h>
 #endif
 
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
+#ifdef HAVE_WINSOCK
+#include <winsock.h>
 #endif
 
 #include "capstr.h"
@@ -63,6 +57,7 @@
 #include "game.h"
 #include "log.h"
 #include "mem.h"
+#include "netintf.h"
 #include "packets.h"
 #include "support.h"
 #include "version.h"
@@ -71,13 +66,40 @@
 #include "civclient.h"
 #include "dialogs_g.h"		/* popdown_races_dialog() */
 #include "gui_main_g.h"		/* add_net_input(), remove_net_input() */
+#include "messagewin_g.h"
 #include "packhand.h"
+#include "plrdlg_g.h"
+#include "repodlgs_g.h"
 
 #include "clinet.h"
 
 struct connection aconnection;
 
-extern char metaserver[];
+/**************************************************************************
+  Close socket and cleanup.  This one doesn't print a message, so should
+  do so before-hand if necessary.
+**************************************************************************/
+static void close_socket_nomessage(struct connection *pc)
+{
+  pc->used = 0;
+  pc->established = 0;
+  my_closesocket(pc->sock);
+
+  /* make sure not to use these accidently: */
+  free_socket_packet_buffer(pc->buffer);
+  free_socket_packet_buffer(pc->send_buffer);
+  pc->buffer = NULL;
+  pc->send_buffer = NULL;
+
+  remove_net_input();
+  popdown_races_dialog(); 
+
+  plrdlg_update_delay_off();
+  report_update_delay_off();
+  meswin_update_delay_off();
+  
+  set_client_state(CLIENT_PRE_GAME_STATE);
+}
 
 /**************************************************************************
 ...
@@ -86,10 +108,7 @@ static void close_socket_callback(struct connection *pc)
 {
   append_output_window(_("Lost connection to server!"));
   freelog(LOG_NORMAL, "lost connection to server");
-  close(pc->sock);
-  remove_net_input();
-  popdown_races_dialog(); 
-  set_client_state(CLIENT_PRE_GAME_STATE);
+  close_socket_nomessage(pc);
 }
 
 /**************************************************************************
@@ -98,10 +117,8 @@ static void close_socket_callback(struct connection *pc)
 int connect_to_server(char *name, char *hostname, int port,
 		      char *errbuf, int n_errbuf)
 {
-  /* use name to find TCPIP address of server */
+  /* use name to find TCP/IP address of server */
   struct sockaddr_in src;
-  struct hostent *ph;
-  long address;
   struct packet_req_join_game req;
 
   if(port==0)
@@ -110,22 +127,10 @@ int connect_to_server(char *name, char *hostname, int port,
   if(!hostname)
     hostname="localhost";
   
-  if(isdigit((size_t)*hostname)) {
-    if((address = inet_addr(hostname)) == -1) {
-      mystrlcpy(errbuf, _("Invalid hostname"), n_errbuf);
-      return -1;
-    }
-    src.sin_addr.s_addr = address;
-    src.sin_family = AF_INET;
-  }
-  else if ((ph = gethostbyname(hostname)) == NULL) {
+  if (!fc_lookup_host(hostname, &src)) {
     mystrlcpy(errbuf, _("Failed looking up host"), n_errbuf);
     return -1;
   }
-  else {
-    src.sin_family = ph->h_addrtype;
-    memcpy((char *) &src.sin_addr, ph->h_addr, ph->h_length);
-  }
   
   src.sin_port = htons(port);
 
@@ -141,11 +146,24 @@ int connect_to_server(char *name, char *hostname, int port,
   
   if(connect(aconnection.sock, (struct sockaddr *) &src, sizeof (src)) < 0) {
     mystrlcpy(errbuf, mystrerror(errno), n_errbuf);
-    close(aconnection.sock);
+    my_closesocket(aconnection.sock);
+    aconnection.sock = -1;
     return -1;
   }
 
-  aconnection.buffer.ndata=0;
+  if (aconnection.buffer) {
+    /* didn't close cleanly previously? */
+    freelog(LOG_ERROR, "Unexpected buffers in connect_to_server()");
+    /* get newly initialized ones instead */
+    free_socket_packet_buffer(aconnection.buffer);
+    free_socket_packet_buffer(aconnection.send_buffer);
+  }
+  aconnection.buffer = new_socket_packet_buffer();
+  aconnection.send_buffer = new_socket_packet_buffer();
+  aconnection.last_write = 0;
+  
+  aconnection.used = 1;
+  aconnection.delayed_disconnect = 0;
 
   /* gui-dependent details now in gui_main.c: */
   add_net_input(aconnection.sock);
@@ -173,9 +191,7 @@ int connect_to_server(char *name, char *hostname, int port,
 void disconnect_from_server(void)
 {
   append_output_window(_("Disconnecting from server."));
-  close(aconnection.sock);
-  remove_net_input();
-  set_client_state(CLIENT_PRE_GAME_STATE);
+  close_socket_nomessage(&aconnection);
 }  
 
 /**************************************************************************
@@ -184,12 +200,17 @@ void disconnect_from_server(void)
 **************************************************************************/
 void input_from_server(int fid)
 {
-  if(read_socket_data(fid, &aconnection.buffer)>=0) {
-    int type;
+  if(read_socket_data(fid, aconnection.buffer)>=0) {
+    int type, result;
     char *packet;
 
-    while((packet=get_packet_from_connection(&aconnection, &type))) {
-      handle_packet_input(packet, type);
+    while (1) {
+      packet = get_packet_from_connection(&aconnection, &type, &result);
+      if (result) {
+	handle_packet_input(packet, type);
+      } else {
+	break;
+      }
     }
   }
   else {
@@ -211,7 +232,6 @@ struct server_list *create_server_list(char *errbuf, int n_errbuf)
 {
   struct server_list *server_list;
   struct sockaddr_in addr;
-  struct hostent *ph;
   int s;
   FILE *f;
   char *proxy_url;
@@ -269,12 +289,9 @@ struct server_list *create_server_list(char *errbuf, int n_errbuf)
     urlpath = s;
   }
 
-  if ((ph = gethostbyname(server)) == NULL) {
+  if (!fc_lookup_host(server, &addr)) {
     mystrlcpy(errbuf, _("Failed looking up host"), n_errbuf);
     return NULL;
-  } else {
-    addr.sin_family = ph->h_addrtype;
-    memcpy((char *) &addr.sin_addr, ph->h_addr, ph->h_length);
   }
   
   addr.sin_port = htons(port);
@@ -286,7 +303,7 @@ struct server_list *create_server_list(char *errbuf, int n_errbuf)
   
   if(connect(s, (struct sockaddr *) &addr, sizeof (addr)) < 0) {
     mystrlcpy(errbuf, mystrerror(errno), n_errbuf);
-    close(s);
+    my_closesocket(s);
     return NULL;
   }
 
@@ -312,7 +329,7 @@ struct server_list *create_server_list(char *errbuf, int n_errbuf)
       fwrite(str,1,i,f);
     fflush(f);
 
-    close(s);
+    my_closesocket(s);
 
     fseek(f,0,SEEK_SET);
   }
