@@ -14,9 +14,9 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 
-#include "capability.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
@@ -30,14 +30,13 @@
 #include "menu_g.h"
 
 #include "civclient.h"
+#include "clinet.h"
+#include "goto.h"
 #include "options.h"
 #include "tilespec.h"
 
 #include "control.h"
 
-extern struct connection aconnection;
-
-
 /* gui-dep code may adjust depending on tile size etc: */
 int num_units_below = MAX_NUM_UNITS_BELOW;
 
@@ -45,18 +44,12 @@ int num_units_below = MAX_NUM_UNITS_BELOW;
 /* unit_focus points to the current unit in focus */
 static struct unit *punit_focus;
 
-/* set high, if the player has selected goto */
-/* actually, set to id of unit goto-ing (id is non-zero) */
-int goto_state;
-
-/* set high, if the player has selected nuke */
-int nuke_state;
-
-/* set high, if the player has selected paradropping */
-int paradrop_state;
-
-/* set high if the player has selected connect */
-int connect_state;
+/* These should be set via set_hover_state() */
+int hover_unit = 0; /* id of unit hover_state applies to */
+enum cursor_hover_state hover_state = HOVER_NONE;
+/* This may only be here until client goto is fully implemented.
+   It is reset each time the hower_state is reset. */
+int draw_goto_line = 1;
 
 /* units involved in current combat */
 static struct unit *punit_attacking;
@@ -66,6 +59,21 @@ static struct unit *punit_defending;
 
 static struct unit *find_best_focus_candidate(void);
 
+/**************************************************************************
+...
+**************************************************************************/
+void set_hover_state(struct unit *punit, enum cursor_hover_state state)
+{
+  assert(punit || state==HOVER_NONE);
+  draw_goto_line = 1;
+  if (punit)
+    hover_unit = punit->id;
+  else
+    hover_unit = 0;
+  hover_state = state;
+  exit_goto_state();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -178,10 +186,7 @@ void advance_unit_focus(void)
 
   punit_focus=find_best_focus_candidate();
 
-  goto_state=0;
-  nuke_state=0;
-  paradrop_state=0;
-  connect_state=0;
+  set_hover_state(NULL, HOVER_NONE);
 
   if(!punit_focus) {
     unit_list_iterate(game.player_ptr->units, punit) {
@@ -468,11 +473,24 @@ void process_diplomat_arrival(struct unit *pdiplomat, int victim_id)
 **************************************************************************/
 void request_unit_goto(void)
 {
-  struct unit *punit=get_unit_in_focus();
-     
-  if(punit) {
-    goto_state=punit->id;
+  struct unit *punit = get_unit_in_focus();
+
+  if (!punit)
+    return;
+
+  if (hover_state != HOVER_GOTO) {
+    set_hover_state(punit, HOVER_GOTO);
     update_unit_info_label(punit);
+    /* Not yet implemented for air units */
+    if (is_air_unit(punit)) {
+      draw_goto_line = 0;
+    } else {
+      enter_goto_state(punit);
+      create_line_at_mouse_pos();
+    }
+  } else if (!is_air_unit(punit)) {
+    assert(goto_is_active());
+    goto_add_waypoint();
   }
 }
 
@@ -484,9 +502,8 @@ void request_unit_connect(void)
 {
   struct unit *punit=get_unit_in_focus();
      
-  if(punit && can_unit_do_connect (punit, ACTIVITY_IDLE)) {
-    goto_state=punit->id;
-    connect_state=1;
+  if (punit && can_unit_do_connect (punit, ACTIVITY_IDLE)) {
+    set_hover_state(punit, HOVER_CONNECT);
     update_unit_info_label(punit);
   }
 }
@@ -499,7 +516,7 @@ void request_unit_unload(struct unit *punit)
   struct packet_unit_request req;
 
   if(!get_transporter_capacity(punit)) {
-    append_output_window(_("Game: You can only unload transporter units."));
+    append_output_window(_("Game: Only transporter units can be unloaded."));
     return;
   }
 
@@ -510,6 +527,18 @@ void request_unit_unload(struct unit *punit)
   send_packet_unit_request(&aconnection, &req, PACKET_UNIT_UNLOAD);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void request_unit_airlift(struct unit *punit, struct city *pcity)
+{
+  struct packet_unit_request p;
+  p.unit_id = punit->id;
+  p.x = pcity->x;
+  p.y = pcity->y;
+  send_packet_unit_request(&aconnection, &p, PACKET_UNIT_AIRLIFT);
+}
+
 /**************************************************************************
 (RP:) un-sentry all my own sentried units on punit's tile
 **************************************************************************/
@@ -729,8 +758,8 @@ void request_unit_auto(struct unit *punit)
     req.name[0]='\0';
     send_packet_unit_request(&aconnection, &req, PACKET_UNIT_AUTO);
   } else {
-    append_output_window(_("Game: Only settlers, and military units"
-			 " in cities, can be put in auto-mode."));
+    append_output_window(_("Game: Only settler units and military units"
+			   " in cities can be put in auto-mode."));
   }
 }
 
@@ -745,7 +774,7 @@ void request_unit_caravan_action(struct unit *punit, enum packet_type action)
   if (!pcity) return;
   if (!(action==PACKET_UNIT_ESTABLISH_TRADE
 	||(action==PACKET_UNIT_HELP_BUILD_WONDER))) {
-    freelog(LOG_NORMAL, "Bad action (%d) in request_unit_caravan_action",
+    freelog(LOG_ERROR, "Bad action (%d) in request_unit_caravan_action",
 	    action);
     return;
   }
@@ -768,8 +797,7 @@ void request_unit_nuke(struct unit *punit)
   if(!(punit->moves_left))
     do_unit_nuke(punit);
   else {
-    nuke_state=1;
-    goto_state=punit->id;
+    set_hover_state(punit, HOVER_NUKE);
     update_unit_info_label(punit);
   }
 }
@@ -786,11 +814,36 @@ void request_unit_paradrop(struct unit *punit)
   if(!can_unit_paradrop(punit))
     return;
 
-  paradrop_state=1;
-  goto_state=punit->id;
+  set_hover_state(punit, HOVER_PARADROP);
   update_unit_info_label(punit);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void request_unit_patrol(void)
+{
+  struct unit *punit = get_unit_in_focus();
+
+  if (!punit)
+    return;
+
+  if (hover_state != HOVER_PATROL) {
+    set_hover_state(punit, HOVER_PATROL);
+    update_unit_info_label(punit);
+    /* Not yet implemented for air units */
+    if (is_air_unit(punit)) {
+      draw_goto_line = 0;
+    } else {
+      enter_goto_state(punit);
+      create_line_at_mouse_pos();
+    }
+  } else {
+    assert(goto_is_active());
+    goto_add_waypoint();
+  }
+}
+
 /**************************************************************************
  Toggle display of grid lines on the map
 **************************************************************************/
@@ -826,6 +879,150 @@ void request_toggle_city_productions(void)
   update_map_canvas_visible();
 }
 
+/**************************************************************************
+ Toggle display of terrain
+**************************************************************************/
+void request_toggle_terrain(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_terrain ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of coastline
+**************************************************************************/
+void request_toggle_coastline(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_coastline ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of roads and rails
+**************************************************************************/
+void request_toggle_roads_rails(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_roads_rails ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of irrigation
+**************************************************************************/
+void request_toggle_irrigation(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_irrigation ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of mines
+**************************************************************************/
+void request_toggle_mines(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_mines ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of fortress and airbase
+**************************************************************************/
+void request_toggle_fortress_airbase(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_fortress_airbase ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of specials
+**************************************************************************/
+void request_toggle_specials(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_specials ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of pollution
+**************************************************************************/
+void request_toggle_pollution(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_pollution ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of cities
+**************************************************************************/
+void request_toggle_cities(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_cities ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of units
+**************************************************************************/
+void request_toggle_units(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_units ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of focus unit
+**************************************************************************/
+void request_toggle_focus_unit(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_focus_unit ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of fog of war
+**************************************************************************/
+void request_toggle_fog_of_war(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_fog_of_war ^= 1;
+  update_map_canvas_visible();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -883,100 +1080,50 @@ void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo)
   if(get_unit_in_focus()==punit) update_menus();
 }
 
-/**************************************************************************
- Finish the goto mode and let the unit which is stored in goto_state move
- to a given location.
- returns 1 if goto mode was activated before calling this function
- otherwise 0 (then this function does nothing)
-**************************************************************************/
-int do_goto(int xtile, int ytile)
-{
-  if(goto_state) {
-    struct unit *punit;
-
-    if((punit=player_find_unit_by_id(game.player_ptr, goto_state))) {
-      struct packet_unit_request req;
-      req.unit_id=punit->id;
-      req.name[0]='\0';
-      req.x=xtile;
-      req.y=ytile;
-      send_packet_unit_request(&aconnection, &req, PACKET_UNIT_GOTO_TILE);
-    }
-
-    goto_state=0;
-    nuke_state=0;
-    paradrop_state=0;
-    connect_state=0;
-
-    return 1;
-  }
-  return 0;
-}
-
 /**************************************************************************
  Handles everything when the user clicked a tile
 **************************************************************************/
 void do_map_click(int xtile, int ytile)
 {
-  struct city *pcity;
-  struct tile *ptile;
+  struct city *pcity = map_get_city(xtile, ytile);
+  struct tile *ptile = map_get_tile(xtile, ytile);
+  struct unit *punit = player_find_unit_by_id(game.player_ptr, hover_unit);
 
-  ptile=map_get_tile(xtile, ytile);
-  pcity=map_get_city(xtile, ytile);
-
-  if(goto_state) { 
-    struct unit *punit;
-
-    if((punit=player_find_unit_by_id(game.player_ptr, goto_state))) {
-      struct packet_unit_request req;
-
-      if(paradrop_state) {
-        do_unit_paradrop_to(punit, xtile, ytile);
-        goto_state=0;
-        nuke_state=0;
-        paradrop_state=0;
-	connect_state=0;
-	update_unit_info_label(punit);
-        return;
-      }
-
-      if(connect_state) {
-        popup_unit_connect_dialog(punit, xtile, ytile);
-        goto_state=0;
-        nuke_state=0;
-        paradrop_state=0;
-        connect_state=0;
-        return;
-      }
-
-      if(nuke_state &&
-	 3*real_map_distance(punit->x,punit->y,xtile,ytile) > punit->moves_left) {
+  if (punit && hover_state != HOVER_NONE) {
+    switch (hover_state) {
+    case HOVER_NONE:
+      abort(); /* well; shouldn't get here :) */
+      break;
+    case HOVER_GOTO:
+      do_unit_goto(xtile, ytile);
+      break;
+    case HOVER_NUKE:
+      if (3 * real_map_distance(punit->x, punit->y, xtile, ytile)
+	  > punit->moves_left) {
         append_output_window(_("Game: Too far for this unit."));
-        goto_state=0;
-        nuke_state=0;
-        paradrop_state=0;
-        connect_state=0;
-        update_unit_info_label(punit);
-        return;
+      } else {
+	do_unit_goto(xtile, ytile);
+	/* note that this will be executed by the server after the goto */
+	if (!pcity)
+	  do_unit_nuke(punit);
       }
-
-      req.unit_id=punit->id;
-      req.name[0]='\0';
-      req.x=xtile;
-      req.y=ytile;
-      send_packet_unit_request(&aconnection, &req, PACKET_UNIT_GOTO_TILE);
-      if(nuke_state && (!pcity))
-	do_unit_nuke(punit);
+      break;
+    case HOVER_PARADROP:
+      do_unit_paradrop_to(punit, xtile, ytile);
+      break;
+    case HOVER_CONNECT:
+      popup_unit_connect_dialog(punit, xtile, ytile);
+      break;
+    case HOVER_PATROL:
+      do_unit_patrol_to(punit, xtile, ytile);
+      break;	
     }
-
-    goto_state=0;
-    nuke_state=0;
-    paradrop_state=0;
-    connect_state=0;
+    set_hover_state(NULL, HOVER_NONE);
+    update_unit_info_label(punit);
     return;
   }
   
-  if(pcity && game.player_idx==pcity->owner) {
+  if (pcity && game.player_idx==pcity->owner) {
     popup_city_dialog(pcity, 0);
     return;
   }
@@ -1063,6 +1210,36 @@ void update_unit_pix_label(struct unit *punit)
   }
 }
 
+
+/**************************************************************************
+ Finish the goto mode and let the unit which is stored in hover_unit move
+ to a given location.
+**************************************************************************/
+void do_unit_goto(int x, int y)
+{
+  struct unit *punit = player_find_unit_by_id(game.player_ptr, hover_unit);
+
+  if (!hover_unit || hover_state != HOVER_GOTO)
+    return;
+
+  if (punit) {
+    if (!draw_goto_line) {
+      send_goto_unit(punit, x, y);
+    } else {
+      int dest_x, dest_y;
+      draw_line(x, y);
+      get_line_dest(&dest_x, &dest_y);
+      if (dest_x == x && dest_y == y) {
+	send_goto_route(punit);
+      } else {
+	append_output_window(_("Game: Didn't find a route to the destination!"));
+      }
+    }
+  }
+
+  set_hover_state(NULL, HOVER_NONE);
+}
+
 /**************************************************************************
 Explode nuclear at a tile without enemy units
 **************************************************************************/
@@ -1089,6 +1266,28 @@ void do_unit_paradrop_to(struct unit *punit, int x, int y)
   send_packet_unit_request(&aconnection, &req, PACKET_UNIT_PARADROP_TO);
 }
  
+/**************************************************************************
+Paradrop to a location
+**************************************************************************/
+void do_unit_patrol_to(struct unit *punit, int x, int y)
+{
+  if (is_air_unit(punit)) {
+    append_output_window(_("Game: Sorry, airunit patrol not yet implemented."));
+    return;
+  } else {
+    int dest_x, dest_y;
+    draw_line(x, y);
+    get_line_dest(&dest_x, &dest_y);
+    if (dest_x == x && dest_y == y) {
+      send_patrol_route(punit);
+    } else {
+      append_output_window(_("Game: Didn't find a route to the destination!"));
+    }
+  }
+
+  set_hover_state(NULL, HOVER_NONE);
+}
+ 
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1129,15 +1328,15 @@ void request_center_focus_unit(void)
 **************************************************************************/
 void key_cancel_action(void)
 {
-  if(goto_state) {
-    struct unit *punit;
+  int popped = 0;
+  if (hover_state == HOVER_GOTO || hover_state == HOVER_PATROL)
+    if (draw_goto_line)
+      popped = goto_pop_waypoint();
 
-    punit=player_find_unit_by_id(game.player_ptr, goto_state);
+  if (hover_state != HOVER_NONE && !popped) {
+    struct unit *punit = player_find_unit_by_id(game.player_ptr, hover_unit);
 
-    goto_state=0;
-    nuke_state=0;
-    paradrop_state=0;
-    connect_state=0;
+    set_hover_state(NULL, HOVER_NONE);
 
     update_unit_info_label(punit);
   }
@@ -1159,6 +1358,102 @@ void key_city_productions_toggle(void)
   request_toggle_city_productions();
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void key_terrain_toggle(void)
+{
+  request_toggle_terrain();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_coastline_toggle(void)
+{
+  request_toggle_coastline();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_roads_rails_toggle(void)
+{
+  request_toggle_roads_rails();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_irrigation_toggle(void)
+{
+  request_toggle_irrigation();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_mines_toggle(void)
+{
+  request_toggle_mines();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_fortress_airbase_toggle(void)
+{
+  request_toggle_fortress_airbase();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_specials_toggle(void)
+{
+  request_toggle_specials();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_pollution_toggle(void)
+{
+  request_toggle_pollution();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_cities_toggle(void)
+{
+  request_toggle_cities();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_units_toggle(void)
+{
+  request_toggle_units();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_focus_unit_toggle(void)
+{
+  request_toggle_focus_unit();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_fog_of_war_toggle(void)
+{
+  request_toggle_fog_of_war();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1318,7 +1613,23 @@ handle user pressing key for 'Connect' command
 **************************************************************************/
 void key_unit_connect(void)
 {
-  request_unit_connect();
+  if(get_unit_in_focus())
+    request_unit_connect();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_unit_diplomat_actions(void)
+{
+  struct city *pcity;		/* need pcity->id */
+  if(get_unit_in_focus()
+     && is_diplomat_unit(punit_focus)
+     && (pcity = map_get_city(punit_focus->x, punit_focus->y))
+     && !diplomat_dialog_is_open()    /* confusing otherwise? */
+     && diplomat_can_do_action(punit_focus, DIPLOMAT_ANY_ACTION,
+			       punit_focus->x, punit_focus->y))
+     process_diplomat_arrival(punit_focus, pcity->id);
 }
 
 /**************************************************************************
@@ -1335,7 +1646,8 @@ void key_unit_disband(void)
 **************************************************************************/
 void key_unit_done(void)
 {
-  request_unit_move_done();
+  if(get_unit_in_focus())
+    request_unit_move_done();
 }
 
 /**************************************************************************
@@ -1373,7 +1685,8 @@ void key_unit_fortress(void)
 **************************************************************************/
 void key_unit_goto(void)
 {
-  request_unit_goto();
+  if(get_unit_in_focus())
+    request_unit_goto();
 }
 
 /**************************************************************************
@@ -1424,6 +1737,15 @@ void key_unit_paradrop(void)
       request_unit_paradrop(punit_focus);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void key_unit_patrol(void)
+{
+  if(get_unit_in_focus())
+    request_unit_patrol();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
