@@ -15,40 +15,69 @@
 
 #include "packets.h"
 
+enum cursor_hover_state {
+  HOVER_NONE = 0,
+  HOVER_GOTO,
+  HOVER_NUKE,
+  HOVER_PARADROP,
+  HOVER_CONNECT,
+  HOVER_PATROL
+};
+
+extern int hover_unit; /* unit hover_state applies to */
+extern enum cursor_hover_state hover_state;
+extern int draw_goto_line;
+
 void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo);
+void do_unit_goto(int x, int y);
 void do_unit_nuke(struct unit *punit);
 void do_unit_paradrop_to(struct unit *punit, int x, int y);
 void do_unit_connect(struct unit *punit, int x, int y);
-int do_goto(int xtile, int ytile);
+void do_unit_patrol_to(struct unit *punit, int x, int y);
 void do_map_click(int xtile, int ytile);
 
+void set_hover_state(struct unit *punit, enum cursor_hover_state state);
 void handle_advance_focus(struct packet_generic_integer *packet);
 void request_center_focus_unit(void);
 void request_move_unit_direction(struct unit *punit, int dx, int dy); /* used by key_xxx */
 void request_new_unit_activity(struct unit *punit, enum unit_activity act);
 void request_new_unit_activity_targeted(struct unit *punit, enum unit_activity act,
 					int tgt);
-void request_unit_selected(struct unit *punit);
-void request_unit_pillage(struct unit *punit);
-void request_unit_sentry(struct unit *punit);
-void request_unit_fortify(struct unit *punit);
 void request_unit_auto(struct unit *punit);
 void request_unit_build_city(struct unit *punit);
 void request_unit_caravan_action(struct unit *punit, enum packet_type action);
 void request_unit_change_homecity(struct unit *punit);
-void request_unit_disband(struct unit *punit);
-void request_unit_goto(void);
 void request_unit_connect(void);
+void request_unit_disband(struct unit *punit);
+void request_unit_fortify(struct unit *punit);
+void request_unit_goto(void);
 void request_unit_move_done(void);
 void request_unit_nuke(struct unit *punit);
 void request_unit_paradrop(struct unit *punit);
+void request_unit_patrol(void);
+void request_unit_pillage(struct unit *punit);
+void request_unit_selected(struct unit *punit);
+void request_unit_sentry(struct unit *punit);
 void request_unit_unload(struct unit *punit);
+void request_unit_airlift(struct unit *punit, struct city *pcity);
 void request_unit_upgrade(struct unit *punit);
 void request_unit_wait(struct unit *punit);
 void request_unit_wakeup(struct unit *punit);
 void request_toggle_map_grid(void);
 void request_toggle_city_names(void);
 void request_toggle_city_productions(void);
+void request_toggle_terrain(void);
+void request_toggle_coastline(void);
+void request_toggle_roads_rails(void);
+void request_toggle_irrigation(void);
+void request_toggle_mines(void);
+void request_toggle_fortress_airbase(void);
+void request_toggle_specials(void);
+void request_toggle_pollution(void);
+void request_toggle_cities(void);
+void request_toggle_units(void);
+void request_toggle_focus_unit(void);
+void request_toggle_fog_of_war(void);
 
 void wakeup_sentried_units(int x, int y);
 
@@ -70,6 +99,18 @@ void process_diplomat_arrival(struct unit *punit, int victim_id);
 void key_cancel_action(void);
 void key_city_names_toggle(void);
 void key_city_productions_toggle(void);
+void key_terrain_toggle(void);
+void key_coastline_toggle(void);
+void key_roads_rails_toggle(void);
+void key_irrigation_toggle(void);
+void key_mines_toggle(void);
+void key_fortress_airbase_toggle(void);
+void key_specials_toggle(void);
+void key_pollution_toggle(void);
+void key_cities_toggle(void);
+void key_units_toggle(void);
+void key_focus_unit_toggle(void);
+void key_fog_of_war_toggle(void);
 void key_end_turn(void);
 void key_map_grid_toggle(void);
 void key_move_north(void);
@@ -87,6 +128,7 @@ void key_unit_auto_settle(void);
 void key_unit_build_city(void);
 void key_unit_build_wonder(void);
 void key_unit_connect(void);
+void key_unit_diplomat_actions(void);
 void key_unit_disband(void);
 void key_unit_done(void);
 void key_unit_fallout(void);
@@ -97,6 +139,7 @@ void key_unit_homecity(void);
 void key_unit_irrigate(void);
 void key_unit_mine(void);
 void key_unit_nuke(void);
+void key_unit_patrol(void);
 void key_unit_paradrop(void);
 void key_unit_pillage(void);
 void key_unit_pollution(void);
