@@ -34,6 +34,8 @@
 #include "chatline.h"
 #include "citydlg.h"
 #include "cityrepdata.h"
+#include "clinet.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "optiondlg.h"
@@ -43,51 +45,44 @@
 
 #include "cityrep.h"
 
-extern GtkWidget *toplevel;
-
-extern struct connection aconnection;
-extern int delay_report_update;
-
 #define NEG_VAL(x)  ((x)<0 ? (x) : (-x))
 
 /******************************************************************/
-GtkWidget *config_shell;
-GtkWidget *config_toggle[NUM_CREPORT_COLS];
-
-void create_city_report_config_dialog(void);
-void popup_city_report_config_dialog(void);
-void config_ok_command_callback(GtkWidget *widget, gpointer data);
-
+static GtkWidget *config_shell;
+static GtkWidget *config_toggle[NUM_CREPORT_COLS];
 
+static void create_city_report_config_dialog(void);
+static void popup_city_report_config_dialog(void);
+static void config_ok_command_callback(GtkWidget *widget, gpointer data);
 
 /******************************************************************/
-void create_city_report_dialog(int make_modal);
-void city_close_callback(GtkWidget *widget, gpointer data);
-void city_center_callback(GtkWidget *widget, gpointer data);
-void city_popup_callback(GtkWidget *widget, gpointer data);
-void city_buy_callback(GtkWidget *widget, gpointer data);
-void city_refresh_callback(GtkWidget *widget, gpointer data);
-void city_change_all_dialog_callback(GtkWidget *w, gpointer data);
-void city_change_all_callback(GtkWidget *w, gpointer data);
-void city_list_callback(GtkWidget *w, gint row, gint column);
-void city_config_callback(GtkWidget *widget, gpointer data);
+static void create_city_report_dialog(int make_modal);
+static void city_close_callback(GtkWidget *widget, gpointer data);
+static void city_center_callback(GtkWidget *widget, gpointer data);
+static void city_popup_callback(GtkWidget *widget, gpointer data);
+static void city_buy_callback(GtkWidget *widget, gpointer data);
+static void city_refresh_callback(GtkWidget *widget, gpointer data);
+static void city_change_all_dialog_callback(GtkWidget *w, gpointer data);
+static void city_change_all_callback(GtkWidget *w, gpointer data);
+static void city_list_callback(GtkWidget *w, gint row, gint column);
+static void city_config_callback(GtkWidget *widget, gpointer data);
 static gboolean city_change_callback(GtkWidget *w, GdkEvent *event, gpointer data);
 static gboolean city_select_callback(GtkWidget *w, GdkEvent *event, gpointer data);
 static void city_report_list_callback(GtkWidget *w, gint row, gint col, GdkEvent *ev, gpointer data);
 
-GtkWidget *city_dialog_shell=NULL;
-GtkWidget *city_label;
-GtkWidget *city_list;
-GtkWidget *city_center_command, *city_popup_command, *city_buy_command,
-	  *city_refresh_command, *city_config_command;
-GtkWidget *city_change_command;
-GtkWidget *city_change_all_command;
-GtkWidget *city_change_all_dialog_shell;
-GtkWidget *city_change_all_from_list;
-GtkWidget *city_change_all_to_list;
-GtkWidget *city_select_command;
+static GtkWidget *city_dialog_shell=NULL;
+static GtkWidget *city_label;
+static GtkWidget *city_list;
+static GtkWidget *city_center_command, *city_popup_command, *city_buy_command,
+  *city_refresh_command, *city_config_command;
+static GtkWidget *city_change_command;
+static GtkWidget *city_change_all_command;
+static GtkWidget *city_change_all_dialog_shell;
+static GtkWidget *city_change_all_from_list;
+static GtkWidget *city_change_all_to_list;
+static GtkWidget *city_select_command;
 
-int city_dialog_shell_is_modal;
+static int city_dialog_shell_is_modal;
 
 /*
  * Sort cities by column...
@@ -137,8 +132,8 @@ static void get_city_table_header(char *text[], int n)
 
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     my_snprintf(text[i], n, "%*s\n%*s",
-            NEG_VAL(spec->width), _(spec->title1),
-	    NEG_VAL(spec->width), _(spec->title2));
+	    NEG_VAL(spec->width), spec->title1 ? _(spec->title1) : "",
+	    NEG_VAL(spec->width), spec->title2 ? _(spec->title2) : "");
   }
 }
 
@@ -157,7 +152,7 @@ void popup_city_report_dialog(int make_modal)
       city_dialog_shell_is_modal=make_modal;
     
       if(make_modal)
-	gtk_widget_set_sensitive(toplevel, FALSE);
+	gtk_widget_set_sensitive(top_vbox, FALSE);
       
       create_city_report_dialog(make_modal);
       gtk_set_relative_position(toplevel, city_dialog_shell, 10, 10);
@@ -185,6 +180,9 @@ city_from_glist(GList* list)
   return retval;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 typedef gboolean TestCityFunc(struct city *, gint);
 
 static gboolean city_can_build_impr_or_unit(struct city *pcity, gint number)
@@ -194,15 +192,16 @@ static gboolean city_can_build_impr_or_unit(struct city *pcity, gint number)
   else
     return can_build_improvement(pcity, number);
 }
-
-static void
-append_impr_or_unit_to_menu_sub(GtkWidget *menu,
-			       gchar *nothing_appended_text,
-			       gboolean append_units,
-			       gboolean append_wonders,
-			       gboolean change_prod,
-			       TestCityFunc test_func,
-			       GtkSignalFunc callback)
+/****************************************************************
+...
+*****************************************************************/
+static void append_impr_or_unit_to_menu_sub(GtkWidget *menu,
+					    gchar *nothing_appended_text,
+					    gboolean append_units,
+					    gboolean append_wonders,
+					    gboolean change_prod,
+					    TestCityFunc test_func,
+					    GtkSignalFunc callback)
 {  
   gint i;
   gint first = append_units ? B_LAST : 0;
@@ -288,8 +287,10 @@ append_impr_or_unit_to_menu_sub(GtkWidget *menu,
     }
 }
 
-static
-void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
+/****************************************************************
+...
+*****************************************************************/
+static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
 {
   gint number = GPOINTER_TO_INT(data);
   gint i;
@@ -334,6 +335,9 @@ void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
     }
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static void
 append_impr_or_unit_to_menu(GtkWidget *menu,
 			   gboolean change_prod,
@@ -377,7 +381,9 @@ append_impr_or_unit_to_menu(GtkWidget *menu,
 		      GINT_TO_POINTER(change_prod));
 }
 
-
+/****************************************************************
+...
+*****************************************************************/
 static gint 
 city_change_callback(GtkWidget *w, GdkEvent *event, gpointer data)
 {
@@ -411,7 +417,7 @@ city_change_callback(GtkWidget *w, GdkEvent *event, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void create_city_report_dialog(int make_modal)
+static void create_city_report_dialog(int make_modal)
 {
   static char *titles	[NUM_CREPORT_COLS];
   static char  buf	[NUM_CREPORT_COLS][64];
@@ -426,9 +432,9 @@ void create_city_report_dialog(int make_modal)
         GTK_SIGNAL_FUNC(city_close_callback),NULL );
   gtk_accel_group_attach(accel, GTK_OBJECT(city_dialog_shell));
 
-  gtk_window_set_title(GTK_WINDOW(city_dialog_shell),_("City Report"));
+  gtk_window_set_title(GTK_WINDOW(city_dialog_shell),_("Cities"));
 
-  report_title=get_report_title(_("City Advisor"));
+  report_title=get_report_title(_("Cities"));
   city_label = gtk_label_new(report_title);
 
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->vbox ),
@@ -484,9 +490,6 @@ void create_city_report_dialog(int make_modal)
         city_change_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( city_change_command, GTK_CAN_DEFAULT );
 
-  gtk_signal_connect( GTK_OBJECT( city_change_command ), "event",
-	GTK_SIGNAL_FUNC( city_change_callback ), NULL );
-
   city_change_all_command = gtk_accelbutton_new(_("Change _All"), accel);
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->action_area ),
         city_change_all_command, TRUE, TRUE, 0 );
@@ -502,14 +505,13 @@ void create_city_report_dialog(int make_modal)
         city_select_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( city_select_command, GTK_CAN_DEFAULT );
 
-  gtk_signal_connect(GTK_OBJECT(city_select_command), "event",
-	GTK_SIGNAL_FUNC(city_select_callback), NULL);
-
   city_config_command	= gtk_accelbutton_new(_("Con_figure"), accel);
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(city_dialog_shell)->action_area ),
         city_config_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( city_config_command, GTK_CAN_DEFAULT );
 
+  gtk_signal_connect(GTK_OBJECT(city_select_command), "event",
+	GTK_SIGNAL_FUNC(city_select_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(close_command), "clicked",
 	GTK_SIGNAL_FUNC(city_close_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(city_center_command), "clicked",
@@ -522,6 +524,8 @@ void create_city_report_dialog(int make_modal)
 	GTK_SIGNAL_FUNC(city_refresh_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(city_config_command), "clicked",
 	GTK_SIGNAL_FUNC(city_config_callback), NULL);
+  gtk_signal_connect( GTK_OBJECT( city_change_command ), "event",
+	GTK_SIGNAL_FUNC( city_change_callback ), NULL );
   gtk_signal_connect(GTK_OBJECT(city_change_all_command), "clicked",
 	GTK_SIGNAL_FUNC(city_change_all_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(city_list), "select_row",
@@ -545,7 +549,7 @@ void create_city_report_dialog(int make_modal)
 /****************************************************************
 ...
 *****************************************************************/
-void city_list_callback(GtkWidget *w, gint row, gint column)
+static void city_list_callback(GtkWidget *w, gint row, gint column)
 {
   if (GTK_CLIST(city_list)->selection)
   {
@@ -563,6 +567,9 @@ void city_list_callback(GtkWidget *w, gint row, gint column)
   }
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_select_all_callback(GtkWidget *w, gpointer data)
 {
@@ -570,6 +577,9 @@ city_select_all_callback(GtkWidget *w, gpointer data)
   return TRUE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_unselect_all_callback(GtkWidget *w, gpointer data)
 {
@@ -577,6 +587,9 @@ city_unselect_all_callback(GtkWidget *w, gpointer data)
   return TRUE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_invert_selection_callback(GtkWidget *w, gpointer data)
 {
@@ -593,6 +606,9 @@ city_invert_selection_callback(GtkWidget *w, gpointer data)
   return TRUE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_select_coastal_callback(GtkWidget *w, gpointer data)
 {
@@ -611,6 +627,9 @@ city_select_coastal_callback(GtkWidget *w, gpointer data)
   return TRUE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_select_same_island_callback(GtkWidget *w, gpointer data)
 {
@@ -642,6 +661,9 @@ city_select_same_island_callback(GtkWidget *w, gpointer data)
   return TRUE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_unit_supported(struct city *pcity, gint unit)
 {
@@ -655,6 +677,9 @@ city_unit_supported(struct city *pcity, gint unit)
   return FALSE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_unit_present(struct city *pcity, gint unit)
 {
@@ -668,6 +693,9 @@ city_unit_present(struct city *pcity, gint unit)
   return FALSE;
 }
       
+/****************************************************************
+...
+*****************************************************************/
 static gboolean
 city_select_callback(GtkWidget *w, GdkEvent *event, gpointer data)
 {
@@ -760,7 +788,7 @@ city_select_callback(GtkWidget *w, GdkEvent *event, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void city_refresh_callback(GtkWidget *w, gpointer data)
+static void city_refresh_callback(GtkWidget *w, gpointer data)
 
 { /* added by Syela - I find this very useful */
   GList *selection = GTK_CLIST(city_list)->selection;
@@ -783,52 +811,50 @@ void city_refresh_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 Handle callbacks from the "change all" dialog.
 *****************************************************************/
-void city_change_all_dialog_callback(GtkWidget *w, gpointer data)
+static void city_change_all_dialog_callback(GtkWidget *w, gpointer data)
 {
-    GList *selection;
+  char *cmd = (char *)data;
+
+  if (cmd != NULL) {
+    GList *selection_from, *selection_to;
     gint row;
     int from, to;
-    char *cmd = (char *)data;
     char buf[512];
 
-    if (cmd != NULL) {
-	if ( !( selection = GTK_CLIST( city_change_all_from_list )->selection ) ) {
-	    append_output_window(_("Game: Select a unit or improvement"
-				   " to change production from."));
-	    return;
-	}
-	else {
-	    row = (gint)selection->data;
-	    from = (int)gtk_clist_get_row_data(GTK_CLIST(city_change_all_from_list),
-					       row);
-	}
-	if ( !( selection = GTK_CLIST( city_change_all_to_list )->selection ) ) {
-	    append_output_window(_("Game: Select a unit or improvement"
-				   " to change production to."));
-	    return;
-	}
-	else {
-	    row = (gint)selection->data;
-	    to = (int)gtk_clist_get_row_data(GTK_CLIST(city_change_all_to_list),
-					       row);
-	}
-	if (from==to) {
-	    append_output_window(_("Game: That's the same thing!"));
-	    return;
-	}
-	my_snprintf(buf, sizeof(buf),
-		    _("Game: Changing production of every %s into %s."),
-		(from >= B_LAST) ?
+    /* What are we changing to? */
+    selection_to = GTK_CLIST(city_change_all_to_list)->selection;
+    if (!selection_to) {
+      append_output_window(_("Game: Select a unit or improvement"
+			     " to change production to."));
+      return;
+    } else {
+      row = (gint)selection_to->data;
+      to = (int)gtk_clist_get_row_data(GTK_CLIST(city_change_all_to_list),
+				       row);
+    }
+
+    /* Iterate over the items we change from */
+    selection_from = GTK_CLIST(city_change_all_from_list)->selection;
+    for (; selection_from; selection_from = g_list_next(selection_from)) {
+      row = (gint)selection_from->data;
+      from = (int)gtk_clist_get_row_data(GTK_CLIST(city_change_all_from_list),
+					 row);
+      if (from == to) {
+	continue;
+      }
+      my_snprintf(buf, sizeof(buf),
+		  _("Game: Changing production of every %s into %s."),
+		  (from >= B_LAST) ?
   		  get_unit_type(from-B_LAST)->name : get_improvement_name(from),
-		(to >= B_LAST) ?
+		  (to >= B_LAST) ?
 		  get_unit_type(to-B_LAST)->name : get_improvement_name(to));
 
-	append_output_window(buf);
-	
-	client_change_all(from,to);
+      append_output_window(buf);
+      client_change_all(from, to);
     }
-    gtk_widget_destroy(city_change_all_dialog_shell);
-    city_change_all_dialog_shell = NULL;
+  }
+  gtk_widget_destroy(city_change_all_dialog_shell);
+  city_change_all_dialog_shell = NULL;
 }
 
 /****************************************************************
@@ -836,7 +862,7 @@ Change all cities building one type of thing to another thing.
 This is a callback for the "change all" button.
 *****************************************************************/
 #define MAX_LEN_BUF 256
-void city_change_all_callback(GtkWidget *w, gpointer data)
+static void city_change_all_callback(GtkWidget *w, gpointer data)
 {
     GList              *selection;
     gint                row;
@@ -874,7 +900,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 	city_change_all_from_list = gtk_clist_new_with_titles(1, title[0]);
 	gtk_clist_column_titles_passive(GTK_CLIST(city_change_all_from_list));
 	gtk_clist_set_selection_mode(GTK_CLIST(city_change_all_from_list),
-				     GTK_SELECTION_SINGLE);
+				     GTK_SELECTION_EXTENDED);
 	scrollpane = gtk_scrolled_window_new(NULL, NULL);
 	gtk_container_add(GTK_CONTAINER(scrollpane),
 			  city_change_all_from_list);
@@ -923,7 +949,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 		j = gtk_clist_append(GTK_CLIST(city_change_all_from_list), buf);
 		gtk_clist_set_row_data(GTK_CLIST(city_change_all_from_list),
 				       j, (gpointer)(i+B_LAST));
-		if (is_building[i] == 2) {
+		if (is_building[B_LAST+i] == 2) {
 		    gtk_clist_select_row(GTK_CLIST(city_change_all_from_list),
 					  j, 0);
 		}
@@ -940,7 +966,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 	/* make a list of everything we could build */
 	city_change_all_to_list = gtk_clist_new_with_titles(1, title[1]);
 	gtk_clist_column_titles_passive(GTK_CLIST(city_change_all_to_list));
-	gtk_clist_set_selection_mode(GTK_CLIST(city_change_all_from_list),
+	gtk_clist_set_selection_mode(GTK_CLIST(city_change_all_to_list),
 				     GTK_SELECTION_SINGLE);
 	scrollpane = gtk_scrolled_window_new(NULL, NULL);
 	gtk_container_add(GTK_CONTAINER(scrollpane),
@@ -1005,7 +1031,7 @@ void city_change_all_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void city_buy_callback(GtkWidget *w, gpointer data)
+static void city_buy_callback(GtkWidget *w, gpointer data)
 {
   GList *current = GTK_CLIST(city_list)->selection;
 
@@ -1045,11 +1071,11 @@ void city_buy_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void city_close_callback(GtkWidget *w, gpointer data)
+static void city_close_callback(GtkWidget *w, gpointer data)
 {
 
   if(city_dialog_shell_is_modal)
-     gtk_widget_set_sensitive(toplevel, TRUE);
+     gtk_widget_set_sensitive(top_vbox, TRUE);
    gtk_widget_destroy(city_dialog_shell);
    city_dialog_shell=NULL;
 }
@@ -1057,7 +1083,7 @@ void city_close_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void city_center_callback(GtkWidget *w, gpointer data)
+static void city_center_callback(GtkWidget *w, gpointer data)
 {
   GList *current = GTK_CLIST(city_list)->selection;
   struct city        *pcity;
@@ -1072,7 +1098,7 @@ void city_center_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void city_popup_callback(GtkWidget *w, gpointer data)
+static void city_popup_callback(GtkWidget *w, gpointer data)
 {
   GList *current = GTK_CLIST(city_list)->selection;
   GList *copy = NULL;
@@ -1099,8 +1125,8 @@ void city_popup_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-
-static void city_report_list_callback(GtkWidget *w, gint row, gint col, GdkEvent *ev, gpointer data)
+static void city_report_list_callback(GtkWidget *w, gint row, gint col,
+				      GdkEvent *ev, gpointer data)
 {
   /* Pops up a city when it's double-clicked */
   if(ev && ev->type==GDK_2BUTTON_PRESS)
@@ -1110,7 +1136,7 @@ static void city_report_list_callback(GtkWidget *w, gint row, gint col, GdkEvent
 /****************************************************************
 ...
 *****************************************************************/
-void city_config_callback(GtkWidget *w, gpointer data)
+static void city_config_callback(GtkWidget *w, gpointer data)
 {
   popup_city_report_config_dialog();
 }
@@ -1132,7 +1158,7 @@ void city_report_dialog_update(void)
   if(delay_report_update) return;
   if(!city_dialog_shell) return;
 
-    report_title=get_report_title(_("City Advisor"));
+    report_title=get_report_title(_("Cities"));
     gtk_set_label(city_label, report_title);
 
     for (i=0, spec=city_report_specs;i<NUM_CREPORT_COLS;i++, spec++)
@@ -1236,7 +1262,7 @@ void city_report_dialog_update_city(struct city *pcity)
 /****************************************************************
 ... 
 *****************************************************************/
-void popup_city_report_config_dialog(void)
+static void popup_city_report_config_dialog(void)
 {
   int i;
 
@@ -1258,14 +1284,14 @@ void popup_city_report_config_dialog(void)
 /****************************************************************
 ...
 *****************************************************************/
-void create_city_report_config_dialog(void)
+static void create_city_report_config_dialog(void)
 {
   GtkWidget *config_label, *config_ok_command;
   struct city_report_spec *spec;
   int i;
   
   config_shell = gtk_dialog_new();
-  gtk_window_set_title (GTK_WINDOW(config_shell), _("Configure City Report"));
+  gtk_window_set_title (GTK_WINDOW(config_shell), _("Configure Cities Report"));
   gtk_window_set_position (GTK_WINDOW(config_shell), GTK_WIN_POS_MOUSE);
 
   config_label = gtk_label_new(_("Set columns shown"));
@@ -1297,7 +1323,7 @@ void create_city_report_config_dialog(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void config_ok_command_callback(GtkWidget *w, gpointer data)
+static void config_ok_command_callback(GtkWidget *w, gpointer data)
 {
   struct city_report_spec *spec;
   gboolean b;
