@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,7 +24,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "capability.h"
 #include "fcintl.h"
 #include "game.h"
 #include "government.h"
@@ -39,99 +39,94 @@
 #include "civclient.h"
 #include "clinet.h"
 #include "control.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
+#include "options.h"
 #include "tilespec.h"
 
 #include "dialogs.h"
 
-
-extern GtkWidget *toplevel;
-extern GdkWindow *root_window;
-extern struct connection aconnection;
-extern int ai_popup_windows;
-extern GdkGC *fill_bg_gc;
-
 /******************************************************************/
 static gint popup_mes_del_callback(GtkWidget *widget, GdkEvent *event,
 				   gpointer data);
 
 /******************************************************************/
-GtkWidget	*races_dialog_shell=NULL;
-GtkWidget	*races_toggles_form;
-GtkWidget       *races_sex_toggles_form;
-GtkWidget       *city_style_toggles_form;
-GtkWidget	*races_ok_command;		/* ok button */
-GtkWidget	*races_disc_command=NULL;	/* disc button */
-GtkWidget	*races_quit_command=NULL;	/* quit button */
-GtkWidget	**races_toggles=NULL,		/* toggle race */
-                *races_sex_toggles       [2],   /* Male/Female */
-                **city_style_toggles = NULL,
-		*races_name;			/* leader name */
-GList           *leader_strings = NULL;
-GList           *sorted_races_list = NULL; /* contains a list of race
+static GtkWidget  *races_dialog_shell=NULL;
+static GtkWidget  *races_toggles_form;
+static GtkWidget  *races_sex_toggles_form;
+static GtkWidget  *city_style_toggles_form;
+static GtkWidget  *races_ok_command;            /* ok button */
+static GtkWidget  *races_disc_command=NULL;     /* disc button */
+static GtkWidget  *races_quit_command=NULL;     /* quit button */
+static GtkWidget **races_toggles=NULL,          /* toggle race */
+                  *races_sex_toggles[2],        /* Male/Female */
+                 **city_style_toggles = NULL,
+                  *races_name;                  /* leader name */
+static GList      *leader_strings = NULL;
+static GList      *sorted_races_list = NULL; /* contains a list of race
 					      ids sorted by the race
 					      name. Is valid as long
 					      as the races_dialog is
 					      poped up. */
 /******************************************************************/
-GtkWidget *spy_tech_shell;
-GtkWidget *spy_advances_list, *spy_advances_list_label;
-GtkWidget *spy_steal_command;
+static GtkWidget  *spy_tech_shell;
+static GtkWidget  *spy_advances_list;
+static GtkWidget  *spy_steal_command;
 
-int spy_tech_shell_is_modal;
-int advance_type[A_LAST+1];
-int steal_advance = 0;
+static int         spy_tech_shell_is_modal;
+static int         advance_type[A_LAST+1];
+static int         steal_advance = 0;
 
 /******************************************************************/
-GtkWidget *spy_sabotage_shell;
-GtkWidget *spy_improvements_list;
-GtkWidget *spy_sabotage_command;
+static GtkWidget  *spy_sabotage_shell;
+static GtkWidget  *spy_improvements_list;
+static GtkWidget  *spy_sabotage_command;
 
-int spy_sabotage_shell_is_modal;
-int improvement_type[B_LAST+1];
-int sabotage_improvement = 0;
+static int         spy_sabotage_shell_is_modal;
+static int         improvement_type[B_LAST+1];
+static int         sabotage_improvement = 0;
 
 /******************************************************************/
 
-GtkWidget *unit_select_dialog_shell;
-GtkWidget *unit_select_commands[100];
-GtkWidget *unit_select_labels[100];
-int unit_select_ids[100];
-int unit_select_no;
+#define MAX_SELECT_UNITS 100
+static GtkWidget  *unit_select_dialog_shell;
+static GtkWidget  *unit_select_commands[MAX_SELECT_UNITS];
+static GtkWidget  *unit_select_labels[MAX_SELECT_UNITS];
+static int         unit_select_ids[MAX_SELECT_UNITS];
+static int         unit_select_no;
 
+static int races_buttons_get_current(void);
+static int sex_buttons_get_current(void);
+static int city_style_get_current(void);
 
-int races_buttons_get_current(void);
-int sex_buttons_get_current(void);
-int city_style_get_current(void);
+static void create_races_dialog	(void);
+static void races_buttons_callback	( GtkWidget *w, gpointer data );
+static void races_toggles_callback	( GtkWidget *w, gpointer data );
+static void races_sex_toggles_callback ( GtkWidget *w, gpointer data );
+static void races_name_callback	( GtkWidget *w, gpointer data );
+static void city_style_toggles_callback ( GtkWidget *w, gpointer data );
 
-void create_races_dialog	(void);
-void races_buttons_callback	( GtkWidget *w, gpointer data );
-void races_toggles_callback	( GtkWidget *w, gpointer data );
-void races_sex_toggles_callback ( GtkWidget *w, gpointer data );
-void races_name_callback	( GtkWidget *w, gpointer data );
-void city_style_toggles_callback ( GtkWidget *w, gpointer data );
-
-int selected_nation;
-int selected_leader;
-int selected_sex;
-int selected_city_style;
-int city_style_idx[64];        /* translation table basic style->city_style  */
-int city_style_ridx[64];       /* translation table the other way            */
+static int selected_nation;
+static int selected_leader;
+static int selected_sex;
+static int selected_city_style;
+static int city_style_idx[64];        /* translation table basic style->city_style  */
+static int city_style_ridx[64];       /* translation table the other way            */
                                /* they in fact limit the num of styles to 64 */
-int b_s_num; /* number of basic city styles, i.e. those that you can start with */
+static int b_s_num; /* number of basic city styles, i.e. those that you can start with */
 
-int is_showing_government_dialog;
+static int is_showing_government_dialog;
 
-int is_showing_pillage_dialog = FALSE;
-int unit_to_use_to_pillage;
+static int is_showing_pillage_dialog = FALSE;
+static int unit_to_use_to_pillage;
 
-int caravan_city_id;
-int caravan_unit_id;
+static int caravan_city_id;
+static int caravan_unit_id;
 
-int diplomat_dialog_open = 0;
-int diplomat_id;
-int diplomat_target_id;
+static int diplomat_dialog_open = 0;
+static int diplomat_id;
+static int diplomat_target_id;
 
 struct city *pcity_caravan_dest;
 struct unit *punit_caravan;
@@ -149,12 +144,15 @@ static int connect_unit_y;
 static void notify_command_callback(GtkWidget *w, GtkWidget *t)
 {
   gtk_widget_destroy( t );
-  gtk_widget_set_sensitive( toplevel, TRUE );
+  gtk_widget_set_sensitive( top_vbox, TRUE );
 }
 
-gint deleted_callback( GtkWidget *widget, GdkEvent *event, gpointer data )
+/****************************************************************
+...
+*****************************************************************/
+gint deleted_callback(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-  gtk_widget_set_sensitive( toplevel, TRUE );
+  gtk_widget_set_sensitive( top_vbox, TRUE );
   return FALSE;
 }
 
@@ -222,7 +220,7 @@ void popup_notify_dialog(char *caption, char *headline, char *lines)
   gtk_set_relative_position (toplevel, notify_dialog_shell, 10, 10);
   gtk_widget_show( notify_dialog_shell );
 
-  gtk_widget_set_sensitive( toplevel, FALSE );
+  gtk_widget_set_sensitive( top_vbox, FALSE );
 }
 
 /****************************************************************
@@ -288,21 +286,21 @@ static void notify_goto_command_callback(GtkWidget *w, gpointer data)
   notify_goto_widget_remove(w);
 
   gtk_widget_destroy(w->parent->parent->parent);
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
 }
 
 static void notify_no_goto_command_callback(GtkWidget *w, gpointer data)
 {
   notify_goto_widget_remove(w);
   gtk_widget_destroy(w->parent->parent->parent);
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
 }
 
 static gint notify_deleted_callback(GtkWidget *widget, GdkEvent *event,
 				    gpointer data)
 {
   notify_goto_widget_remove(widget);
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
   return FALSE;
 }
 
@@ -314,7 +312,7 @@ void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
   GtkWidget *notify_dialog_shell, *notify_command, *notify_goto_command;
   GtkWidget *notify_label;
   
-  if (x == 0 && y == 0) {
+  if (y < 0 || y >= map.ysize) {
     popup_notify_dialog(_("Message:"), headline, lines);
     return;
   }
@@ -348,7 +346,7 @@ void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
   gtk_widget_show_all( GTK_DIALOG(notify_dialog_shell)->action_area );
   gtk_widget_show(notify_dialog_shell);
 
-  gtk_widget_set_sensitive(toplevel, FALSE);
+  gtk_widget_set_sensitive(top_vbox, FALSE);
 }
 
 
@@ -406,7 +404,7 @@ void popup_bribe_dialog(struct unit *punit)
     my_snprintf(buf, sizeof(buf),
 		_("Bribe unit for %d gold?\nTreasury contains %d gold."), 
 		punit->bribe_cost, game.player_ptr->economic.gold);
-    popup_message_dialog(toplevel, /*"diplomatbribedialog"*/_("Bribe Enemy Unit"), buf,
+    popup_message_dialog(top_vbox, /*"diplomatbribedialog"*/_("Bribe Enemy Unit"), buf,
 			_("_Yes"), diplomat_bribe_yes_callback, 0,
 			_("_No"), diplomat_bribe_no_callback, 0, 0);
   } else {
@@ -414,7 +412,7 @@ void popup_bribe_dialog(struct unit *punit)
 		_("Bribing the unit costs %d gold.\n"
 		  "Treasury contains %d gold."), 
 		punit->bribe_cost, game.player_ptr->economic.gold);
-    popup_message_dialog(toplevel, /*"diplomatnogolddialog"*/_("Traitors Demand Too Much!"), buf,
+    popup_message_dialog(top_vbox, /*"diplomatnogolddialog"*/_("Traitors Demand Too Much!"), buf,
 			_("Darn"), diplomat_bribe_no_callback, 0, 
 			0);
   }
@@ -556,7 +554,7 @@ static void spy_close_tech_callback(GtkWidget *w, gpointer data)
 {
 
   if(spy_tech_shell_is_modal)
-     gtk_widget_set_sensitive(toplevel, TRUE);
+     gtk_widget_set_sensitive(top_vbox, TRUE);
    gtk_widget_destroy(spy_tech_shell);
    spy_tech_shell=0;
 
@@ -570,7 +568,7 @@ static void spy_close_sabotage_callback(GtkWidget *w, gpointer data)
 {
 
   if(spy_sabotage_shell_is_modal)
-     gtk_widget_set_sensitive(toplevel, TRUE);
+     gtk_widget_set_sensitive(top_vbox, TRUE);
    gtk_widget_destroy(spy_sabotage_shell);
    spy_sabotage_shell=0;
 
@@ -625,7 +623,7 @@ static void spy_steal_callback(GtkWidget *w, gpointer data)
   spy_tech_shell = 0l;
   
   if(!steal_advance){
-    freelog(LOG_NORMAL,"Bug in spy steal tech code.");
+    freelog(LOG_ERROR, "Bug in spy steal tech code.");
     process_diplomat_arrival(NULL, 0);
     return;
   }
@@ -654,7 +652,7 @@ static void spy_sabotage_callback(GtkWidget *w, gpointer data)
   spy_sabotage_shell = 0l;
   
   if(!sabotage_improvement){
-    freelog(LOG_NORMAL, "Bug in spy sabotage code");
+    freelog(LOG_ERROR, "Bug in spy sabotage code");
     process_diplomat_arrival(NULL, 0);
     return;
   }
@@ -833,7 +831,7 @@ static int create_improvements_list(struct player *pplayer,
   gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
   improvement_type[j++] = -1;
 
-  for(i=0; i<B_LAST; i++) {
+  for(i=0; i<game.num_impr_types; i++) {
     if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i)) {
       row[0] = get_impr_name_ex(pcity, i);
       gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
@@ -982,7 +980,7 @@ void popup_incite_dialog(struct city *pcity)
 		_("Incite a revolt for %d gold?\nTreasury contains %d gold."), 
 		pcity->incite_revolt_cost, game.player_ptr->economic.gold);
    diplomat_target_id = pcity->id;
-   popup_message_dialog(toplevel, /*"diplomatrevoltdialog"*/_("Incite a Revolt!"), buf,
+   popup_message_dialog(top_vbox, /*"diplomatrevoltdialog"*/_("Incite a Revolt!"), buf,
 		       _("_Yes"), diplomat_incite_yes_callback, 0,
 		       _("_No"), diplomat_incite_no_callback, 0, 0);
   } else {
@@ -990,13 +988,37 @@ void popup_incite_dialog(struct city *pcity)
 		_("Inciting a revolt costs %d gold.\n"
 		  "Treasury contains %d gold."), 
 		pcity->incite_revolt_cost, game.player_ptr->economic.gold);
-   popup_message_dialog(toplevel, /*"diplomatnogolddialog"*/_("Traitors Demand Too Much!"), buf,
+   popup_message_dialog(top_vbox, /*"diplomatnogolddialog"*/_("Traitors Demand Too Much!"), buf,
 		       _("Darn"), diplomat_incite_no_callback, 0, 
 		       0);
   }
 }
 
 
+/****************************************************************
+  Callback from diplomat/spy dialog for "keep moving".
+  (This should only occur when entering allied city.)
+*****************************************************************/
+static void diplomat_keep_moving_callback(GtkWidget *w, gpointer data)
+{
+  struct unit *punit;
+  struct city *pcity;
+  
+  destroy_message_dialog(w);
+  diplomat_dialog_open=0;
+
+  if( (punit=find_unit_by_id(diplomat_id))
+      && (pcity=find_city_by_id(diplomat_target_id))
+      && !same_pos(punit->x, punit->y, pcity->x, pcity->y)) {
+    struct packet_diplomat_action req;
+    req.action_type = DIPLOMAT_MOVE;
+    req.diplomat_id = diplomat_id;
+    req.target_id = diplomat_target_id;
+    send_packet_diplomat_action(&aconnection, &req);
+  }
+  process_diplomat_arrival(NULL, 0);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1030,13 +1052,14 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 		unit_name(punit->type), pcity->name);
 
     if(!unit_flag(punit->type, F_SPY)){
-      shl=popup_message_dialog(toplevel, /*"diplomatdialog"*/
+      shl=popup_message_dialog(top_vbox, /*"diplomatdialog"*/
 			       _(" Choose Your Diplomat's Strategy"), buf,
          		     _("Establish _Embassy"), diplomat_embassy_callback, 0,
          		     _("_Investigate City"), diplomat_investigate_callback, 0,
          		     _("_Sabotage City"), diplomat_sabotage_callback, 0,
          		     _("Steal _Technology"), diplomat_steal_callback, 0,
          		     _("Incite a _Revolt"), diplomat_incite_callback, 0,
+         		     _("_Keep moving"), diplomat_keep_moving_callback, 0,
          		     _("_Cancel"), diplomat_cancel_callback, 0,
          		     0);
       
@@ -1050,8 +1073,10 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
        message_dialog_button_set_sensitive(shl,"button3",FALSE);
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
        message_dialog_button_set_sensitive(shl,"button4",FALSE);
+      if(!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_x, dest_y))
+       message_dialog_button_set_sensitive(shl,"button5",FALSE);
     }else{
-       shl=popup_message_dialog(toplevel, /*"spydialog"*/
+       shl=popup_message_dialog(top_vbox, /*"spydialog"*/
 				_("Choose Your Spy's Strategy"), buf,
  			      _("Establish _Embassy"), diplomat_embassy_callback, 0,
  			      _("_Investigate City (free)"), diplomat_investigate_callback, 0,
@@ -1059,6 +1084,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
  			      _("Industrial _Sabotage"), spy_request_sabotage_list, 0,
  			      _("Steal _Technology"), spy_steal_popup, 0,
  			      _("Incite a _Revolt"), diplomat_incite_callback, 0,
+ 			      _("_Keep moving"), diplomat_keep_moving_callback, 0,
  			      _("_Cancel"), diplomat_cancel_callback, 0,
  			      0);
  
@@ -1074,6 +1100,8 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
        message_dialog_button_set_sensitive(shl,"button4",FALSE);
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
        message_dialog_button_set_sensitive(shl,"button5",FALSE);
+      if(!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_x, dest_y))
+       message_dialog_button_set_sensitive(shl,"button6",FALSE);
      }
 
     diplomat_dialog_open=1;
@@ -1083,7 +1111,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
        
        diplomat_target_id=ptunit->id;
  
-       shl=popup_message_dialog(toplevel, /*"spybribedialog"*/_("Subvert Enemy Unit"),
+       shl=popup_message_dialog(top_vbox, /*"spybribedialog"*/_("Subvert Enemy Unit"),
                               (!unit_flag(punit->type, F_SPY))?
  			      _("Sir, the diplomat is waiting for your command"):
  			      _("Sir, the spy is waiting for your command"),
@@ -1108,7 +1136,6 @@ int diplomat_dialog_is_open(void)
   return diplomat_dialog_open;
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -1186,7 +1213,7 @@ void popup_caravan_dialog(struct unit *punit,
   caravan_city_id=pdestcity->id; /* callbacks need these */
   caravan_unit_id=punit->id;
   
-  caravan_dialog=popup_message_dialog(toplevel,
+  caravan_dialog=popup_message_dialog(top_vbox,
 			   /*"caravandialog"*/_("Your Caravan Has Arrived"), 
 			   buf,
 			   _("Establish _Traderoute"),caravan_establish_trade_callback, 0,
@@ -1242,7 +1269,7 @@ void popup_government_dialog(void)
   if(!is_showing_government_dialog) {
     is_showing_government_dialog=1;
   
-    gtk_widget_set_sensitive(toplevel, FALSE);
+    gtk_widget_set_sensitive(top_vbox, FALSE);
   
     dshell=gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_position (GTK_WINDOW(dshell), GTK_WIN_POS_MOUSE);
@@ -1250,7 +1277,7 @@ void popup_government_dialog(void)
     gtk_signal_connect(
       GTK_OBJECT(dshell),
       "delete_event",
-      GTK_SIGNAL_FUNC(popup_mes_del_callback),
+      GTK_SIGNAL_FUNC(gtk_true),
       (gpointer) toplevel
     );
 
@@ -1268,7 +1295,7 @@ void popup_government_dialog(void)
     dlabel = gtk_label_new(_("Select government type:"));
     gtk_box_pack_start(GTK_BOX(vbox), dlabel, TRUE, FALSE, 0);
 
-    gtk_object_set_data(GTK_OBJECT(vbox), "parent",(gpointer) toplevel);
+    gtk_object_set_data(GTK_OBJECT(vbox), "parent",(gpointer) top_vbox);
 
     for (i=0; i<game.government_count; ++i) {
       struct government *g = &governments[i];
@@ -1320,7 +1347,7 @@ static void revolution_callback_no(GtkWidget *w, gpointer data)
 *****************************************************************/
 void popup_revolution_dialog(void)
 {
-  popup_message_dialog(toplevel, /*"revolutiondialog"*/_("Revolution!"), 
+  popup_message_dialog(top_vbox, /*"revolutiondialog"*/_("Revolution!"), 
 		       _("You say you wanna revolution?"),
 		       _("_Yes"),revolution_callback_yes, 0,
 		       _("_No"),revolution_callback_no, 0, 
@@ -1351,6 +1378,16 @@ static void pillage_callback(GtkWidget *w, gpointer data)
   is_showing_pillage_dialog = FALSE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static gint pillage_del_callback(GtkWidget *widget, GdkEvent *event,
+				 gpointer data)
+{
+  pillage_callback((GtkWidget *)data, NULL);
+  return FALSE;
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1363,18 +1400,10 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
     is_showing_pillage_dialog = TRUE;
     unit_to_use_to_pillage = punit->id;
 
-    gtk_widget_set_sensitive (toplevel, FALSE);
+    gtk_widget_set_sensitive (top_vbox, FALSE);
 
     dshell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_position (GTK_WINDOW(dshell), GTK_WIN_POS_MOUSE);
-
-    gtk_signal_connect (
-      GTK_OBJECT (dshell),
-      "delete_event",
-      GTK_SIGNAL_FUNC (popup_mes_del_callback),
-      (gpointer)toplevel
-    );
-
     gtk_window_set_title (GTK_WINDOW(dshell), _("What To Pillage"));
 
     vbox = gtk_vbox_new (0,TRUE);
@@ -1385,7 +1414,7 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
     dlabel = gtk_label_new (_("Select what to pillage:"));
     gtk_box_pack_start (GTK_BOX (vbox), dlabel, TRUE, FALSE, 0);
 
-    gtk_object_set_data (GTK_OBJECT (vbox), "parent", (gpointer)toplevel);
+    gtk_object_set_data (GTK_OBJECT (vbox), "parent", (gpointer)top_vbox);
 
     while (may_pillage) {
       what = get_preferred_pillage (may_pillage);
@@ -1408,6 +1437,13 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
       NULL
     );
 
+    gtk_signal_connect (
+      GTK_OBJECT (dshell),
+      "delete_event",
+      GTK_SIGNAL_FUNC (pillage_del_callback),
+      (gpointer)button
+    );
+
     gtk_widget_show_all (vbox);
     gtk_widget_show (dshell);
   }
@@ -1446,6 +1482,16 @@ static void unit_connect_callback (GtkWidget *w, gpointer data)
   is_showing_unit_connect_dialog = FALSE;
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static gint unit_connect_del_callback(GtkWidget *widget, GdkEvent *event,
+				      gpointer data)
+{
+  unit_connect_callback((GtkWidget *)data, (gpointer)ACTIVITY_IDLE);
+  return FALSE;
+}
+
 /****************************************************************
 popup dialog which prompts for activity type (unit connect)
 *****************************************************************/
@@ -1462,18 +1508,11 @@ void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
   connect_unit_x = dest_x;
   connect_unit_y = dest_y;
 
-  gtk_widget_set_sensitive (toplevel, FALSE);
+  gtk_widget_set_sensitive (top_vbox, FALSE);
 
   dshell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_position (GTK_WINDOW(dshell), GTK_WIN_POS_MOUSE);
 
-  gtk_signal_connect (
-    GTK_OBJECT (dshell),
-    "delete_event",
-    GTK_SIGNAL_FUNC (popup_mes_del_callback),
-    (gpointer)toplevel
-    );
-
   gtk_window_set_title (GTK_WINDOW(dshell), _("Connect"));
 
   vbox = gtk_vbox_new (0,TRUE);
@@ -1484,7 +1523,7 @@ void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
   dlabel = gtk_label_new (_("Choose unit activity:"));
   gtk_box_pack_start (GTK_BOX (vbox), dlabel, TRUE, FALSE, 0);
 
-  gtk_object_set_data (GTK_OBJECT (vbox), "parent", (gpointer)toplevel);
+  gtk_object_set_data (GTK_OBJECT (vbox), "parent", (gpointer)top_vbox);
 
   for (activity = ACTIVITY_IDLE + 1; activity < ACTIVITY_LAST; activity++) {
     if (! can_unit_do_connect (punit, activity)) continue;
@@ -1507,6 +1546,13 @@ void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
     (gpointer)ACTIVITY_IDLE
     );
 
+  gtk_signal_connect (
+    GTK_OBJECT (dshell),
+    "delete_event",
+    GTK_SIGNAL_FUNC (unit_connect_del_callback),
+    (gpointer)button
+  );
+
   gtk_widget_show_all (vbox);
   gtk_widget_show (dshell);
 }
@@ -1536,7 +1582,7 @@ void message_dialog_button_set_sensitive(GtkWidget *shl, char *bname,int state)
 ...
 *****************************************************************/
 GtkWidget *popup_message_dialog(GtkWidget *parent, char *dialogname,
-								char *text, ...)
+				char *text, ...)
 {
   va_list args;
   GtkWidget *dshell, *button, *dlabel, *vbox;
@@ -1616,7 +1662,7 @@ static void unit_select_all_callback(GtkWidget *w, gpointer data)
 {
   int i;
 
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
   gtk_widget_destroy(unit_select_dialog_shell);
   unit_select_dialog_shell=0;
   
@@ -1633,14 +1679,16 @@ static void unit_select_all_callback(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-static void unit_select_callback(GtkWidget *w, struct unit *punit)
+static void unit_select_callback(GtkWidget *w, int id)
 {
-  if(punit && punit->owner == game.player_idx) {
+  struct unit *punit = player_find_unit_by_id(game.player_ptr, id);
+
+  if (punit) {
     request_new_unit_activity(punit, ACTIVITY_IDLE);
     set_unit_focus(punit);
   }
 
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
   gtk_widget_destroy(unit_select_dialog_shell);
   unit_select_dialog_shell=0;
 }
@@ -1649,11 +1697,12 @@ static int number_of_columns(int n)
 {
 #if 0
   /* This would require libm, which isn't worth it for this one little
-   * function.  Since the number of units is limited to 100 already, the ifs
+   * function.  Since MAX_SELECT_UNITS is 100 already, the ifs
    * work fine.  */
   double sqrt(); double ceil();
   return ceil(sqrt((double)n/5.0));
 #else
+  assert(MAX_SELECT_UNITS == 100);
   if(n<=5) return 1;
   else if(n<=20) return 2;
   else if(n<=45) return 3;
@@ -1678,18 +1727,18 @@ void popup_unit_select_dialog(struct tile *ptile)
   GtkWidget *unit_select_all_command, *unit_select_close_command;
 
   if (!unit_select_dialog_shell){
-  gtk_widget_set_sensitive(toplevel, FALSE);
+  gtk_widget_set_sensitive(top_vbox, FALSE);
 
   unit_select_dialog_shell = gtk_dialog_new();
-  gtk_signal_connect( GTK_OBJECT(unit_select_dialog_shell),"delete_event",
-	GTK_SIGNAL_FUNC(deleted_callback),NULL );
+  gtk_signal_connect(GTK_OBJECT(unit_select_dialog_shell), "delete_event",
+		     unit_select_callback, NULL);
   gtk_window_set_position (GTK_WINDOW(unit_select_dialog_shell), GTK_WIN_POS_MOUSE);
 
   gtk_window_set_title(GTK_WINDOW(unit_select_dialog_shell),
 	_("Unit selection") );
 
-  n=unit_list_size(&ptile->units);
-  r=number_of_rows(n);
+  n = MIN(MAX_SELECT_UNITS, unit_list_size(&ptile->units));
+  r = number_of_rows(n);
 
   table=gtk_table_new(r, number_of_columns(n), FALSE);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(unit_select_dialog_shell)->vbox),
@@ -1715,13 +1764,13 @@ void popup_unit_select_dialog(struct tile *ptile)
 	    pcity ? pcity->name : "",
 	    unit_activity_text(punit));
 
-    pix = gtk_pixcomm_new(root_window, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    pix = gtk_pixcomm_new(root_window, UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT);
 
     unit_select_commands[i]=gtk_button_new();
     gtk_widget_set_sensitive(unit_select_commands[i],
        can_unit_do_activity(punit, ACTIVITY_IDLE) );
     gtk_widget_set_usize(unit_select_commands[i],
-       NORMAL_TILE_WIDTH+4, NORMAL_TILE_HEIGHT+4);
+			 UNIT_TILE_WIDTH+4, UNIT_TILE_HEIGHT+4);
     gtk_container_add(GTK_CONTAINER(unit_select_commands[i]), pix);
     gtk_box_pack_start(GTK_BOX(hbox),unit_select_commands[i],
        FALSE, FALSE, 0);
@@ -1729,14 +1778,14 @@ void popup_unit_select_dialog(struct tile *ptile)
     if (flags_are_transparent) {
        gtk_pixcomm_clear(GTK_PIXCOMM(pix), FALSE);
     }
-    put_unit_gpixmap(punit, GTK_PIXCOMM(pix), 0, 0);
+    put_unit_gpixmap(punit, GTK_PIXCOMM(pix));
 
     unit_select_labels[i]=gtk_label_new(buffer);
     gtk_box_pack_start(GTK_BOX(hbox),unit_select_labels[i],
        FALSE, FALSE, 0);
 
     gtk_signal_connect(GTK_OBJECT(unit_select_commands[i]), "clicked",
-       GTK_SIGNAL_FUNC(unit_select_callback), (gpointer)punit);
+       GTK_SIGNAL_FUNC(unit_select_callback), (gpointer)punit->id);
   }
   unit_select_no=i;
 
@@ -1771,7 +1820,7 @@ popup the dialog 10% inside the main-window
 *****************************************************************/
 void popup_races_dialog(void)
 {
-  gtk_widget_set_sensitive (toplevel, FALSE);
+  gtk_widget_set_sensitive (top_vbox, FALSE);
 
   create_races_dialog ();
 
@@ -1784,7 +1833,7 @@ void popup_races_dialog(void)
 void popdown_races_dialog(void)
 {
   if (races_dialog_shell) {
-    gtk_widget_set_sensitive (toplevel, TRUE);
+    gtk_widget_set_sensitive (top_vbox, TRUE);
     gtk_widget_destroy (races_dialog_shell);
     races_dialog_shell = NULL;
     g_list_free(sorted_races_list);
@@ -1805,7 +1854,7 @@ static gint cmp_func(gconstpointer a_p, gconstpointer b_p)
 Selectes a leader and the appropriate sex.
 Updates the gui elements and the selected_* variables.
 *****************************************************************/
-static void select_random_leader()
+static void select_random_leader(void)
 {
   int j, leader_num;
   char **leaders;
@@ -1821,12 +1870,12 @@ static void select_random_leader()
   gtk_combo_set_value_in_list( GTK_COMBO(races_name), FALSE, FALSE);
   gtk_combo_set_popdown_strings( GTK_COMBO(races_name), leader_strings);
 
-  /* initalize leader names */
+  /* initialize leader names */
   selected_leader = myrand(leader_num);
   gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(races_name)->entry),
 		      leaders[selected_leader]);
 
-  /* initalize leader sex */
+  /* initialize leader sex */
   selected_sex = get_nation_leader_sex(selected_nation,
 				       leaders[selected_leader]);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
@@ -1837,7 +1886,7 @@ static void select_random_leader()
 Selectes a random race and the appropriate city style.
 Updates the gui elements and the selected_* variables.
 *****************************************************************/
-static void select_random_race()
+static void select_random_race(void)
 {
   /* try to find a free nation */
   while(1) {
@@ -1847,12 +1896,12 @@ static void select_random_race()
       break;
   }
 
-  /* initalize nation toggle array */
+  /* initialize nation toggle array */
   gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON(
 		 races_toggles[g_list_index(sorted_races_list,
 			    GINT_TO_POINTER(selected_nation))] ), TRUE );
 
-  /* initalize city style */
+  /* initialize city style */
   selected_city_style =
     city_style_ridx[get_nation_city_style(selected_nation)];
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
@@ -1920,7 +1969,7 @@ void create_races_dialog(void)
 
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->vbox ),
 	fs, FALSE, FALSE, 0 );
-  races_sex_toggles_form = gtk_table_new( 2, 1, FALSE );
+  races_sex_toggles_form = gtk_table_new( 1, 2, FALSE );
   gtk_container_add( GTK_CONTAINER( fs ), races_sex_toggles_form ); 
 
   races_sex_toggles[0]= gtk_radio_button_new_with_label( sgroup, _("Male") );
@@ -2020,6 +2069,8 @@ void create_races_dialog(void)
   select_random_race();
   select_random_leader();
 
+  gtk_widget_grab_default(races_ok_command);
+
   gtk_widget_show_all( GTK_DIALOG(races_dialog_shell)->vbox );
   gtk_widget_show_all( GTK_DIALOG(races_dialog_shell)->action_area );
 }
@@ -2076,7 +2127,7 @@ void races_toggles_set_sensitive(int bits1, int bits2)
 /**************************************************************************
 ...
 **************************************************************************/
-void races_name_callback( GtkWidget *w, gpointer data )
+static void races_name_callback( GtkWidget *w, gpointer data )
 {
   char *lead;
   int nat;
@@ -2091,7 +2142,7 @@ void races_name_callback( GtkWidget *w, gpointer data )
 /**************************************************************************
 ...
 **************************************************************************/
-void races_toggles_callback( GtkWidget *w, gpointer race_id_p )
+static void races_toggles_callback( GtkWidget *w, gpointer race_id_p )
 {
   selected_nation = GPOINTER_TO_INT(race_id_p);
 
@@ -2105,7 +2156,7 @@ void races_toggles_callback( GtkWidget *w, gpointer race_id_p )
 /**************************************************************************
 ...
 **************************************************************************/
-void races_sex_toggles_callback( GtkWidget *w, gpointer data )
+static void races_sex_toggles_callback( GtkWidget *w, gpointer data )
 {
   if(w==races_sex_toggles[0]) 
       selected_sex = 1;
@@ -2116,7 +2167,7 @@ void races_sex_toggles_callback( GtkWidget *w, gpointer data )
 /**************************************************************************
 ...
 **************************************************************************/
-void city_style_toggles_callback( GtkWidget *w, gpointer data )
+static void city_style_toggles_callback( GtkWidget *w, gpointer data )
 {
   int i;
 
@@ -2130,17 +2181,17 @@ void city_style_toggles_callback( GtkWidget *w, gpointer data )
 /**************************************************************************
 ...
 **************************************************************************/
-int races_buttons_get_current(void)
+static int races_buttons_get_current(void)
 {
   return selected_nation;
 }
 
-int sex_buttons_get_current(void)
+static int sex_buttons_get_current(void)
 {
   return selected_sex;
 }
 
-int city_style_get_current(void)
+static int city_style_get_current(void)
 {
   return selected_city_style;
 }
@@ -2148,7 +2199,7 @@ int city_style_get_current(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void races_buttons_callback( GtkWidget *w, gpointer data )
+static void races_buttons_callback( GtkWidget *w, gpointer data )
 {
   int selected, selected_sex, selected_style;
   char *s;
@@ -2210,7 +2261,7 @@ void destroy_me_callback( GtkWidget *w, gpointer data)
 /**************************************************************************
   Adjust tax rates from main window
 **************************************************************************/
-void taxrates_callback( GtkWidget *w, GdkEventButton *event, gpointer data )
+void taxrates_callback(GtkWidget *w, GdkEventButton *event, gpointer data)
 {
   int tax_end,lux_end,sci_end;
   size_t i;
