@@ -20,13 +20,13 @@
 
 #include <gtk/gtk.h>
 
-#include "capability.h"
 #include "fcintl.h"
 #include "game.h"
 #include "genlist.h"
 #include "government.h"
 #include "map.h"
 #include "mem.h"
+#include "packets.h"
 #include "player.h"
 #include "shared.h"
 #include "support.h"
@@ -35,13 +35,12 @@
 #include "climisc.h"
 #include "clinet.h"
 #include "diptreaty.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 
 #include "diplodlg.h"
 
-extern GtkWidget *toplevel;
-
 #define MAX_NUM_CLAUSES 64
 
 struct Diplomacy_dialog {
@@ -65,6 +64,8 @@ struct Diplomacy_dialog {
   GtkWidget *dip_gold_entry0;
   GtkWidget *dip_gold_entry1;
   GtkWidget *dip_pact_menu;
+  GtkWidget *dip_vision_button0;
+  GtkWidget *dip_vision_button1;
 
   GtkWidget *dip_label;
   GtkWidget *dip_clauselabel;
@@ -78,31 +79,30 @@ struct Diplomacy_dialog {
   GtkWidget *dip_erase_clause_command;
 };
 
-char *dummy_clause_list_strings[]={ "\n", "\n", "\n", "\n", "\n", "\n", 0};
+static struct genlist diplomacy_dialogs;
+static int diplomacy_dialogs_list_has_been_initialised;
 
-struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
+static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 						 struct player *plr1);
 
-struct genlist diplomacy_dialogs;
-int diplomacy_dialogs_list_has_been_initialised;
-
-struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
+static struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
 					       struct player *plr1);
-void popup_diplomacy_dialog(struct player *plr0, struct player *plr1);
-void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_accept_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data);
-void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data);
-void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
-void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
+static void popup_diplomacy_dialog(struct player *plr0, struct player *plr1);
+static void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_accept_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data);
+static void diplomacy_dialog_vision_callback(GtkWidget *w, gpointer data);
+static void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
+static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 static gint diplomacy_dialog_mbutton_callback(GtkWidget *w, GdkEvent *event);
-
+static void diplo_dialog_returnkey(GtkWidget *w, gpointer data);
 
 /****************************************************************
 ...
@@ -181,7 +181,7 @@ void handle_diplomacy_remove_clause(struct packet_diplomacy_info *pa)
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-void popup_diplomacy_dialog(struct player *plr0, struct player *plr1)
+static void popup_diplomacy_dialog(struct player *plr0, struct player *plr1)
 {
   struct Diplomacy_dialog *pdialog;
   
@@ -262,8 +262,8 @@ static int fill_diplomacy_city_menu(GtkWidget *popupmenu,
 /****************************************************************
 ...
 *****************************************************************/
-struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
-						 struct player *plr1)
+static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
+							struct player *plr1)
 {
   char buf[512];
   static gchar *titles_[1]
@@ -417,21 +417,39 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 	pdialog->dip_gold_frame1, FALSE,FALSE,2);
 
 
+  pdialog->dip_vision_button0=gtk_button_new_with_label(_("Give shared vision"));
+  gtk_box_pack_start(GTK_BOX(pdialog->dip_vbox0), pdialog->dip_vision_button0,
+		     FALSE, FALSE, 2);
+  gtk_signal_connect(GTK_OBJECT(pdialog->dip_vision_button0), "clicked",
+		     GTK_SIGNAL_FUNC(diplomacy_dialog_vision_callback),
+		     (gpointer)pdialog);
+  if (plr0->gives_shared_vision & (1<<plr1->player_no))
+    gtk_widget_set_sensitive(pdialog->dip_vision_button0, FALSE);
+
+  pdialog->dip_vision_button1=gtk_button_new_with_label(_("Give shared vision"));
+  gtk_box_pack_start(GTK_BOX(pdialog->dip_vbox1), pdialog->dip_vision_button1,
+		     FALSE, FALSE, 2);
+  gtk_signal_connect(GTK_OBJECT(pdialog->dip_vision_button1), "clicked",
+		     GTK_SIGNAL_FUNC(diplomacy_dialog_vision_callback),
+		     (gpointer)pdialog);
+  if (plr1->gives_shared_vision & (1<<plr0->player_no))
+    gtk_widget_set_sensitive(pdialog->dip_vision_button1, FALSE);
+
   /* Start of pact button insertion */
 
   pdialog->dip_pact_menu=gtk_menu_new();
 
-  item=gtk_menu_item_new_with_label(_("Cease-fire"));
+  item=gtk_menu_item_new_with_label(Q_("?diplomatic_state:Cease-fire"));
   gtk_menu_append(GTK_MENU(pdialog->dip_pact_menu),item);
   gtk_signal_connect(GTK_OBJECT(item),"activate",
 	GTK_SIGNAL_FUNC(diplomacy_dialog_ceasefire_callback),(gpointer)pdialog);
 
-  item=gtk_menu_item_new_with_label(_("Peace"));
+  item=gtk_menu_item_new_with_label(Q_("?diplomatic_state:Peace"));
   gtk_menu_append(GTK_MENU(pdialog->dip_pact_menu),item);
   gtk_signal_connect(GTK_OBJECT(item),"activate",
 	GTK_SIGNAL_FUNC(diplomacy_dialog_peace_callback),(gpointer)pdialog);
 
-  item=gtk_menu_item_new_with_label(_("Alliance"));
+  item=gtk_menu_item_new_with_label(Q_("?diplomatic_state:Alliance"));
   gtk_menu_append(GTK_MENU(pdialog->dip_pact_menu),item);
   gtk_signal_connect(GTK_OBJECT(item),"activate",
 	GTK_SIGNAL_FUNC(diplomacy_dialog_alliance_callback),(gpointer)pdialog);
@@ -536,7 +554,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 /**************************************************************************
 ...
 **************************************************************************/
-void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
+static void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
          int               i;
          char              buf		[64];
@@ -569,7 +587,7 @@ void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
 {
   size_t choice;
   struct packet_diplomacy_info pa;
@@ -590,7 +608,7 @@ void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
 Callback for trading cities
 			      - Kris Bubendorfer
 *****************************************************************/
-void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data)
 {
   size_t choice;
   struct packet_diplomacy_info pa;
@@ -614,7 +632,7 @@ void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
   GList              *selection;
@@ -666,7 +684,7 @@ static gint diplomacy_dialog_mbutton_callback(GtkWidget *w, GdkEvent *event)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
   struct packet_diplomacy_info pa;
@@ -687,7 +705,7 @@ void diplomacy_dialog_map_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
   struct packet_diplomacy_info pa;
@@ -709,7 +727,7 @@ void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
 ...
 *****************************************************************/
 static void diplomacy_dialog_add_pact_clause(GtkWidget *w, gpointer data,
-					     int type)
+						    int type)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
   struct packet_diplomacy_info pa;
@@ -726,7 +744,7 @@ static void diplomacy_dialog_add_pact_clause(GtkWidget *w, gpointer data,
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data)
 {
   diplomacy_dialog_add_pact_clause(w, data, CLAUSE_CEASEFIRE);
 }
@@ -734,7 +752,7 @@ void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data)
 {
   diplomacy_dialog_add_pact_clause(w, data, CLAUSE_PEACE);
 }
@@ -742,17 +760,37 @@ void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data)
 {
   diplomacy_dialog_add_pact_clause(w, data, CLAUSE_ALLIANCE);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void diplomacy_dialog_vision_callback(GtkWidget *w, gpointer data)
+{
+  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
+  struct packet_diplomacy_info pa;
+  struct player *pgiver;
+
+  pgiver = (w == pdialog->dip_vision_button0) ? 
+    pdialog->treaty.plr0 : pdialog->treaty.plr1;
+
+  pa.plrno0 = pdialog->treaty.plr0->player_no;
+  pa.plrno1 = pdialog->treaty.plr1->player_no;
+  pa.clause_type = CLAUSE_VISION;
+  pa.plrno_from = pgiver->player_no;
+  pa.value = 0;
+  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
+			     &pa);
+}
 
 
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
   struct packet_diplomacy_info pa;
@@ -769,7 +807,7 @@ void diplomacy_dialog_close_callback(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void diplomacy_dialog_accept_callback(GtkWidget *w, gpointer data)
+static void diplomacy_dialog_accept_callback(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
   struct packet_diplomacy_info pa;
@@ -796,8 +834,8 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 /*****************************************************************
 ...
 *****************************************************************/
-struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
-					       struct player *plr1)
+static struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
+						      struct player *plr1)
 {
   struct genlist_iterator myiter;
 
@@ -840,7 +878,7 @@ static struct Diplomacy_dialog *find_diplomacy_by_input(GtkWidget *w)
 /*****************************************************************
 ...
 *****************************************************************/
-void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
+static void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
 {
   struct Diplomacy_dialog *pdialog;
   
@@ -870,3 +908,19 @@ void diplo_dialog_returnkey(GtkWidget *w, gpointer data)
       append_output_window(_("Game: Invalid amount of gold specified."));
   }
 }
+
+/*****************************************************************
+  Close all dialogs, for when client disconnects from game.
+*****************************************************************/
+void close_all_diplomacy_dialogs(void)
+{
+  struct Diplomacy_dialog *pdialog;
+  
+  if (!diplomacy_dialogs_list_has_been_initialised) {
+    return;
+  }
+  while (genlist_size(&diplomacy_dialogs)) {
+    pdialog = genlist_get(&diplomacy_dialogs, 0);
+    close_diplomacy_dialog(pdialog);
+  }
+}
