@@ -27,23 +27,22 @@
 #include "player.h"
 
 #include "dialogs.h"
+#include "gui_main.h"
 #include "mapview.h"
 
 #include "finddlg.h"
 
-extern GtkWidget *toplevel;
+static GtkWidget *find_dialog_shell;
+static GtkWidget *find_label;
+static GtkWidget *find_list;
+static GtkWidget *find_center_command;
+static GtkWidget *find_cancel_command;
 
-GtkWidget *find_dialog_shell;
-GtkWidget *find_label;
-GtkWidget *find_list;
-GtkWidget *find_center_command;
-GtkWidget *find_cancel_command;
+static void update_find_dialog(GtkWidget *find_list);
 
-void update_find_dialog(GtkWidget *find_list);
-
-void find_center_command_callback(GtkWidget *w, gpointer data);
-void find_cancel_command_callback(GtkWidget *w, gpointer data);
-void find_list_callback(GtkWidget *w, gint row, gint column);
+static void find_center_command_callback(GtkWidget *w, gpointer data);
+static void find_cancel_command_callback(GtkWidget *w, gpointer data);
+static void find_list_callback(GtkWidget *w, gint row, gint column);
 
 static int original_x, original_y;
 
@@ -57,7 +56,7 @@ void popup_find_dialog(void)
 
   get_center_tile_mapcanvas(&original_x, &original_y);
 
-  gtk_widget_set_sensitive(toplevel, FALSE);
+  gtk_widget_set_sensitive(top_vbox, FALSE);
   
   find_dialog_shell=gtk_widget_new(GTK_TYPE_DIALOG,
 				   "GtkWindow::title", _("Find City"),
@@ -130,7 +129,7 @@ void popup_find_dialog(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_find_dialog(GtkWidget *find_list)
+static void update_find_dialog(GtkWidget *find_list)
 {
   int i;
   
@@ -156,13 +155,13 @@ void update_find_dialog(GtkWidget *find_list)
 static void popdown_find_dialog(void)
 {
   gtk_widget_destroy(find_dialog_shell);
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void find_center_command_callback(GtkWidget *w, gpointer data)
+static void find_center_command_callback(GtkWidget *w, gpointer data)
 {
   struct city *pcity;
   GList *selection;
@@ -184,7 +183,7 @@ void find_center_command_callback(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-void find_cancel_command_callback(GtkWidget *w, gpointer data)
+static void find_cancel_command_callback(GtkWidget *w, gpointer data)
 {
   center_tile_mapcanvas(original_x, original_y);
   popdown_find_dialog();
@@ -193,7 +192,7 @@ void find_cancel_command_callback(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-void find_list_callback(GtkWidget *w, gint row, gint column)
+static void find_list_callback(GtkWidget *w, gint row, gint column)
 {
   struct city *pcity;
   gchar *string;
