@@ -34,32 +34,34 @@
 #include "civclient.h"
 #include "control.h"
 #include "dialogs.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 
 #include "gotodlg.h"
 
-extern GtkWidget *toplevel;
+static GtkWidget *goto_dialog_shell;
+static GtkWidget *goto_label;
+static GtkWidget *goto_list;
+static GtkWidget *goto_center_command;
+static GtkWidget *goto_airlift_command;
+static GtkWidget *goto_all_toggle;
+static GtkWidget *goto_cancel_command;
 
-GtkWidget *goto_dialog_shell;
-GtkWidget *goto_label;
-GtkWidget *goto_list;
-GtkWidget *goto_center_command;
-GtkWidget *goto_airlift_command;
-GtkWidget *goto_all_toggle;
-GtkWidget *goto_cancel_command;
+static void update_goto_dialog			(GtkWidget *goto_list);
 
-void update_goto_dialog			(GtkWidget *goto_list);
-
-void goto_cancel_command_callback	(GtkWidget *w, gpointer data);
-void goto_goto_command_callback		(GtkWidget *w, gpointer data);
-void goto_airlift_command_callback	(GtkWidget *w, gpointer data);
-void goto_all_toggle_callback		(GtkWidget *w, gpointer data);
-void goto_list_callback			(GtkWidget *w, gint row, gint column);
-void goto_list_ucallback		(GtkWidget *w, gint row, gint column);
+static void goto_cancel_command_callback	(GtkWidget *w, gpointer data);
+static void goto_goto_command_callback		(GtkWidget *w, gpointer data);
+static void goto_airlift_command_callback	(GtkWidget *w, gpointer data);
+static void goto_all_toggle_callback		(GtkWidget *w, gpointer data);
+static void goto_list_callback			(GtkWidget *w, gint row, gint column);
+static void goto_list_ucallback		(GtkWidget *w, gint row, gint column);
 
 static int original_x, original_y;
 
+/****************************************************************
+...
+*****************************************************************/
 void popup_goto_dialog_action(void)
 {
   popup_goto_dialog();
@@ -80,7 +82,7 @@ void popup_goto_dialog(void)
 
   get_center_tile_mapcanvas(&original_x, &original_y);
   
-  gtk_widget_set_sensitive(toplevel, FALSE);
+  gtk_widget_set_sensitive(top_vbox, FALSE);
   
   goto_dialog_shell=gtk_dialog_new();
   gtk_signal_connect(GTK_OBJECT(goto_dialog_shell), "delete_event",
@@ -176,7 +178,7 @@ static struct city *get_selected_city(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_goto_dialog(GtkWidget *goto_list)
+static void update_goto_dialog(GtkWidget *goto_list)
 {
   int    i, j;
   int    all_cities;
@@ -210,13 +212,13 @@ void update_goto_dialog(GtkWidget *goto_list)
 static void popdown_goto_dialog(void)
 {
   gtk_widget_destroy(goto_dialog_shell);
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void goto_list_callback(GtkWidget *w, gint row, gint column)
+static void goto_list_callback(GtkWidget *w, gint row, gint column)
 {
   struct city *pdestcity;
 
@@ -233,7 +235,7 @@ void goto_list_callback(GtkWidget *w, gint row, gint column)
 /**************************************************************************
 ...
 **************************************************************************/
-void goto_list_ucallback(GtkWidget *w, gint row, gint column)
+static void goto_list_ucallback(GtkWidget *w, gint row, gint column)
 {
   gtk_widget_set_sensitive(goto_airlift_command, FALSE);
 }
@@ -241,17 +243,13 @@ void goto_list_ucallback(GtkWidget *w, gint row, gint column)
 /**************************************************************************
 ...
 **************************************************************************/
-void goto_airlift_command_callback(GtkWidget *w, gpointer data)
+static void goto_airlift_command_callback(GtkWidget *w, gpointer data)
 {
   struct city *pdestcity=get_selected_city();
-  if(pdestcity) {
+  if (pdestcity) {
     struct unit *punit=get_unit_in_focus();
-    if(punit) {
-      struct unit req_unit;
-      req_unit=*punit;
-      req_unit.x=pdestcity->x;
-      req_unit.y=pdestcity->y;
-      send_unit_info(&req_unit);
+    if (punit) {
+      request_unit_airlift(punit, pdestcity);
     }
   }
   popdown_goto_dialog();
@@ -260,7 +258,7 @@ void goto_airlift_command_callback(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-void goto_all_toggle_callback(GtkWidget *w, gpointer data)
+static void goto_all_toggle_callback(GtkWidget *w, gpointer data)
 {
   update_goto_dialog(goto_list);
 }
@@ -268,18 +266,13 @@ void goto_all_toggle_callback(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-void goto_goto_command_callback(GtkWidget *w, gpointer data)
+static void goto_goto_command_callback(GtkWidget *w, gpointer data)
 {
   struct city *pdestcity=get_selected_city();
-  if(pdestcity) {
+  if (pdestcity) {
     struct unit *punit=get_unit_in_focus();
-    if(punit) {
-      struct packet_unit_request req;
-      req.unit_id=punit->id;
-      req.name[0]='\0';
-      req.x=pdestcity->x;
-      req.y=pdestcity->y;
-      send_packet_unit_request(&aconnection, &req, PACKET_UNIT_GOTO_TILE);
+    if (punit) {
+      send_goto_unit(punit, pdestcity->x, pdestcity->y);
     }
   }
   popdown_goto_dialog();
@@ -288,7 +281,7 @@ void goto_goto_command_callback(GtkWidget *w, gpointer data)
 /**************************************************************************
 ...
 **************************************************************************/
-void goto_cancel_command_callback(GtkWidget *w, gpointer data)
+static void goto_cancel_command_callback(GtkWidget *w, gpointer data)
 {
   center_tile_mapcanvas(original_x, original_y);
   popdown_goto_dialog();
