@@ -15,6 +15,9 @@
 #endif
 
 #include <assert.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -64,19 +67,6 @@
 /*void file_quit_cmd_callback( GtkWidget *widget, gpointer data )*/
 void game_rates( GtkWidget *widget, gpointer data );
 
-extern SPRITE *		intro_gfx_sprite;
-extern SPRITE *		radar_gfx_sprite;
-
-
-extern enum client_states	client_state;
-
-extern int			seconds_to_turndone;
-
-extern int			last_turn_gold_amount;
-extern int			did_advance_tech_this_turn;
-
-
-
 GtkWidget *	map_canvas;			/* GtkDrawingArea */
 GtkWidget *	overview_canvas;		/* GtkDrawingArea */
 
@@ -94,11 +84,13 @@ int		single_tile_pixmap_width, single_tile_pixmap_height;
 
 GtkWidget *	drawing;
 GtkWidget *	toplevel;
+GtkWidget *	top_vbox;
 
 GdkGC *		civ_gc;
 GdkGC *		fill_bg_gc;
+GdkGC *         thin_line_gc, *thick_line_gc;
 GdkGC *		fill_tile_gc;
-GdkPixmap *	gray50, *gray25;
+GdkPixmap *	gray50, *gray25, *black50;
 GdkGC *		mask_fg_gc;
 GdkGC *		mask_bg_gc;
 GdkPixmap *	mask_bitmap;
@@ -125,7 +117,7 @@ GtkWidget *	unit_below_pixmap		[MAX_NUM_UNITS_BELOW];
 GtkWidget *     unit_below_pixmap_button        [MAX_NUM_UNITS_BELOW];
 GtkWidget *	more_arrow_pixmap;
 
-gint		gtk_interval_id;
+static gint	gtk_interval_id;
 
 GtkWidget *	unit_info_label, *unit_info_frame;
 GtkWidget *	text_scrollbar;
@@ -181,17 +173,65 @@ static void parse_options(int argc, char **argv)
 **************************************************************************/
 static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
 {
-    if (GTK_WIDGET_HAS_FOCUS(inputline))
-	return FALSE;
+  if (GTK_WIDGET_HAS_FOCUS(inputline) || !GTK_WIDGET_IS_SENSITIVE(top_vbox))
+    return FALSE;
 
-    switch (event->keyval)
-    {
+  if (is_isometric) {
+    switch (event->keyval) {
+    case GDK_Up:
+    case GDK_8:
+    case GDK_KP_8:
+    case GDK_KP_Up:		key_move_north_west();		break;
+    case GDK_9:
+    case GDK_KP_9:
+    case GDK_Page_Up:
+    case GDK_KP_Page_Up:	key_move_north();		break;
+    case GDK_Right:
+    case GDK_6:
+    case GDK_KP_6:
+    case GDK_KP_Right:		key_move_north_east();		break;
+    case GDK_3:
+    case GDK_KP_3:
+    case GDK_Page_Down:
+    case GDK_KP_Page_Down:	key_move_east();		break;
+    case GDK_Down:
+    case GDK_2:
+    case GDK_KP_2:
+    case GDK_KP_Down:		key_move_south_east();		break;
+    case GDK_1:
+    case GDK_KP_1:
+    case GDK_End:
+    case GDK_KP_End:		key_move_south();		break;
+    case GDK_Left:
+    case GDK_4:
+    case GDK_KP_4:
+    case GDK_KP_Left:		key_move_south_west();		break;
+    case GDK_7:
+    case GDK_KP_7:
+    case GDK_Home:
+    case GDK_KP_Home:		key_move_west();		break;
+
+    case GDK_KP_Begin:
+    case GDK_5:
+    case GDK_KP_5:		focus_to_next_unit();		break;
+
+    case GDK_Return:
+    case GDK_KP_Enter:		key_end_turn();			break;
+
+    case GDK_Escape:		key_cancel_action();		break;
+
+    case GDK_t:			key_city_workers(widget,event);	break;
+    default:							return FALSE;
+    }
+  } else {
+    switch (event->keyval) {
     case GDK_Up:
     case GDK_8:
     case GDK_KP_8:
     case GDK_KP_Up:		key_move_north();		break;
     case GDK_9:
     case GDK_KP_9:
+    case GDK_Page_Up:
     case GDK_KP_Page_Up:	key_move_north_east();		break;
     case GDK_Right:
     case GDK_6:
@@ -199,6 +239,7 @@ static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
     case GDK_KP_Right:		key_move_east();		break;
     case GDK_3:
     case GDK_KP_3:
+    case GDK_Page_Down:
     case GDK_KP_Page_Down:	key_move_south_east();		break;
     case GDK_Down:
     case GDK_2:
@@ -206,6 +247,7 @@ static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
     case GDK_KP_Down:		key_move_south();		break;
     case GDK_1:
     case GDK_KP_1:
+    case GDK_End:
     case GDK_KP_End:		key_move_south_west();		break;
     case GDK_Left:
     case GDK_4:
@@ -213,6 +255,7 @@ static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
     case GDK_KP_Left:		key_move_west();		break;
     case GDK_7:
     case GDK_KP_7:
+    case GDK_Home:
     case GDK_KP_Home:		key_move_north_west();		break;
     case GDK_KP_Begin:
     case GDK_5:
@@ -226,9 +269,10 @@ static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
     case GDK_t:			key_city_workers(widget,event);	break;
     default:							return FALSE;
     }
+  }
 
-    gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
-    return TRUE;
+  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
+  return TRUE;
 }
 
 static gint tearoff_delete(GtkWidget *w, GdkEvent *ev, GtkWidget *box)
@@ -301,8 +345,10 @@ static void select_pixmap_callback(GtkWidget *w, GdkEvent *ev, int i)
     return;
   punit=find_unit_by_id(unit_ids[i]);
   if(punit) { /* should always be true at this point */
-     request_new_unit_activity(punit, ACTIVITY_IDLE);
-     set_unit_focus(punit);
+    if (punit->owner == game.player_idx) {  /* may be non-true if alliance */
+      request_new_unit_activity(punit, ACTIVITY_IDLE);
+      set_unit_focus(punit);
+    }
   }
 }
 
@@ -311,7 +357,6 @@ static void select_pixmap_callback(GtkWidget *w, GdkEvent *ev, int i)
 **************************************************************************/
 static void setup_widgets(void)
 {
-  GtkWidget	      *vbox;
   GtkWidget	      *hbox;
   GtkWidget	      *vbox1;
   GtkWidget           *avbox;
@@ -324,17 +369,17 @@ static void setup_widgets(void)
   GtkStyle	      *text_style;
   int		       i;
 
-  vbox = gtk_vbox_new( FALSE, 5 );
-  gtk_container_add( GTK_CONTAINER( toplevel ), vbox );
-  
-  setup_menus( toplevel, &menubar );
-  gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
-
   paned = gtk_vpaned_new();
-  gtk_box_pack_start( GTK_BOX( vbox ), paned, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(toplevel), paned);
 
-  hbox = gtk_hbox_new( FALSE, 0 );
-  gtk_paned_pack1(GTK_PANED(paned), hbox, TRUE, FALSE);
+  top_vbox = gtk_vbox_new(FALSE, 5);
+  gtk_paned_pack1(GTK_PANED(paned), top_vbox, TRUE, FALSE);
+  
+  setup_menus(toplevel, &menubar);
+  gtk_box_pack_start(GTK_BOX(top_vbox), menubar, FALSE, FALSE, 0);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(top_vbox), hbox, TRUE, TRUE, 0);
 
   vbox1 = gtk_vbox_new( FALSE, 3 );
   gtk_box_pack_start( GTK_BOX( hbox ), vbox1, FALSE, FALSE, 0 );
@@ -389,7 +434,7 @@ static void setup_widgets(void)
       box4 = gtk_hbox_new( FALSE, 0 );
       gtk_box_pack_start(GTK_BOX(avbox), box4, FALSE, FALSE, 0);
 
-      table = gtk_table_new(10, 3, FALSE);
+      table = gtk_table_new(3, 10, FALSE);
       gtk_table_set_row_spacing(GTK_TABLE(table), 0, 0);
       gtk_table_set_col_spacing(GTK_TABLE(table), 0, 0);
       gtk_box_pack_start(GTK_BOX(box4), table, TRUE, FALSE, 0);
@@ -462,13 +507,13 @@ static void setup_widgets(void)
     ubox = gtk_hbox_new(FALSE,0);
     gtk_box_pack_start(GTK_BOX(avbox), ubox, FALSE, FALSE, 0);
 
-    table = gtk_table_new( 2, 5, FALSE );
+    table = gtk_table_new( 2, num_units_below, FALSE );
     gtk_box_pack_start(GTK_BOX(ubox),table,FALSE,FALSE,5);
 
     gtk_table_set_row_spacings(GTK_TABLE(table), 2);
     gtk_table_set_col_spacings(GTK_TABLE(table), 2);
 
-    unit_pixmap=gtk_pixcomm_new(root_window, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    unit_pixmap=gtk_pixcomm_new(root_window, UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT);
     gtk_pixcomm_clear(GTK_PIXCOMM(unit_pixmap), TRUE);
     unit_pixmap_button=gtk_event_box_new();
     gtk_container_add(GTK_CONTAINER(unit_pixmap_button),unit_pixmap);
@@ -479,7 +524,8 @@ static void setup_widgets(void)
 
     for (i=0; i<num_units_below; i++)
     {
-      unit_below_pixmap[i]=gtk_pixcomm_new(root_window, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+      unit_below_pixmap[i]=gtk_pixcomm_new(root_window, UNIT_TILE_WIDTH,
+					   UNIT_TILE_HEIGHT);
       unit_below_pixmap_button[i]=gtk_event_box_new();
       gtk_container_add(GTK_CONTAINER(unit_below_pixmap_button[i]),
         unit_below_pixmap[i]);
@@ -490,7 +536,7 @@ static void setup_widgets(void)
       gtk_table_attach_defaults(GTK_TABLE(table), unit_below_pixmap_button[i],
     	  i, i+1, 1, 2);
       gtk_widget_set_usize(unit_below_pixmap[i],
-            NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+			   UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT);
       gtk_pixcomm_clear(GTK_PIXCOMM(unit_below_pixmap[i]), TRUE);
     }
 
@@ -511,8 +557,9 @@ static void setup_widgets(void)
   map_canvas		      = gtk_drawing_area_new();
 
   gtk_widget_set_events(map_canvas, GDK_EXPOSURE_MASK
-        			      |GDK_BUTTON_PRESS_MASK
-        			      |GDK_KEY_PRESS_MASK);
+        			   |GDK_BUTTON_PRESS_MASK
+        			   |GDK_KEY_PRESS_MASK
+                                   |GDK_POINTER_MOTION_MASK);
 
   gtk_drawing_area_size(GTK_DRAWING_AREA(map_canvas), 510, 300);
   gtk_container_add(GTK_CONTAINER(frame), map_canvas);
@@ -532,6 +579,8 @@ static void setup_widgets(void)
         	      GTK_SIGNAL_FUNC( butt_down_mapcanvas ), NULL );
   gtk_signal_connect( GTK_OBJECT( map_canvas ), "button_press_event",
         	      GTK_SIGNAL_FUNC( butt_down_wakeup ), NULL );
+  gtk_signal_connect(GTK_OBJECT(map_canvas), "motion_notify_event",
+		     GTK_SIGNAL_FUNC(move_mapcanvas), NULL);
 
   {   /* the message window */
       GtkWidget *text;
@@ -558,6 +607,7 @@ static void setup_widgets(void)
       text_style->text[GTK_STATE_INSENSITIVE]=
 					text_style->text[GTK_STATE_NORMAL];
       gtk_widget_set_style (text, text_style);
+      gtk_text_set_word_wrap(GTK_TEXT(text), 1);
 
       main_message_area = GTK_TEXT (text);
 
@@ -577,7 +627,7 @@ static void setup_widgets(void)
       GTK_SIGNAL_FUNC(keyboard_handler), NULL);
 /*  gtk_key_snooper_install(keyboard_handler, NULL);*/
 
-  gtk_widget_show_all(vbox);
+  gtk_widget_show_all(paned);
   gtk_widget_hide(more_arrow_pixmap);
 }
 
@@ -592,9 +642,19 @@ void ui_main(int argc, char **argv)
   parse_options(argc, argv);
 
   /* tell GTK+ library which locale */
-#ifdef ENABLE_NLS
-  gtk_set_locale();
+
+#ifdef HAVE_LOCALE_H
+  /* Freeciv assumes that all the world uses ISO-8859-1.  This
+     assumption is not quite true, but used to work more or less.
+     This is a workaround until we can fix the problem properly
+     in the next release.  */
+#ifdef HAVE_PUTENV
+  if (!strcmp(setlocale(LC_CTYPE, (const char *)NULL), "C"))
+    putenv("LC_CTYPE=en_US.ISO8859-1");
 #endif
+#endif
+
+  gtk_set_locale();
 
   /* GTK withdraw gtk options */
   /* Process GTK arguments */
@@ -643,14 +703,31 @@ void ui_main(int argc, char **argv)
   }
 
   fill_bg_gc = gdk_gc_new(root_window);
+  if (is_isometric) {
+    thin_line_gc = gdk_gc_new(root_window);
+    thick_line_gc = gdk_gc_new(root_window);
+    gdk_gc_set_line_attributes(thin_line_gc,
+			       1,
+			       GDK_LINE_SOLID,
+			       GDK_CAP_NOT_LAST,
+			       GDK_JOIN_MITER);
+    gdk_gc_set_line_attributes(thick_line_gc,
+			       2,
+			       GDK_LINE_SOLID,
+			       GDK_CAP_NOT_LAST,
+			       GDK_JOIN_MITER);
+  }
+
   fill_tile_gc = gdk_gc_new(root_window);
   gdk_gc_set_fill(fill_tile_gc, GDK_STIPPLED);
 
   {
     unsigned char d1[]={0x03,0x0c,0x03,0x0c};
     unsigned char d2[]={0x08,0x02,0x08,0x02};
+    unsigned char d3[]={0xAA,0x55,0xAA,0x55};
     gray50=gdk_bitmap_create_from_data(root_window,d1,4,4);
     gray25=gdk_bitmap_create_from_data(root_window,d2,4,4);
+    black50 = gdk_bitmap_create_from_data(root_window,d3,4,4);
   }
 
   {
@@ -707,8 +784,11 @@ void ui_main(int argc, char **argv)
   gdk_draw_rectangle(overview_canvas_store, fill_bg_gc, TRUE, 0, 0,
               overview_canvas_store_width, overview_canvas_store_height);
 
+  single_tile_pixmap_width = UNIT_TILE_WIDTH;
+  single_tile_pixmap_height = UNIT_TILE_HEIGHT;
   single_tile_pixmap	      = gdk_pixmap_new(root_window, 
-        			NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT, -1);
+					       single_tile_pixmap_width,
+					       single_tile_pixmap_height, -1);
 
   load_options();
 
@@ -894,7 +974,7 @@ void set_unit_icon(int idx, struct unit *punit)
   
   if (punit) {
     gtk_pixcomm_clear(GTK_PIXCOMM(w), FALSE);
-    put_unit_gpixmap(punit, GTK_PIXCOMM(w), 0, 0);
+    put_unit_gpixmap(punit, GTK_PIXCOMM(w));
   } else {
     gtk_pixcomm_clear(GTK_PIXCOMM(w), TRUE);
   }
