@@ -20,6 +20,7 @@
 #include <assert.h>
 
 #include <gtk/gtk.h>
+#include "gtkpixcomm.h"
 
 #include "city.h"
 #include "fcintl.h"
@@ -37,6 +38,7 @@
 #include "climisc.h"
 #include "colors.h"
 #include "graphics.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdata.h"
 #include "tilespec.h"
@@ -46,38 +48,33 @@
 #define TECH_TREE_DEPTH         20
 #define TECH_TREE_EXPANDED_DEPTH 2
 
-extern GtkWidget *toplevel;
-extern GdkGC *civ_gc;
-
-extern char long_buffer[64000];	      /* helpdata.c */
-
 /*
  * Globals.
  */
-GtkWidget *		help_dialog_shell;
-GtkWidget *		help_clist;
-GtkWidget *		help_clist_scrolled;
-GtkWidget *		help_frame;
-GtkWidget *		help_text;
-GtkWidget *		help_text_scrolled;
-GtkWidget *		help_vbox;
-GtkWidget *		unit_tile;
-GtkWidget *		help_box;
-GtkWidget *		help_itable;
-GtkWidget *		help_wtable;
-GtkWidget *		help_utable;
-GtkWidget *		help_ttable;
-GtkWidget *		help_tree;
-GtkWidget *		help_tree_scrolled;
-GtkWidget *             help_tree_expand;
-GtkWidget *             help_tree_expand_unknown;
-GtkWidget *             help_tree_collapse;
-GtkWidget *             help_tree_reset;
-GtkWidget *             help_tree_buttons_hbox;
-GtkWidget *		help_ilabel	[6];
-GtkWidget *		help_wlabel	[6];
-GtkWidget *		help_ulabel	[5][5];
-GtkWidget *		help_tlabel	[4][5];
+static GtkWidget * help_dialog_shell;
+static GtkWidget * help_clist;
+static GtkWidget * help_clist_scrolled;
+static GtkWidget * help_frame;
+static GtkWidget * help_text;
+static GtkWidget * help_text_scrolled;
+static GtkWidget * help_vbox;
+static GtkWidget * unit_tile;
+static GtkWidget * help_box;
+static GtkWidget * help_itable;
+static GtkWidget * help_wtable;
+static GtkWidget * help_utable;
+static GtkWidget * help_ttable;
+static GtkWidget * help_tree;
+static GtkWidget * help_tree_scrolled;
+static GtkWidget * help_tree_expand;
+static GtkWidget * help_tree_expand_unknown;
+static GtkWidget * help_tree_collapse;
+static GtkWidget * help_tree_reset;
+static GtkWidget * help_tree_buttons_hbox;
+static GtkWidget * help_ilabel[6];
+static GtkWidget * help_wlabel[6];
+static GtkWidget * help_ulabel[5][5];
+static GtkWidget * help_tlabel[4][5];
 
 typedef struct help_tree_node {
   int tech;
@@ -85,26 +82,26 @@ typedef struct help_tree_node {
 } help_tndata;
 
 char *help_ilabel_name[6] =
-{ N_("Cost:"), "", N_("Upkeep:"), "", N_("Requirement:"), "" };
+{ N_("Cost:"), NULL, N_("Upkeep:"), NULL, N_("Requirement:"), NULL };
 
 char *help_wlabel_name[6] =
-{ N_("Cost:"), "", N_("Requirement:"), "", N_("Obsolete by:"), "" };
+{ N_("Cost:"), NULL, N_("Requirement:"), NULL, N_("Obsolete by:"), NULL };
 
 char *help_ulabel_name[5][5] =
 {
-    { N_("Cost:"),		"", "",	N_("Attack:"),	"" },
-    { N_("Defense:"),	"", "",	N_("Move:")	,	"" },
-    { N_("FirePower:"),	"", "",	N_("Hitpoints:"),	"" },
-    { N_("Basic Upkeep:"),	"", "",	N_("Vision:"),	"" },
-    { N_("Requirement:"),	"", "",	N_("Obsolete by:"),	"" }
+    { N_("Cost:"),		NULL, NULL, N_("Attack:"),	NULL },
+    { N_("Defense:"),		NULL, NULL, N_("Move:")	,	NULL },
+    { N_("FirePower:"),		NULL, NULL, N_("Hitpoints:"),	NULL },
+    { N_("Basic Upkeep:"),	NULL, NULL, N_("Vision:"),	NULL },
+    { N_("Requirement:"),	NULL, NULL, N_("Obsolete by:"),	NULL }
 };
 
 char *help_tlabel_name[4][5] =
 {
-    { N_("Move/Defense:"),	"", "",	N_("Food/Res/Trade:"),	"" },
-    { N_("Sp1 F/R/T:"),	"", "",	N_("Sp2 F/R/T:"),		"" },
-    { N_("Road Rslt/Time:"),"", "",	N_("Irrig. Rslt/Time:"),	"" },
-    { N_("Mine Rslt/Time:"),"", "",	N_("Trans. Rslt/Time:"),	"" }
+    { N_("Move/Defense:"),	NULL, NULL, N_("Food/Res/Trade:"),	NULL },
+    { N_("Sp1 F/R/T:"),		NULL, NULL, N_("Sp2 F/R/T:"),		NULL },
+    { N_("Road Rslt/Time:"),	NULL, NULL, N_("Irrig. Rslt/Time:"),	NULL },
+    { N_("Mine Rslt/Time:"),	NULL, NULL, N_("Trans. Rslt/Time:"),	NULL }
 };
 
 
@@ -124,7 +121,7 @@ static void set_title_topic(char *topic)
   if(!strcmp(topic, "Freeciv")
      || !strcmp(topic, "About")
      || !strcmp(topic, _("About"))) {
-    gtk_frame_set_label(GTK_FRAME(help_frame), FREECIV_NAME_VERSION);
+    gtk_frame_set_label(GTK_FRAME(help_frame), freeciv_name_version());
   } else {
     gtk_frame_set_label(GTK_FRAME(help_frame), topic);
   }
@@ -480,15 +477,16 @@ static void create_help_dialog(void)
   gtk_container_add( GTK_CONTAINER( help_frame ), help_box );
 
 
-  unit_tile = gtk_pixmap_new( create_overlay_unit( -1 ), NULL );
-  gtk_pixmap_set_build_insensitive(GTK_PIXMAP(unit_tile), FALSE);
+  unit_tile = gtk_pixcomm_new(root_window, UNIT_TILE_WIDTH,
+  					   UNIT_TILE_HEIGHT);
   gtk_box_pack_start( GTK_BOX( help_box ), unit_tile, FALSE, FALSE, 0 );
 
-  help_itable = gtk_table_new(6, 1, FALSE);
+  help_itable = gtk_table_new(1, 6, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_itable, FALSE, FALSE, 0);
 
   for (i=0; i<6; i++) {
-    help_ilabel[i] = gtk_label_new(_(help_ilabel_name[i]));
+    help_ilabel[i] =
+	gtk_label_new(help_ilabel_name[i] ? _(help_ilabel_name[i]) : "");
     gtk_widget_set_name(help_ilabel[i], "help label");
 
     if (i==5) {
@@ -500,11 +498,12 @@ static void create_help_dialog(void)
 					help_ilabel[i], i, i+1, 0, 1);
   }
 
-  help_wtable = gtk_table_new(6, 1, FALSE);
+  help_wtable = gtk_table_new(1, 6, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_wtable, FALSE, FALSE, 0);
 
   for (i=0; i<6; i++) {
-    help_wlabel[i] = gtk_label_new(_(help_wlabel_name[i]));
+    help_wlabel[i] =
+	gtk_label_new(help_wlabel_name[i] ? _(help_wlabel_name[i]) : "");
     gtk_widget_set_name(help_wlabel[i], "help label");
 
     if (i==3 || i==5) {
@@ -517,13 +516,14 @@ static void create_help_dialog(void)
   }
 
 
-  help_utable = gtk_table_new(5, 4, FALSE);
+  help_utable = gtk_table_new(5, 5, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_utable, FALSE, FALSE, 0);
 
   for (i=0; i<5; i++)
     for (j=0; j<5; j++)
     {
-      help_ulabel[j][i] = gtk_label_new(_(help_ulabel_name[j][i]));
+      help_ulabel[j][i] =
+	  gtk_label_new(help_ulabel_name[j][i] ? _(help_ulabel_name[j][i]) : "");
       gtk_widget_set_name(help_ulabel[j][i], "help label");
 
       if (j==4 && (i==1 || i==4))
@@ -542,12 +542,13 @@ static void create_help_dialog(void)
     }
 
 
-  help_ttable = gtk_table_new(5, 4, FALSE);
+  help_ttable = gtk_table_new(5, 5, FALSE);
   gtk_box_pack_start(GTK_BOX(help_box), help_ttable, FALSE, FALSE, 0);
 
   for (i=0; i<5; i++) {
     for (j=0; j<4; j++) {
-      help_tlabel[j][i] = gtk_label_new(_(help_tlabel_name[j][i]));
+      help_tlabel[j][i] =
+	  gtk_label_new(help_tlabel_name[j][i] ? _(help_tlabel_name[j][i]) : "");
       gtk_widget_set_name(help_tlabel[j][i], "help label");
 
       gtk_table_attach_defaults(GTK_TABLE(help_ttable),
@@ -648,7 +649,7 @@ static void help_update_improvement(const struct help_item *pitem,
   
   create_help_page(HELP_IMPROVEMENT);
   
-  if (which<B_LAST) {
+  if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
     sprintf(buf, "%d", imp->build_cost);
     gtk_set_label(help_ilabel[1], buf);
@@ -688,7 +689,7 @@ static void help_update_wonder(const struct help_item *pitem,
   
   create_help_page(HELP_WONDER);
 
-  if (which<B_LAST) {
+  if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
     sprintf(buf, "%d", imp->build_cost);
     gtk_set_label(help_wlabel[1], buf);
@@ -764,7 +765,7 @@ static void help_update_unit_type(const struct help_item *pitem,
     gtk_widget_show(help_text);
     gtk_widget_show(help_text_scrolled);
 
-    gtk_pixmap_set(GTK_PIXMAP(unit_tile), create_overlay_unit(i), NULL);
+    create_overlay_unit(unit_tile, i);
     gtk_widget_show(unit_tile);
   }
   else {
@@ -824,7 +825,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
     gtk_container_add(GTK_CONTAINER(help_vbox), w);
 
-    for(j=0; j<B_LAST; ++j) {
+    for(j=0; j<game.num_impr_types; ++j) {
       if(i==improvement_types[j].tech_req) {
         hbox = gtk_hbox_new(FALSE, 0);
         gtk_container_add(GTK_CONTAINER(help_vbox), hbox);
@@ -883,7 +884,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
           w = help_slink_new(advances[advances[j].req[1]].name, HELP_TECH);
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-          w = gtk_label_new(").");
+          w = gtk_label_new(Q_("?techhelp:)."));
           gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	}
       }
@@ -898,7 +899,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
         w = help_slink_new(advances[advances[j].req[0]].name, HELP_TECH);
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
-        w = gtk_label_new(").");
+        w = gtk_label_new(Q_("?techhelp:)."));
         gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       }
     }
