@@ -29,16 +29,15 @@
 #include "shared.h"
 #include "support.h"
 
+#include "clinet.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 
 #include "inteldlg.h"
 
-extern GtkWidget *toplevel;
-extern struct connection aconnection;
-
 /******************************************************************/
-GtkWidget *intel_dialog_shell;
+static GtkWidget *intel_dialog_shell;
 /******************************************************************/
 
 
@@ -55,7 +54,7 @@ void popup_intel_dialog(struct player *p)
     intel_create_dialog(p);
     gtk_set_relative_position(toplevel, intel_dialog_shell, 25, 25);
     gtk_widget_show(intel_dialog_shell);
-    gtk_widget_set_sensitive(toplevel, FALSE);
+    gtk_widget_set_sensitive(top_vbox, FALSE);
   }
 }
 
@@ -203,7 +202,7 @@ void intel_create_dialog(struct player *p)
 **************************************************************************/
 void intel_close_command_callback(GtkWidget *w, gpointer data)
 { 
-  gtk_widget_set_sensitive(toplevel, TRUE);
+  gtk_widget_set_sensitive(top_vbox, TRUE);
   gtk_widget_destroy(intel_dialog_shell);
   intel_dialog_shell=NULL;
 }
