@@ -25,9 +25,14 @@ gint adjust_workers(GtkWidget *widget, GdkEventButton *ev);
 gint butt_down_mapcanvas(GtkWidget *w, GdkEventButton *ev);
 gint butt_down_wakeup(GtkWidget *w, GdkEventButton *ev);
 gint butt_down_overviewcanvas(GtkWidget *w, GdkEventButton *ev);
+gint move_mapcanvas(GtkWidget *widget, GdkEventButton *event);
+void update_line(int window_x, int window_y);
 
 void center_on_unit(void);
 void focus_to_next_unit(void);
 void popupinfo_popdown_callback(GtkWidget *w, gpointer data);
 
+/* Color to use to display the workers */
+extern int city_workers_color;
+
 #endif  /* FC__MAPCTRL_H */
