@@ -33,23 +33,20 @@
 #include "citydlg.h"
 #include "clinet.h"
 #include "colors.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "options.h"
 
 #include "messagewin.h"
 
-extern GtkWidget *toplevel;
-
-GtkWidget *meswin_dialog_shell;
-GtkWidget *meswin_list;
-GtkWidget *meswin_close_command;
-GtkWidget *meswin_int_command;
-GtkWidget *meswin_meet_command;
-GtkWidget *meswin_goto_command;
-GtkWidget *meswin_popcity_command;
-GtkStyle *meswin_visited_style;
-GtkStyle *meswin_not_visited_style;
+static GtkWidget *meswin_dialog_shell;
+static GtkWidget *meswin_list;
+static GtkWidget *meswin_close_command;
+static GtkWidget *meswin_goto_command;
+static GtkWidget *meswin_popcity_command;
+static GtkStyle *meswin_visited_style;
+static GtkStyle *meswin_not_visited_style;
 
 void create_meswin_dialog(void);
 void meswin_scroll_down(void);
@@ -127,12 +124,14 @@ void create_meswin_dialog(void)
   static gchar *titles_[1] = { N_("Messages") };
   static gchar **titles;
   GtkWidget *scrolled;
+  GtkAccelGroup *accel = gtk_accel_group_new();
 
   if (!titles) titles = intl_slist(1, titles_);
 
   meswin_dialog_shell = gtk_dialog_new();
   gtk_signal_connect( GTK_OBJECT(meswin_dialog_shell),"delete_event",
         GTK_SIGNAL_FUNC(meswin_close_callback),NULL );
+  gtk_accel_group_attach(accel, GTK_OBJECT(meswin_dialog_shell));
 
   gtk_window_set_title( GTK_WINDOW(meswin_dialog_shell), _("Messages") );
 
@@ -143,36 +142,43 @@ void create_meswin_dialog(void)
   gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolled ),
                           GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
   gtk_widget_set_usize(scrolled, 500, 250);
-  gtk_box_pack_start( GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->vbox), scrolled, TRUE, TRUE, 0 );
-					 
-  meswin_close_command = gtk_button_new_with_label(_("Close"));
-  gtk_box_pack_start( GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->action_area), meswin_close_command, TRUE, TRUE, 0 );
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->vbox),
+		     scrolled, TRUE, TRUE, 0 );
+
+  meswin_close_command = gtk_accelbutton_new(_("_Close"), accel);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->action_area),
+		     meswin_close_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( meswin_close_command, GTK_CAN_DEFAULT );
   gtk_widget_grab_default( meswin_close_command );
 
-  meswin_goto_command = gtk_button_new_with_label(_("Goto location"));
+  meswin_goto_command = gtk_accelbutton_new(_("_Goto location"), accel);
   gtk_widget_set_sensitive( meswin_goto_command, FALSE );
-  gtk_box_pack_start( GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->action_area), meswin_goto_command, TRUE, TRUE, 0 );
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->action_area),
+		     meswin_goto_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( meswin_goto_command, GTK_CAN_DEFAULT );
 
-  meswin_popcity_command = gtk_button_new_with_label(_("Popup City"));
+  meswin_popcity_command = gtk_accelbutton_new(_("_Popup City"), accel);
   gtk_widget_set_sensitive( meswin_popcity_command, FALSE );
-  gtk_box_pack_start( GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->action_area), meswin_popcity_command, TRUE, TRUE, 0 );
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(meswin_dialog_shell)->action_area),
+		     meswin_popcity_command, TRUE, TRUE, 0 );
   GTK_WIDGET_SET_FLAGS( meswin_popcity_command, GTK_CAN_DEFAULT );
 
   gtk_signal_connect(GTK_OBJECT(meswin_list), "select_row",
 		GTK_SIGNAL_FUNC(meswin_list_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(meswin_list), "unselect_row",
 		GTK_SIGNAL_FUNC(meswin_list_ucallback), NULL);
-	       
+
   gtk_signal_connect(GTK_OBJECT(meswin_close_command), "clicked",
 		GTK_SIGNAL_FUNC(meswin_close_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(meswin_goto_command), "clicked",
 		GTK_SIGNAL_FUNC(meswin_goto_callback), NULL);
   gtk_signal_connect(GTK_OBJECT(meswin_popcity_command), "clicked",
 		GTK_SIGNAL_FUNC(meswin_popcity_callback), NULL);
-  
-  meswin_visited_style = gtk_style_new ();
+
+  if (gtk_rc_get_style(meswin_list))
+    meswin_visited_style = gtk_style_copy(gtk_rc_get_style(meswin_list));
+  else
+    meswin_visited_style = gtk_style_new();
   meswin_visited_style->fg[GTK_STATE_NORMAL]=
 					*colors_standard[COLOR_STD_RACE8];
   meswin_visited_style->fg[GTK_STATE_SELECTED]=
@@ -180,7 +186,7 @@ void create_meswin_dialog(void)
   meswin_visited_style->bg[GTK_STATE_SELECTED]=
 					*colors_standard[COLOR_STD_RACE13];
 
-  meswin_not_visited_style = gtk_style_new ();
+  meswin_not_visited_style = gtk_style_copy(meswin_visited_style);
   meswin_not_visited_style->fg[GTK_STATE_NORMAL]=
 					*colors_standard[COLOR_STD_OCEAN];
   meswin_not_visited_style->fg[GTK_STATE_SELECTED]=
@@ -357,10 +363,7 @@ void meswin_list_callback (GtkWidget *w, gint row, gint column, GdkEvent *ev)
 
   x = xpos[row];
   y = ypos[row];
-  location_ok = (x || y);
-  /* Note: bad luck if tile (x=0,y=0) has a message, but can't
-   * easily fix this without making the protocol incompatibile --dwp
-   */
+  location_ok = (x != -1 && y != -1);
   city_ok = (location_ok && (pcity=map_get_city(x,y))
 	    && (pcity->owner == game.player_idx));
 
@@ -435,7 +438,7 @@ void meswin_goto_callback(GtkWidget *w, gpointer data)
 
   row = (gint)selection->data;
 
-  if(xpos[row] != 0 || ypos[row]!=0)
+  if(xpos[row]!=-1 && ypos[row]!=-1)
     center_tile_mapcanvas(xpos[row], ypos[row]);
   meswin_visited_item (row);
 }
@@ -457,7 +460,7 @@ void meswin_popcity_callback(GtkWidget *w, gpointer data)
 
   x = xpos[row];
   y = ypos[row];
-  if((x || y) && (pcity=map_get_city(x,y))
+  if((x!=-1 && y!=-1) && (pcity=map_get_city(x,y))
      && (pcity->owner == game.player_idx)) {
       if (center_when_popup_city) {
        center_tile_mapcanvas(x,y);
