@@ -24,6 +24,7 @@
 #include "city.h"
 #include "fcintl.h"
 #include "game.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "inputdlg.h"
@@ -31,10 +32,9 @@
 #include "packets.h"
 #include "worklist.h"
 
-#include "wldlg.h"
+#include "clinet.h"
 
-extern struct connection aconnection;
-extern GtkWidget *toplevel;
+#include "wldlg.h"
 
 #define WORKLIST_ADVANCED_TARGETS  1
 #define WORKLIST_CURRENT_TARGETS   0
@@ -83,11 +83,15 @@ struct worklist_dialog {
   int worklist_avail_num_improvements;
   int worklist_avail_num_targets;
 };
+#define WORKLIST_END (-1)
+
 
 static GtkWidget *worklist_report_shell = NULL;
 static struct worklist_report_dialog *report_dialog;
 
 
+static int uni_id(struct worklist *pwl, int wlinx);
+
 static void worklist_id_to_name(char buf[], int id, int is_unit, 
 				struct city *pcity);
 
@@ -103,9 +107,9 @@ static void populate_worklist_report_list(struct worklist_report_dialog *pdialog
 
 /* Callbacks for the worklist dialog */
 static void worklist_list_callback(GtkWidget *w, gint row, gint column,
-			    GdkEventButton *e, gpointer data);
+				   GdkEvent *ev, gpointer data);
 static void worklist_avail_callback(GtkWidget *w, gint row, gint column,
-			    GdkEventButton *e, gpointer data);
+				    GdkEvent *ev, gpointer data);
 static void insert_into_worklist(struct worklist_dialog *pdialog, 
 				 int before, int id);
 static void worklist_prepend_callback(GtkWidget *w, gpointer data);
@@ -309,6 +313,7 @@ GtkWidget* popup_worklist(struct worklist *pwl, struct city *pcity,
   pdialog->cancel_callback = cancel_cb;
   pdialog->toggle_show_advanced = NULL;
 
+  gtk_widget_set_sensitive(parent, FALSE);
   wshell = gtk_dialog_new();
   gtk_signal_connect(GTK_OBJECT(wshell), "delete_event",
 		     GTK_SIGNAL_FUNC(worklist_dialog_delete_callback),
@@ -504,6 +509,22 @@ void update_worklist_report_dialog(void)
 
 /****************************************************************
 
+*****************************************************************/
+int uni_id(struct worklist *pwl, int inx)
+{
+  if ((inx < 0) || (inx >= MAX_LEN_WORKLIST)) {
+    return WORKLIST_END;
+  } else if (pwl->wlefs[inx] == WEF_UNIT) {
+    return pwl->wlids[inx] + B_LAST;
+  } else if (pwl->wlefs[inx] == WEF_IMPR) {
+    return pwl->wlids[inx];
+  } else {
+    return WORKLIST_END;
+  }
+}
+
+/****************************************************************
+
 *****************************************************************/
 void worklist_id_to_name(char buf[], int id, int is_unit, 
 			 struct city *pcity)
@@ -743,11 +764,17 @@ void populate_worklist_report_list(struct worklist_report_dialog *pdialog)
   User selected one of the worklist items
 *****************************************************************/
 void worklist_list_callback(GtkWidget *w, gint row, gint column,
-			    GdkEventButton *e, gpointer data)
+			    GdkEvent *ev, gpointer data)
 {
   GList *selection;
   struct worklist_dialog *pdialog;
-  
+
+  if (ev && ev->type == GDK_2BUTTON_PRESS) {
+    /* Double-click to remove item from worklist */
+    worklist_delete_callback(w, data);
+    return;
+  }
+
   pdialog=(struct worklist_dialog *)data;
   
   selection = GTK_CLIST(pdialog->worklist)->selection;
@@ -768,13 +795,19 @@ void worklist_list_callback(GtkWidget *w, gint row, gint column,
   User selected one of the available items
 *****************************************************************/
 void worklist_avail_callback(GtkWidget *w, gint row, gint column,
-			    GdkEventButton *e, gpointer data)
+			    GdkEvent *ev, gpointer data)
 {
   struct worklist_dialog *pdialog;
   GList *selection;
-  
+
+  if (ev && ev->type == GDK_2BUTTON_PRESS) {
+    /* Double-click to insert item in worklist */
+    worklist_insert_callback(w, data);
+    return;
+  }
+
   pdialog=(struct worklist_dialog *)data;
-  
+
   selection = GTK_CLIST(pdialog->avail)->selection;
 
   if (! selection) {
@@ -913,8 +946,8 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
     int wl_idx = pdialog->worklist_avail_ids[idx];
     struct worklist *pwl = &pplr->worklists[wl_idx];
 
-    for (i = 0; i < MAX_LEN_WORKLIST && pwl->ids[i] != WORKLIST_END; i++) {
-      insert_into_worklist(pdialog, where, pwl->ids[i]);
+    for (i = 0; i < MAX_LEN_WORKLIST && uni_id(pwl, i) != WORKLIST_END; i++) {
+      insert_into_worklist(pdialog, where, uni_id(pwl, i));
       if (where < MAX_LEN_WORKLIST)
 	where++;
     }
@@ -1069,7 +1102,19 @@ void worklist_ok_callback(GtkWidget *w, gpointer data)
   init_worklist(&wl);
   
   for (i = 0; i < MAX_LEN_WORKLIST; i++) {
-    wl.ids[i] = pdialog->worklist_ids[i];
+    if (pdialog->worklist_ids[i] == WORKLIST_END) {
+      wl.wlefs[i] = WEF_END;
+      wl.wlids[i] = 0;
+    } else if (pdialog->worklist_ids[i] >= B_LAST) {
+      wl.wlefs[i] = WEF_UNIT;
+      wl.wlids[i] = pdialog->worklist_ids[i] - B_LAST;
+    } else if (pdialog->worklist_ids[i] >= 0) {
+      wl.wlefs[i] = WEF_IMPR;
+      wl.wlids[i] = pdialog->worklist_ids[i];
+    } else {
+      wl.wlefs[i] = WEF_END;
+      wl.wlids[i] = 0;
+    }
   }
   strcpy(wl.name, pdialog->pwl->name);
   wl.is_valid = pdialog->pwl->is_valid;
@@ -1145,8 +1190,13 @@ void worklist_avail_help_callback(GtkWidget *w, gpointer data)
 
   selection = GTK_CLIST(pdialog->avail)->selection;
   if(selection) {
-    id = pdialog->worklist_avail_ids[(int)selection->data];
-    is_unit = (int)selection->data >= pdialog->worklist_avail_num_improvements;
+    if ((int)selection->data >= pdialog->worklist_avail_num_targets) {
+      /* target is a global worklist id */
+      id = -1;
+    } else {
+      id = pdialog->worklist_avail_ids[(int)selection->data];
+      is_unit = (int)selection->data >= pdialog->worklist_avail_num_improvements;
+    }
   } else {
     id = -1;
   }
@@ -1208,9 +1258,9 @@ void worklist_populate_worklist(struct worklist_dialog *pdialog)
 
   /* Fill in the rest of the worklist list */
   for (i = 0; n < MAX_LEN_WORKLIST &&
-	 pdialog->pwl->ids[i] != WORKLIST_END; i++, n++) {
+	 uni_id(pdialog->pwl, i) != WORKLIST_END; i++, n++) {
     worklist_peek_ith(pdialog->pwl, &target, &is_unit, i);
-    id = worklist_peek_id_ith(pdialog->pwl, i);
+    id = uni_id(pdialog->pwl, i);
 
     worklist_id_to_name(pdialog->worklist_names[n],
 			target, is_unit, pdialog->pcity);
@@ -1248,7 +1298,7 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
     advanced_tech = FALSE;
  
   /*     + First, improvements and Wonders. */
-  for(i=0; i<B_LAST; i++) {
+  for(i=0; i<game.num_impr_types; i++) {
     /* Can the player (eventually) build this improvement? */
     can_build = can_player_build_improvement(pplr,i);
     can_eventually_build = could_player_eventually_build_improvement(pplr,i);
