@@ -58,12 +58,26 @@ popup_caravan_dialog(struct unit *punit,
 	/* PORTME */
 }
 
+int
+caravan_dialog_is_open(void)
+{
+	/* PORTME */
+	return 0;
+}
+
 void
 popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 {
 	/* PORTME */
 }
 
+int
+diplomat_dialog_is_open(void)
+{
+	/* PORTME */
+	return 0;
+}
+
 void
 popup_incite_dialog(struct city *pcity)
 {
@@ -76,6 +90,12 @@ popup_bribe_dialog(struct unit *punit)
 	/* PORTME */
 }
 
+void
+popup_sabotage_dialog(struct city *pcity)
+{
+	/* PORTME */
+}
+
 void
 popup_pillage_dialog(struct unit *punit, int may_pillage)
 {
@@ -83,7 +103,7 @@ popup_pillage_dialog(struct unit *punit, int may_pillage)
 }
 
 void
-process_caravan_arrival(struct unit *punit)
+popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
 {
 	/* PORTME */
 }
