@@ -12,6 +12,12 @@ ui_main(int argc, char *argv[])
 	fprintf( stderr, "Freeciv rules!\n" );
 }
 
+void
+sound_bell(void)
+{
+	/* PORTME */
+}
+
 void
 enable_turn_done_button(void)
 {
@@ -29,3 +35,15 @@ remove_net_input(void)
 {
 	/* PORTME */
 }
+
+void
+set_unit_icon(int idx, struct unit *punit)
+{
+	/* PORTME */
+}
+
+void
+set_unit_icons_more_arrow(int onoff)
+{
+	/* PORTME */
+}
