@@ -158,3 +158,13 @@ refresh_tile_mapcanvas(int x, int y, int write_to_screen)
 {
 	/* PORTME */
 }
+
+void draw_segment(int src_x, int src_y, int dir)
+{
+	/* PORTME */
+}
+
+void undraw_segment(int src_x, int src_y, int dir)
+{
+	/* PORTME */
+}
