@@ -41,6 +41,12 @@
 /*******************************************************************************
  Action Routines!!
 *******************************************************************************/
+static void xaw_mouse_moved(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  XButtonEvent *ev=&event->xbutton;
+  update_line(ev->x, ev->y);
+}
+
 static void xaw_btn_adjust_workers(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   mapctrl_btn_adjust_workers(event);
@@ -184,7 +190,7 @@ static void xaw_key_move_north_west(Widget w, XEvent *event, String *argv, Cardi
 static void xaw_key_open_city_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
-     is_menu_item_active(MENU_REPORT, MENU_REPORT_CITY))
+     is_menu_item_active(MENU_REPORT, MENU_REPORT_CITIES))
     popup_city_report_dialog(0);
 }
 
@@ -195,6 +201,13 @@ static void xaw_key_open_demographics(Widget w, XEvent *event, String *argv, Car
     send_report_request(REPORT_DEMOGRAPHIC);
 }
 
+static void xaw_key_open_economy_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
+     is_menu_item_active(MENU_REPORT, MENU_REPORT_ECONOMY))
+    popup_economy_report_dialog(0);
+}
+
 static void xaw_key_open_find_city(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
@@ -212,21 +225,14 @@ static void xaw_key_open_goto_airlift(Widget w, XEvent *event, String *argv, Car
 static void xaw_key_open_messages(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
-     is_menu_item_active(MENU_GAME, MENU_GAME_MESSAGES))
+     is_menu_item_active(MENU_REPORT, MENU_REPORT_MESSAGES))
     popup_meswin_dialog();
 }
 
-static void xaw_key_open_military_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
-{
-  if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
-     is_menu_item_active(MENU_REPORT, MENU_REPORT_MILITARY))
-    popup_activeunits_report_dialog(0);
-}
-
 static void xaw_key_open_players(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
-     is_menu_item_active(MENU_GAME, MENU_GAME_PLAYERS))
+     is_menu_item_active(MENU_REPORT, MENU_REPORT_PLAYERS))
     popup_players_dialog();
 }
 
@@ -258,13 +264,6 @@ static void xaw_key_open_spaceship(Widget w, XEvent *event, String *argv, Cardin
     popup_spaceship_dialog(game.player_ptr);
 }
 
-static void xaw_key_open_trade_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
-{
-  if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
-     is_menu_item_active(MENU_REPORT, MENU_REPORT_TRADE))
-    popup_trade_report_dialog(0);
-}
-
 static void xaw_key_open_top_five(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
@@ -272,6 +271,13 @@ static void xaw_key_open_top_five(Widget w, XEvent *event, String *argv, Cardina
     send_report_request(REPORT_TOP_5_CITIES);
 }
 
+static void xaw_key_open_units_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
+     is_menu_item_active(MENU_REPORT, MENU_REPORT_UNITS))
+    popup_activeunits_report_dialog(0);
+}
+
 static void xaw_key_open_wonders(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE &&
@@ -356,6 +362,12 @@ static void xaw_key_unit_connect(Widget w, XEvent *event, String *argv, Cardinal
     key_unit_connect();
 }
 
+static void xaw_key_unit_diplomat_spy_action(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  if(is_menu_item_active(MENU_ORDER, MENU_ORDER_DIPLOMAT_DLG))
+    key_unit_diplomat_actions();
+}
+
 static void xaw_key_unit_disband(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(is_menu_item_active(MENU_ORDER, MENU_ORDER_DISBAND))
@@ -462,6 +474,12 @@ static void xaw_key_unit_pollution(Widget w, XEvent *event, String *argv, Cardin
     key_unit_pollution();
 }
 
+static void xaw_key_unit_patrol(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  if(is_menu_item_active(MENU_ORDER, MENU_ORDER_PATROL))
+    key_unit_patrol();
+}
+
 static void xaw_key_unit_road(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(is_menu_item_active(MENU_ORDER, MENU_ORDER_ROAD))
@@ -528,6 +546,11 @@ static void xaw_msg_close_city_report(Widget w, XEvent *event, String *argv, Car
   cityrep_msg_close(w);
 }
 
+static void xaw_msg_close_economy_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  economyreport_msg_close(w);
+}
+
 static void xaw_msg_close_help(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   helpdlg_msg_close(w);
@@ -538,11 +561,6 @@ static void xaw_msg_close_messages(Widget w, XEvent *event, String *argv, Cardin
   meswin_msg_close(w);
 }
 
-static void xaw_msg_close_military_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
-{
-  activeunits_msg_close(w);
-}
-
 static void xaw_msg_close_players(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   plrdlg_msg_close(w);
@@ -558,9 +576,9 @@ static void xaw_msg_close_spaceship(Widget w, XEvent *event, String *argv, Cardi
   spaceshipdlg_msg_close(w);
 }
 
-static void xaw_msg_close_trade_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_msg_close_units_report(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  tradereport_msg_close(w);
+  activeunits_msg_close(w);
 }
 
 static void xaw_msg_quit_freeciv(Widget w, XEvent *event, String *argv, Cardinal *argc)
@@ -572,6 +590,7 @@ static void xaw_msg_quit_freeciv(Widget w, XEvent *event, String *argv, Cardinal
  Table of Actions!!
 *******************************************************************************/
 static XtActionsRec Actions[] = {
+  { "mouse-moved", xaw_mouse_moved },
   { "btn-adjust-workers", xaw_btn_adjust_workers },
   { "btn-select-citymap", xaw_btn_select_citymap },
   { "btn-select-mapcanvas", xaw_btn_select_mapcanvas },
@@ -602,17 +621,17 @@ static XtActionsRec Actions[] = {
   { "key-move-north-west", xaw_key_move_north_west },
   { "key-open-city-report", xaw_key_open_city_report },
   { "key-open-demographics", xaw_key_open_demographics },
+  { "key-open-economy-report", xaw_key_open_economy_report },
   { "key-open-find-city", xaw_key_open_find_city },
   { "key-open-goto-airlift", xaw_key_open_goto_airlift },
   { "key-open-messages", xaw_key_open_messages },
-  { "key-open-military-report", xaw_key_open_military_report },
   { "key-open-players", xaw_key_open_players },
   { "key-open-rates", xaw_key_open_rates },
   { "key-open-revolution", xaw_key_open_revolution },
   { "key-open-science-report", xaw_key_open_science_report },
   { "key-open-spaceship", xaw_key_open_spaceship },
-  { "key-open-trade-report", xaw_key_open_trade_report },
   { "key-open-top-five", xaw_key_open_top_five },
+  { "key-open-units-report", xaw_key_open_units_report },
   { "key-open-wonders", xaw_key_open_wonders },
   { "key-open-worklists", xaw_key_open_worklists },
   { "key-unit-airbase", xaw_key_unit_airbase },
@@ -624,6 +643,7 @@ static XtActionsRec Actions[] = {
   { "key-unit-build-city-or-wonder", xaw_key_unit_build_city_or_wonder },
   { "key-unit-build-wonder", xaw_key_unit_build_wonder },
   { "key-unit-connect", xaw_key_unit_connect },
+  { "key-unit-diplomat-spy-action", xaw_key_unit_diplomat_spy_action },
   { "key-unit-disband", xaw_key_unit_disband },
   { "key-unit-done", xaw_key_unit_done },
   { "key-unit-fallout", xaw_key_unit_fallout },
@@ -639,6 +659,7 @@ static XtActionsRec Actions[] = {
   { "key-unit-paradrop-or-pollution", xaw_key_unit_paradrop_or_pollution },
   { "key-unit-pillage", xaw_key_unit_pillage },
   { "key-unit-pollution", xaw_key_unit_pollution },
+  { "key-unit-patrol", xaw_key_unit_patrol },
   { "key-unit-road", xaw_key_unit_road },
   { "key-unit-road-or-traderoute", xaw_key_unit_road_or_traderoute },
   { "key-unit-traderoute", xaw_key_unit_traderoute },
@@ -649,13 +670,13 @@ static XtActionsRec Actions[] = {
   { "key-unit-wakeup-others", xaw_key_unit_wakeup_others },
   { "msg-close-city", xaw_msg_close_city },
   { "msg-close-city-report", xaw_msg_close_city_report },
+  { "msg-close-economy-report", xaw_msg_close_economy_report },
   { "msg-close-help", xaw_msg_close_help },
   { "msg-close-messages", xaw_msg_close_messages },
-  { "msg-close-military-report", xaw_msg_close_military_report },
   { "msg-close-players", xaw_msg_close_players },
   { "msg-close-science-report", xaw_msg_close_science_report },
   { "msg-close-spaceship", xaw_msg_close_spaceship },
-  { "msg-close-trade-report", xaw_msg_close_trade_report },
+  { "msg-close-units-report", xaw_msg_close_units_report },
   { "msg-quit-freeciv", xaw_msg_quit_freeciv }
 };
 
