@@ -30,12 +30,11 @@
 #include "support.h"
 
 #include "clinet.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 
 #include "chatline.h"
 
-extern Widget inputline_text, outputwindow_text;
-
 /**************************************************************************
 ...
 **************************************************************************/
