@@ -48,10 +48,12 @@
 #include "support.h"
 
 #include "cityrep.h"
+#include "clinet.h"
 #include "colors.h"
 #include "control.h" /* request_xxx and set_unit_focus */
 #include "dialogs.h"
 #include "graphics.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "inputdlg.h"
@@ -67,14 +69,6 @@
 
 #include "cityicon.ico"
 
-extern Display	*display;
-extern Widget toplevel, main_form, map_canvas;
-extern int display_depth;
-extern struct connection aconnection;
-extern int map_view_x0, map_view_y0;
-extern GC fill_bg_gc;
-
-
 #define MIN_NUM_CITIZENS	22
 #define MAX_NUM_CITIZENS	50
 #define DEFAULT_NUM_CITIZENS	38
@@ -182,10 +176,6 @@ static void cityopt_callback(Widget w, XtPointer client_data,
 			     XtPointer call_data);
 static void popdown_cityopt_dialog(void);
 
-
-extern Atom wm_delete_window;
-
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -218,7 +208,7 @@ static void get_contents_of_storage(struct city_dialog *pdialog,
     pcity=pdialog->pcity;
     granary=(city_got_effect(pcity, B_GRANARY) ? '*' : ' ');
     foodstock=pcity->food_stock;
-    foodbox=game.foodbox*(pcity->size+1);
+    foodbox=city_granary_size(pcity->size);
   }
 
   my_snprintf(retbuf, n, _("Granary: %c%3d/%-3d"),
@@ -360,6 +350,18 @@ struct city_dialog *get_city_dialog(struct city *pcity)
   return 0;
 }
 
+/****************************************************************
+...
+*****************************************************************/
+int city_dialog_is_open(struct city *pcity)
+{
+  if (get_city_dialog(pcity)) {
+    return 1;
+  } else {
+    return 0;
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -392,7 +394,7 @@ void refresh_city_dialog(struct city *pcity)
   }
   if(pcity->owner == game.player_idx)  {
     city_report_dialog_update_city(pcity);
-    trade_report_dialog_update();
+    economy_report_dialog_update();
   } else {
     if(pdialog)  {
       /* Set the buttons we do not want live while a Diplomat investigates */
@@ -1420,8 +1422,8 @@ void rename_callback(Widget w, XtPointer client_data, XtPointer call_data)
 		      "shellrenamecity", 
 		      _("What should we rename the city to?"),
 		      pdialog->pcity->name,
-		      (void*)rename_city_callback, (XtPointer)pdialog,
-		      (void*)rename_city_callback, (XtPointer)0);
+		      rename_city_callback, (XtPointer)pdialog,
+		      rename_city_callback, (XtPointer)0);
 }
 
 /****************************************************************
@@ -1566,19 +1568,23 @@ void city_dialog_update_map(struct city_dialog *pdialog)
 	 !(x==CITY_MAP_SIZE-1 && y==CITY_MAP_SIZE-1) &&
 	 tile_is_known(pcity->x+x-CITY_MAP_SIZE/2, 
 		       pcity->y+y-CITY_MAP_SIZE/2)) {
-	pixmap_put_tile(XtWindow(pdialog->map_canvas), x, y, 
+	pixmap_put_tile(XtWindow(pdialog->map_canvas),
 	                pcity->x+x-CITY_MAP_SIZE/2,
-			pcity->y+y-CITY_MAP_SIZE/2, 1);
+			pcity->y+y-CITY_MAP_SIZE/2,
+			x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT, 1);
 	if(pcity->city_map[x][y]==C_TILE_WORKER)
-	  put_city_tile_output(XtWindow(pdialog->map_canvas), x, y, 
-			       get_food_tile(x, y, pcity),
-			       get_shields_tile(x, y, pcity), 
-			       get_trade_tile(x, y, pcity) );
+	  put_city_tile_output(XtWindow(pdialog->map_canvas),
+			       x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT,
+			       city_get_food_tile(x, y, pcity),
+			       city_get_shields_tile(x, y, pcity), 
+			       city_get_trade_tile(x, y, pcity) );
 	else if(pcity->city_map[x][y]==C_TILE_UNAVAILABLE)
-	  pixmap_frame_tile_red(XtWindow(pdialog->map_canvas), x, y);
+	  pixmap_frame_tile_red(XtWindow(pdialog->map_canvas),
+				x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
       }
       else {
-	pixmap_put_black_tile(XtWindow(pdialog->map_canvas), x, y);
+	pixmap_put_black_tile(XtWindow(pdialog->map_canvas),
+			      x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
       }
     }
 }
@@ -1883,6 +1889,7 @@ void city_dialog_update_title(struct city_dialog *pdialog)
   if(strcmp(now, buf)) {
     XtVaSetValues(pdialog->cityname_label, XtNlabel, (XtArgVal)buf, NULL);
     xaw_horiz_center(pdialog->cityname_label);
+    XtVaSetValues(pdialog->shell, XtNtitle, (XtArgVal)pdialog->pcity->name, NULL);
   }
 }
 
@@ -1893,7 +1900,7 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
 {
   int i, n, flag;
 
-  for(i=0, n=0, flag=0; i<B_LAST; ++i)
+  for(i=0, n=0, flag=0; i<game.num_impr_types; ++i)
     if(pdialog->pcity->improvements[i]) {
       if(!pdialog->improvlist_names_ptrs[n] ||
 	 strcmp(pdialog->improvlist_names_ptrs[n], get_impr_name_ex(pdialog->pcity, i)))
@@ -2333,7 +2340,7 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
   
   XtSetSensitive(pdialog->shell, FALSE);
 
-  for(i=0, n=0; i<B_LAST; i++)
+  for(i=0, n=0; i<game.num_impr_types; i++)
     if(can_build_improvement(pdialog->pcity, i)) {
       if (i==B_CAPITAL) {
 	my_snprintf(pdialog->change_list_names[n],
@@ -2416,15 +2423,25 @@ void commit_city_worklist(struct worklist *pwl, void *data)
      the player doesn't even yet have the Map Making tech).  */
 
   for (k = 0; k < MAX_LEN_WORKLIST; k++) {
+    int same_as_current_build;
     if (! worklist_peek_ith(pwl, &id, &is_unit, k))
       break;
+    same_as_current_build = id == pdialog->pcity->currently_building
+      && is_unit == pdialog->pcity->is_building_unit;
+
+    /* Very special case: If we are currently building a wonder we
+       allow the construction to continue, even if we the wonder is
+       finished elsewhere, ie unbuildable. */
+    if (k == 0 && !is_unit && is_wonder(id) && same_as_current_build) {
+      worklist_remove(pwl, k);
+      break;
+    }
 
     /* If it can be built... */
     if (( is_unit && can_build_unit(pdialog->pcity, id)) ||
 	(!is_unit && can_build_improvement(pdialog->pcity, id))) {
       /* ...but we're not yet building it, then switch. */
-      if (id != pdialog->pcity->currently_building || 
-	  is_unit != pdialog->pcity->is_building_unit) {
+      if (!same_as_current_build) {
 
 	/* Change the current target */
 	packet.city_id=pdialog->pcity->id;
@@ -2448,7 +2465,8 @@ void commit_city_worklist(struct worklist *pwl, void *data)
   packet.worklist.name[0] = '\0';
   packet.worklist.is_valid = 1;
   for (i = 0; i < MAX_LEN_WORKLIST; i++) {
-    packet.worklist.ids[i] = pwl->ids[i];
+    packet.worklist.wlefs[i] = pwl->wlefs[i];
+    packet.worklist.wlids[i] = pwl->wlids[i];
   }
     
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_WORKLIST);
@@ -2507,7 +2525,7 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   if(ret->list_index!=XAW_LIST_NONE) {
     int i, n;
-    for(i=0, n=0; i<B_LAST; i++)
+    for(i=0, n=0; i<game.num_impr_types; i++)
       if(pdialog->pcity->improvements[i]) {
 	if(n==ret->list_index) {
 	  char buf[512];
