@@ -43,10 +43,12 @@
 #include "unit.h"
 
 #include "climisc.h"
+#include "clinet.h"
 
 #include "chatline.h"
 #include "citydlg.h"
 #include "cityrepdata.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "optiondlg.h"
@@ -55,19 +57,12 @@
 
 #include "cityrep.h"
 
-extern Widget toplevel, main_form;
-extern Display *display;
-extern Atom wm_delete_window;
-
-extern struct connection aconnection;
-extern int delay_report_update;
-
 /******************************************************************/
 static void popup_chgall_dialog (Widget parent);
 
 /******************************************************************/
-Widget config_shell;
-Widget config_toggle[NUM_CREPORT_COLS];
+static Widget config_shell;
+static Widget config_toggle[NUM_CREPORT_COLS];
 
 void create_city_report_config_dialog(void);
 void popup_city_report_config_dialog(void);
@@ -96,17 +91,17 @@ void city_list_callback(Widget w, XtPointer client_data,
 void city_config_callback(Widget w, XtPointer client_data, 
 			  XtPointer call_data);
 
-Widget city_form;
-Widget city_dialog_shell;
-Widget city_label;
-Widget city_viewport;
-Widget city_list, city_list_label;
-Widget city_center_command, city_popup_command, city_buy_command,
+static Widget city_form;
+static Widget city_dialog_shell;
+static Widget city_label;
+static Widget city_viewport;
+static Widget city_list, city_list_label;
+static Widget city_center_command, city_popup_command, city_buy_command,
        city_chgall_command, city_refresh_command, city_config_command;
-Widget city_change_command, city_popupmenu;
+static Widget city_change_command, city_popupmenu;
 
-int city_dialog_shell_is_modal;
-struct city **cities_in_list = NULL;
+static int city_dialog_shell_is_modal;
+static struct city **cities_in_list = NULL;
 
 static char *dummy_city_list[]={ 
   "    "
@@ -152,7 +147,8 @@ static char *get_city_table_header(void)
 	cat_snprintf(text, sizeof(text), "%*s", spec->space, " ");
 
       cat_snprintf(text, sizeof(text), "%*s", spec->width,
-		   (j ? _(spec->title2) : _(spec->title1)));
+		   (j ? (spec->title2 ? _(spec->title2) : "")
+		    : (spec->title1 ? _(spec->title1) : "")));
     }
     if (j==0) sz_strlcat(text, "\n");
   }
@@ -215,7 +211,7 @@ void create_city_report_dialog(int make_modal)
 				      city_dialog_shell,
 				      NULL);   
 
-  report_title=get_report_title(_("City Advisor"));
+  report_title=get_report_title(_("Cities"));
   city_label = XtVaCreateManagedWidget("reportcitylabel", 
 				       labelWidgetClass, 
 				       city_form,
@@ -329,7 +325,7 @@ void city_list_callback(Widget w, XtPointer client_data,
 				        city_change_command,
 				        NULL);
     flag = 0;
-    for(i=0; i<B_LAST; i++)
+    for(i=0; i<game.num_impr_types; i++)
       if(can_build_improvement(pcity, i)) {
 	Widget entry;
 	if (i==B_CAPITAL) {
@@ -572,7 +568,7 @@ void city_report_dialog_update(void)
       }
     }
        
-    report_title=get_report_title(_("City Advisor"));
+    report_title=get_report_title(_("Cities"));
     xaw_set_label(city_label, report_title);
     free(report_title);
 
@@ -782,8 +778,6 @@ void config_ok_command_callback(Widget w, XtPointer client_data,
  
 **************************************************************************/
 
-extern XtAppContext app_context;
-
 struct chgall_data
 {
   struct
