@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -31,7 +32,6 @@
 #include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/Toggle.h>     
 
-#include "capability.h"
 #include "fcintl.h"
 #include "game.h"
 #include "government.h"
@@ -48,21 +48,15 @@
 #include "clinet.h"
 #include "control.h" /* request_xxx and set_unit_focus */
 #include "graphics.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapctrl.h"
 #include "mapview.h"
+#include "options.h"
 #include "tilespec.h"
 
 #include "dialogs.h"
 
-extern Widget toplevel, main_form, map_canvas;
-
-extern struct connection aconnection;
-extern Display	*display;
-extern int display_depth;
-extern int ai_popup_windows;
-extern GC fill_bg_gc;
-
 /******************************************************************/
 static Widget races_dialog_shell=NULL;
 static Widget races_form, races_toggles_form, races_label;
@@ -77,40 +71,32 @@ static Widget races_action_form;
 static Widget races_ok_command, races_disconnect_command, races_quit_command;
 
 /******************************************************************/
-Widget spy_tech_shell;
-Widget spy_advances_list, spy_advances_list_label;
-Widget spy_steal_command;
+static Widget spy_tech_shell;
+static Widget spy_advances_list, spy_advances_list_label;
+static Widget spy_steal_command;
 
-int spy_tech_shell_is_modal;
-int advance_type[A_LAST+1];
-int steal_advance = 0;
+static int spy_tech_shell_is_modal;
+static int advance_type[A_LAST+1];
+static int steal_advance = 0;
 
 /******************************************************************/
-Widget spy_sabotage_shell;
-Widget spy_improvements_list, spy_improvements_list_label;
-Widget spy_sabotage_command;
+static Widget spy_sabotage_shell;
+static Widget spy_improvements_list, spy_improvements_list_label;
+static Widget spy_sabotage_command;
 
-int spy_sabotage_shell_is_modal;
-int improvement_type[B_LAST+1];
-int sabotage_improvement = 0;
+static int spy_sabotage_shell_is_modal;
+static int improvement_type[B_LAST+1];
+static int sabotage_improvement = 0;
 
 /******************************************************************/
-
-Widget about_dialog_shell;
-Widget about_form, about_label, about_command;
-
-/******************************************************************/
-Widget help_copying_dialog_shell, help_keys_dialog_shell;
-
-
-/******************************************************************/
-Widget unit_select_dialog_shell;
-Widget unit_select_form;
-Widget unit_select_commands[100];
-Widget unit_select_labels[100];
-Pixmap unit_select_pixmaps[100];
-int unit_select_ids[100];
-int unit_select_no;
+#define MAX_SELECT_UNITS 100
+static Widget unit_select_dialog_shell;
+static Widget unit_select_form;
+static Widget unit_select_commands[MAX_SELECT_UNITS];
+static Widget unit_select_labels[MAX_SELECT_UNITS];
+static Pixmap unit_select_pixmaps[MAX_SELECT_UNITS];
+static int unit_select_ids[MAX_SELECT_UNITS];
+static int unit_select_no;
 
 void about_button_callback(Widget w, XtPointer client_data, 
 			    XtPointer call_data);
@@ -325,7 +311,7 @@ void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
   Widget notify_headline, notify_label;
   Dimension width, width2, width_1, width_2;
   
-  if (x == 0 && y == 0) {
+  if (y < 0 || y >= map.ysize) {
     popup_notify_dialog("Message:", headline, lines);
     return;
   }
@@ -654,7 +640,7 @@ static void spy_steal_callback(Widget w, XtPointer client_data,
   spy_tech_shell = 0l;
   
   if(!steal_advance){
-    freelog(LOG_NORMAL, "Bug in spy steal tech code");
+    freelog(LOG_ERROR, "Bug in spy steal tech code");
     process_diplomat_arrival(NULL, 0);
     return;
   }
@@ -684,7 +670,7 @@ static void spy_sabotage_callback(Widget w, XtPointer client_data,
   spy_sabotage_shell = 0l;
   
   if(!sabotage_improvement){
-    freelog(LOG_NORMAL, "Bug in spy sabotage code");
+    freelog(LOG_ERROR, "Bug in spy sabotage code");
     process_diplomat_arrival(NULL, 0);
     return;
   }
@@ -849,7 +835,7 @@ static int create_improvements_list(struct player *pplayer,
   improvements_can_sabotage[j] = _("City Production");
   improvement_type[j++] = -1;
 
-  for(i=0; i<B_LAST; i++) 
+  for(i=0; i<game.num_impr_types; i++) 
     if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i)) {
       improvements_can_sabotage[j] = get_impr_name_ex(pcity, i);
       improvement_type[j++] = i;
@@ -1035,6 +1021,31 @@ void popup_incite_dialog(struct city *pcity)
 }
 
 
+/****************************************************************
+  Callback from diplomat/spy dialog for "keep moving".
+  (This should only occur when entering allied city.)
+*****************************************************************/
+static void diplomat_keep_moving_callback(Widget w, XtPointer client_data, 
+					  XtPointer call_data)
+{
+  struct unit *punit;
+  struct city *pcity;
+  
+  destroy_message_dialog(w);
+  diplomat_dialog_open=0;
+
+  if( (punit=find_unit_by_id(diplomat_id))
+      && (pcity=find_city_by_id(diplomat_target_id))
+      && !same_pos(punit->x, punit->y, pcity->x, pcity->y)) {
+    struct packet_diplomat_action req;
+    req.action_type = DIPLOMAT_MOVE;
+    req.diplomat_id = diplomat_id;
+    req.target_id = diplomat_target_id;
+    send_packet_diplomat_action(&aconnection, &req);
+  }
+  process_diplomat_arrival(NULL, 0);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1074,6 +1085,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 			       diplomat_sabotage_callback, 0, 1,
 			       diplomat_steal_callback, 0, 1,
 			       diplomat_incite_callback, 0, 1,
+			       diplomat_keep_moving_callback, 0, 1,
 			       diplomat_cancel_callback, 0, 0,
 			       NULL);
       
@@ -1087,6 +1099,8 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 	XtSetSensitive(XtNameToWidget(shl, "*button3"), FALSE);
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
 	XtSetSensitive(XtNameToWidget(shl, "*button4"), FALSE);
+      if(!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_x, dest_y))
+	XtSetSensitive(XtNameToWidget(shl, "*button5"), FALSE);
     }else{
       shl=popup_message_dialog(toplevel, "spydialog", buf,
 			       diplomat_embassy_callback, 0,  1,
@@ -1095,6 +1109,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 			       spy_request_sabotage_list, 0, 1,
 			       spy_steal_popup, 0, 1,
 			       diplomat_incite_callback, 0, 1,
+			       diplomat_keep_moving_callback, 0, 1,
 			       diplomat_cancel_callback, 0, 0,
 			       NULL);
       
@@ -1110,6 +1125,8 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 	XtSetSensitive(XtNameToWidget(shl, "*button4"), FALSE);
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
 	XtSetSensitive(XtNameToWidget(shl, "*button5"), FALSE);
+      if(!diplomat_can_do_action(punit, DIPLOMAT_MOVE, dest_x, dest_y))
+	XtSetSensitive(XtNameToWidget(shl, "*button6"), FALSE);
     }
 
     diplomat_dialog_open=1;
@@ -1559,11 +1576,12 @@ static int number_of_columns(int n)
 {
 #if 0
   /* This would require libm, which isn't worth it for this one little
-   * function.  Since the number of units is limited to 100 already, the ifs
+   * function.  Since MAX_SELECT_UNITS is 100 already, the ifs
    * work fine.  */
   double sqrt(); double ceil();
   return ceil(sqrt((double)n/5.0));
 #else
+  assert(MAX_SELECT_UNITS == 100);
   if(n<=5) return 1;
   else if(n<=20) return 2;
   else if(n<=45) return 3;
@@ -1606,8 +1624,8 @@ void popup_unit_select_dialog(struct tile *ptile)
     XSetForeground(display, fill_bg_gc, bg);
   }
 
-  n=unit_list_size(&ptile->units);
-  r=number_of_rows(n);
+  n = MIN(MAX_SELECT_UNITS, unit_list_size(&ptile->units));
+  r = number_of_rows(n);
 
   for(i=0; i<n; i++) {
     struct unit *punit=unit_list_get(&ptile->units, i);
