@@ -33,13 +33,13 @@
 #include <X11/Xaw/AsciiText.h>  
 #include <X11/Xaw/Viewport.h>
 
-#include "capability.h"
 #include "fcintl.h"
 #include "game.h"
 #include "genlist.h"
 #include "government.h"
 #include "map.h"
 #include "mem.h"
+#include "packets.h"
 #include "player.h"
 #include "shared.h"
 #include "support.h"
@@ -48,13 +48,12 @@
 #include "climisc.h"
 #include "clinet.h"
 #include "diptreaty.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 
 #include "diplodlg.h"
 
-extern Widget toplevel;
-
 #define MAX_NUM_CLAUSES 64
 
 struct Diplomacy_dialog {
@@ -78,6 +77,8 @@ struct Diplomacy_dialog {
   Widget dip_gold_label1;
   Widget dip_gold_input0;
   Widget dip_gold_input1;
+  Widget dip_vision_button0;
+  Widget dip_vision_button1;
   Widget dip_pact_menubutton;
   
   Widget dip_label;
@@ -97,14 +98,15 @@ struct Diplomacy_dialog {
   char *clauselist_strings_ptrs[MAX_NUM_CLAUSES+1];
 };
 
-char *dummy_clause_list_strings[]={ "\n", "\n", "\n", "\n", "\n", "\n", 0};
+static char *dummy_clause_list_strings[]
+    = { "\n", "\n", "\n", "\n", "\n", "\n", 0 };
+
+static struct genlist diplomacy_dialogs;
+static int diplomacy_dialogs_list_has_been_initialised;
 
 struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 						 struct player *plr1);
 
-struct genlist diplomacy_dialogs;
-int diplomacy_dialogs_list_has_been_initialised;
-
 struct Diplomacy_dialog *find_diplomacy_dialog(struct player *plr0, 
 					       struct player *plr1);
 void popup_diplomacy_dialog(struct player *plr0, struct player *plr1);
@@ -128,6 +130,8 @@ void diplomacy_dialog_peace_callback(Widget w, XtPointer client_data,
 					XtPointer call_data);
 void diplomacy_dialog_alliance_callback(Widget w, XtPointer client_data,
 					XtPointer call_data);
+void diplomacy_dialog_vision_callback(Widget w, XtPointer client_data,
+				      XtPointer call_data);
 void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 
@@ -473,6 +477,29 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 						   XtNlabel, buf,
 						   NULL);
 
+
+  pdialog->dip_vision_button0 =
+    I_L(XtVaCreateManagedWidget("dipvisionbutton0",
+				commandWidgetClass, 
+				pdialog->dip_form0,
+				NULL));
+  if (plr0->gives_shared_vision & (1<<plr1->player_no))
+    XtSetSensitive(pdialog->dip_vision_button0, FALSE);
+
+  pdialog->dip_vision_button1 =
+    I_L(XtVaCreateManagedWidget("dipvisionbutton1",
+				commandWidgetClass, 
+				pdialog->dip_form1,
+				NULL));
+  if (plr1->gives_shared_vision & (1<<plr0->player_no))
+    XtSetSensitive(pdialog->dip_vision_button1, FALSE);
+
+  XtAddCallback(pdialog->dip_vision_button0, XtNcallback, 
+		diplomacy_dialog_vision_callback, (XtPointer)pdialog);
+  XtAddCallback(pdialog->dip_vision_button1, XtNcallback, 
+		diplomacy_dialog_vision_callback, (XtPointer)pdialog);
+
+
   pdialog->dip_pact_menubutton=
     I_L(XtVaCreateManagedWidget("dippactmenubutton",
 				menuButtonWidgetClass,
@@ -482,16 +509,16 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 				 simpleMenuWidgetClass, 
 				 pdialog->dip_pact_menubutton, 
 				 NULL);
-  entry=XtVaCreateManagedWidget(_("Cease-fire"), smeBSBObjectClass, popupmenu,
-				NULL);
+  entry=XtVaCreateManagedWidget(Q_("?diplomatic_state:Cease-fire"),
+				smeBSBObjectClass, popupmenu, NULL);
   XtAddCallback(entry, XtNcallback, diplomacy_dialog_ceasefire_callback,
 		(XtPointer)pdialog);
-  entry=XtVaCreateManagedWidget(_("Peace"), smeBSBObjectClass, popupmenu,
-				NULL);
+  entry=XtVaCreateManagedWidget(Q_("?diplomatic_state:Peace"),
+				smeBSBObjectClass, popupmenu, NULL);
   XtAddCallback(entry, XtNcallback, diplomacy_dialog_peace_callback,
 		(XtPointer)pdialog);
-  entry=XtVaCreateManagedWidget(_("Alliance"), smeBSBObjectClass, popupmenu,
-				NULL);
+  entry=XtVaCreateManagedWidget(Q_("?diplomatic_state:Alliance"),
+				smeBSBObjectClass, popupmenu, NULL);
   XtAddCallback(entry, XtNcallback, diplomacy_dialog_alliance_callback,
 		(XtPointer)pdialog);
   
@@ -774,6 +801,28 @@ void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data,
 			     &pa);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+void diplomacy_dialog_vision_callback(Widget w, XtPointer client_data, 
+				   XtPointer call_data)
+{
+  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)client_data;
+  struct packet_diplomacy_info pa;
+  struct player *pgiver;
+  
+  pgiver = w==pdialog->dip_vision_button0 ? 
+    pdialog->treaty.plr0 : pdialog->treaty.plr1;
+  
+  pa.plrno0=pdialog->treaty.plr0->player_no;
+  pa.plrno1=pdialog->treaty.plr1->player_no;
+  pa.clause_type=CLAUSE_VISION;
+  pa.plrno_from=pgiver->player_no;
+  pa.value=0;
+  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
+			     &pa);
+}
+
 /****************************************************************
 Generic add-a-clause function for adding pact types
 *****************************************************************/
@@ -953,3 +1002,19 @@ void diplodlg_key_gold(Widget w)
       append_output_window(_("Game: Invalid amount of gold specified."));
   }
 }
+
+/*****************************************************************
+  Close all dialogs, for when client disconnects from game.
+*****************************************************************/
+void close_all_diplomacy_dialogs(void)
+{
+  struct Diplomacy_dialog *pdialog;
+  
+  if (!diplomacy_dialogs_list_has_been_initialised) {
+    return;
+  }
+  while (genlist_size(&diplomacy_dialogs)) {
+    pdialog = genlist_get(&diplomacy_dialogs, 0);
+    close_diplomacy_dialog(pdialog);
+  }
+}
