@@ -40,23 +40,22 @@
 #include "civclient.h"
 #include "clinet.h"
 #include "control.h" /* get_unit_in_focus */
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapctrl.h"
 #include "mapview.h"
 
 #include "gotodlg.h"
 
-extern Widget toplevel, main_form;
-
-Widget goto_dialog_shell;
-Widget goto_form;
-Widget goto_label;
-Widget goto_viewport;
-Widget goto_list;
-Widget goto_center_command;
-Widget goto_airlift_command;
-Widget goto_all_toggle;
-Widget goto_cancel_command;
+static Widget goto_dialog_shell;
+static Widget goto_form;
+static Widget goto_label;
+static Widget goto_viewport;
+static Widget goto_list;
+static Widget goto_center_command;
+static Widget goto_airlift_command;
+static Widget goto_all_toggle;
+static Widget goto_cancel_command;
 
 void update_goto_dialog(Widget goto_list);
 
@@ -231,7 +230,7 @@ void update_goto_dialog(Widget goto_list)
   }
 
   if(ncities_total) {
-    qsort(city_name_ptrs,ncities_total,sizeof(char *),string_ptr_compare);
+    qsort(city_name_ptrs, ncities_total, sizeof(char *), compare_strings_ptrs);
     XawListChange(goto_list, city_name_ptrs, ncities_total, 0, True);
   }
 }
@@ -279,11 +278,7 @@ void goto_airlift_command_callback(Widget w, XtPointer client_data,
   if(pdestcity) {
     struct unit *punit=get_unit_in_focus();
     if(punit) {
-      struct unit req_unit;
-      req_unit=*punit;
-      req_unit.x=pdestcity->x;
-      req_unit.y=pdestcity->y;
-      send_unit_info(&req_unit);
+      request_unit_airlift(punit, pdestcity);
     }
   }
   popdown_goto_dialog();
@@ -304,16 +299,11 @@ void goto_all_toggle_callback(Widget w, XtPointer client_data,
 void goto_goto_command_callback(Widget w, XtPointer client_data, 
 				  XtPointer call_data)
 {
-  struct city *pdestcity=get_selected_city();
-  if(pdestcity) {
-    struct unit *punit=get_unit_in_focus();
-    if(punit) {
-      struct packet_unit_request req;
-      req.unit_id=punit->id;
-      req.name[0]='\0';
-      req.x=pdestcity->x;
-      req.y=pdestcity->y;
-      send_packet_unit_request(&aconnection, &req, PACKET_UNIT_GOTO_TILE);
+  struct city *pdestcity = get_selected_city();
+  if (pdestcity) {
+    struct unit *punit = get_unit_in_focus();
+    if (punit) {
+      send_goto_unit(punit, pdestcity->x, pdestcity->y);
     }
   }
   popdown_goto_dialog();
