@@ -39,7 +39,9 @@
 
 #include "climisc.h"
 #include "colors.h"
+#include "gui_main.h"
 #include "mapview_g.h"
+#include "options.h"
 #include "tilespec.h"
 
 #include "graphics.h"
@@ -50,15 +52,8 @@
 #include "drop_cursor_mask.xbm"
 #include "nuke_cursor.xbm"
 #include "nuke_cursor_mask.xbm"
-
-extern int display_depth;
-extern Display *display;
-extern GC fill_bg_gc;
-extern GC civ_gc, font_gc;
-extern Colormap cmap;
-extern Window root_window;
-extern XFontStruct *main_font_struct;
-extern int solid_color_behind_units;
+#include "patrol_cursor.xbm"
+#include "patrol_cursor_mask.xbm"
 
 struct Sprite *intro_gfx_sprite;
 struct Sprite *radar_gfx_sprite;
@@ -66,10 +61,27 @@ struct Sprite *radar_gfx_sprite;
 Cursor goto_cursor;
 Cursor drop_cursor;
 Cursor nuke_cursor;
+Cursor patrol_cursor;
 
 static struct Sprite *ctor_sprite_mask(Pixmap mypixmap, Pixmap mask, 
  				       int width, int height);
 
+/***************************************************************************
+...
+***************************************************************************/
+int isometric_view_supported(void)
+{
+  return 0;
+}
+
+/***************************************************************************
+...
+***************************************************************************/
+int overhead_view_supported(void)
+{
+  return 1;
+}
+
 /***************************************************************************
 ...
 ***************************************************************************/
@@ -117,15 +129,15 @@ void load_intro_gfx(void)
 
   y=radar_gfx_sprite->height-(lin+((int)(1.5*main_font_struct->descent)));
 
-  w=XTextWidth(main_font_struct, WORD_VERSION, strlen(WORD_VERSION));
+  w=XTextWidth(main_font_struct, word_version(), strlen(word_version()));
   XSetForeground(display, font_gc, colors_standard[COLOR_STD_BLACK]);
   XDrawString(display, radar_gfx_sprite->pixmap, font_gc, 
 	      (tot/2-w/2)+1, y+1, 
-	      WORD_VERSION, strlen(WORD_VERSION));
+	      word_version(), strlen(word_version()));
   XSetForeground(display, font_gc, colors_standard[COLOR_STD_WHITE]);
   XDrawString(display, radar_gfx_sprite->pixmap, font_gc, 
 	      tot/2-w/2, y, 
-	      WORD_VERSION, strlen(WORD_VERSION));
+	      word_version(), strlen(word_version()));
 
   y+=lin;
 
@@ -219,6 +231,17 @@ void load_cursors(void)
 				    nuke_cursor_x_hot, nuke_cursor_y_hot);
   XFreePixmap(display, pixmap);
   XFreePixmap(display, mask);
+
+  /* patrol */
+  pixmap = XCreateBitmapFromData(display, root_window, patrol_cursor_bits,
+				 patrol_cursor_width, patrol_cursor_height);
+  mask   = XCreateBitmapFromData(display, root_window, patrol_cursor_mask_bits,
+				 patrol_cursor_mask_width, patrol_cursor_mask_height);
+  patrol_cursor = XCreatePixmapCursor(display, pixmap, mask,
+				      &white, &black,
+				      patrol_cursor_x_hot, patrol_cursor_y_hot);
+  XFreePixmap(display, pixmap);
+  XFreePixmap(display, mask);
 }
 
 #ifdef UNUSED
