@@ -64,18 +64,13 @@
 
 #include "freeciv.ico"
 
-AppResources appResources;
-
-extern String fallback_resources[];
-
+static AppResources appResources;
 
 /* ids of the units icons in information display: (or 0) */
 static int unit_ids[MAX_NUM_UNITS_BELOW];  
 
-
 static void setup_widgets(void);
 
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -137,7 +132,7 @@ static XrmOptionDescRec cmd_options[] = {
 static void end_turn_callback(Widget w, XtPointer client_data,
 			      XtPointer call_data);
 
-static void timer_callback(caddr_t client_data, XtIntervalId *id);
+static void timer_callback(XtPointer client_data, XtIntervalId *id);
 
 /**************************************************************************/
 
@@ -156,8 +151,6 @@ int display_depth;
 int screen_number;
 enum Display_color_type display_color_type;
 XtAppContext app_context;
-extern Colormap cmap;
-extern Widget info_command;
 
 /* this GC will be the default one all thru freeciv */
 GC civ_gc; 
@@ -174,7 +167,6 @@ XFontStruct *main_font_struct;
 GC prod_font_gc;
 XFontStruct *prod_font_struct;
 
-
 Widget toplevel, main_form, menu_form, below_menu_form, left_column_form;
 Widget bottom_form;
 Widget map_form;
@@ -206,8 +198,6 @@ int overview_canvas_store_width, overview_canvas_store_height;
 Pixmap single_tile_pixmap;
 int single_tile_pixmap_width, single_tile_pixmap_height;
 
-extern int seconds_to_turndone;
-
 XtInputId x_input_id;
 XtIntervalId x_interval_id;
 Atom wm_delete_window;
@@ -448,7 +438,6 @@ void ui_main(int argc, char *argv[])
 				   NORMAL_TILE_HEIGHT,
 				   display_depth);
 
-
   for(i=0; i<num_units_below; i++)
     unit_below_pixmap[i]=XCreatePixmap(display, XtWindow(overview_canvas), 
 				       NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT, 
@@ -485,8 +474,10 @@ static void unit_icon_callback(Widget w, XtPointer client_data,
     return;
   punit=find_unit_by_id(unit_ids[i]);
   if(punit) { /* should always be true at this point */
-     request_new_unit_activity(punit, ACTIVITY_IDLE);
-     set_unit_focus(punit);
+    if (punit->owner == game.player_idx) {  /* may be non-true if alliance */
+      request_new_unit_activity(punit, ACTIVITY_IDLE);
+      set_unit_focus(punit);
+    }
   }
 }
 
@@ -793,7 +784,7 @@ void end_turn_callback(Widget w, XtPointer client_data, XtPointer call_data)
 /**************************************************************************
 ...
 **************************************************************************/
-void timer_callback(caddr_t client_data, XtIntervalId *id)
+void timer_callback(XtPointer client_data, XtIntervalId *id)
 {
   static int flip;
   
