@@ -44,55 +44,51 @@
 #include "climisc.h"
 #include "dialogs.h"
 #include "graphics.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdata.h"
 #include "tilespec.h"
 
 #include "helpdlg.h"
 
-extern Widget toplevel, main_form;
-extern Display *display;
-extern Atom wm_delete_window;
+static Widget help_dialog_shell;
+static Widget help_form;
+static Widget help_viewport;
+static Widget help_list;
+static Widget help_text;
+static Widget help_close_command;
 
-Widget help_dialog_shell;
-Widget help_form;
-Widget help_viewport;
-Widget help_list;
-Widget help_text;
-Widget help_close_command;
+static Widget help_right_form;
+static Widget help_title;
 
-Widget help_right_form;
-Widget help_title;
+static Widget help_improvement_cost, help_improvement_cost_data;
+static Widget help_improvement_upkeep, help_improvement_upkeep_data;
+static Widget help_improvement_req, help_improvement_req_data;
+static Widget help_improvement_variant, help_improvement_variant_data;
+static Widget help_wonder_obsolete, help_wonder_obsolete_data;
+static Widget help_wonder_variant, help_wonder_variant_data;
 
+static Widget help_tech_tree;
+static Widget help_tree_viewport;
 
-Widget help_improvement_cost, help_improvement_cost_data;
-Widget help_improvement_upkeep, help_improvement_upkeep_data;
-Widget help_improvement_req, help_improvement_req_data;
-Widget help_improvement_variant, help_improvement_variant_data;
-Widget help_wonder_obsolete, help_wonder_obsolete_data;
-Widget help_wonder_variant, help_wonder_variant_data;
+static Widget help_unit_attack, help_unit_attack_data;
+static Widget help_unit_def, help_unit_def_data;
+static Widget help_unit_move, help_unit_move_data;
+static Widget help_unit_hp, help_unit_hp_data;
+static Widget help_unit_fp, help_unit_fp_data;
+static Widget help_unit_cost, help_unit_cost_data;
+static Widget help_unit_visrange, help_unit_visrange_data;
+static Widget help_unit_upkeep, help_unit_upkeep_data;
+static Widget help_unit_tile;
 
-Widget help_tech_tree;
-Widget help_tree_viewport;
-
-Widget help_unit_attack, help_unit_attack_data;
-Widget help_unit_def, help_unit_def_data;
-Widget help_unit_move, help_unit_move_data;
-Widget help_unit_hp, help_unit_hp_data;
-Widget help_unit_fp, help_unit_fp_data;
-Widget help_unit_cost, help_unit_cost_data;
-Widget help_unit_visrange, help_unit_visrange_data;
-Widget help_unit_upkeep, help_unit_upkeep_data;
-Widget help_unit_tile;
-
-Widget help_terrain_movement_defense, help_terrain_movement_defense_data;
-Widget help_terrain_food_shield_trade, help_terrain_food_shield_trade_data;
-Widget help_terrain_special_1, help_terrain_special_1_data;
-Widget help_terrain_special_2, help_terrain_special_2_data;
-Widget help_terrain_road_result_time, help_terrain_road_result_time_data;
-Widget help_terrain_irrigation_result_time, help_terrain_irrigation_result_time_data;
-Widget help_terrain_mining_result_time, help_terrain_mining_result_time_data;
-Widget help_terrain_transform_result_time, help_terrain_transform_result_time_data;
+static Widget help_terrain_movement_defense, help_terrain_movement_defense_data;
+static Widget help_terrain_food_shield_trade, help_terrain_food_shield_trade_data;
+static Widget help_terrain_special_1, help_terrain_special_1_data;
+static Widget help_terrain_special_2, help_terrain_special_2_data;
+static Widget help_terrain_road_result_time, help_terrain_road_result_time_data;
+static Widget help_terrain_irrigation_result_time, help_terrain_irrigation_result_time_data;
+static Widget help_terrain_mining_result_time, help_terrain_mining_result_time_data;
+static Widget help_terrain_transform_result_time, help_terrain_transform_result_time_data;
 
 static void create_help_page(enum help_page_type type);
 
@@ -111,7 +107,6 @@ static void select_help_item_string(const char *item,
 				    enum help_page_type htype);
 
 static char *topic_list[1024];
-extern char long_buffer[64000];	      /* helpdata.c */
 
 #define TREE_NODE_UNKNOWN_TECH_BG "red"
 #define TREE_NODE_KNOWN_TECH_BG "green"
@@ -127,7 +122,7 @@ static void set_title_topic(const struct help_item *pitem)
   if(!strcmp(pitem->topic, "Freeciv")
      || !strcmp(pitem->topic, "About")
      || !strcmp(pitem->topic, _("About"))) {
-    xaw_set_label(help_title, FREECIV_NAME_VERSION);
+    xaw_set_label(help_title, freeciv_name_version());
   } else {
     xaw_set_label(help_title, pitem->topic);
   }
@@ -513,8 +508,8 @@ static void create_help_page(enum help_page_type type)
       XtVaCreateManagedWidget("helpunittile",
 			      labelWidgetClass,
 			      help_right_form,
-			      XtNwidth, NORMAL_TILE_WIDTH,
-			      XtNheight, NORMAL_TILE_HEIGHT,
+			      XtNwidth, UNIT_TILE_WIDTH,
+			      XtNheight, UNIT_TILE_HEIGHT,
 			      NULL);  
     XtAddCallback(help_unit_tile,
                   XtNdestroyCallback,free_bitmap_destroy_callback,
@@ -770,7 +765,7 @@ static void help_update_improvement(const struct help_item *pitem,
   
   create_help_page(HELP_IMPROVEMENT);
   
-  if (which<B_LAST) {
+  if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
     sprintf(buf, "%d ", imp->build_cost);
     xaw_set_label(help_improvement_cost_data, buf);
@@ -808,7 +803,7 @@ static void help_update_wonder(const struct help_item *pitem,
   
   create_help_page(HELP_WONDER);
 
-  if (which<B_LAST) {
+  if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
     sprintf(buf, "%d ", imp->build_cost);
     xaw_set_label(help_improvement_cost_data, buf);
@@ -912,7 +907,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     create_tech_tree(help_tech_tree, 0, i, 3);
     helptext_tech(buf, i, pitem->text);
 
-    for(j=0; j<B_LAST; ++j) {
+    for(j=0; j<game.num_impr_types; ++j) {
       if(i==improvement_types[j].tech_req) 
 	sprintf(buf+strlen(buf), _("Allows %s.\n"),
 		improvement_types[j].name);
