@@ -35,18 +35,16 @@
 #include "shared.h"
 #include "support.h"
 
+#include "clinet.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "optiondlg.h"
 
 #include "inteldlg.h"
 
-extern Widget toplevel, main_form;
-extern struct connection aconnection;
-extern Display	*display;
-
 /******************************************************************/
-Widget intel_dialog_shell;
+static Widget intel_dialog_shell;
 /******************************************************************/
 
 
