@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -39,7 +40,9 @@
 #include "climisc.h"
 #include "colors.h"
 #include "control.h" /* set_unit_focus_no_center and get_unit_in_focus */
+#include "goto.h"
 #include "graphics.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapctrl.h"
 #include "options.h"
@@ -63,59 +66,56 @@ map, and I added 1 (using the EXTRA_BOTTOM_ROW constant).
 */
 #define EXTRA_BOTTOM_ROW 1
 
-
-extern Display	*display;
-extern GC civ_gc, font_gc, prod_font_gc;
-extern GC fill_bg_gc;
-extern GC fill_tile_gc;
-extern XFontStruct *main_font_struct;
-extern XFontStruct *prod_font_struct;
-extern Window root_window;
-
-extern Widget map_canvas, map_form, info_command, turn_done_button;
-extern Widget map_vertical_scrollbar, map_horizontal_scrollbar;
-extern Widget overview_canvas, main_form, left_column_form;
-extern Widget menu_form, below_menu_form, bottom_form;
-extern Widget econ_label[10];
-extern Widget bulb_label, sun_label, flake_label, government_label, timeout_label;
-extern Widget unit_info_label;
-extern int display_depth;
-extern Pixmap single_tile_pixmap;
-extern Pixmap gray50,gray25;
-extern int city_workers_color;
-
-extern int seconds_to_turndone;
-
-extern struct Sprite *intro_gfx_sprite;
-extern struct Sprite *radar_gfx_sprite;
-extern Cursor goto_cursor;
-extern Cursor drop_cursor;
-extern Cursor nuke_cursor;
-
-extern Pixmap map_canvas_store;
-extern int map_canvas_store_twidth, map_canvas_store_theight;
-extern Pixmap overview_canvas_store;
-extern int overview_canvas_store_width, overview_canvas_store_height;
-
 /* contains the x0, y0 coordinates of the upper left corner block */
 int map_view_x0, map_view_y0;
 
 /* used by map_canvas expose func */ 
 int force_full_repaint;
 
-extern int goto_state;
-extern int paradrop_state;
-extern int nuke_state;
-
 static void pixmap_put_overlay_tile(Pixmap pixmap, int x, int y,
  				    struct Sprite *ssprite);
 static void show_city_descriptions(void);
+static void put_line(Pixmap pm, int x, int y, int dir);
 
 /* the intro picture is held in this pixmap, which is scaled to
    the screen size */
 Pixmap scaled_intro_pixmap;
 int scaled_intro_pixmap_width, scaled_intro_pixmap_height;
 
+/**************************************************************************
+Finds the pixel coordinates of a tile.
+Beside setting the results in canvas_x,canvas_y it returns whether the tile
+is inside the visible map.
+**************************************************************************/
+static int get_canvas_xy(int map_x, int map_y, int *canvas_x, int *canvas_y)
+{
+  if (map_view_x0+map_canvas_store_twidth <= map.xsize)
+    *canvas_x = map_x-map_view_x0;
+  else if(map_x >= map_view_x0)
+    *canvas_x = map_x-map_view_x0;
+  else if(map_x < map_adjust_x(map_view_x0+map_canvas_store_twidth))
+    *canvas_x = map_x+map.xsize-map_view_x0;
+  else *canvas_x = -1;
+
+  *canvas_y = map_y - map_view_y0;
+
+  *canvas_x *= NORMAL_TILE_WIDTH;
+  *canvas_y *= NORMAL_TILE_HEIGHT;
+
+  return *canvas_x >= 0
+    && *canvas_x < map_canvas_store_twidth*NORMAL_TILE_WIDTH
+    && *canvas_y >= 0
+    && *canvas_y < map_canvas_store_twidth*NORMAL_TILE_HEIGHT;
+}
+
+/**************************************************************************
+Finds the map coordinates corresponding to pixel coordinates.
+**************************************************************************/
+void get_map_xy(int canvas_x, int canvas_y, int *map_x, int *map_y)
+{
+  *map_x = map_adjust_x(map_view_x0 + canvas_x/NORMAL_TILE_WIDTH);
+  *map_y = map_adjust_y(map_view_y0 + canvas_y/NORMAL_TILE_HEIGHT);
+}
 
 /**************************************************************************
 ...
@@ -126,6 +126,7 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
   static struct timer *anim_timer = NULL; 
   struct unit *losing_unit = (hp0 == 0 ? punit0 : punit1);
   int i;
+  int canvas_x, canvas_y;
 
   if (!do_combat_animation) {
     punit0->hp = hp0;
@@ -159,6 +160,7 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
 
   } while (punit0->hp > hp0 || punit1->hp > hp1);
 
+  get_canvas_xy(losing_unit->x, losing_unit->y, &canvas_x, &canvas_y);
   for (i = 0; i < num_tiles_explode_unit; i++) {
     anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
 
@@ -170,8 +172,7 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
     XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas), civ_gc,
 	      0, 0,
 	      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
-	      map_canvas_adjust_x(losing_unit->x) * NORMAL_TILE_WIDTH,
-	      map_canvas_adjust_y(losing_unit->y) * NORMAL_TILE_HEIGHT );
+	      canvas_x, canvas_y);
 
     XSync(display, 0);
     usleep_since_timer_start(anim_timer, 20000);
@@ -308,29 +309,39 @@ void update_unit_info_label(struct unit *punit)
     struct city *pcity;
     pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
     my_snprintf(buffer, sizeof(buffer), "%s %s\n%s\n%s\n%s", 
-	    get_unit_type(punit->type)->name,
-	    (punit->veteran) ? _("(veteran)") : "",
-	    (goto_state==punit->id) ? 
-	    _("Select destination") : unit_activity_text(punit), 
-	    map_get_tile_info_text(punit->x, punit->y),
-	    pcity ? pcity->name : "");
+		get_unit_type(punit->type)->name,
+		(punit->veteran) ? _("(veteran)") : "",
+		(hover_unit==punit->id) ? 
+		_("Select destination") : unit_activity_text(punit), 
+		map_get_tile_info_text(punit->x, punit->y),
+		pcity ? pcity->name : "");
     xaw_set_label(unit_info_label, buffer);
 
-    if ((goto_cursor != (Cursor) None) && (drop_cursor != (Cursor) None)) {
-      if (goto_state == punit->id) {
-	if (paradrop_state)
-	  XDefineCursor(display, XtWindow(map_canvas), drop_cursor);
-	else if (nuke_state)
-	  XDefineCursor(display, XtWindow(map_canvas), nuke_cursor);
-	else
-	  XDefineCursor(display, XtWindow(map_canvas), goto_cursor);
-      } else {
-	XUndefineCursor(display, XtWindow(map_canvas));
-      }
+    if (hover_unit != punit->id)
+      set_hover_state(NULL, HOVER_NONE);
+
+    switch (hover_state) {
+    case HOVER_NONE:
+      XUndefineCursor(display, XtWindow(map_canvas));
+      break;
+    case HOVER_PATROL:
+      XDefineCursor(display, XtWindow(map_canvas), patrol_cursor);
+      break;
+    case HOVER_GOTO:
+    case HOVER_CONNECT:
+      XDefineCursor(display, XtWindow(map_canvas), goto_cursor);
+      break;
+    case HOVER_NUKE:
+      XDefineCursor(display, XtWindow(map_canvas), nuke_cursor);
+      break;
+    case HOVER_PARADROP:
+      XDefineCursor(display, XtWindow(map_canvas), drop_cursor);
+      break;
     }
-  }
-  else
+  } else {
     xaw_set_label(unit_info_label, "");
+    XUndefineCursor(display, XtWindow(map_canvas));
+  }
 
   update_unit_pix_label(punit);
 }
@@ -377,29 +388,6 @@ void set_indicator_icons(int bulb, int sol, int flake, int gov)
   xaw_set_bitmap(government_label, gov_sprite->pixmap);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-int map_canvas_adjust_x(int x)
-{
-  if(map_view_x0+map_canvas_store_twidth<=map.xsize)
-     return x-map_view_x0;
-  else if(x>=map_view_x0)
-     return x-map_view_x0;
-  else if(x<map_adjust_x(map_view_x0+map_canvas_store_twidth))
-     return x+map.xsize-map_view_x0;
-
-  return -1;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int map_canvas_adjust_y(int y)
-{
-  return y-map_view_y0;
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -409,8 +397,7 @@ void refresh_tile_mapcanvas(int x, int y, int write_to_screen)
   y=map_adjust_y(y);
 
   if(tile_visible_mapcanvas(x, y)) {
-    update_map_canvas(map_canvas_adjust_x(x), 
-		      map_canvas_adjust_y(y), 1, 1, write_to_screen);
+    update_map_canvas(x, y, 1, 1, write_to_screen);
   }
   overview_update_tile(x, y);
 }
@@ -475,12 +462,7 @@ void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
       steps = smooth_move_unit_steps;
     }
 
-    if(x0 >= map_view_x0) {
-      start_x = (x0 - map_view_x0) * NORMAL_TILE_WIDTH;
-    } else {
-      start_x = (map.xsize - map_view_x0 + x0) * NORMAL_TILE_WIDTH;
-    }
-    start_y = (y0 - map_view_y0) * NORMAL_TILE_HEIGHT;
+    get_canvas_xy(x0, y0, &start_x, & start_y);
 
     this_x = start_x;
     this_y = start_y;
@@ -517,8 +499,8 @@ void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
 **************************************************************************/
 void get_center_tile_mapcanvas(int *x, int *y)
 {
-  *x=map_adjust_x(map_view_x0+map_canvas_store_twidth/2);
-  *y=map_adjust_y(map_view_y0+map_canvas_store_theight/2);
+  *x = map_adjust_x(map_view_x0+map_canvas_store_twidth/2);
+  *y = map_adjust_y(map_view_y0+map_canvas_store_theight/2);
 }
 
 /**************************************************************************
@@ -537,10 +519,12 @@ void center_tile_mapcanvas(int x, int y)
   map_view_x0=new_map_view_x0;
   map_view_y0=new_map_view_y0;
 
-  update_map_canvas(0, 0, map_canvas_store_twidth,map_canvas_store_theight, 1);
+  update_map_canvas_visible();
   update_map_canvas_scrollbars();
   
   refresh_overview_viewrect();
+  if (hover_state == HOVER_GOTO || hover_state == HOVER_PATROL)
+    create_line_at_mouse_pos();
 }
 
 /**************************************************************************
@@ -711,8 +695,7 @@ void map_canvas_expose(Widget w, XEvent *event, Region exposed,
 		     NORMAL_TILE_WIDTH*map_canvas_store_twidth,
 		     NORMAL_TILE_HEIGHT*map_canvas_store_theight);
 
-      update_map_canvas(0, 0, map_canvas_store_twidth,
-			map_canvas_store_theight, 1);
+      update_map_canvas_visible();
 
       update_map_canvas_scrollbars();
       refresh_overview_viewrect();
@@ -751,30 +734,32 @@ void map_canvas_resize(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_map_canvas(int tile_x, int tile_y, int width, int height, 
+void update_map_canvas(int x, int y, int width, int height, 
 		       int write_to_screen)
 {
-  int x, y;
+  int x_itr, y_itr;
+  int canvas_x, canvas_y;
 
-  for(y=tile_y; y<tile_y+height; y++)
-    for(x=tile_x; x<tile_x+width; x++)
-      pixmap_put_tile(map_canvas_store, x, y, 
-		      (map_view_x0+x)%map.xsize, map_view_y0+y, 0);
+  for (y_itr=y; y_itr<y+height; y_itr++) {
+    for (x_itr=x; x_itr<x+width; x_itr++) {
+      int map_x = map_adjust_x(x_itr);
+      int map_y = y_itr;
+      get_canvas_xy(map_x, map_y, &canvas_x, &canvas_y);
 
-  if(write_to_screen) {
+      if (tile_visible_mapcanvas(map_x, map_y)) {
+	pixmap_put_tile(map_canvas_store, map_x, map_y,
+			canvas_x, canvas_y, 0);
+      }
+    }
+  }
+
+  get_canvas_xy(x, y, &canvas_x, &canvas_y);
+  if (write_to_screen) {
     XCopyArea(display, map_canvas_store, XtWindow(map_canvas), 
 	      civ_gc, 
-	      tile_x*NORMAL_TILE_WIDTH, 
-	      tile_y*NORMAL_TILE_HEIGHT, 
-	      width*NORMAL_TILE_WIDTH,
-	      height*NORMAL_TILE_HEIGHT,
-	      tile_x*NORMAL_TILE_WIDTH, 
-	      tile_y*NORMAL_TILE_HEIGHT);
-
-    if(width==map_canvas_store_twidth && height==map_canvas_store_theight) {
-      show_city_descriptions();
-    }
-    
+	      canvas_x, canvas_y,
+	      width*NORMAL_TILE_WIDTH, height*NORMAL_TILE_HEIGHT,
+	      canvas_x, canvas_y);
   }
 }
 
@@ -783,7 +768,9 @@ void update_map_canvas(int tile_x, int tile_y, int width, int height,
 **************************************************************************/
 void update_map_canvas_visible(void)
 {
-  update_map_canvas(0,0, map_canvas_store_twidth,map_canvas_store_theight, 1);
+  update_map_canvas(map_view_x0, map_view_y0,
+		    map_canvas_store_twidth, map_canvas_store_theight, 1);
+  show_city_descriptions();
 }
 
 /**************************************************************************
@@ -805,8 +792,7 @@ Update display of descriptions associated with cities on the main map.
 **************************************************************************/
 void update_city_descriptions(void)
 {
-  update_map_canvas(0, 0, map_canvas_store_twidth,
-		    map_canvas_store_theight, 1);
+  update_map_canvas_visible();
 }
 
 /**************************************************************************
@@ -894,62 +880,58 @@ static void show_city_descriptions(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void put_city_tile_output(Pixmap pm, int x, int y, 
+void put_city_tile_output(Pixmap pm, int canvas_x, int canvas_y, 
 			  int food, int shield, int trade)
 {
   food = CLIP(0, food, NUM_TILES_DIGITS-1);
   trade = CLIP(0, trade, NUM_TILES_DIGITS-1);
   shield = CLIP(0, shield, NUM_TILES_DIGITS-1);
   
-  pixmap_put_overlay_tile(pm, x, y, sprites.city.tile_foodnum[food]);
-  pixmap_put_overlay_tile(pm, x, y, sprites.city.tile_shieldnum[shield]);
-  pixmap_put_overlay_tile(pm, x, y, sprites.city.tile_tradenum[trade]);
+  pixmap_put_overlay_tile(pm, canvas_x, canvas_y, sprites.city.tile_foodnum[food]);
+  pixmap_put_overlay_tile(pm, canvas_x, canvas_y, sprites.city.tile_shieldnum[shield]);
+  pixmap_put_overlay_tile(pm, canvas_x, canvas_y, sprites.city.tile_tradenum[trade]);
 }
 
 
 /**************************************************************************
 ...
 **************************************************************************/
-void put_unit_pixmap(struct unit *punit, Pixmap pm, int xtile, int ytile)
+void put_unit_pixmap(struct unit *punit, Pixmap pm,
+		     int canvas_x, int canvas_y)
 {
   struct Sprite *sprites[40];
-  int count = fill_unit_sprite_array(sprites, punit);
+  int solid_bg;
+  int count = fill_unit_sprite_array(sprites, punit, &solid_bg);
 
-  if(count)
-  {
-    int i;
+  if (count) {
+    int i = 0;
 
-    if(sprites[0])
-    {
-      if(flags_are_transparent)
-      {
-        pixmap_put_overlay_tile(pm, xtile, ytile, sprites[0]);
-      } else
-      {
-        XCopyArea(display, sprites[0]->pixmap, pm, civ_gc, 0, 0,
-                  sprites[0]->width, sprites[0]->height, 
-                  xtile*NORMAL_TILE_WIDTH, ytile*NORMAL_TILE_HEIGHT);
-      }
-    } else
-    {
+    if (solid_bg) {
       XSetForeground(display, fill_bg_gc,
-		     colors_standard[player_color(get_player(punit->owner))]);
+		     colors_standard[player_color(unit_owner(punit))]);
       XFillRectangle(display, pm, fill_bg_gc, 
-                     xtile*NORMAL_TILE_WIDTH, ytile*NORMAL_TILE_HEIGHT, 
-                     NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+		     canvas_x, canvas_y,
+		     NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    } else {
+      if (flags_are_transparent) {
+	pixmap_put_overlay_tile(pm, canvas_x, canvas_y, sprites[0]);
+      } else {
+	XCopyArea(display, sprites[0]->pixmap, pm, civ_gc, 0, 0,
+		  sprites[0]->width, sprites[0]->height, 
+		  canvas_x, canvas_y);
+      }
+      i++;
     }
 
-    for(i=1;i<count;i++)
-    {
-      if(sprites[i])
-        pixmap_put_overlay_tile(pm, xtile, ytile, sprites[i]);
+    for (; i<count; i++) {
+      if (sprites[i])
+	pixmap_put_overlay_tile(pm, canvas_x, canvas_y, sprites[i]);
     }
   }
 }
 
 
 /**************************************************************************
-  ...
   FIXME: 
   For now only two food, one shield and two masks can be drawn per unit,
   the proper way to do this is probably something like what Civ II does.
@@ -977,43 +959,44 @@ void put_unit_pixmap_city_overlays(struct unit *punit, Pixmap pm)
 /**************************************************************************
 ...
 **************************************************************************/
-void put_nuke_mushroom_pixmaps(int abs_x0, int abs_y0)
+void put_nuke_mushroom_pixmaps(int x, int y)
 {
-  int x, y;
+  int x_itr, y_itr;
 
-  for(y=0; y<3; y++) {
-    for(x=0; x<3; x++) {
-      int map_x = map_canvas_adjust_x(x-1+abs_x0)*NORMAL_TILE_WIDTH;
-      int map_y = map_canvas_adjust_y(y-1+abs_y0)*NORMAL_TILE_HEIGHT;
-      struct Sprite *mysprite = sprites.explode.nuke[y][x];
+  for (x_itr = 0; x_itr<3; x_itr++) {
+    for (y_itr = 0; y_itr<3; y_itr++) {
+      int x1 = x + x_itr -1;
+      int y1 = y + y_itr -1;
+      if (normalize_map_pos(&x1, &y1)) {
+	int canvas_x, canvas_y;
+	struct Sprite *mysprite = sprites.explode.nuke[y_itr][x_itr];
 
-      XCopyArea(display, map_canvas_store, single_tile_pixmap, civ_gc,
-		map_x, map_y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
-		0, 0);
-      pixmap_put_overlay_tile(single_tile_pixmap, 0, 0, mysprite);
-      XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas), civ_gc,
-		0, 0, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
-		map_x, map_y);
+	get_canvas_xy(x1, y1, &canvas_x, &canvas_y);
+	XCopyArea(display, map_canvas_store, single_tile_pixmap, civ_gc,
+		  canvas_x, canvas_y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		  0, 0);
+	pixmap_put_overlay_tile(single_tile_pixmap, 0, 0, mysprite);
+	XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas), civ_gc,
+		  0, 0, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		  canvas_x, canvas_y);
+      }
     }
   }
-
   XSync(display, 0);
   sleep(1);
 
-  update_map_canvas(map_canvas_adjust_x(abs_x0-1),
-                    map_canvas_adjust_y(abs_y0-1),
-		    3, 3, 1);
+  update_map_canvas(x-1, y-1, 3, 3, 1);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void pixmap_put_black_tile(Pixmap pm, int x, int y)
+void pixmap_put_black_tile(Pixmap pm, int canvas_x, int canvas_y)
 {
   XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_BLACK]);
 
   XFillRectangle(display, pm, fill_bg_gc,  
-		 x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT,
+		 canvas_x, canvas_y,
 		 NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
 }
 		     
@@ -1021,131 +1004,153 @@ void pixmap_put_black_tile(Pixmap pm, int x, int y)
 /**************************************************************************
 ...
 **************************************************************************/
-void pixmap_frame_tile_red(Pixmap pm, int x, int y)
+void pixmap_frame_tile_red(Pixmap pm, int canvas_x, int canvas_y)
 {
   XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_RED]);
 
   XDrawRectangle(display, pm, fill_bg_gc,  
-		 x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT,
+		 canvas_x, canvas_y,
 		 NORMAL_TILE_WIDTH-1, NORMAL_TILE_HEIGHT-1);
   XDrawRectangle(display, pm, fill_bg_gc,  
-		 x*NORMAL_TILE_WIDTH+1, y*NORMAL_TILE_HEIGHT+1,
+		 canvas_x+1, canvas_y+1,
 		 NORMAL_TILE_WIDTH-3, NORMAL_TILE_HEIGHT-3);
   XDrawRectangle(display, pm, fill_bg_gc,  
-		 x*NORMAL_TILE_WIDTH+2, y*NORMAL_TILE_HEIGHT+2,
+		 canvas_x+2, canvas_y+2,
 		 NORMAL_TILE_WIDTH-5, NORMAL_TILE_HEIGHT-5);
 }
 
 
-
 /**************************************************************************
 ...
 **************************************************************************/
-void pixmap_put_tile(Pixmap pm, int x, int y, int abs_x0, int abs_y0, 
+void pixmap_put_tile(Pixmap pm, int x, int y, int canvas_x, int canvas_y, 
 		     int citymode)
 {
-  struct Sprite *sprites[80];
-  int count = fill_tile_sprite_array(sprites, abs_x0, abs_y0, citymode);
+  struct Sprite *tile_sprs[80];
+  int fill_bg;
+  struct player *pplayer;
 
-  int x1 = x * NORMAL_TILE_WIDTH;
-  int y1 = y * NORMAL_TILE_HEIGHT;
+  if (normalize_map_pos(&x, &y) && tile_is_known(x, y)) {
+    int count = fill_tile_sprite_array(tile_sprs, x, y, citymode, &fill_bg, &pplayer);
+    int i = 0;
 
-  if(count)
-  {
-    int i;
-
-    if(sprites[0])
-    {
-      /* first tile without mask */
-      XCopyArea(display, sprites[0]->pixmap, pm, 
-              civ_gc, 0, 0,
-              sprites[0]->width, sprites[0]->height, x1,y1);
-    } else
-    {
-      /* normally when solid_color_behind_units */
-      struct city *pcity;
-      struct player *pplayer=NULL;
-
-      if(count>1 && !sprites[1])
-      {
-        /* it's the unit */
-        struct tile *ptile;
-        struct unit *punit;
-        ptile=map_get_tile(abs_x0, abs_y0);
-        if ((punit=find_visible_unit(ptile)))
-          pplayer = &game.players[punit->owner];
-
-      } else
-      {
-        /* it's the city */
-        if((pcity=map_get_city(abs_x0, abs_y0)))
-          pplayer = &game.players[pcity->owner];
-      }
-
-      if(pplayer)
-      {
+    if (fill_bg) {
+      if (pplayer) {
         XSetForeground(display, fill_bg_gc,
 		       colors_standard[player_color(pplayer)]);
-        XFillRectangle(display, pm, fill_bg_gc, 
-                       x1,y1,NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+      } else {
+        XSetForeground(display, fill_bg_gc,
+		       colors_standard[COLOR_STD_BACKGROUND]);
+      }
+      XFillRectangle(display, pm, fill_bg_gc, 
+		     canvas_x, canvas_y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    } else {
+      /* first tile without mask */
+      XCopyArea(display, tile_sprs[0]->pixmap, pm, 
+		civ_gc, 0, 0,
+		tile_sprs[0]->width, tile_sprs[0]->height, canvas_x, canvas_y);
+      i++;
+    }
+
+    for (;i<count; i++) {
+      if (tile_sprs[i]) {
+        pixmap_put_overlay_tile(pm, canvas_x, canvas_y, tile_sprs[i]);
       }
     }
 
-    for(i=1;i<count;i++)
-    {
-      if(sprites[i])
-        pixmap_put_overlay_tile(pm, x, y, sprites[i]);
-    }
-
-    if(draw_map_grid && !citymode) {
+    if (draw_map_grid && !citymode) {
       int here_in_radius =
-	player_in_city_radius(game.player_ptr, abs_x0, abs_y0);
+	player_in_city_radius(game.player_ptr, x, y);
       /* left side... */
-      if((map_get_tile(abs_x0-1, abs_y0))->known &&
-	 (here_in_radius ||
-	  player_in_city_radius(game.player_ptr, abs_x0-1, abs_y0))) {
+      if ((map_get_tile(x-1, y))->known &&
+	  (here_in_radius ||
+	   player_in_city_radius(game.player_ptr, x-1, y))) {
 	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_WHITE]);
       } else {
 	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_BLACK]);
       }
       XDrawLine(display, pm, civ_gc,
-		x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT,
-		x*NORMAL_TILE_WIDTH, (y+1)*NORMAL_TILE_HEIGHT);
+		canvas_x, canvas_y,
+		canvas_x, canvas_y + NORMAL_TILE_HEIGHT);
       /* top side... */
-      if((map_get_tile(abs_x0, abs_y0-1))->known &&
+      if((map_get_tile(x, y-1))->known &&
 	 (here_in_radius ||
-	  player_in_city_radius(game.player_ptr, abs_x0, abs_y0-1))) {
+	  player_in_city_radius(game.player_ptr, x, y-1))) {
 	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_WHITE]);
       } else {
 	XSetForeground(display, civ_gc, colors_standard[COLOR_STD_BLACK]);
       }
       XDrawLine(display, pm, civ_gc,
-		x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT,
-		(x+1)*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
+		canvas_x, canvas_y,
+		canvas_x + NORMAL_TILE_WIDTH, canvas_y);
+    }
+    if (draw_coastline && !draw_terrain) {
+      enum tile_terrain_type t1 = map_get_terrain(x, y), t2;
+      int x1 = x-1, y1 = y;
+      XSetForeground(display, civ_gc, colors_standard[COLOR_STD_OCEAN]);
+      if (normalize_map_pos(&x1, &y1)) {
+	t2 = map_get_terrain(x1, y1);
+	/* left side */
+	if ((t1 == T_OCEAN) ^ (t2 == T_OCEAN))
+	  XDrawLine(display, pm, civ_gc,
+		    canvas_x, canvas_y,
+		    canvas_x, canvas_y + NORMAL_TILE_HEIGHT);
+      }
+      /* top side */
+      x1 = x; y1 = y-1;
+      if (normalize_map_pos(&x1, &y1)) {
+	t2 = map_get_terrain(x1, y1);
+	if ((t1 == T_OCEAN) ^ (t2 == T_OCEAN))
+	  XDrawLine(display, pm, civ_gc,
+		    canvas_x, canvas_y,
+		    canvas_x + NORMAL_TILE_WIDTH, canvas_y);
+      }
+    }
+  } else {
+    /* tile is unknown */
+    pixmap_put_black_tile(pm, canvas_x, canvas_y);
+  }
+
+  if (!citymode) {
+    /* put any goto lines on the tile. */
+    if (y >= 0 && y < map.ysize) {
+      int dir;
+      for (dir = 0; dir < 8; dir++) {
+	if (get_drawn(x, y, dir)) {
+	  put_line(map_canvas_store, x, y, dir);
+	}
+      }
     }
 
-  } else
-  {
-    /* tile is unknow */
-    pixmap_put_black_tile(pm, x,y);
+    /* Some goto lines overlap onto the tile... */
+    if (NORMAL_TILE_WIDTH%2 == 0 || NORMAL_TILE_HEIGHT%2 == 0) {
+      int line_x = x - 1;
+      int line_y = y;
+      if (normalize_map_pos(&line_x, &line_y)
+	  && get_drawn(line_x, line_y, 2)) {
+	/* it is really only one pixel in the top right corner */
+	put_line(map_canvas_store, line_x, line_y, 2);
+      }
+    }
   }
 }
 
+  
 /**************************************************************************
 ...
 **************************************************************************/
-static void pixmap_put_overlay_tile(Pixmap pixmap, int x, int y,
+static void pixmap_put_overlay_tile(Pixmap pixmap, int canvas_x, int canvas_y,
  				    struct Sprite *ssprite)
 {
   if (!ssprite) return;
       
-  XSetClipOrigin(display, civ_gc, x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
+  XSetClipOrigin(display, civ_gc, canvas_x, canvas_y);
   XSetClipMask(display, civ_gc, ssprite->mask);
       
   XCopyArea(display, ssprite->pixmap, pixmap, 
 	    civ_gc, 0, 0,
 	    ssprite->width, ssprite->height, 
-	    x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
+	    canvas_x, canvas_y);
   XSetClipMask(display, civ_gc, None); 
 }
 
@@ -1154,12 +1159,14 @@ static void pixmap_put_overlay_tile(Pixmap pixmap, int x, int y,
 **************************************************************************/
 void put_cross_overlay_tile(int x,int y)
 {
+  int canvas_x, canvas_y;
   x=map_adjust_x(x);
   y=map_adjust_y(y);
+  get_canvas_xy(x, y, &canvas_x, &canvas_y);
 
-  if(tile_visible_mapcanvas(x, y)) {
-    pixmap_put_overlay_tile(XtWindow(map_canvas),map_canvas_adjust_x(x),
-			    map_canvas_adjust_y(y), sprites.user.attention);
+  if (tile_visible_mapcanvas(x, y)) {
+    pixmap_put_overlay_tile(XtWindow(map_canvas), canvas_x, canvas_y,
+			    sprites.user.attention);
   }
 }
 
@@ -1169,44 +1176,48 @@ void put_cross_overlay_tile(int x,int y)
 **************************************************************************/
 void put_city_workers(struct city *pcity, int color)
 {
-  int x,y;
-  int i,j;
-  static struct city *last_pcity=NULL;
+  int canvas_x, canvas_y;
+  static struct city *last_pcity = NULL;
 
-  if(color==-1) {
-    if(pcity!=last_pcity)  city_workers_color = (city_workers_color%3)+1;
-    color=city_workers_color;
+  if (color == -1) {
+    if (pcity != last_pcity)
+      city_workers_color = (city_workers_color%3)+1;
+    color = city_workers_color;
   }
 
   XSetForeground(display, fill_tile_gc, colors_standard[color]);
-  x=map_canvas_adjust_x(pcity->x); y=map_canvas_adjust_y(pcity->y);
+  get_canvas_xy(pcity->x, pcity->y, &canvas_x, &canvas_y);
   city_map_iterate(i, j)  {
-    enum city_tile_type t=get_worker_city(pcity, i, j);
-    enum city_tile_type last_t=-1;
-    if(!(i==2 && j==2)) {
-      if(t==C_TILE_EMPTY) {
-	if(last_t!=t) XSetStipple(display,fill_tile_gc,gray25);
-      } else if(t==C_TILE_WORKER) {
-	if(last_t!=t) XSetStipple(display,fill_tile_gc,gray50);
-      } else continue;
-      last_t=t;
-      XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc, 
-		(x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT, 
-		NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
-		(x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT);
-      XFillRectangle(display, XtWindow(map_canvas), fill_tile_gc,
-		     (x+i-2)*NORMAL_TILE_WIDTH, (y+j-2)*NORMAL_TILE_HEIGHT,
-		     NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    enum city_tile_type worked = get_worker_city(pcity, i, j);
+    int x = pcity->x + i - CITY_MAP_SIZE/2;
+    int y = pcity->y + j - CITY_MAP_SIZE/2;
+    if (normalize_map_pos(&x, &y)) {
+      get_canvas_xy(x, y, &canvas_x, &canvas_y);
+      if (!(i==2 && j==2)) {
+	if (worked == C_TILE_EMPTY) {
+	  XSetStipple(display, fill_tile_gc,gray25);
+	} else if (worked == C_TILE_WORKER) {
+	  XSetStipple(display, fill_tile_gc,gray50);
+	} else continue;
+	XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc, 
+		  canvas_x, canvas_y,
+		  NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		  canvas_x, canvas_y);
+	XFillRectangle(display, XtWindow(map_canvas), fill_tile_gc,
+		       canvas_x, canvas_y,
+		       NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+      }
+      if (worked == C_TILE_WORKER) {
+	put_city_tile_output(XtWindow(map_canvas),
+			     canvas_x, canvas_y, 
+			     city_get_food_tile(i, j, pcity),
+			     city_get_shields_tile(i, j, pcity), 
+			     city_get_trade_tile(i, j, pcity) );
+      }
     }
-    if(t==C_TILE_WORKER) {
-      put_city_tile_output(XtWindow(map_canvas), x+i-2, y+j-2, 
-			   get_food_tile(i, j, pcity),
-			   get_shields_tile(i, j, pcity), 
-			   get_trade_tile(i, j, pcity) );
-    }
-  }
+  } city_map_iterate_end;
 
-  last_pcity=pcity;
+  last_pcity = pcity;
 }
 
 
@@ -1232,7 +1243,7 @@ void scrollbar_jump_callback(Widget w, XtPointer client_data,
 	map_view_y0;
   }
 
-  update_map_canvas(0, 0, map_canvas_store_twidth, map_canvas_store_theight, 1);
+  update_map_canvas_visible();
   /* The scrollbar tracks by itself, while calling the jumpProc,
      so there's no need to call update_map_canvas_scrollbars() here. */
   refresh_overview_viewrect();
@@ -1268,7 +1279,97 @@ void scrollbar_scroll_callback(Widget w, XtPointer client_data,
   map_view_y0=map_adjust_y(map_view_y0);
 
 
-  update_map_canvas(0, 0, map_canvas_store_twidth, map_canvas_store_theight, 1);
+  update_map_canvas_visible();
   update_map_canvas_scrollbars();
   refresh_overview_viewrect();
 }
+
+/**************************************************************************
+...
+**************************************************************************/
+static void put_line(Pixmap pm, int x, int y, int dir)
+{
+  int canvas_src_x, canvas_src_y, canvas_dest_x, canvas_dest_y;
+  get_canvas_xy(x, y, &canvas_src_x, &canvas_src_y);
+  canvas_src_x += NORMAL_TILE_WIDTH/2;
+  canvas_src_y += NORMAL_TILE_HEIGHT/2;
+  canvas_dest_x = canvas_src_x + (NORMAL_TILE_WIDTH * DIR_DX[dir])/2;
+  canvas_dest_y = canvas_src_y + (NORMAL_TILE_WIDTH * DIR_DY[dir])/2;
+
+  XSetForeground(display, civ_gc, colors_standard[COLOR_STD_CYAN]);
+
+  XDrawLine(display, pm, civ_gc, canvas_src_x, canvas_src_y,
+	    canvas_dest_x, canvas_dest_y);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void draw_segment(int src_x, int src_y, int dir)
+{
+  int dest_x, dest_y;
+
+  dest_x = src_x + DIR_DX[dir];
+  dest_y = src_y + DIR_DY[dir];
+
+  assert(is_real_tile(dest_x, dest_y));
+  normalize_map_pos(&dest_x, &dest_y);
+
+  /* A previous line already marks the place */
+  if (get_drawn(src_x, src_y, dir)) {
+    increment_drawn(src_x, src_y, dir);
+    return;
+  }
+
+  if (tile_visible_mapcanvas(src_x, src_y)) {
+    put_line(map_canvas_store, src_x, src_y, dir);
+    put_line(XtWindow(map_canvas), src_x, src_y, dir);
+  }
+
+  if (tile_visible_mapcanvas(dest_x, dest_y)) {
+    put_line(map_canvas_store, dest_x, dest_y, 7-dir);
+    put_line(XtWindow(map_canvas), dest_x, dest_y, 7-dir);
+  }
+
+  increment_drawn(src_x, src_y, dir);
+}
+
+/**************************************************************************
+This is somewhat inefficient, but I simply can't feel any performance
+penalty so I will be lazy...
+**************************************************************************/
+void undraw_segment(int src_x, int src_y, int dir)
+{
+  int dest_x = src_x + DIR_DX[dir];
+  int dest_y = src_y + DIR_DY[dir];
+  int drawn = get_drawn(src_x, src_y, dir);
+
+  assert(drawn > 0);
+  /* If we walk on a path twice it looks just like walking on it once. */
+  if (drawn > 1) {
+    decrement_drawn(src_x, src_y, dir);
+    return;
+  }
+
+  decrement_drawn(src_x, src_y, dir);
+  refresh_tile_mapcanvas(src_x, src_y, 1);
+  assert(is_real_tile(dest_x, dest_y));
+  normalize_map_pos(&dest_x, &dest_y);
+  refresh_tile_mapcanvas(dest_x, dest_y, 1);
+  if (NORMAL_TILE_WIDTH%2 == 0 || NORMAL_TILE_HEIGHT%2 == 0) {
+    if (dir == 2) { /* Since the tle doesn't have a middle we draw an extra pixel
+		       on the adjacent tile when drawing in this direction. */
+      dest_x = src_x + 1;
+      dest_y = src_y;
+      assert(is_real_tile(dest_x, dest_y));
+      normalize_map_pos(&dest_x, &dest_y);
+      refresh_tile_mapcanvas(dest_x, dest_y, 1);
+    } else if (dir == 5) { /* the same */
+      dest_x = src_x;
+      dest_y = src_y + 1;
+      assert(is_real_tile(dest_x, dest_y));
+      normalize_map_pos(&dest_x, &dest_y);
+      refresh_tile_mapcanvas(dest_x, dest_y, 1);
+    }
+  }
+}
