@@ -24,7 +24,6 @@
 #include <X11/Xaw/SmeBSB.h>
 #include <X11/Xaw/SmeLine.h>
 
-#include "capability.h"
 #include "fcintl.h"
 #include "map.h"
 #include "mem.h"
@@ -54,7 +53,6 @@
 
 #include "menu.h"
 
-
 /* stuff for run-time mutable menu text */
 
 #define MAX_LEN_TERR_NAME_DISP  9
@@ -106,9 +104,6 @@ static struct MenuEntry game_menu_entries[]={
     { { N_("Message Options"), 0      },      "", MENU_GAME_MSG_OPTIONS, 0 },
     { { N_("Save Settings"), 0        },      "", MENU_GAME_SAVE_SETTINGS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Players"), 0              },    "F3", MENU_GAME_PLAYERS, 0 },
-    { { N_("Messages"), 0             },   "F10", MENU_GAME_MESSAGES, 0 },
-    { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Server Opt initial"), 0   },      "", MENU_GAME_SERVER_OPTIONS1, 0 },
     { { N_("Server Opt ongoing"), 0   },      "", MENU_GAME_SERVER_OPTIONS2, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
@@ -123,7 +118,7 @@ static struct MenuEntry game_menu_entries[]={
 static struct MenuEntry kingdom_menu_entries[]={
     { { N_("Tax Rates"), 0            },     "T", MENU_KINGDOM_RATES, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Find City"), 0            }, "ctl-f", MENU_KINGDOM_FIND_CITY, 0 },
+    { { N_("Find City"), 0            },     "F", MENU_KINGDOM_FIND_CITY, 0 },
     { { N_("Worklists"), 0            },     "L", MENU_KINGDOM_WORKLISTS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Revolution"), 0           },     "R", MENU_KINGDOM_REVOLUTION, 0 },
@@ -135,6 +130,19 @@ static struct MenuEntry view_menu_entries[]={
     { { N_("City Names"), 0           }, "ctl-n", MENU_VIEW_SHOW_CITY_NAMES, 0 },
     { { N_("City Productions"), 0     }, "ctl-p", MENU_VIEW_SHOW_CITY_PRODUCTIONS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
+    { { N_("Terrain"), 0              },      "", MENU_VIEW_SHOW_TERRAIN, 0 },
+    { { N_("Coastline"), 0            },      "", MENU_VIEW_SHOW_COASTLINE, 0 },
+    { { N_("Roads and Rails"), 0      },      "", MENU_VIEW_SHOW_ROADS_RAILS, 0 },
+    { { N_("Irrigation"), 0           },      "", MENU_VIEW_SHOW_IRRIGATION, 0 },
+    { { N_("Mines"), 0                },      "", MENU_VIEW_SHOW_MINES, 0 },
+    { { N_("Fortress and Airbase"), 0 },      "", MENU_VIEW_SHOW_FORTRESS_AIRBASE, 0 },
+    { { N_("Specials"), 0             },      "", MENU_VIEW_SHOW_SPECIALS, 0 },
+    { { N_("Pollution & Fallout"), 0  },      "", MENU_VIEW_SHOW_POLLUTION, 0 },
+    { { N_("Cities"), 0               },      "", MENU_VIEW_SHOW_CITIES, 0 },
+    { { N_("Units"), 0                },      "", MENU_VIEW_SHOW_UNITS, 0 },
+    { { N_("Focus Unit"), 0           },      "", MENU_VIEW_SHOW_FOCUS_UNIT, 0 },
+    { { N_("Fog of War"), 0           },      "", MENU_VIEW_SHOW_FOG_OF_WAR, 0 },
+    { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Center View"), 0          },     "c", MENU_VIEW_CENTER_VIEW, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
@@ -169,12 +177,14 @@ static struct MenuEntry order_menu_entries[]={
     { { N_("Auto Attack"), 0          },     "a", MENU_ORDER_AUTO_ATTACK, 0 },
     { { N_("Auto Explore"), 0         },     "x", MENU_ORDER_AUTO_EXPLORE, 0 },
     { { N_("Connect"), 0              },     "C", MENU_ORDER_CONNECT, 0 },
+    { { N_("Patrol"), 0               },     "q", MENU_ORDER_PATROL, 0 },
     { { N_("Go to"), 0                },     "g", MENU_ORDER_GOTO, 0 },
     { { N_("Go/Airlift to City"), 0   },     "l", MENU_ORDER_GOTO_CITY, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Disband Unit"), 0         },     "D", MENU_ORDER_DISBAND, 0 },
     { { N_("Help Build Wonder"), 0    },     "b", MENU_ORDER_BUILD_WONDER, 0 },
     { { N_("Make Trade Route"), 0     },     "r", MENU_ORDER_TRADEROUTE, 0 },
+    { { N_("Diplomat/Spy Actions"), 0 },     "d", MENU_ORDER_DIPLOMAT_DLG, 0},
     { { N_("Explode Nuclear"), 0      },     "N", MENU_ORDER_NUKE, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Wait"), 0                 },     "w", MENU_ORDER_WAIT, 0 },
@@ -183,13 +193,15 @@ static struct MenuEntry order_menu_entries[]={
 };
 
 static struct MenuEntry reports_menu_entries[]={
-    { { N_("City Report"), 0          },    "F1", MENU_REPORT_CITY, 0 },
-    { { N_("Military Report"), 0      },    "F2", MENU_REPORT_MILITARY, 0 },
-    { { N_("Trade Report"), 0         },    "F5", MENU_REPORT_TRADE, 0 },
-    { { N_("Science Report"), 0       },    "F6", MENU_REPORT_SCIENCE, 0 },
+    { { N_("Cities"), 0               },    "F1", MENU_REPORT_CITIES, 0 },
+    { { N_("Units"), 0                },    "F2", MENU_REPORT_UNITS, 0 },
+    { { N_("Players"), 0              },    "F3", MENU_REPORT_PLAYERS, 0 },
+    { { N_("Economy"), 0              },    "F5", MENU_REPORT_ECONOMY, 0 },
+    { { N_("Science"), 0              },    "F6", MENU_REPORT_SCIENCE, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Wonders of the World"), 0 },    "F7", MENU_REPORT_WOW, 0 },
     { { N_("Top Five Cities"), 0      },    "F8", MENU_REPORT_TOP_CITIES, 0 },
+    { { N_("Messages"), 0             },   "F10", MENU_REPORT_MESSAGES, 0 },
     { { N_("Demographics"), 0         },   "F11", MENU_REPORT_DEMOGRAPHIC, 0 },
     { { N_("Spaceship"), 0            },   "F12", MENU_REPORT_SPACESHIP, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
@@ -240,8 +252,6 @@ void update_menus(void)
     menu_entry_sensitive(MENU_GAME, MENU_GAME_OPTIONS, 0);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_MSG_OPTIONS, 0);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_SAVE_SETTINGS, 0);
-    menu_entry_sensitive(MENU_GAME, MENU_GAME_PLAYERS, 0);
-    menu_entry_sensitive(MENU_GAME, MENU_GAME_MESSAGES, 0);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_SERVER_OPTIONS1, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_SERVER_OPTIONS2, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_OUTPUT_LOG, 1);
@@ -266,11 +276,22 @@ void update_menus(void)
     XtSetSensitive(menus[MENU_VIEW]->button, True);
     XtSetSensitive(menus[MENU_KINGDOM]->button, True);
 
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_TERRAIN, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_COASTLINE, !draw_terrain);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_ROADS_RAILS, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_IRRIGATION, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_MINES, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_FORTRESS_AIRBASE, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_SPECIALS, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_POLLUTION, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_CITIES, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_UNITS, 1);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_FOCUS_UNIT, !draw_units);
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_FOG_OF_WAR, 1);
+
     menu_entry_sensitive(MENU_GAME, MENU_GAME_OPTIONS, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_MSG_OPTIONS, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_SAVE_SETTINGS, 1);
-    menu_entry_sensitive(MENU_GAME, MENU_GAME_PLAYERS, 1);
-    menu_entry_sensitive(MENU_GAME, MENU_GAME_MESSAGES, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_SERVER_OPTIONS1, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_SERVER_OPTIONS2, 1);
     menu_entry_sensitive(MENU_GAME, MENU_GAME_OUTPUT_LOG, 1);
@@ -331,12 +352,18 @@ void update_menus(void)
 			   can_unit_do_activity(punit, ACTIVITY_EXPLORE));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_CONNECT, 
 			   can_unit_do_connect(punit, ACTIVITY_IDLE));
+      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_PATROL, 
+			   can_unit_do_activity(punit, ACTIVITY_PATROL));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_GOTO_CITY,
 			   any_cities);
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_BUILD_WONDER,
 			   unit_can_help_build_wonder_here(punit));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_TRADEROUTE,
 			   unit_can_est_traderoute_here(punit));
+      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_DIPLOMAT_DLG,
+			   (is_diplomat_unit(punit)
+			    && diplomat_can_do_action(punit, DIPLOMAT_ANY_ACTION,
+						      punit->x, punit->y)));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_NUKE,
                            unit_flag(punit->type, F_NUCLEAR));
 
@@ -417,12 +444,6 @@ static void game_menu_callback(Widget w, XtPointer client_data,
   case MENU_GAME_SAVE_SETTINGS:
     save_options();
     break;
-  case MENU_GAME_PLAYERS:
-    popup_players_dialog();
-    break;
-  case MENU_GAME_MESSAGES:
-    popup_meswin_dialog();
-    break;
   case MENU_GAME_SERVER_OPTIONS1:
     send_report_request(REPORT_SERVER_OPTIONS1);
     break;
@@ -486,6 +507,44 @@ static void view_menu_callback(Widget w, XtPointer client_data,
   case MENU_VIEW_SHOW_CITY_PRODUCTIONS:
     key_city_productions_toggle();
     break;
+  case MENU_VIEW_SHOW_TERRAIN:
+    key_terrain_toggle();
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_COASTLINE, !draw_terrain);
+    break;
+  case MENU_VIEW_SHOW_COASTLINE:
+    key_coastline_toggle();
+    break;
+  case MENU_VIEW_SHOW_ROADS_RAILS:
+    key_roads_rails_toggle();
+    break;
+  case MENU_VIEW_SHOW_IRRIGATION:
+    key_irrigation_toggle();
+    break;
+  case MENU_VIEW_SHOW_MINES:
+    key_mines_toggle();
+    break;
+  case MENU_VIEW_SHOW_FORTRESS_AIRBASE:
+    key_fortress_airbase_toggle();
+    break;
+  case MENU_VIEW_SHOW_SPECIALS:
+    key_specials_toggle();
+    break;
+  case MENU_VIEW_SHOW_POLLUTION:
+    key_pollution_toggle();
+    break;
+  case MENU_VIEW_SHOW_CITIES:
+    key_cities_toggle();
+    break;
+  case MENU_VIEW_SHOW_UNITS:
+    key_units_toggle();
+    menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_FOCUS_UNIT, !draw_units);
+    break;
+  case MENU_VIEW_SHOW_FOCUS_UNIT:
+    key_focus_unit_toggle();
+    break;
+  case MENU_VIEW_SHOW_FOG_OF_WAR:
+    key_fog_of_war_toggle();
+    break;
   case MENU_VIEW_CENTER_VIEW:
     request_center_focus_unit();
     break;
@@ -564,6 +623,9 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
   case MENU_ORDER_CONNECT:
     key_unit_connect();
     break;
+  case MENU_ORDER_PATROL:
+    key_unit_patrol();
+    break;
   case MENU_ORDER_GOTO:
     key_unit_goto();
     break;
@@ -580,6 +642,9 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
   case MENU_ORDER_TRADEROUTE:
     key_unit_traderoute();
     break;
+  case MENU_ORDER_DIPLOMAT_DLG:
+    key_unit_diplomat_actions();
+    break;
   case MENU_ORDER_NUKE:
     key_unit_nuke();
     break;
@@ -601,14 +666,17 @@ static void reports_menu_callback(Widget w, XtPointer client_data,
   size_t pane_num = (size_t)client_data;
 
   switch(pane_num) {
-   case MENU_REPORT_CITY:
+   case MENU_REPORT_CITIES:
     popup_city_report_dialog(0);
     break;
-   case MENU_REPORT_MILITARY:
+   case MENU_REPORT_UNITS:
     popup_activeunits_report_dialog(0);
     break;
-   case MENU_REPORT_TRADE:
-    popup_trade_report_dialog(0);
+  case MENU_REPORT_PLAYERS:
+    popup_players_dialog();
+    break;
+   case MENU_REPORT_ECONOMY:
+    popup_economy_report_dialog(0);
     break;
    case MENU_REPORT_SCIENCE:
     popup_science_dialog(0);
@@ -619,6 +687,9 @@ static void reports_menu_callback(Widget w, XtPointer client_data,
    case MENU_REPORT_TOP_CITIES:
     send_report_request(REPORT_TOP_5_CITIES);
     break;
+  case MENU_REPORT_MESSAGES:
+    popup_meswin_dialog();
+    break;
    case MENU_REPORT_DEMOGRAPHIC:
     send_report_request(REPORT_DEMOGRAPHIC);
     break;
