@@ -37,24 +37,21 @@
 #include "chatline.h"
 #include "citydlg.h"
 #include "clinet.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
 #include "options.h"
 
 #include "messagewin.h"
 
-extern Widget toplevel;
-extern Display *display;
-extern Atom wm_delete_window;
-
-Widget meswin_dialog_shell;
-Widget meswin_form;
-Widget meswin_label;
-Widget meswin_list;
-Widget meswin_viewport;
-Widget meswin_close_command;
-Widget meswin_goto_command;
-Widget meswin_popcity_command;
+static Widget meswin_dialog_shell;
+static Widget meswin_form;
+static Widget meswin_label;
+static Widget meswin_list;
+static Widget meswin_viewport;
+static Widget meswin_close_command;
+static Widget meswin_goto_command;
+static Widget meswin_popcity_command;
 
 void create_meswin_dialog(void);
 void meswin_scroll_down(void);
@@ -374,10 +371,7 @@ void meswin_list_callback(Widget w, XtPointer client_data,
     int x, y;
     x = xpos[ret->list_index];
     y = ypos[ret->list_index];
-    location_ok = (x || y);
-    /* Note: bad luck if tile (x=0,y=0) has a message, but can't
-     * easily fix this without making the protocol incompatibile --dwp
-     */
+    location_ok = (x != -1 && y != -1);
     city_ok = (location_ok && (pcity=map_get_city(x,y))
 	       && (pcity->owner == game.player_idx));
   }    
@@ -414,7 +408,8 @@ void meswin_goto_callback(Widget w, XtPointer client_data,
 
   ret=XawListShowCurrent(meswin_list);
 
-  if(ret->list_index!=XAW_LIST_NONE && (xpos[ret->list_index] != 0 || ypos[ret->list_index]!=0))
+  if(ret->list_index!=XAW_LIST_NONE
+     && (xpos[ret->list_index]!=-1 && ypos[ret->list_index]!=-1))
     center_tile_mapcanvas(xpos[ret->list_index], ypos[ret->list_index]);
 }
 
@@ -432,7 +427,7 @@ void meswin_popcity_callback(Widget w, XtPointer client_data,
   if(ret->list_index!=XAW_LIST_NONE) {
     x = xpos[ret->list_index];
     y = ypos[ret->list_index];
-    if((x || y) && (pcity=map_get_city(x,y))
+    if((x!=-1 && y!=-1) && (pcity=map_get_city(x,y))
        && (pcity->owner == game.player_idx)) {
       if (center_when_popup_city) {
 	center_tile_mapcanvas(x,y);
