@@ -33,19 +33,16 @@
 #include "support.h"
 
 #include "chatline.h"
+#include "climisc.h"
 #include "clinet.h"
 #include "diplodlg.h"
+#include "gui_main.h"
 #include "gui_stuff.h"
 #include "inteldlg.h"
 #include "spaceshipdlg.h"
 
 #include "plrdlg.h"
 
-
-extern Widget toplevel, main_form;
-extern Display *display;
-extern Atom wm_delete_window;
-
 static Widget players_dialog_shell;
 static Widget players_form;
 static Widget players_label;
@@ -54,6 +51,7 @@ static Widget players_close_command;
 static Widget players_int_command;
 static Widget players_meet_command;
 static Widget players_war_command;
+static Widget players_vision_command;
 static Widget players_sship_command;
 
 static int list_index_to_player_index[MAX_NUM_PLAYERS];
@@ -70,19 +68,42 @@ static void players_list_callback(Widget w, XtPointer client_data,
 				  XtPointer call_data);
 static void players_war_callback(Widget w, XtPointer client_data, 
 				 XtPointer call_data);
+static void players_vision_callback(Widget w, XtPointer client_data, 
+				 XtPointer call_data);
 static void players_sship_callback(Widget w, XtPointer client_data, 
 				   XtPointer call_data);
 
 
+/******************************************************************/
+
+static int delay_plrdlg_update=0;
+
+/******************************************************************
+ Turn off updating of player dialog
+*******************************************************************/
+void plrdlg_update_delay_on(void)
+{
+  delay_plrdlg_update=1;
+}
+
+/******************************************************************
+ Turn on updating of player dialog
+*******************************************************************/
+void plrdlg_update_delay_off(void)
+{
+  delay_plrdlg_update=0;
+}
+
+
 /****************************************************************
-popup the dialog 10% inside the main-window 
+popup the dialog somewhat inside the main-window 
 *****************************************************************/
 void popup_players_dialog(void)
 {
   if(!players_dialog_shell)
     create_players_dialog();
 
-  xaw_set_relative_position(toplevel, players_dialog_shell, 25, 25);
+  xaw_set_relative_position(toplevel, players_dialog_shell, 5, 25);
   XtPopup(players_dialog_shell, XtGrabNone);
 }
 
@@ -133,6 +154,12 @@ void create_players_dialog(void)
 				XtNsensitive, False,
 				NULL));
 
+  players_vision_command =
+    I_L(XtVaCreateManagedWidget("playersvisioncommand", commandWidgetClass,
+				players_form,
+				XtNsensitive, False,
+				NULL));
+
   players_sship_command =
     I_L(XtVaCreateManagedWidget("playerssshipcommand", commandWidgetClass,
 				players_form,
@@ -154,6 +181,9 @@ void create_players_dialog(void)
   XtAddCallback(players_war_command, XtNcallback, players_war_callback, 
 		NULL);
 
+  XtAddCallback(players_vision_command, XtNcallback, players_vision_callback, 
+		NULL);
+
   XtAddCallback(players_sship_command, XtNcallback, players_sship_callback, 
 		NULL);
   
@@ -173,7 +203,7 @@ void create_players_dialog(void)
 **************************************************************************/
 void update_players_dialog(void)
 {
-   if(players_dialog_shell) {
+   if(players_dialog_shell && !delay_plrdlg_update) {
     int i,j;
     Dimension width;
     static char *namelist_ptrs[MAX_NUM_PLAYERS];
@@ -236,14 +266,15 @@ void update_players_dialog(void)
 
       /* assemble the whole lot */
       my_snprintf(namelist_text[j], sizeof(namelist_text[j]),
-	      "%-16s %-12s    %c     %-15s %-13s %-6s   %-15s%s", 
+	      "%-16s %-12s %-8s %-15s %-8s %-13s %-6s   %-15s%s", 
 	      namebuf,
 	      get_nation_name(game.players[i].nation), 
-	      player_has_embassy(game.player_ptr, &game.players[i]) ? 'X':' ',
+	      get_embassy_status(game.player_ptr, &game.players[i]),
 	      dsbuf,
+	      get_vision_status(game.player_ptr, &game.players[i]),
 	      repbuf,
 	      statebuf,
-	      game.players[i].addr, 
+	      player_addr_hack(&game.players[i]),  /* Fixme for multi-conn */
 	      idlebuf);
 
       namelist_ptrs[j]=namelist_text[j];
@@ -286,6 +317,12 @@ void players_list_callback(Widget w, XtPointer client_data,
 	XtSetSensitive(players_war_command, TRUE);
     }
 
+    if (game.player_ptr->gives_shared_vision & (1<<player_index)) {
+      XtSetSensitive(players_vision_command, TRUE);
+    } else {
+      XtSetSensitive(players_vision_command, FALSE);
+    }
+
     if(pplayer->is_alive && player_has_embassy(game.player_ptr, pplayer)) {
       if(pplayer->is_connected)
 	XtSetSensitive(players_meet_command, TRUE);
@@ -379,6 +416,23 @@ void players_war_callback(Widget w, XtPointer client_data,
   }
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void players_vision_callback(Widget w, XtPointer client_data, 
+                          XtPointer call_data)
+{
+  XawListReturnStruct *ret;
+  
+  ret=XawListShowCurrent(players_list);
+  if(ret->list_index!=XAW_LIST_NONE) {
+    struct packet_generic_integer pa;    
+    pa.value=ret->list_index;
+    send_packet_generic_integer(&aconnection, PACKET_PLAYER_REMOVE_VISION,
+				&pa);
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
