@@ -20,7 +20,8 @@
 
 void activeunits_msg_close(Widget w);
 void sciencereport_msg_close(Widget w);
-void tradereport_msg_close(Widget w);
+void economyreport_msg_close(Widget w);
 
+extern int delay_report_update;
 
 #endif  /* FC__REPODLGS_H */
