@@ -113,9 +113,7 @@ static void insert_generated_table(const char* name, char* outbuf)
 	  if (*(tile_types[i].terrain_name))
 	    {
 	      outbuf = strchr (outbuf, '\0');
-	      sprintf
-		(
-		 outbuf,
+	      sprintf(outbuf,
 		 "%-10s %3d    %3d %-10s %3d %-10s %3d %-10s\n",
 		 tile_types[i].terrain_name,
 		 tile_types[i].road_time,
@@ -155,7 +153,7 @@ static struct help_item *new_help_item(int type)
 }
 
 /****************************************************************
- for genlist_sort(); sort by topic via strcmp
+ for genlist_sort(); sort by topic via compare_strings()
  (sort topics with more leading spaces after those with fewer)
 *****************************************************************/
 static int help_item_compar(const void *a, const void *b)
@@ -173,7 +171,7 @@ static int help_item_compar(const void *a, const void *b)
       break;
     }
   }
-  return strcmp(ta, tb);
+  return compare_strings(ta, tb);
 }
 
 /****************************************************************
@@ -185,7 +183,7 @@ void boot_help_texts(void)
 
   struct section_file file, *sf = &file;
   char *filename;
-  struct help_item *pitem = NULL;
+  struct help_item *pitem;
   int i, isec;
   char **sec, **paras;
   int nsec, npara;
@@ -205,33 +203,31 @@ void boot_help_texts(void)
   
   filename = datafilename("helpdata.txt");
   if (filename == NULL) {
-    freelog(LOG_NORMAL, "Could not find readable helpdata.txt in data path");
-    freelog(LOG_NORMAL, "The data path may be set via"
+    freelog(LOG_ERROR, "Could not find readable helpdata.txt in data path");
+    freelog(LOG_ERROR, "The data path may be set via"
 	                " the environment variable FREECIV_PATH");
-    freelog(LOG_NORMAL, "Current data path is: \"%s\"", datafilename(NULL));
-    freelog(LOG_NORMAL, "Did not read help texts");
+    freelog(LOG_ERROR, "Current data path is: \"%s\"", datafilename(NULL));
+    freelog(LOG_ERROR, "Did not read help texts");
     return;
   }
   /* after following call filename may be clobbered; use sf->filename instead */
   if (!section_file_load(sf, filename)) {
     /* this is now unlikely to happen */
-    freelog(LOG_NORMAL, "failed reading help-texts");
+    freelog(LOG_ERROR, "failed reading help-texts");
     return;
   }
 
   sec = secfile_get_secnames_prefix(sf, "help_", &nsec);
   
   for(isec=0; isec<nsec; isec++) {
-    
-    enum help_page_type current_type = HELP_ANY;
     char *gen_str =
       secfile_lookup_str_default(sf, NULL, "%s.generate", sec[isec]);
     
     if (gen_str) {
+      enum help_page_type current_type = HELP_ANY;
       if (!booted) {
 	continue; /* on initial boot data tables are empty */
       }
-      current_type = HELP_ANY;
       for(i=2; help_type_names[i]; i++) {
 	if(strcmp(gen_str, help_type_names[i])==0) {
 	  current_type = i;
@@ -239,7 +235,7 @@ void boot_help_texts(void)
 	}
       }
       if (current_type == HELP_ANY) {
-	freelog(LOG_NORMAL, "bad help-generate category \"%s\"", gen_str);
+	freelog(LOG_ERROR, "bad help-generate category \"%s\"", gen_str);
 	continue;
       }
       {
@@ -299,7 +295,7 @@ void boot_help_texts(void)
 	      genlist_insert(&category_nodes, pitem, -1);
 	  }
 	} else if(current_type==HELP_IMPROVEMENT) {
-	  for(i=0; i<B_LAST; i++) {
+	  for(i=0; i<game.num_impr_types; i++) {
 	    if(improvement_exists(i) && !is_wonder(i)) {
 	      pitem = new_help_item(current_type);
 	      my_snprintf(name, sizeof(name), " %s", improvement_types[i].name);
@@ -309,7 +305,7 @@ void boot_help_texts(void)
 	    }
 	  }
 	} else if(current_type==HELP_WONDER) {
-	  for(i=0; i<B_LAST; i++) {
+	  for(i=0; i<game.num_impr_types; i++) {
 	    if(improvement_exists(i) && is_wonder(i)) {
 	      pitem = new_help_item(current_type);
 	      my_snprintf(name, sizeof(name), " %s", improvement_types[i].name);
@@ -392,7 +388,7 @@ const struct help_item *get_help_item(int pos)
   check_help_nodes_init();
   size = genlist_size(&help_nodes);
   if (pos < 0 || pos > size) {
-    freelog(LOG_NORMAL, "Bad index %d to get_help_item (size %d)", pos, size);
+    freelog(LOG_ERROR, "Bad index %d to get_help_item (size %d)", pos, size);
     return NULL;
   }
   if (pos == size) {
@@ -499,31 +495,6 @@ const struct help_item *help_iter_next(void)
   have been built (or are being built and by who/where?)
 *****************************************************************/
 
-/****************************************************************
-  Write text about cathedral techs: used by both
-  Cathedrals proper and Mich's Chapel Wonder.
-  Returns new end-of-string ptr.
-*****************************************************************/
-static void helptext_cathedral_techs(char *buf)
-{
-  int t;
-
-  assert(buf);
-  buf[0] = '\0';
-  t=game.rtech.cathedral_minus;
-  if(tech_exists(t)) {
-    sprintf(buf, _("The discovery of %s will reduce this by 1.  "),
-	    advances[t].name);
-    buf += strlen(buf);
-  }
-  t=game.rtech.cathedral_plus;
-  if(tech_exists(t)) {
-    sprintf(buf, _("The discovery of %s will increase this by 1.  "),
-	   advances[t].name);
-    buf += strlen(buf);
-  }
-}
-
 /****************************************************************
   Write misc dynamic text for improvements (not wonders).
   user_text is written after some extra, and before others.
@@ -540,9 +511,8 @@ void helptext_improvement(char *buf, int which, const char *user_text)
     if(improvement_exists(B_SEWER)) {
       char *s = improvement_types[B_SEWER].name;
       sprintf(buf+strlen(buf),
-	      _("(A%s %s is also required for a city to grow larger"
-		" than size %d.)  "),
-	      n_if_vowel(*s), s, game.sewer_size);
+	      _("(The %s improvement is also required for a city to grow"
+		" larger than size %d.)  "), s, game.sewer_size);
     }
     strcat(buf,"\n");
   }
@@ -553,18 +523,6 @@ void helptext_improvement(char *buf, int which, const char *user_text)
   if (imp->helptext) {
     sprintf(buf+strlen(buf), "%s  ", _(imp->helptext));
   }
-  if(which==B_CATHEDRAL) {
-    helptext_cathedral_techs(buf+strlen(buf));
-  }
-  if(which==B_COLOSSEUM) {
-    int t=game.rtech.colosseum_plus;
-    if(tech_exists(t)) {
-      int n = strlen(buf);
-      if(n && buf[n-1] == '\n') buf[n-1] = ' ';
-      sprintf(buf+n, _("The discovery of %s will increase this by 1.  "),
-	     advances[t].name);
-    }
-  }
   if(which==B_BARRACKS
      && tech_exists(improvement_types[B_BARRACKS].obsolete_by)
      && tech_exists(improvement_types[B_BARRACKS2].obsolete_by)) {
@@ -611,10 +569,6 @@ void helptext_wonder(char *buf, int which,
   if (imp->helptext) {
     sprintf(buf+strlen(buf), "%s  ", _(imp->helptext));
   }
-  if(which==B_MICHELANGELO) {
-    if (improvement_variant(B_MICHELANGELO)==0)
-      helptext_cathedral_techs(buf+strlen(buf));
-  }
   if (strcmp(user_text, "")!=0) {
     sprintf(buf+strlen(buf), "\n\n%s", user_text);
   }
@@ -900,39 +854,3 @@ char *helptext_unit_upkeep_str(int i)
   }
   return buf;
 }
-
-#if 0
-/****************************************************************
-  Similar to above, with eg: "1s/1h"
-*****************************************************************/
-static char *helptext_unit_upkeep_str_short(int i)
-{
-  static char buf[32];
-  struct unit_type *utype = get_unit_type(i);
-
-  if (utype->shield_cost || utype->food_cost
-      || utype->gold_cost || utype->happy_cost) {
-    int any = 0;
-    buf[0] = '\0';
-    if (utype->shield_cost) {
-      sprintf(buf+strlen(buf), /*s=shields*/ _("%s%ds"),
-	      (any++ ? "/" : ""), utype->shield_cost);
-    }
-    if (utype->food_cost) {
-      sprintf(buf+strlen(buf), /*f=food*/ _("%s%df"),
-	      (any++ ? "/" : ""), utype->food_cost);
-    }
-    if (utype->happy_cost) {
-      sprintf(buf+strlen(buf), /*h=(un)happy*/ _("%s%dh"),
-	      (any++ ? "/" : ""), utype->happy_cost);
-    }
-    if (utype->gold_cost) {
-      sprintf(buf+strlen(buf), /*g=gold*/ _("%s%dg"),
-	      (any++ ? "/" : ""), utype->gold_cost);
-    }
-  } else {
-    sprintf(buf, "%d", 0);
-  }
-  return buf;
-}
-#endif
