@@ -31,11 +31,13 @@ void popup_revolution_dialog(void);
 void popup_government_dialog(void);
 void popup_caravan_dialog(struct unit *punit,
 			  struct city *phomecity, struct city *pdestcity);
+int caravan_dialog_is_open(void);
 void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y);
+int diplomat_dialog_is_open(void);
 void popup_incite_dialog(struct city *pcity);
 void popup_bribe_dialog(struct unit *punit);
+void popup_sabotage_dialog(struct city *pcity);
 void popup_pillage_dialog(struct unit *punit, int may_pillage);
 void popup_unit_connect_dialog (struct unit *punit, int dest_x, int dest_y);
-void process_caravan_arrival(struct unit *punit);
 
 #endif  /* FC__DIALOGS_G_H */
