@@ -13,9 +13,15 @@
 #ifndef FC__GUI_MAIN_G_H
 #define FC__GUI_MAIN_G_H
 
+struct unit;
+
 void ui_main(int argc, char *argv[]);
+void sound_bell(void);
 void enable_turn_done_button(void);
 void add_net_input(int);
 void remove_net_input(void);
 
+void set_unit_icon(int idx, struct unit *punit);
+void set_unit_icons_more_arrow(int onoff);
+
 #endif  /* FC__GUI_MAIN_G_H */
