@@ -16,9 +16,6 @@
 struct unit;
 struct city;
 
-int map_canvas_adjust_x(int x);
-int map_canvas_adjust_y(int y);
-
 int tile_visible_mapcanvas(int x, int y);
 int tile_visible_and_not_on_border_mapcanvas(int x, int y);
 
@@ -35,7 +32,7 @@ void overview_update_tile(int x, int y);
 void center_tile_mapcanvas(int x, int y);
 void get_center_tile_mapcanvas(int *x, int *y);
 
-void update_map_canvas(int tile_x, int tile_y, int width, int height,
+void update_map_canvas(int x, int y, int width, int height,
 		       int write_to_screen);
 void update_map_canvas_visible(void);
 void update_map_canvas_scrollbars(void);
@@ -46,10 +43,13 @@ void put_city_workers(struct city *pcity, int color);
 void move_unit_map_canvas(struct unit *punit, int x0, int y0, int x1, int y1);
 void decrease_unit_hp_smooth(struct unit *punit0, int hp0, 
 			     struct unit *punit1, int hp1);
-void put_nuke_mushroom_pixmaps(int abs_x0, int abs_y0);
+void put_nuke_mushroom_pixmaps(int x, int y);
 
 void refresh_overview_canvas(void);
 void refresh_overview_viewrect(void);
 void refresh_tile_mapcanvas(int x, int y, int write_to_screen);
 
+void draw_segment(int src_x, int src_y, int dir);
+void undraw_segment(int src_x, int src_y, int dir);
+
 #endif  /* FC__MAPVIEW_G_H */
