@@ -17,9 +17,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <assert.h>
 
-#include "capability.h"
 #include "capstr.h"
 #include "events.h"
 #include "fcintl.h"
@@ -44,6 +44,7 @@
 #include "clinet.h"		/* aconnection */
 #include "control.h"
 #include "dialogs_g.h"
+#include "goto.h"               /* client_goto_init() */
 #include "graphics_g.h"
 #include "gui_main_g.h"
 #include "helpdata.h"		/* boot_help_texts() */
@@ -60,11 +61,8 @@
 
 #include "packhand.h"
 
-extern int seconds_to_turndone;
-extern int last_turn_gold_amount;
-extern int did_advance_tech_this_turn;
-extern char name[512];
-
+static void handle_city_packet_common(struct city *pcity, int is_new,
+                                      int popup, int investigate);
 
 /**************************************************************************
 ...
@@ -115,10 +113,13 @@ void handle_join_game_reply(struct packet_join_game_reply *packet)
 
   if (packet->you_can_join) {
     freelog(LOG_VERBOSE, "join game accept:%s", packet->message);
+    aconnection.established = 1;
+    game.conn_id = packet->conn_id;
   } else {
     my_snprintf(msg, sizeof(msg),
 		_("You were rejected from the game: %s"), packet->message);
     append_output_window(msg);
+    game.conn_id = 0;
   }
   if (strcmp(s_capability, our_capability)==0)
     return;
@@ -135,8 +136,23 @@ void handle_join_game_reply(struct packet_join_game_reply *packet)
 **************************************************************************/
 void handle_remove_city(struct packet_generic_integer *packet)
 {
-  struct city *pcity=find_city_by_id(packet->value);
-  if(pcity!=NULL) client_remove_city(pcity);
+  struct city *pcity = find_city_by_id(packet->value);
+  int x, y;
+
+  if (pcity==NULL)
+    return;
+
+  x = pcity->x; y = pcity->y;
+  client_remove_city(pcity);
+  reset_move_costs(x, y);
+
+  /* update menus if the focus unit is on the tile. */
+  {
+    struct unit *punit = get_unit_in_focus();
+    if (punit && same_pos(punit->x, punit->y, x, y)) {
+      update_menus();
+    }
+  }
 }
 
 /**************************************************************************
@@ -189,7 +205,6 @@ void handle_unit_combat(struct packet_unit_combat *packet)
 **************************************************************************/
 void handle_game_state(struct packet_generic_integer *packet)
 {
-  
   if(get_client_state()==CLIENT_SELECT_RACE_STATE && 
      packet->value==CLIENT_GAME_RUNNING_STATE &&
      game.player_ptr->nation == MAX_NUM_NATIONS) {
@@ -199,6 +214,8 @@ void handle_game_state(struct packet_generic_integer *packet)
   set_client_state(packet->value);
 
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE) {
+    struct city *pcity;
+    struct unit *punit;
     refresh_overview_canvas();
     refresh_overview_viewrect();
     enable_turn_done_button();
@@ -208,8 +225,38 @@ void handle_game_state(struct packet_generic_integer *packet)
     update_unit_focus();
     update_unit_info_label(get_unit_in_focus());
 
-    if(get_unit_in_focus())
-      center_tile_mapcanvas(get_unit_in_focus()->x, get_unit_in_focus()->y);
+    /* Find something sensible to display instead of the intro gfx. */
+    if ((punit = get_unit_in_focus())) {
+      center_tile_mapcanvas(punit->x, punit->y);
+    } else if ((pcity = find_palace(game.player_ptr))) {
+      /* Else focus on the capital. */
+      center_tile_mapcanvas(pcity->x, pcity->y);
+    } else if (city_list_size(&game.player_ptr->cities) > 0) {
+      /* Just focus on any city. */
+      pcity = city_list_get(&game.player_ptr->cities, 0);
+      assert(pcity);
+      center_tile_mapcanvas(pcity->x, pcity->y);
+    } else if (unit_list_size(&game.player_ptr->units) > 0) {
+      /* Just focus on any unit. */
+      punit = unit_list_get(&game.player_ptr->units, 0);
+      assert(punit);
+      center_tile_mapcanvas(punit->x, punit->y);
+    } else {
+      /* Just any known tile will do; search near the middle first. */
+      iterate_outward(map.xsize/2, map.ysize/2,
+		      MAX(map.xsize/2, map.ysize/2),
+		      x, y) {
+	if (tile_is_known(x, y)) {
+	  center_tile_mapcanvas(x, y);
+	  goto OUT;
+	}
+      } iterate_outward_end;
+      /* If we get here we didn't find a known tile.
+	 Refresh a random place to clear the intro gfx. */
+      center_tile_mapcanvas(map.xsize/2, map.ysize/2);
+    OUT:
+      ; /* do nothing */
+    }
     
     free_intro_radar_sprites();
   }
@@ -222,21 +269,14 @@ void handle_city_info(struct packet_city_info *packet)
 {
   int i, x, y, city_is_new;
   struct city *pcity;
+  int popup;
+  int update_descriptions = 0;
 
   pcity=find_city_by_id(packet->id);
 
   if (pcity && (pcity->owner != packet->owner)) {
-    /* With current server, this shouldn't happen: when city changes
-     * owner it is re-created with a new id.  Unclear what to do here;
-     * try to cope...
-     */
-    freelog(LOG_NORMAL,
-	    "Got existing city id (%d %s) with wrong owner (%d %s, old %d %s)",
-	    packet->id, pcity->name, packet->owner,
-	    get_player(packet->owner)->name, pcity->owner,
-	    get_player(pcity->owner)->name);
     client_remove_city(pcity);
-    pcity = 0;
+    pcity = NULL;
   }
 
   if(!pcity) {
@@ -247,6 +287,17 @@ void handle_city_info(struct packet_city_info *packet)
   }
   else {
     city_is_new=0;
+
+    /* Check if city desciptions should be updated */
+    if (draw_city_names && strcmp(pcity->name,packet->name)) {
+      update_descriptions = 1;
+    }
+    if (draw_city_productions &&
+        ((pcity->is_building_unit != packet->is_building_unit) ||
+         (pcity->currently_building != packet->currently_building))) {
+      update_descriptions = 1;
+    }
+
     assert(pcity->id == packet->id);
   }
   
@@ -256,15 +307,17 @@ void handle_city_info(struct packet_city_info *packet)
   sz_strlcpy(pcity->name, packet->name);
   
   pcity->size=packet->size;
-  pcity->ppl_happy[4]=packet->ppl_happy;
-  pcity->ppl_content[4]=packet->ppl_content;
-  pcity->ppl_unhappy[4]=packet->ppl_unhappy;
+  for (i=0;i<5;i++) {
+    pcity->ppl_happy[i]=packet->ppl_happy[i];
+    pcity->ppl_content[i]=packet->ppl_content[i];
+    pcity->ppl_unhappy[i]=packet->ppl_unhappy[i];
+  }
   pcity->ppl_elvis=packet->ppl_elvis;
   pcity->ppl_scientist=packet->ppl_scientist;
   pcity->ppl_taxman=packet->ppl_taxman;
 
   pcity->city_options=packet->city_options;
-  
+
   for (i=0;i<4;i++)  {
     pcity->trade[i]=packet->trade[i];
     pcity->trade_value[i]=packet->trade_value[i];
@@ -284,7 +337,7 @@ void handle_city_info(struct packet_city_info *packet)
   pcity->food_stock=packet->food_stock;
   pcity->shield_stock=packet->shield_stock;
   pcity->pollution=packet->pollution;
-    
+
   pcity->is_building_unit=packet->is_building_unit;
   pcity->currently_building=packet->currently_building;
   if (city_is_new)
@@ -300,6 +353,8 @@ void handle_city_info(struct packet_city_info *packet)
   pcity->changed_from_id=packet->changed_from_id;
   pcity->changed_from_is_unit=packet->changed_from_is_unit;
   pcity->before_change_shields=packet->before_change_shields;
+  pcity->disbanded_shields=packet->disbanded_shields;
+  pcity->caravan_shields=packet->caravan_shields;
 
   i=0;
   for(y=0; y<CITY_MAP_SIZE; y++) {
@@ -312,14 +367,53 @@ void handle_city_info(struct packet_city_info *packet)
     }
   }
     
-  for(i=0; i<B_LAST; i++)
+  for(i=0; i<game.num_impr_types; i++)
     pcity->improvements[i]=(packet->improvements[i]=='1') ? 1 : 0;
-  
-  if(city_is_new) {
+
+  popup = (city_is_new && get_client_state()==CLIENT_GAME_RUNNING_STATE && 
+           pcity->owner==game.player_idx) || packet->diplomat_investigate;
+
+  handle_city_packet_common(pcity, city_is_new, popup, packet->diplomat_investigate);
+
+  /* update the descriptions if necessary */
+  if (update_descriptions && tile_visible_mapcanvas(pcity->x,pcity->y)) {
+    /* update it only every second (because of ChangeAll), this is not
+       a perfect solution at all! */
+    static int timer_initialized;
+    static time_t timer;
+    int really_update = 1;
+    time_t new_timer = time(NULL);
+
+    if (timer_initialized) {
+      double diff;
+      timer = time(NULL);
+      diff = difftime(new_timer,timer);
+      if (diff < 1.0) {
+	really_update = 0;
+      }
+    }
+
+    if (really_update) {
+      update_city_descriptions();
+      timer = new_timer;
+      timer_initialized = 1;
+    }
+  }
+}
+
+/**************************************************************************
+  ...
+**************************************************************************/
+static void handle_city_packet_common(struct city *pcity, int is_new,
+                                      int popup, int investigate)
+{
+  int i;
+
+  if(is_new) {
     unit_list_init(&pcity->units_supported);
     unit_list_init(&pcity->info_units_supported);
     unit_list_init(&pcity->info_units_present);
-    city_list_insert(&game.players[packet->owner].cities, pcity);
+    city_list_insert(&game.players[pcity->owner].cities, pcity);
     map_set_city(pcity->x, pcity->y, pcity);
     if(pcity->owner==game.player_idx)
       city_report_dialog_update();
@@ -333,7 +427,7 @@ void handle_city_info(struct packet_city_info *packet)
   }
 
   if(draw_map_grid &&
-     city_is_new && get_client_state()==CLIENT_GAME_RUNNING_STATE) {
+     is_new && get_client_state()==CLIENT_GAME_RUNNING_STATE) {
     /* just to update grid; slow, but doesn't happen very often --jjm */
     int r=((CITY_MAP_SIZE+1)/2);
     int d=(2*r)+1;
@@ -341,7 +435,7 @@ void handle_city_info(struct packet_city_info *packet)
     int y=map_adjust_y(pcity->y - r);
     update_map_canvas
     (
-     map_canvas_adjust_x(x), map_canvas_adjust_y(y),
+     x, y,
      d, d,
      1
     );
@@ -349,31 +443,143 @@ void handle_city_info(struct packet_city_info *packet)
     refresh_tile_mapcanvas(pcity->x, pcity->y, 1);
   }
 
-  if(city_workers_display==pcity)  {
+  if (city_workers_display==pcity)  {
     put_city_workers(pcity, -1);
     city_workers_display=NULL;
   }
 
-  if(((city_is_new && get_client_state()==CLIENT_GAME_RUNNING_STATE && 
-      pcity->owner==game.player_idx) || packet->diplomat_investigate) &&
-     (!game.player_ptr->ai.control || ai_popup_windows)) {
+  if (popup &&
+      (!game.player_ptr->ai.control || ai_popup_windows)) {
     update_menus();
-    popup_city_dialog(pcity, 0);
+    if (!city_dialog_is_open(pcity)) {
+      popup_city_dialog(pcity, 0);
+    }
   }
 
-  if(!city_is_new && pcity->owner==game.player_idx) {
-    struct unit *punit = get_unit_in_focus();
-    if (punit && (punit->x == pcity->x) && (punit->y == pcity->y)) {
-      update_menus();
-    }
+  if (!is_new && (pcity->owner==game.player_idx
+		  || popup)) {
     refresh_city_dialog(pcity);
   }
 
-  if(city_is_new) {
+  /* update menus if the focus unit is on the tile. */
+  {
+    struct unit *punit = get_unit_in_focus();
+    if (punit && same_pos(punit->x, punit->y, pcity->x, pcity->y)) {
+      update_menus();
+    }
+  }
+
+  if(is_new) {
     freelog(LOG_DEBUG, "New %s city %s id %d (%d %d)",
 	 get_nation_name(city_owner(pcity)->nation),
 	 pcity->name, pcity->id, pcity->x, pcity->y);
   }
+
+  reset_move_costs(pcity->x, pcity->y);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void handle_short_city(struct packet_short_city *packet)
+{
+  struct city *pcity;
+  int city_is_new;
+  int update_descriptions = 0;
+
+  pcity=find_city_by_id(packet->id);
+
+  if (pcity && (pcity->owner != packet->owner)) {
+    client_remove_city(pcity);
+    pcity = 0;
+  }
+
+  if(!pcity) {
+    city_is_new=1;
+    pcity=fc_malloc(sizeof(struct city));
+    pcity->id=packet->id;
+    idex_register_city(pcity);
+  }
+  else {
+    city_is_new=0;
+
+    /* Check if city desciptions should be updated */
+    if (draw_city_names && strcmp(pcity->name,packet->name)) {
+      update_descriptions = 1;
+    }
+    
+    assert(pcity->id == packet->id);
+  }
+
+  pcity->owner=packet->owner;
+  pcity->x=packet->x;
+  pcity->y=packet->y;
+  sz_strlcpy(pcity->name, packet->name);
+  
+  pcity->size=packet->size;
+
+  if (packet->happy) {
+    pcity->ppl_happy[4]   = pcity->size;
+    pcity->ppl_unhappy[4] = 0;
+  } else {
+    pcity->ppl_unhappy[4] = pcity->size;
+    pcity->ppl_happy[4]   = 0;
+  }
+
+  pcity->improvements[B_PALACE] = packet->capital;
+  pcity->improvements[B_CITY]   = packet->walls;
+
+  if (city_is_new)
+    pcity->worklist = create_worklist();
+
+  /* This sets dumb values for everything else. This is not really required,
+     but just want to be at the safe side. */
+  {
+    int i;
+    int x, y;
+
+    pcity->ppl_elvis          = pcity->size;
+    pcity->ppl_scientist      = 0;
+    pcity->ppl_taxman         = 0;
+    for (i=0;i<4;i++)  {
+      pcity->trade[i]=0;
+      pcity->trade_value[i]     = 0;
+    }
+    pcity->food_prod          = 0;
+    pcity->food_surplus       = 0;
+    pcity->shield_prod        = 0;
+    pcity->shield_surplus     = 0;
+    pcity->trade_prod         = 0;
+    pcity->corruption         = 0;
+    pcity->luxury_total       = 0;
+    pcity->tax_total          = 0;
+    pcity->science_total      = 0;
+    pcity->food_stock         = 0;
+    pcity->shield_stock       = 0;
+    pcity->pollution          = 0;
+    pcity->city_options       = 0;
+    pcity->is_building_unit    = 0;
+    pcity->currently_building = 0;
+    init_worklist(pcity->worklist);
+    pcity->airlift            = 0;
+    pcity->did_buy            = 0;
+    pcity->did_sell           = 0;
+    pcity->was_happy          = 0;
+    for(i=0; i<game.num_impr_types; i++) {
+      if (i != B_PALACE && i != B_CITY)
+        pcity->improvements[i] = 0;
+    }
+    for(y=0; y<CITY_MAP_SIZE; y++)
+      for(x=0; x<CITY_MAP_SIZE; x++)
+	pcity->city_map[x][y] = C_TILE_EMPTY;
+  } /* Dumb values */
+
+  handle_city_packet_common(pcity, city_is_new, 0, 0);
+
+  /* update the descriptions if necessary */
+  if (update_descriptions && tile_visible_mapcanvas(pcity->x,pcity->y)) {
+    update_city_descriptions();
+  }
 }
 
 /**************************************************************************
@@ -398,6 +604,9 @@ void handle_new_year(struct packet_new_year *ppacket)
   turn_gold_difference=game.player_ptr->economic.gold-last_turn_gold_amount;
   last_turn_gold_amount=game.player_ptr->economic.gold;
 
+  plrdlg_update_delay_off();
+  update_players_dialog();
+
   report_update_delay_off();
   update_report_dialogs();
 
@@ -420,6 +629,7 @@ void handle_new_year(struct packet_new_year *ppacket)
 void handle_before_new_year(void)
 {
   clear_notify_window();
+  plrdlg_update_delay_on();
   report_update_delay_on();
   meswin_update_delay_on();
 }
@@ -432,7 +642,8 @@ void handle_chat_msg(struct packet_generic_message *packet)
   int where = MW_OUTPUT;	/* where to display the message */
   
   if (packet->event >= E_LAST)  {
-    freelog(LOG_NORMAL,"Unknown event type %d!", packet->event);
+    /* Server may have added a new event; leave as MW_OUTPUT */
+    freelog(LOG_NORMAL, "Unknown event type %d!", packet->event);
   } else if (packet->event >= 0)  {
     where = messages_where[packet->event];
   }
@@ -442,7 +653,7 @@ void handle_chat_msg(struct packet_generic_message *packet)
     add_notify_window(packet);
   if ((where & MW_POPUP) &&
       (!game.player_ptr->ai.control || ai_popup_windows))
-    popup_notify_goto_dialog("Popup Request", packet->message, 
+    popup_notify_goto_dialog(_("Popup Request"), packet->message, 
 			     packet->x, packet->y);
 }
  
@@ -728,6 +939,7 @@ void handle_map_info(struct packet_map_info *pinfo)
 
   map_allocate();
   climap_init_continents();
+  init_client_goto();
   
   set_overview_dimensions(map.xsize, map.ysize);
 }
@@ -740,7 +952,7 @@ void handle_game_info(struct packet_game_info *pinfo)
   int i, boot_help;
   game.gold=pinfo->gold;
   game.tech=pinfo->tech;
-  game.techlevel=pinfo->techlevel;
+  game.researchcost=pinfo->researchcost;
   game.skill_level=pinfo->skill_level;
   game.timeout=pinfo->timeout;
 
@@ -754,12 +966,12 @@ void handle_game_info(struct packet_game_info *pinfo)
   game.nuclearwinter=pinfo->nuclearwinter;
   game.cooling=pinfo->cooling;
   if(get_client_state()!=CLIENT_GAME_RUNNING_STATE) {
-    game.player_idx=pinfo->player_idx;
-    game.player_ptr=&game.players[game.player_idx];
+    game.player_idx = pinfo->player_idx;
+    game.player_ptr = &game.players[game.player_idx];
   }
   for(i=0; i<A_LAST/*game.num_tech_types*/; i++)
     game.global_advances[i]=pinfo->global_advances[i];
-  for(i=0; i<B_LAST; i++)
+  for(i=0; i<B_LAST/*game.num_impr_types*/; i++)
     game.global_wonders[i]=pinfo->global_wonders[i];
   
   if(get_client_state()!=CLIENT_GAME_RUNNING_STATE) {
@@ -769,29 +981,8 @@ void handle_game_info(struct packet_game_info *pinfo)
   game.techpenalty=pinfo->techpenalty;
   game.foodbox=pinfo->foodbox;
   game.civstyle=pinfo->civstyle;
-/* when removing "game_ruleset" capability,
-remove this *entire* piece of code (to REMOVE TO HERE, below) */
-if (!has_capability("game_ruleset", aconnection.capability)) {
-if (game.civstyle == 1) {
-game.rgame.min_city_center_food = 0;
-game.rgame.min_city_center_shield = 0;
-game.rgame.min_city_center_trade = 0;
-game.rgame.min_dist_bw_cities = 1;
-game.rgame.init_vis_radius_sq = 2;
-game.rgame.hut_overflight = OVERFLIGHT_NOTHING;
-game.rgame.pillage_select = 0;
-} else {
-game.rgame.min_city_center_food = 1;
-game.rgame.min_city_center_shield = 1;
-game.rgame.min_city_center_trade = 0;
-game.rgame.min_dist_bw_cities = 2;
-game.rgame.init_vis_radius_sq = 5;
-game.rgame.hut_overflight = OVERFLIGHT_FRIGHTEN;
-game.rgame.pillage_select = 1;
-}
-game.rgame.nuke_contamination = CONTAMINATION_POLLUTION;
-}
-/* REMOVE TO HERE */
+  game.unhappysize = pinfo->unhappysize;
+  game.cityfactor = pinfo->cityfactor;
 
   boot_help = (get_client_state() == CLIENT_GAME_RUNNING_STATE
 	       && game.spacerace != pinfo->spacerace);
@@ -816,7 +1007,7 @@ void handle_player_info(struct packet_player_info *pinfo)
   int i;
   int poptechup;
   char msg[MAX_LEN_MSG];
-  struct player *pplayer=&game.players[pinfo->playerno];
+  struct player *pplayer = &game.players[pinfo->playerno];
 
   sz_strlcpy(pplayer->name, pinfo->name);
   pplayer->nation=pinfo->nation;
@@ -828,6 +1019,7 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->economic.luxury=pinfo->luxury;
   pplayer->government=pinfo->government;
   pplayer->embassy=pinfo->embassy;
+  pplayer->gives_shared_vision = pinfo->gives_shared_vision;
   pplayer->city_style=pinfo->city_style;
 
   for (i = 0; i < MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS; i++) {
@@ -853,25 +1045,19 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->research.researching=pinfo->researching;
   pplayer->future_tech=pinfo->future_tech;
   pplayer->ai.tech_goal=pinfo->tech_goal;
-
-  if(!pplayer->conn){
-    /* It is only the client that does this */
-
-    pplayer->conn = fc_malloc(sizeof(struct connection));
-    pplayer->conn->sock = 0;
-    pplayer->conn->used = 0;
-    pplayer->conn->player = NULL;
-  }
-
-  sz_strlcpy(pplayer->conn->capability, pinfo->capability);
   
   if(get_client_state()==CLIENT_GAME_RUNNING_STATE && pplayer==game.player_ptr) {
-    sz_strlcpy(name, pplayer->name);
     if(poptechup) {
       if(!game.player_ptr->ai.control || ai_popup_windows)
 	popup_science_dialog(1);
       did_advance_tech_this_turn=game.year;
       science_dialog_update();
+
+      /* If we just learned bridge building and focus is on a settler
+	 on a river the road menu item will remain disabled unless we
+	 do this. (applys in other cases as well.) */
+      if (get_unit_in_focus())
+	update_menus();
     } 
   }
 
@@ -880,7 +1066,6 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->is_alive=pinfo->is_alive;
   
   pplayer->is_connected=pinfo->is_connected;
-  sz_strlcpy(pplayer->addr, pinfo->addr);
 
   pplayer->ai.is_barbarian=pinfo->is_barbarian;
   pplayer->revolution=pinfo->revolution;
@@ -915,6 +1100,72 @@ void handle_player_info(struct packet_player_info *pinfo)
   }
 }
 
+/**************************************************************************
+  Remove, add, or update dummy connection struct representing some
+  connection to the server, with info from packet_conn_info.
+  Updates player and game connection lists.
+  Calls update_players_dialog() in case info for that has changed.
+**************************************************************************/
+void handle_conn_info(struct packet_conn_info *pinfo)
+{
+  struct connection *pconn = find_conn_by_id(pinfo->id);
+  
+  freelog(LOG_DEBUG, "conn_info id%d used%d est%d plr%d obs%d acc%d",
+	  pinfo->id, pinfo->used, pinfo->established, pinfo->player_num,
+	  pinfo->observer, (int)pinfo->access_level);
+  freelog(LOG_DEBUG, "conn_info \"%s\" \"%s\" \"%s\"",
+	  pinfo->name, pinfo->addr, pinfo->capability);
+  
+  if (pinfo->used==0) {
+    /* Forget the connection */
+    if (!pconn) {
+      freelog(LOG_VERBOSE, "Server removed unknown connection %d", pinfo->id);
+      return;
+    }
+    client_remove_cli_conn(pconn);
+  }
+  else {
+    /* Add or update the connection */
+    struct player *pplayer =
+      ((pinfo->player_num >= 0 && pinfo->player_num < game.nplayers)
+       ? get_player(pinfo->player_num) : NULL);
+    
+    if (!pconn) {
+      freelog(LOG_VERBOSE, "Server reports new connection %d %s",
+	      pinfo->id, pinfo->name);
+      pconn = fc_calloc(1, sizeof(struct connection));
+      pconn->buffer = NULL;
+      pconn->send_buffer = NULL;
+      if (pplayer) {
+	conn_list_insert_back(&pplayer->connections, pconn);
+      }
+      conn_list_insert_back(&game.all_connections, pconn);
+      conn_list_insert_back(&game.est_connections, pconn);
+      conn_list_insert_back(&game.game_connections, pconn);
+    } else {
+      freelog(LOG_DEBUG, "Server reports updated connection %d %s",
+	      pinfo->id, pinfo->name);
+      if (pplayer != pconn->player) {
+	if (pconn->player) {
+	  conn_list_unlink(&pconn->player->connections, pconn);
+	}
+	if (pplayer) {
+	  conn_list_insert_back(&pplayer->connections, pconn);
+	}
+      }
+    }
+    pconn->id = pinfo->id;
+    pconn->established = pinfo->established;
+    pconn->observer = pinfo->observer;
+    pconn->access_level = pinfo->access_level;
+    pconn->player = pplayer;
+    sz_strlcpy(pconn->name, pinfo->name);
+    sz_strlcpy(pconn->addr, pinfo->addr);
+    sz_strlcpy(pconn->capability, pinfo->capability);
+  }
+  update_players_dialog();
+}
+
 /**************************************************************************
 Ideally the client should let the player choose which type of
 modules and components to build, and (possibly) where to extend
@@ -1067,7 +1318,7 @@ static int spaceship_autoplace(struct player *pplayer,
 void handle_spaceship_info(struct packet_spaceship_info *p)
 {
   int i;
-  struct player *pplayer=&game.players[p->player_num];
+  struct player *pplayer = &game.players[p->player_num];
   struct player_spaceship *ship = &pplayer->spaceship;
   
   ship->state        = p->sship_state;
@@ -1123,6 +1374,8 @@ void handle_tile_info(struct packet_tile_info *packet)
   }
   ptile->known = packet->known;
 
+  reset_move_costs(packet->x, packet->y);
+
   /* fog of war remove units */
   if (ptile->known <= TILE_KNOWN_FOGGED && old_known == TILE_KNOWN) {
     unit_list_iterate(ptile->units, punit) {
@@ -1192,7 +1445,15 @@ void handle_tile_info(struct packet_tile_info *packet)
 	refresh_tile_mapcanvas(x, y-1, 1);
       if(tile_is_known(x, y+1)>=TILE_KNOWN_FOGGED)
 	refresh_tile_mapcanvas(x, y+1, 1);
-    } 
+    }
+  }
+
+  /* update menus if the focus unit is on the tile. */
+  if (tile_changed) {
+    struct unit *punit = get_unit_in_focus();
+    if (punit && same_pos(punit->x, punit->y, packet->x, packet->y)) {
+      update_menus();
+    }
   }
 }
 
@@ -1223,7 +1484,7 @@ void handle_select_nation(struct packet_generic_values *packet)
     races_toggles_set_sensitive( packet->value1, packet->value2);
   }
   else
-    freelog(LOG_VERBOSE, "got a select nation packet in an incompatible state");
+    freelog(LOG_ERROR, "got a select nation packet in an incompatible state");
 }
 
 /**************************************************************************
@@ -1261,6 +1522,7 @@ void handle_ruleset_control(struct packet_ruleset_control *packet)
   game.default_government = packet->default_government;
 
   game.num_unit_types = packet->num_unit_types;
+  game.num_impr_types = packet->num_impr_types;
   game.num_tech_types = packet->num_tech_types;
 
   governments = fc_calloc(game.government_count, sizeof(struct government));
@@ -1289,7 +1551,7 @@ void handle_ruleset_unit(struct packet_ruleset_unit *p)
   struct unit_type *u;
 
   if(p->id < 0 || p->id >= game.num_unit_types || p->id >= U_LAST) {
-    freelog(LOG_NORMAL, "Received bad unit_type id %d in handle_ruleset_unit()",
+    freelog(LOG_ERROR, "Received bad unit_type id %d in handle_ruleset_unit()",
 	    p->id);
     return;
   }
@@ -1334,7 +1596,7 @@ void handle_ruleset_tech(struct packet_ruleset_tech *p)
   struct advance *a;
 
   if(p->id < 0 || p->id >= game.num_tech_types || p->id >= A_LAST) {
-    freelog(LOG_NORMAL, "Received bad advance id %d in handle_ruleset_tech()",
+    freelog(LOG_ERROR, "Received bad advance id %d in handle_ruleset_tech()",
 	    p->id);
     return;
   }
@@ -1356,8 +1618,8 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
 {
   struct impr_type *b;
 
-  if(p->id < 0 || p->id >= B_LAST) {
-    freelog(LOG_NORMAL,
+  if(p->id < 0 || p->id >= game.num_impr_types || p->id >= B_LAST) {
+    freelog(LOG_ERROR,
 	    "Received bad building id %d in handle_ruleset_building()",
 	    p->id);
     return;
@@ -1388,9 +1650,9 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
   b->helptext = p->helptext;		/* pointer assignment */
 
 #ifdef DEBUG
-  if(p->id == B_LAST-1) {
+  if(p->id == game.num_impr_types-1) {
     int id;
-    for (id = 0; id < B_LAST; id++) {
+    for (id = 0; id < game.num_impr_types; id++) {
       int inx;
       b = &improvement_types[id];
       freelog(LOG_DEBUG, "Impr: %s...",
@@ -1448,6 +1710,9 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
 	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " amount=%d",
 		    b->effect[inx].amount);
 	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " survives=%d",
+		    b->effect[inx].survives);
+	ptr = strchr(ptr, '\0');
 	freelog(LOG_DEBUG, "   %2d. %s", inx, buf);
 	ptr = buf;
 	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " cond_bldg=%d/%s",
@@ -1517,7 +1782,7 @@ void handle_ruleset_government(struct packet_ruleset_government *p)
   struct government *gov;
 
   if (p->id < 0 || p->id >= game.government_count) {
-    freelog(LOG_NORMAL,
+    freelog(LOG_ERROR,
 	    "Received bad government id %d in handle_ruleset_government",
 	    p->id);
     return;
@@ -1588,12 +1853,12 @@ void handle_ruleset_government_ruler_title
   struct government *gov;
 
   if(p->gov < 0 || p->gov >= game.government_count) {
-    freelog(LOG_NORMAL, "Received bad government num %d for title", p->gov);
+    freelog(LOG_ERROR, "Received bad government num %d for title", p->gov);
     return;
   }
   gov = &governments[p->gov];
   if(p->id < 0 || p->id >= gov->num_ruler_titles) {
-    freelog(LOG_NORMAL, "Received bad ruler title num %d for %s title",
+    freelog(LOG_ERROR, "Received bad ruler title num %d for %s title",
 	    p->id, gov->name);
     return;
   }
@@ -1611,7 +1876,7 @@ void handle_ruleset_terrain(struct packet_ruleset_terrain *p)
   int j;
 
   if (p->id < T_FIRST || p->id >= T_COUNT) {
-    freelog(LOG_NORMAL,
+    freelog(LOG_ERROR,
 	    "Received bad terrain id %d in handle_ruleset_terrain",
 	    p->id);
     return;
@@ -1697,7 +1962,7 @@ void handle_ruleset_nation(struct packet_ruleset_nation *p)
   struct nation_type *pl;
 
   if(p->id < 0 || p->id >= game.nation_count || p->id >= MAX_NUM_NATIONS) {
-    freelog(LOG_NORMAL, "Received bad nation id %d in handle_ruleset_nation()",
+    freelog(LOG_ERROR, "Received bad nation id %d in handle_ruleset_nation()",
 	    p->id);
     return;
   }
@@ -1727,7 +1992,7 @@ void handle_ruleset_city(struct packet_ruleset_city *packet)
 
   id = packet->style_id;
   if (id < 0 || id >= game.styles_count) {
-    freelog(LOG_NORMAL, "Received bad citystyle id %d in handle_ruleset_city()",
+    freelog(LOG_ERROR, "Received bad citystyle id %d in handle_ruleset_city()",
 	    id);
     return;
   }
@@ -1756,6 +2021,8 @@ void handle_ruleset_game(struct packet_ruleset_game *packet)
   game.rgame.hut_overflight = packet->hut_overflight;
   game.rgame.pillage_select = packet->pillage_select;
   game.rgame.nuke_contamination = packet->nuke_contamination;
+  game.rgame.granary_food_ini = packet->granary_food_ini;
+  game.rgame.granary_food_inc = packet->granary_food_inc;
 }
 
 /**************************************************************************
@@ -1814,7 +2081,7 @@ void handle_diplomat_action(struct packet_diplomat_action *packet)
   struct unit *pdiplomat=player_find_unit_by_id(game.player_ptr, packet->diplomat_id);
 
   if (!pdiplomat) {
-    freelog(LOG_NORMAL, "Received bad diplomat id %d in handle_diplomat_action()",
+    freelog(LOG_ERROR, "Received bad diplomat id %d in handle_diplomat_action()",
 	    packet->diplomat_id);
     return;
   }
@@ -1824,7 +2091,7 @@ void handle_diplomat_action(struct packet_diplomat_action *packet)
     process_diplomat_arrival(pdiplomat, packet->target_id);
     break;
   default:
-    freelog(LOG_NORMAL, "Received bad action %d in handle_diplomat_action()",
+    freelog(LOG_ERROR, "Received bad action %d in handle_diplomat_action()",
 	    packet->action_type);
     break;
   }
@@ -1840,7 +2107,7 @@ void handle_sabotage_list(struct packet_sabotage_list *packet)
 
   if (punit && pcity) {
     int i;
-    for(i=0; i<B_LAST; i++)
+    for(i=0; i<game.num_impr_types; i++)
       pcity->improvements[i] = (packet->improvements[i]=='1') ? 1 : 0;
 
     popup_sabotage_dialog(pcity);
