@@ -17,6 +17,15 @@
 
 void handle_join_game_reply(struct packet_join_game_reply *packet);
 
+void handle_tile_info(struct packet_tile_info *packet);
+void handle_player_info(struct packet_player_info *packet);
+void handle_conn_info(struct packet_conn_info *packet);
+void handle_game_info(struct packet_game_info *packet);
+void handle_map_info(struct packet_map_info *pinfo);
+void handle_select_nation(struct packet_generic_values *packet);
+void handle_unit_info(struct packet_unit_info *packet);
+void handle_chat_msg(struct packet_generic_message *packet);
+
 void handle_remove_city(struct packet_generic_integer *packet);
 void handle_remove_unit(struct packet_generic_integer *packet);
 void handle_incite_cost(struct packet_generic_values *packet);
@@ -28,6 +37,7 @@ void handle_spaceship_info(struct packet_spaceship_info *packet);
 void handle_move_unit(struct packet_move_unit *packet);
 void handle_new_year(struct packet_new_year *ppacket);
 void handle_city_info(struct packet_city_info *packet);
+void handle_short_city(struct packet_short_city *packet);
 void handle_unit_combat(struct packet_unit_combat *packet);
 void handle_game_state(struct packet_generic_integer *packet);
 void handle_nuke_tile(struct packet_nuke_tile *packet);
