@@ -55,9 +55,13 @@ struct named_sprites sprites;
 
 int NORMAL_TILE_WIDTH;
 int NORMAL_TILE_HEIGHT;
+int UNIT_TILE_WIDTH;
+int UNIT_TILE_HEIGHT;
 int SMALL_TILE_WIDTH;
 int SMALL_TILE_HEIGHT;
 
+int is_isometric;
+
 char *city_names_font;
 char *city_productions_font_name;
 
@@ -115,10 +119,16 @@ static int no_backdrop = 0;
 ***********************************************************************/
 static char *tilespec_fullname(const char *tileset_name)
 {
-  char *tileset_default = "trident";    /* Do not i18n! --dwp */
+  char *tileset_default;
   char *fname, *dname;
   int level;
 
+  if (isometric_view_supported()) {
+    tileset_default = "hires";    /* Do not i18n! --dwp */
+  } else {
+    tileset_default = "trident";    /* Do not i18n! --dwp */
+  }
+
   if (!tileset_name) {
     tileset_name = tileset_default;
   }
@@ -134,7 +144,7 @@ static char *tilespec_fullname(const char *tileset_name)
   if (strcmp(tileset_name, tileset_default)==0) {
     level = LOG_FATAL;
   } else {
-    level = LOG_NORMAL;
+    level = LOG_ERROR;
   }
   freelog(level, _("Could not find readable file \"%s\" in data path."),
 	  fname);
@@ -226,12 +236,40 @@ void tilespec_read_toplevel(const char *tileset_name)
 
   section_file_lookup(file, "tilespec.name"); /* currently unused */
 
+  is_isometric = secfile_lookup_int_default(file, 0, "tilespec.is_isometric");
+  if (is_isometric && !isometric_view_supported()) {
+    freelog(LOG_ERROR, _("Client does not support isometric tilesets."
+	    " Using default tileset instead."));
+    assert(tileset_name);
+    section_file_free(file);
+    free(fname);
+    tilespec_read_toplevel(NULL);
+    return;
+  }
+  if (!is_isometric && !overhead_view_supported()) {
+    freelog(LOG_ERROR, _("Client does not support overhead view tilesets."
+	    " Using default tileset instead."));
+    assert(tileset_name);
+    section_file_free(file);
+    free(fname);
+    tilespec_read_toplevel(NULL);
+    return;
+  }
+
   NORMAL_TILE_WIDTH = secfile_lookup_int(file, "tilespec.normal_tile_width");
   NORMAL_TILE_HEIGHT = secfile_lookup_int(file, "tilespec.normal_tile_height");
+  if (is_isometric) {
+    UNIT_TILE_WIDTH = NORMAL_TILE_WIDTH;
+    UNIT_TILE_HEIGHT = 3 * NORMAL_TILE_HEIGHT/2;
+  } else {
+    UNIT_TILE_WIDTH = NORMAL_TILE_WIDTH;
+    UNIT_TILE_HEIGHT = NORMAL_TILE_HEIGHT;
+  }
   SMALL_TILE_WIDTH = secfile_lookup_int(file, "tilespec.small_tile_width");
   SMALL_TILE_HEIGHT = secfile_lookup_int(file, "tilespec.small_tile_height");
-  freelog(LOG_VERBOSE, "tile sizes %dx%d, %dx%d small",
+  freelog(LOG_VERBOSE, "tile sizes %dx%d, %d%d unit, %d%d small",
 	  NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+	  UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT,
 	  SMALL_TILE_WIDTH, SMALL_TILE_HEIGHT);
 
   c = secfile_lookup_str_default(file, "10x20", "tilespec.city_names_font");
@@ -323,6 +361,7 @@ static void tilespec_load_one(const char *spec_filename)
     exit(1);
   }
 
+
   gridnames = secfile_get_secnames_prefix(file, "grid_", &num_grids);
   if (num_grids==0) {
     freelog(LOG_FATAL, "spec %s has no grid_* sections", spec_filename);
@@ -330,6 +369,8 @@ static void tilespec_load_one(const char *spec_filename)
   }
 
   for(i=0; i<num_grids; i++) {
+    int is_pixel_border =
+      secfile_lookup_int_default(file, 0, "%s.is_pixel_border", gridnames[i]);
     x_top_left = secfile_lookup_int(file, "%s.x_top_left", gridnames[i]);
     y_top_left = secfile_lookup_int(file, "%s.y_top_left", gridnames[i]);
     dx = secfile_lookup_int(file, "%s.dx", gridnames[i]);
@@ -341,12 +382,12 @@ static void tilespec_load_one(const char *spec_filename)
       column = secfile_lookup_int(file, "%s.tiles%d.column", gridnames[i], j);
       tags = secfile_lookup_str_vec(file, &num_tags, "%s.tiles%d.tag",
 				    gridnames[i], j);
-      
+
       /* there must be at least 1 because of the while(): */
       assert(num_tags>0);
 
-      x1 = x_top_left + column * dx;
-      y1 = y_top_left + row * dy;
+      x1 = x_top_left + column * dx + (is_pixel_border ? column : 0);
+      y1 = y_top_left + row * dy + (is_pixel_border ? row : 0);
       small_sprite = crop_sprite(big_sprite, x1, y1, dx, dy);
 
       for(k=0; k<num_tags; k++) {
@@ -373,7 +414,7 @@ static void tilespec_load_one(const char *spec_filename)
 ***********************************************************************/
 static char *nsew_str(int idx)
 {
-  static char c[] = "n0s0e0w0";
+  static char c[9]; /* strlen("n0s0e0w0") + 1 == 9 */
 
   sprintf(c, "n%ds%de%dw%d", BOOL_VAL(idx&BIT_NORTH), BOOL_VAL(idx&BIT_SOUTH),
      	                     BOOL_VAL(idx&BIT_EAST),  BOOL_VAL(idx&BIT_WEST));
@@ -391,7 +432,7 @@ static char *nsew_str(int idx)
 static void tilespec_lookup_sprite_tags(void)
 {
   char buffer[512];
-  char dir_char[] = "nsew";
+  const char dir_char[] = "nsew";
   int i, j;
   
   assert(sprite_hash);
@@ -409,6 +450,11 @@ static void tilespec_lookup_sprite_tags(void)
   }
 
   SET_SPRITE(right_arrow, "s.right_arrow");
+  if (is_isometric) {
+    SET_SPRITE(black_tile, "t.black_tile");
+    SET_SPRITE(dither_tile, "t.dither_tile");
+    SET_SPRITE(coast_color, "t.coast_color");
+  }
 
   /* This uses internal code for citizens array/index: */
   SET_SPRITE(citizen[0], "citizen.entertainer");
@@ -431,24 +477,37 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(road.isolated, "r.road_isolated");
   SET_SPRITE(rail.isolated, "r.rail_isolated");
   
-  for(i=1; i<NUM_DIRECTION_NSEW; i++) {
-    my_snprintf(buffer, sizeof(buffer), "r.c_road_%s", nsew_str(i));
-    SET_SPRITE(road.cardinal[i], buffer);
-    
-    my_snprintf(buffer, sizeof(buffer), "r.d_road_%s", nsew_str(i));
-    SET_SPRITE(road.diagonal[i], buffer);
-    
-    my_snprintf(buffer, sizeof(buffer), "r.c_rail_%s", nsew_str(i));
-    SET_SPRITE(rail.cardinal[i], buffer);
-    
-    my_snprintf(buffer, sizeof(buffer), "r.d_rail_%s", nsew_str(i));
-    SET_SPRITE(rail.diagonal[i], buffer);
+  if (is_isometric) {
+    for (i=0; i<8; i++) {
+      my_snprintf(buffer, sizeof(buffer), "r.road%d", i);
+      SET_SPRITE(road.dir[i], buffer);
+      my_snprintf(buffer, sizeof(buffer), "r.rail%d", i);
+      SET_SPRITE(rail.dir[i], buffer);
+    }
+  } else {
+    for(i=1; i<NUM_DIRECTION_NSEW; i++) {
+      my_snprintf(buffer, sizeof(buffer), "r.c_road_%s", nsew_str(i));
+      SET_SPRITE(road.cardinal[i], buffer);
+
+      my_snprintf(buffer, sizeof(buffer), "r.d_road_%s", nsew_str(i));
+      SET_SPRITE(road.diagonal[i], buffer);
+
+      my_snprintf(buffer, sizeof(buffer), "r.c_rail_%s", nsew_str(i));
+      SET_SPRITE(rail.cardinal[i], buffer);
+
+      my_snprintf(buffer, sizeof(buffer), "r.d_rail_%s", nsew_str(i));
+      SET_SPRITE(rail.diagonal[i], buffer);
+    }
   }
 
-  for(i=0; i<3; i++) {
-    for(j=0; j<3; j++) {
-      my_snprintf(buffer, sizeof(buffer), "explode.nuke_%d%d", i, j);
-      SET_SPRITE(explode.nuke[i][j], buffer);
+  if (is_isometric) {
+    SET_SPRITE(explode.iso_nuke, "explode.iso_nuke");
+  } else {
+    for(i=0; i<3; i++) {
+      for(j=0; j<3; j++) {
+	my_snprintf(buffer, sizeof(buffer), "explode.nuke_%d%d", i, j);
+	SET_SPRITE(explode.nuke[i][j], buffer);
+      }
     }
   }
 
@@ -487,8 +546,9 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(unit.road,	        "unit.road");
   SET_SPRITE(unit.sentry,	"unit.sentry");      
   SET_SPRITE(unit.stack,	"unit.stack");
-  SET_SPRITE(unit.transform,    "unit.transform");   
-  SET_SPRITE(unit.connect,      "unit.connect");   
+  SET_SPRITE(unit.transform,    "unit.transform");
+  SET_SPRITE(unit.connect,      "unit.connect");
+  SET_SPRITE(unit.patrol,       "unit.patrol");
 
   for(i=0; i<NUM_TILES_HP_BAR; i++) {
     my_snprintf(buffer, sizeof(buffer), "unit.hp_%d", i*10);
@@ -524,7 +584,9 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(tx.farmland,   "tx.farmland");
   SET_SPRITE(tx.irrigation, "tx.irrigation");
   SET_SPRITE(tx.mine,       "tx.mine");
-  SET_SPRITE(tx.oil_mine,   "tx.oil_mine");
+  if (!is_isometric) {
+    SET_SPRITE(tx.oil_mine,   "tx.oil_mine");
+  }
   SET_SPRITE(tx.pollution,  "tx.pollution");
   SET_SPRITE(tx.village,    "tx.village");
   SET_SPRITE(tx.fortress,   "tx.fortress");
@@ -535,24 +597,56 @@ static void tilespec_lookup_sprite_tags(void)
     my_snprintf(buffer, sizeof(buffer), "tx.s_river_%s", nsew_str(i));
     SET_SPRITE(tx.spec_river[i], buffer);
   }
-  for(i=1; i<NUM_DIRECTION_NSEW; i++) {
-    my_snprintf(buffer, sizeof(buffer), "tx.darkness_%s", nsew_str(i));
-    SET_SPRITE(tx.darkness[i], buffer);
-    my_snprintf(buffer, sizeof(buffer), "tx.coast_cape_%s", nsew_str(i));
-    SET_SPRITE(tx.coast_cape[i], buffer);
+
+  if (is_isometric) {
+    for(i=0; i<NUM_DIRECTION_NSEW; i++) {
+      my_snprintf(buffer, sizeof(buffer), "tx.s_forest_%s", nsew_str(i));
+      SET_SPRITE(tx.spec_forest[i], buffer);
+    }
+
+    for(i=0; i<NUM_DIRECTION_NSEW; i++) {
+      my_snprintf(buffer, sizeof(buffer), "tx.s_mountain_%s", nsew_str(i));
+      SET_SPRITE(tx.spec_mountain[i], buffer);
+    }
+
+    for(i=0; i<NUM_DIRECTION_NSEW; i++) {
+      my_snprintf(buffer, sizeof(buffer), "tx.s_hill_%s", nsew_str(i));
+      SET_SPRITE(tx.spec_hill[i], buffer);
+    }
+
+    for (i=0; i<4; i++) {
+      for (j=0; j<8; j++) {
+	char *dir2 = "udlr";
+	my_snprintf(buffer, sizeof(buffer), "tx.coast_cape_%c%d", dir2[i], j);
+	SET_SPRITE(tx.coast_cape_iso[j][i], buffer);
+      }
+    }
+  } else {
+    for(i=1; i<NUM_DIRECTION_NSEW; i++) {
+      my_snprintf(buffer, sizeof(buffer), "tx.darkness_%s", nsew_str(i));
+      SET_SPRITE(tx.darkness[i], buffer);
+    }
+
+    for(i=1; i<NUM_DIRECTION_NSEW; i++) {
+      my_snprintf(buffer, sizeof(buffer), "tx.coast_cape_%s", nsew_str(i));
+      SET_SPRITE(tx.coast_cape[i], buffer);
+    }
+
+    for(i=0; i<2; i++) {
+      for(j=0; j<3; j++) {
+	my_snprintf(buffer, sizeof(buffer), "tx.denmark_%d%d", i, j);
+	SET_SPRITE(tx.denmark[i][j], buffer);
+      }
+    }
   }
+
   for(i=0; i<4; i++) {
     my_snprintf(buffer, sizeof(buffer), "tx.river_outlet_%c", dir_char[i]);
     SET_SPRITE(tx.river_outlet[i], buffer);
   }
-  for(i=0; i<2; i++) {
-    for(j=0; j<3; j++) {
-      my_snprintf(buffer, sizeof(buffer), "tx.denmark_%d%d", i, j);
-      SET_SPRITE(tx.denmark[i][j], buffer);
-    }
-  }
 
-  sprites.city.tile = 0;       /* no place to initialize this variable */
+  sprites.city.tile_wall = 0;       /* no place to initialize this variable */
+  sprites.city.tile = 0;            /* no place to initialize this variable */
 }
 
 /**********************************************************************
@@ -596,7 +690,7 @@ static struct Sprite* lookup_sprite_tag_alt(const char *tag, const char *alt,
 
   sp = hash_lookup_data(sprite_hash, tag);
   if (sp) return sp;
-  
+
   sp = hash_lookup_data(sprite_hash, alt);
   if (sp) {
     freelog(loglevel, "Using alternate graphic %s (instead of %s) for %s %s",
@@ -631,8 +725,9 @@ void tilespec_setup_unit_type(int id)
 void tilespec_setup_tile_type(int id)
 {
   struct tile_type *tt = get_tile_type(id);
+  char buffer1[MAX_LEN_NAME+20];
+  char buffer2[MAX_LEN_NAME+20];
   char *nsew;
-  char buffer1[MAX_LEN_NAME+20], buffer2[MAX_LEN_NAME+20];
   int i;
   
   if(tt->terrain_name[0] == '\0') {
@@ -644,23 +739,40 @@ void tilespec_setup_tile_type(int id)
     }
     return;
   }
-  for(i=0; i<NUM_DIRECTION_NSEW; i++) {
-    nsew = nsew_str(i);
-    my_snprintf(buffer1, sizeof(buffer1), "%s_%s", tt->graphic_str, nsew);
-    my_snprintf(buffer2, sizeof(buffer1), "%s_%s", tt->graphic_alt, nsew);
-    
-    tt->sprite[i] = lookup_sprite_tag_alt(buffer1, buffer2, 1, "tile_type",
-					  tt->terrain_name);
-    
-    /* should probably do something if NULL, eg generic default? */
+
+  if (is_isometric) {
+    my_snprintf(buffer1, sizeof(buffer1), "%s1", tt->graphic_str);
+    if (id != T_RIVER) {
+      tt->sprite[0] = lookup_sprite_tag_alt(buffer1, NULL, 1, "tile_type",
+					    tt->terrain_name);
+    } else {
+      tt->sprite[0] = NULL;
+    }
+  } else {
+    for(i=0; i<NUM_DIRECTION_NSEW; i++) {
+      nsew = nsew_str(i);
+      my_snprintf(buffer1, sizeof(buffer1), "%s_%s", tt->graphic_str, nsew);
+      my_snprintf(buffer2, sizeof(buffer2), "%s_%s", tt->graphic_alt, nsew);
+
+      tt->sprite[i] = lookup_sprite_tag_alt(buffer1, buffer2, 1, "tile_type",
+					    tt->terrain_name);
+
+      assert(tt->sprite[i]);
+      /* should probably do something if NULL, eg generic default? */
+    }
   }
-  for(i=0; i<2; i++) {
+
+  for (i=0; i<2; i++) {
     char *name = i ? tt->special_2_name : tt->special_1_name;
-    tt->special[i].sprite
-      = lookup_sprite_tag_alt(tt->special[i].graphic_str,
-			      tt->special[i].graphic_alt,
-			      name[0], "tile_type special", name);
-    
+    if (name[0]) {
+      tt->special[i].sprite
+	= lookup_sprite_tag_alt(tt->special[i].graphic_str,
+				tt->special[i].graphic_alt,
+				name[0], "tile_type special", name);
+      assert(tt->special[i].sprite);
+    } else {
+      tt->special[i].sprite = NULL;
+    }
     /* should probably do something if NULL, eg generic default? */
   }
 }
@@ -722,11 +834,20 @@ static struct Sprite *get_city_sprite(struct city *pcity)
   for( size=0; size < city_styles[style].tiles_num; size++)
     if( pcity->size < city_styles[style].tresh[size]) 
       break;
-  return sprites.city.tile[style][size-1];
+
+  if (is_isometric) {
+    if (city_got_citywalls(pcity))
+      return sprites.city.tile_wall[style][size-1];
+    else
+      return sprites.city.tile[style][size-1];
+  } else {
+    return sprites.city.tile[style][size-1];
+  }
 }
 
 /**************************************************************************
 Return the sprite needed to draw the city wall
+Not used for isometric view.
 **************************************************************************/
 static struct Sprite *get_city_wall_sprite(struct city *pcity)
 {
@@ -748,24 +869,28 @@ static struct Sprite *get_city_occupied_sprite(struct city *pcity)
 /**********************************************************************
   Fill in the sprite array for the city
 ***********************************************************************/
-static int fill_city_sprite_array(struct Sprite **sprs, struct city *pcity)
+static int fill_city_sprite_array(struct Sprite **sprs, struct city *pcity,
+				  int *solid_bg)
 {
   struct Sprite **save_sprs=sprs;
   struct tile *ptile = map_get_tile(pcity->x, pcity->y);
-  
-  if(!no_backdrop) {
+
+  *solid_bg = 0;
+  if (!no_backdrop) {
     if(!solid_color_behind_units) {
       /* will be the first sprite if flags_are_transparent == FALSE */
       *sprs++ = get_city_nation_flag_sprite(pcity);
-    } else *sprs++ = NULL;
+    } else {
+      *solid_bg = 1;
+    }
   }
-  
-  if(genlist_size(&(ptile->units.list)) > 0)
+
+  if (genlist_size(&(ptile->units.list)) > 0)
     *sprs++ = get_city_occupied_sprite(pcity);
 
   *sprs++ = get_city_sprite(pcity);
 
-  if(city_got_citywalls(pcity))
+  if (city_got_citywalls(pcity))
     *sprs++ = get_city_wall_sprite(pcity);
 
   if(map_get_special(pcity->x, pcity->y) & S_POLLUTION)
@@ -776,13 +901,13 @@ static int fill_city_sprite_array(struct Sprite **sprs, struct city *pcity)
   if(city_unhappy(pcity))
     *sprs++ = sprites.city.disorder;
 
-  if(ptile->known==TILE_KNOWN_FOGGED)
+  if(ptile->known==TILE_KNOWN_FOGGED && draw_fog_of_war)
     *sprs++ = sprites.tx.fog;
 
   /* Put the size sprites last, so that they are not obscured
    * (and because they can be hard to read if fogged).
    */
-  if(pcity->size>=10)
+  if (pcity->size>=10)
     *sprs++ = sprites.city.size_tens[pcity->size/10];
 
   *sprs++ = sprites.city.size[pcity->size%10];
@@ -790,22 +915,52 @@ static int fill_city_sprite_array(struct Sprite **sprs, struct city *pcity)
   return sprs - save_sprs;
 }
 
+/**********************************************************************
+  Fill in the sprite array for the city
+  (small because things have to be done later in isometric view.)
+***********************************************************************/
+int fill_city_sprite_array_iso(struct Sprite **sprs, struct city *pcity)
+{
+  struct Sprite **save_sprs=sprs;
+  struct tile *ptile = map_get_tile(pcity->x, pcity->y);
+
+  if (!no_backdrop) {
+    *sprs++ = get_city_nation_flag_sprite(pcity);
+  }
+
+  if (genlist_size(&(ptile->units.list)) > 0)
+    *sprs++ = get_city_occupied_sprite(pcity);
+
+  *sprs++ = get_city_sprite(pcity);
+
+  if (city_unhappy(pcity))
+    *sprs++ = sprites.city.disorder;
+
+  return sprs - save_sprs;
+}
+
 /**********************************************************************
   Fill in the sprite array for the unit
 ***********************************************************************/
-int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
+int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit,
+			   int *solid_bg)
 {
   struct Sprite **save_sprs=sprs;
   int ihp;
+  *solid_bg = 0;
 
-  if(!no_backdrop) {
-    if(!solid_color_behind_units) {
-      /* will be the first sprite if flags_are_transparent == FALSE */
+  if (is_isometric) {
+    if (!no_backdrop) {
       *sprs++ = get_unit_nation_flag_sprite(punit);
-    } else {
-      /* Two NULLs means unit */
-      *sprs++ = NULL;
-      *sprs++ = NULL;
+    }
+  } else {
+    if (!no_backdrop) {
+      if (!solid_color_behind_units) {
+	/* will be the first sprite if flags_are_transparent == FALSE */
+	*sprs++ = get_unit_nation_flag_sprite(punit);
+      } else {
+	*solid_bg = 1;
+      }
     }
   }
 
@@ -876,6 +1031,10 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
     *sprs++ = sprites.unit.connect;
   }
 
+  if (punit->activity == ACTIVITY_PATROL) {
+    *sprs++ = sprites.unit.patrol;
+  }
+
   ihp = ((NUM_TILES_HP_BAR-1)*punit->hp) / get_unit_type(punit->type)->hp;
   ihp = CLIP(0, ihp, NUM_TILES_HP_BAR-1);
   *sprs++ = sprites.unit.hp_bar[ihp];
@@ -883,10 +1042,255 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
   return sprs - save_sprs;
 }
 
+/**********************************************************************
+Only used for isometric view.
+***********************************************************************/
+static struct Sprite *get_dither(int ttype, int ttype_other)
+{
+  if (ttype_other == T_UNKNOWN)
+    return sprites.black_tile;
+
+  if (ttype == T_OCEAN || ttype == T_JUNGLE)
+    return NULL;
+
+  if (ttype_other == T_OCEAN)
+    return sprites.coast_color;
+
+  if (ttype_other != T_UNKNOWN && ttype_other != T_LAST)
+    return get_tile_type(ttype_other)->sprite[0];
+  else
+    return NULL;
+}
+
+/**********************************************************************
+Fill in the sprite array for the tile at position (abs_x0,abs_y0).
+Does not fill in the city or unit; that have to be done seperatly in
+isometric view. Also, no fog here.
+
+A return of -1 means the tile should be black.
+***********************************************************************/
+int fill_tile_sprite_array_iso(struct Sprite **sprs, struct Sprite **coasts,
+			       struct Sprite **dither,
+			       int x, int y, int citymode,
+			       int *solid_bg)
+{
+  int ttype, ttype_near[8];
+  int tspecial, tspecial_near[8];
+  int ttype_north, ttype_south, ttype_east, ttype_west;
+  int ttype_north_east, ttype_south_east, ttype_south_west, ttype_north_west;
+  int tspecial_north, tspecial_south, tspecial_east, tspecial_west;
+  int tspecial_north_east, tspecial_south_east, tspecial_south_west, tspecial_north_west;
+
+  int tileno;
+  struct tile *ptile;
+  struct city *pcity;
+  struct Sprite **save_sprs = sprs;
+  int dir, i;
+
+  *solid_bg = 0;
+
+  if (!normalize_map_pos(&x, &y))
+    return -1;
+
+  ptile = map_get_tile(x, y);
+  if (!ptile->known)
+    return -1;
+
+  pcity = ptile->city;
+  ttype = map_get_terrain(x, y);
+  tspecial = map_get_special(x, y);
+
+  /* A little hack to avoid drawing seperate T_RIVER isometric tiles. */
+  if (ttype == T_RIVER) {
+    ttype = T_GRASSLAND;
+    tspecial |= S_RIVER;
+  }
+
+  for (dir=0; dir<8; dir++) {
+    int x1 = x + DIR_DX2[dir];
+    int y1 = y + DIR_DY2[dir];
+    if (normalize_map_pos(&x1, &y1)) {
+      ttype_near[dir] = map_get_terrain(x1, y1);
+      tspecial_near[dir] = map_get_special(x1, y1);
+
+      /* hacking away the river here... */
+      if (ttype_near[dir] == T_RIVER) {
+	ttype_near[dir] = T_GRASSLAND;
+	tspecial_near[dir] |= S_RIVER;
+      }
+    } else {
+      ttype_near[dir] = T_UNKNOWN;
+      tspecial_near[dir] = S_NO_SPECIAL;
+    }
+  }
+  ttype_north      = ttype_near[0];
+  ttype_north_east = ttype_near[1];
+  ttype_east       = ttype_near[2];
+  ttype_south_east = ttype_near[3];
+  ttype_south      = ttype_near[4];
+  ttype_south_west = ttype_near[5];
+  ttype_west       = ttype_near[6];
+  ttype_north_west = ttype_near[7];
+  tspecial_north      = tspecial_near[0];
+  tspecial_north_east = tspecial_near[1];
+  tspecial_east       = tspecial_near[2];
+  tspecial_south_east = tspecial_near[3];
+  tspecial_south      = tspecial_near[4];
+  tspecial_south_west = tspecial_near[5];
+  tspecial_west       = tspecial_near[6];
+  tspecial_north_west = tspecial_near[7];
+  
+  if (draw_terrain) {
+    if (ttype != T_OCEAN) /* painted via coasts. */
+      *sprs++ = get_tile_type(ttype)->sprite[0];
+
+    if (ttype == T_HILLS) {
+      tileno = INDEX_NSEW((ttype_north==T_HILLS || ttype_north==T_HILLS),
+			  (ttype_south==T_HILLS || ttype_south==T_HILLS),
+			  (ttype_east==T_HILLS || ttype_east==T_HILLS),
+			  (ttype_west==T_HILLS || ttype_west==T_HILLS));
+      *sprs++=sprites.tx.spec_hill[tileno];
+    }
+
+    if (ttype == T_FOREST) {
+      tileno = INDEX_NSEW((ttype_north==T_FOREST || ttype_north==T_FOREST),
+			  (ttype_south==T_FOREST || ttype_south==T_FOREST),
+			  (ttype_east==T_FOREST || ttype_east==T_FOREST),
+			  (ttype_west==T_FOREST || ttype_west==T_FOREST));
+      *sprs++=sprites.tx.spec_forest[tileno];
+    }
+
+    if (ttype == T_MOUNTAINS) {
+      tileno = INDEX_NSEW((ttype_north==T_MOUNTAINS || ttype_north==T_MOUNTAINS),
+			  (ttype_south==T_MOUNTAINS || ttype_south==T_MOUNTAINS),
+			  (ttype_east==T_MOUNTAINS || ttype_east==T_MOUNTAINS),
+			  (ttype_west==T_MOUNTAINS || ttype_west==T_MOUNTAINS));
+      *sprs++=sprites.tx.spec_mountain[tileno];
+    }
+
+    if (tspecial&S_RIVER) {
+      tileno = INDEX_NSEW((tspecial_north&S_RIVER || ttype_north==T_OCEAN),
+			  (tspecial_south&S_RIVER || ttype_south==T_OCEAN),
+			  (tspecial_east&S_RIVER || ttype_east==T_OCEAN),
+			  (tspecial_west&S_RIVER || ttype_west==T_OCEAN));
+      *sprs++=sprites.tx.spec_river[tileno];
+    }
+
+    if (ttype == T_OCEAN) {
+      if(tspecial_north&S_RIVER || ttype_north==T_RIVER)
+	*sprs++ = sprites.tx.river_outlet[DIR_NORTH];
+      if(tspecial_west&S_RIVER || ttype_west==T_RIVER)
+	*sprs++ = sprites.tx.river_outlet[DIR_WEST];
+      if(tspecial_south&S_RIVER || ttype_south==T_RIVER)
+	*sprs++ = sprites.tx.river_outlet[DIR_SOUTH];
+      if(tspecial_east&S_RIVER || ttype_east==T_RIVER)
+	*sprs++ = sprites.tx.river_outlet[DIR_EAST];
+    }
+  } else {
+    *solid_bg = 1;
+  }
+
+  if (draw_specials) {
+    if (tspecial & S_SPECIAL_1)
+      *sprs++ = tile_types[ttype].special[0].sprite;
+    else if (tspecial & S_SPECIAL_2)
+      *sprs++ = tile_types[ttype].special[1].sprite;
+  }
+
+  if (tspecial & S_MINE && draw_mines) {
+    /* We do not have an oil tower in isometric view yet... */
+    *sprs++ = sprites.tx.mine;
+  }
+
+  if (tspecial & S_IRRIGATION && !pcity && draw_irrigation) {
+    if (tspecial & S_FARMLAND) {
+      *sprs++=sprites.tx.farmland;
+    } else {
+      *sprs++=sprites.tx.irrigation;
+    }
+  }
+
+  if (tspecial & S_RAILROAD && draw_roads_rails) {
+    int found = 0;
+    for (dir=0; dir<8; dir++) {
+      if (tspecial_near[dir] & S_RAILROAD) {
+	*sprs++ = sprites.rail.dir[dir];
+	found = 1;
+      } else if (tspecial_near[dir] & S_ROAD) {
+	*sprs++ = sprites.road.dir[dir];
+	found = 1;
+      }
+    }
+    if (!found && !pcity)
+      *sprs++ = sprites.rail.isolated;
+  } else if (tspecial & S_ROAD && draw_roads_rails) {
+    int found = 0;
+    for (dir=0; dir<8; dir++) {
+      if (tspecial_near[dir] & S_ROAD) {
+	*sprs++ = sprites.road.dir[dir];
+	found = 1;
+      }
+    }
+    if (!found && !pcity)
+      *sprs++ = sprites.road.isolated;
+  }
+
+  if (tspecial & S_HUT && draw_specials) *sprs++ = sprites.tx.village;
+  /* These are drawn later in isometric view (on top of city.)
+     if (tspecial & S_POLLUTION) *sprs++ = sprites.tx.pollution;
+     if (tspecial & S_FALLOUT) *sprs++ = sprites.tx.fallout;
+  */
+
+  /* put coasts */
+  if (ttype == T_OCEAN) {
+    for (i = 0; i < 4; i++) {
+      int ttype_adj[3];
+      int array_index;
+      switch (i) {
+      case 0: /* up */
+	ttype_adj[0] = ttype_west;
+	ttype_adj[1] = ttype_north_west;
+	ttype_adj[2] = ttype_north;
+	break;
+      case 1: /* down */
+	ttype_adj[0] = ttype_east;
+	ttype_adj[1] = ttype_south_east;
+	ttype_adj[2] = ttype_south;
+	break;
+      case 2: /* left */
+	ttype_adj[0] = ttype_south;
+	ttype_adj[1] = ttype_south_west;
+	ttype_adj[2] = ttype_west;
+	break;
+      case 3: /* right*/
+	ttype_adj[0] = ttype_north;
+	ttype_adj[1] = ttype_north_east;
+	ttype_adj[2] = ttype_east;
+	break;
+      default:
+	abort();
+      }
+
+      array_index = (ttype_adj[0] != T_OCEAN ? 1 : 0)
+	+  (ttype_adj[1] != T_OCEAN ? 2 : 0)
+	+  (ttype_adj[2] != T_OCEAN ? 4 : 0);
+      coasts[i] = sprites.tx.coast_cape_iso[array_index][i];
+    }
+  }
+
+  dither[0] = get_dither(ttype, tile_is_known(x, y-1) ? ttype_north : T_UNKNOWN);
+  dither[1] = get_dither(ttype, tile_is_known(x, y+1) ? ttype_south : T_UNKNOWN);
+  dither[2] = get_dither(ttype, tile_is_known(x+1, y) ? ttype_east : T_UNKNOWN);
+  dither[3] = get_dither(ttype, tile_is_known(x-1, y) ? ttype_west : T_UNKNOWN);
+
+  return sprs - save_sprs;
+}
+
 /**********************************************************************
   Fill in the sprite array for the tile at position (abs_x0,abs_y0)
 ***********************************************************************/
-int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int citymode)
+int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0,
+			   int citymode, int *solid_bg, struct player **pplayer)
 {
   int ttype, ttype_north, ttype_south, ttype_east, ttype_west;
   int ttype_north_east, ttype_south_east, ttype_south_west, ttype_north_west;
@@ -904,6 +1308,8 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
   int den_y=map.ysize*.24;
 
   struct Sprite **save_sprs=sprs;
+  *solid_bg = 0;
+  *pplayer = NULL;
 
   ptile=map_get_tile(abs_x0, abs_y0);
 
@@ -917,20 +1323,18 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
   if(!flags_are_transparent || solid_color_behind_units) {
     /* non-transparent flags -> just draw city or unit */
 
-    if((punit=find_visible_unit(ptile))) {
-      if(!citymode || punit->owner!=game.player_idx) {
-	if(!focus_unit_hidden || !pfocus ||
-	   punit->x!=pfocus->x || punit->y!=pfocus->y) {
-	  sprs+=fill_unit_sprite_array(sprs,punit);
-	  if(unit_list_size(&ptile->units)>1)  
-	    *sprs++ = sprites.unit.stack;
-	  return sprs - save_sprs;
-	}
-      }
+    punit = get_drawable_unit(abs_x0, abs_y0, citymode);
+    if (punit && (draw_units || (draw_focus_unit && pfocus == punit))) {
+      sprs += fill_unit_sprite_array(sprs, punit, solid_bg);
+      *pplayer = unit_owner(punit);
+      if (unit_list_size(&ptile->units) > 1)  
+	*sprs++ = sprites.unit.stack;
+      return sprs - save_sprs;
     }
 
-    if(pcity) {
-      sprs+=fill_city_sprite_array(sprs,pcity);
+    if (pcity && draw_cities) {
+      sprs+=fill_city_sprite_array(sprs, pcity, solid_bg);
+      *pplayer = city_owner(pcity);
       return sprs - save_sprs;
     }
   }
@@ -975,21 +1379,22 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
     mysprite = sprites.tx.denmark[abs_y0-den_y][abs_x0-34];
   } else {
     tileno = INDEX_NSEW((ttype_north==ttype),
-                        (ttype_south==ttype),
-                        (ttype_east==ttype),	
-                        (ttype_west==ttype));
+			(ttype_south==ttype),
+			(ttype_east==ttype),	
+			(ttype_west==ttype));
     if(ttype==T_RIVER) {
       tileno |= INDEX_NSEW((ttype_north==T_OCEAN),
-                           (ttype_south==T_OCEAN),
-                           (ttype_east==T_OCEAN),
-                           (ttype_west==T_OCEAN));
+			   (ttype_south==T_OCEAN),
+			   (ttype_east==T_OCEAN),
+			   (ttype_west==T_OCEAN));
     }
     mysprite = get_tile_type(ttype)->sprite[tileno];
   }
 
-  if (mysprite) *sprs++=mysprite;
+  if (draw_terrain) *sprs++=mysprite;
+  else *solid_bg = 1;
 
-  if(ttype==T_OCEAN) {
+  if(ttype==T_OCEAN && draw_terrain) {
     tileno = INDEX_NSEW((ttype_north==T_OCEAN && ttype_east==T_OCEAN && 
                          ttype_north_east!=T_OCEAN),
                         (ttype_south==T_OCEAN && ttype_west==T_OCEAN && 
@@ -1011,7 +1416,7 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
       *sprs++ = sprites.tx.river_outlet[DIR_EAST];
   }
 
-  if (tspecial&S_RIVER) {
+  if (tspecial&S_RIVER && draw_terrain) {
     tileno = INDEX_NSEW((tspecial_north&S_RIVER || ttype_north==T_OCEAN),
 			(tspecial_south&S_RIVER || ttype_south==T_OCEAN),
 			(tspecial_east&S_RIVER  || ttype_east==T_OCEAN),
@@ -1019,12 +1424,12 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
     *sprs++=sprites.tx.spec_river[tileno];
   }
 
-  if(tspecial & S_IRRIGATION) {
+  if(tspecial & S_IRRIGATION && draw_irrigation) {
     if(tspecial & S_FARMLAND) *sprs++=sprites.tx.farmland;
     else *sprs++=sprites.tx.irrigation;
   }
 
-  if((tspecial & S_ROAD) || (tspecial & S_RAILROAD)) {
+  if(((tspecial & S_ROAD) || (tspecial & S_RAILROAD)) && draw_roads_rails) {
     int n, s, e, w;
     
     n = BOOL_VAL(tspecial_north&S_RAILROAD);
@@ -1096,39 +1501,43 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
     }
   }
 
-  if(tspecial & S_SPECIAL_1)
-    *sprs++ = tile_types[ttype].special[0].sprite;
-  else if(tspecial & S_SPECIAL_2)
-    *sprs++ = tile_types[ttype].special[1].sprite;
+  if(draw_specials) {
+    if(tspecial & S_SPECIAL_1)
+      *sprs++ = tile_types[ttype].special[0].sprite;
+    else if(tspecial & S_SPECIAL_2)
+      *sprs++ = tile_types[ttype].special[1].sprite;
+  }
 
-  if(tspecial & S_MINE) {
+  if(tspecial & S_MINE && draw_mines) {
     if(ttype==T_HILLS || ttype==T_MOUNTAINS)
       *sprs++ = sprites.tx.mine;
     else /* desert */
       *sprs++ = sprites.tx.oil_mine;
   }
-
-  if (tspecial & S_RAILROAD) {
-    int adjacent = rail_card_tileno;
-    if (draw_diagonal_roads)
-      adjacent |= rail_semi_tileno;
-    if (!adjacent)
-      *sprs++ = sprites.rail.isolated;
-  }
-  else if (tspecial & S_ROAD) {
-    int adjacent = (rail_card_tileno | road_card_tileno);
-    if (draw_diagonal_roads)
-      adjacent |= (rail_semi_tileno | road_semi_tileno);
-    if (!adjacent)
-      *sprs++ = sprites.road.isolated;
+  
+  if (draw_roads_rails) {
+    if (tspecial & S_RAILROAD) {
+      int adjacent = rail_card_tileno;
+      if (draw_diagonal_roads)
+	adjacent |= rail_semi_tileno;
+      if (!adjacent)
+	*sprs++ = sprites.rail.isolated;
+    }
+    else if (tspecial & S_ROAD) {
+      int adjacent = (rail_card_tileno | road_card_tileno);
+      if (draw_diagonal_roads)
+	adjacent |= (rail_semi_tileno | road_semi_tileno);
+      if (!adjacent)
+	*sprs++ = sprites.road.isolated;
+    }
   }
 
-  if(tspecial & S_HUT) *sprs++ = sprites.tx.village;
-  if(tspecial & S_FORTRESS) *sprs++ = sprites.tx.fortress;
-  if(tspecial & S_AIRBASE) *sprs++ = sprites.tx.airbase;
-  if(tspecial & S_POLLUTION) *sprs++ = sprites.tx.pollution;
-  if(tspecial & S_FALLOUT) *sprs++ = sprites.tx.fallout;
-  if(ptile->known==TILE_KNOWN_FOGGED) *sprs++ = sprites.tx.fog;
+  if(tspecial & S_HUT && draw_specials) *sprs++ = sprites.tx.village;
+  if(tspecial & S_FORTRESS && draw_fortress_airbase) *sprs++ = sprites.tx.fortress;
+  if(tspecial & S_AIRBASE && draw_fortress_airbase) *sprs++ = sprites.tx.airbase;
+  if(tspecial & S_POLLUTION && draw_pollution) *sprs++ = sprites.tx.pollution;
+  if(tspecial & S_FALLOUT && draw_pollution) *sprs++ = sprites.tx.fallout;
+  if(ptile->known==TILE_KNOWN_FOGGED && draw_fog_of_war) *sprs++ = sprites.tx.fog;
 
   if(!citymode) {
     tileno = INDEX_NSEW((tile_is_known(abs_x0, abs_y0-1)==TILE_UNKNOWN),
@@ -1139,21 +1548,23 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
       *sprs++ = sprites.tx.darkness[tileno];
   }
 
-  if(flags_are_transparent) {
+  if (flags_are_transparent) {
+    int dummy;
     /* transparent flags -> draw city or unit last */
 
-    if(pcity) {
-      sprs+=fill_city_sprite_array(sprs,pcity);
+    if (pcity && draw_cities) {
+      sprs+=fill_city_sprite_array(sprs, pcity, &dummy);
     }
 
-    if((punit=find_visible_unit(ptile))) {
-      if(!citymode || punit->owner!=game.player_idx) {
-	if(!focus_unit_hidden || !pfocus ||
-	   punit->x!=pfocus->x || punit->y!=pfocus->y) {
+    punit = find_visible_unit(ptile);
+    if (punit) {
+      if (!citymode || punit->owner != game.player_idx) {
+	if ((!focus_unit_hidden || pfocus != punit) &&
+	    (draw_units || (draw_focus_unit && !focus_unit_hidden && punit == pfocus))) {
 	  no_backdrop=BOOL_VAL(pcity);
-	  sprs+=fill_unit_sprite_array(sprs,punit);
+	  sprs += fill_unit_sprite_array(sprs, punit, &dummy);
 	  no_backdrop=FALSE;
-	  if(unit_list_size(&ptile->units)>1)  
+	  if (unit_list_size(&ptile->units)>1)  
 	    *sprs++ = sprites.unit.stack;
 	}
       }
@@ -1172,14 +1583,24 @@ static void tilespec_setup_style_tile(int style, char *graphics)
   struct Sprite *sp;
   char buffer[128];
   int j;
+  struct Sprite *sp_wall = NULL;
+  char buffer_wall[128];
 
   city_styles[style].tiles_num = 0;
 
   for(j=0; j<32 && city_styles[style].tiles_num < MAX_CITY_TILES; j++) {
     my_snprintf(buffer, sizeof(buffer), "%s_%d", graphics, j);
     sp = hash_lookup_data(sprite_hash, buffer);
+    if (is_isometric) {
+      my_snprintf(buffer, sizeof(buffer_wall), "%s_%d_wall", graphics, j);
+      sp_wall = hash_lookup_data(sprite_hash, buffer);
+    }
     if (sp) {
       sprites.city.tile[style][city_styles[style].tiles_num] = sp;
+      if (is_isometric) {
+	assert(sp_wall);
+	sprites.city.tile_wall[style][city_styles[style].tiles_num] = sp_wall;
+      }
       city_styles[style].tresh[city_styles[style].tiles_num] = j;
       city_styles[style].tiles_num++;
       freelog(LOG_DEBUG, "Found tile %s_%d", graphics, j);
@@ -1189,13 +1610,15 @@ static void tilespec_setup_style_tile(int style, char *graphics)
   if(city_styles[style].tiles_num == 0)      /* don't waste more time */
     return;
 
-  /* the wall tile */
-  my_snprintf(buffer, sizeof(buffer), "%s_wall", graphics);
-  sp = hash_lookup_data(sprite_hash, buffer);
-  if (sp) {
-    sprites.city.tile[style][city_styles[style].tiles_num] = sp;
-  } else {
-    freelog(LOG_NORMAL, "Warning: no wall tile for graphic %s", graphics);
+  if (!is_isometric) {
+    /* the wall tile */
+    my_snprintf(buffer, sizeof(buffer), "%s_wall", graphics);
+    sp = hash_lookup_data(sprite_hash, buffer);
+    if (sp) {
+      sprites.city.tile[style][city_styles[style].tiles_num] = sp;
+    } else {
+      freelog(LOG_NORMAL, "Warning: no wall tile for graphic %s", graphics);
+    }
   }
 
   /* occupied tile */
@@ -1248,10 +1671,15 @@ void tilespec_alloc_city_tiles(int count)
 {
   int i;
 
+  if (is_isometric)
+    sprites.city.tile_wall = fc_calloc( count, sizeof(struct Sprite**) );
   sprites.city.tile = fc_calloc( count, sizeof(struct Sprite**) );
 
-  for( i=0; i<count; i++ ) 
+  for (i=0; i<count; i++) {
+    if (is_isometric)
+      sprites.city.tile_wall[i] = fc_calloc(MAX_CITY_TILES+2, sizeof(struct Sprite*));
     sprites.city.tile[i] = fc_calloc(MAX_CITY_TILES+2, sizeof(struct Sprite*));
+  }
 }
 
 /**********************************************************************
@@ -1261,10 +1689,15 @@ void tilespec_free_city_tiles(int count)
 {
   int i;
 
-  for( i=0; i<count; i++ ) 
+  for (i=0; i<count; i++) {
+    if (is_isometric)
+      free(sprites.city.tile_wall[i]);
     free(sprites.city.tile[i]);
+  }
 
-  free (sprites.city.tile);
+  if (is_isometric)
+    free(sprites.city.tile_wall);
+  free(sprites.city.tile);
 }
 
 /**********************************************************************
@@ -1321,3 +1754,25 @@ void set_focus_unit_hidden_state(int hide)
 {
   focus_unit_hidden = hide;
 }
+
+/**********************************************************************
+...
+***********************************************************************/
+struct unit *get_drawable_unit(int x, int y, int citymode)
+{
+  struct unit *punit = find_visible_unit(map_get_tile(x, y));
+  struct unit *pfocus = get_unit_in_focus();
+
+  if (!punit)
+    return NULL;
+
+  if (citymode && punit->owner == game.player_idx)
+    return NULL;
+
+  if (punit != pfocus
+      || !focus_unit_hidden
+      || !same_pos(punit->x, punit->y, pfocus->x, pfocus->y))
+    return punit;
+  else
+    return NULL;
+}
