@@ -14,172 +14,19 @@
 #define FC__CITY_H
 
 #include "genlist.h"
-#include "shared.h"		/* MAX_LEN_NAME */
-#include "tech.h"
-#include "terrain.h"
+#include "improvement.h"
 #include "unit.h"		/* struct unit_list */
 #include "worklist.h"
 
+
 struct player;
 struct government;
+struct tile;
 
-
-/* Improvement types.
- * (This will later not be an enum.)
- */
-enum improvement_type_id {
-  B_AIRPORT=0, B_AQUEDUCT, B_BANK, B_BARRACKS, B_BARRACKS2, B_BARRACKS3, 
-  B_CATHEDRAL, B_CITY, B_COASTAL, B_COLOSSEUM, B_COURTHOUSE,  B_FACTORY, 
-  B_GRANARY, B_HARBOUR, B_HYDRO, B_LIBRARY, B_MARKETPLACE, B_MASS, B_MFG, 
-  B_NUCLEAR, B_OFFSHORE, B_PALACE, B_POLICE, B_PORT, B_POWER,
-  B_RECYCLING, B_RESEARCH, B_SAM, B_SDI, B_SEWER, B_SOLAR, B_SCOMP, 
-  B_SMODULE, B_SSTRUCTURAL, B_STOCK, B_SUPERHIGHWAYS, B_SUPERMARKET, B_TEMPLE,
-  B_UNIVERSITY,  
-  
-  B_APOLLO, B_ASMITHS, B_COLLOSSUS, B_COPERNICUS, B_CURE, B_DARWIN, B_EIFFEL,
-  B_GREAT, B_WALL, B_HANGING, B_HOOVER, B_ISAAC, B_BACH, B_RICHARDS, 
-  B_LEONARDO, B_LIGHTHOUSE, B_MAGELLAN, B_MANHATTEN, B_MARCO, B_MICHELANGELO, 
-  B_ORACLE, B_PYRAMIDS, B_SETI, B_SHAKESPEARE, B_LIBERTY, B_SUNTZU, 
-  B_UNITED, B_WOMENS,
-  B_CAPITAL, B_LAST
+enum production_class_type {
+  TYPE_UNIT, TYPE_NORMAL_IMPROVEMENT, TYPE_WONDER
 };
 
-typedef int Impr_Type_id;
-
-/* Range of effects.
- * Used in equiv_range and effect.range fields.
- * (These must correspond to effect_range_names[] in city.c.)
- */
-enum effect_range_id {
-  EFR_NONE,
-  EFR_BUILDING,
-  EFR_CITY,
-  EFR_ISLAND,
-  EFR_PLAYER,
-  EFR_WORLD,
-  EFR_LAST	/* keep this last */
-};
-
-typedef enum effect_range_id Eff_Range_id;
-
-/* Type of effects.
- * Used in effect.type field.
- * (These must correspond to effect_type_names[] in city.c.)
- */
-enum effect_type_id {
-  EFT_ADV_PARASITE,
-  EFT_AIRLIFT,
-  EFT_ANY_GOVERNMENT,
-  EFT_BARB_ATTACK,
-  EFT_BARB_DEFEND,
-  EFT_CAPITAL_CITY,
-  EFT_CAPITAL_EXISTS,
-  EFT_ENABLE_NUKE,
-  EFT_ENABLE_SPACE,
-  EFT_ENEMY_PEACEFUL,
-  EFT_FOOD_ADD_TILE,
-  EFT_FOOD_BONUS,
-  EFT_FOOD_INC_TILE,
-  EFT_FOOD_PER_TILE,
-  EFT_GIVE_IMM_ADV,
-  EFT_GROWTH_FOOD,
-  EFT_HAVE_EMBASSIES,
-  EFT_IMPROVE_REP,
-  EFT_LUXURY_BONUS,
-  EFT_LUXURY_PCT,
-  EFT_MAKE_CONTENT,
-  EFT_MAKE_CONTENT_MIL,
-  EFT_MAKE_CONTENT_PCT,
-  EFT_MAKE_HAPPY,
-  EFT_MAY_DECLARE_WAR,
-  EFT_NO_ANARCHY,
-  EFT_NO_SINK_DEEP,
-  EFT_NUKE_PROOF,
-  EFT_POLLU_ADJ,
-  EFT_POLLU_ADJ_POP,
-  EFT_POLLU_ADJ_PROD,
-  EFT_POLLU_DECREASE,
-  EFT_PROD_ADD_TILE,
-  EFT_PROD_BONUS,
-  EFT_PROD_INC_TILE,
-  EFT_PROD_PER_TILE,
-  EFT_PROD_TO_GOLD,
-  EFT_REDUCE_CORRUPT,
-  EFT_REDUCE_WASTE,
-  EFT_REVEAL_CITIES,
-  EFT_REVEAL_MAP,
-  EFT_REVOLT_DIST,
-  EFT_SCIENCE_BONUS,
-  EFT_SCIENCE_PCT,
-  EFT_SIZE_UNLIMIT,
-  EFT_SLOW_NUKE_WINTER,
-  EFT_SLOW_GLOBAL_WARM,
-  EFT_SPACE_PART,
-  EFT_SPY_RESISTANT,
-  EFT_TAX_BONUS,
-  EFT_TAX_PCT,
-  EFT_TRADE_ADD_TILE,
-  EFT_TRADE_BONUS,
-  EFT_TRADE_INC_TILE,
-  EFT_TRADE_PER_TILE,
-  EFT_TRADE_ROUTE_PCT,
-  EFT_UNIT_DEFEND,
-  EFT_UNIT_MOVE,
-  EFT_UNIT_NO_LOSE_POP,
-  EFT_UNIT_RECOVER,
-  EFT_UNIT_REPAIR,
-  EFT_UNIT_VET_COMBAT,
-  EFT_UNIT_VETERAN,
-  EFT_UPGRADE_UNITS,
-  EFT_UPGRADE_ONE_STEP,
-  EFT_UPGRADE_ONE_LEAP,
-  EFT_UPGRADE_ALL_STEP,
-  EFT_UPGRADE_ALL_LEAP,
-  EFT_UPKEEP_FREE,
-  EFT_LAST	/* keep this last */
-};
-
-typedef enum effect_type_id Eff_Type_id;
-
-/* An effect conferred by an improvement.
- */
-struct impr_effect {
-  Eff_Type_id type;
-  Eff_Range_id range;
-  int amount;
-  Impr_Type_id cond_bldg;		/* B_LAST = unconditional */
-  int cond_gov;				/* game.government_count = unconditional */
-  Tech_Type_id cond_adv;		/* A_NONE = unconditional; A_LAST = never */
-  Eff_Type_id cond_eff;			/* EFT_LAST = unconditional */
-  Unit_Class_id aff_unit;		/* UCL_LAST = all */
-  enum tile_terrain_type aff_terr;	/* T_UNKNOWN = all; T_LAST = none */
-  enum tile_special_type aff_spec;	/* S_* bit mask of specials affected */
-};
-
-/* Type of improvement.
- * (Read from buildings.ruleset file.)
- */
-struct impr_type {
-  char name[MAX_LEN_NAME];
-  char name_orig[MAX_LEN_NAME];		/* untranslated */
-  Tech_Type_id tech_req;		/* A_LAST = never; A_NONE = always */
-  Impr_Type_id bldg_req;		/* B_LAST = none required */
-  enum tile_terrain_type *terr_gate;	/* list; T_LAST terminated */
-  enum tile_special_type *spec_gate;	/* list; S_NO_SPECIAL terminated */
-  Eff_Range_id equiv_range;
-  Impr_Type_id *equiv_dupl;		/* list; B_LAST terminated */
-  Impr_Type_id *equiv_repl;		/* list; B_LAST terminated */
-  Tech_Type_id obsolete_by;		/* A_NONE = never obsolete */
-  int is_wonder;
-  int build_cost;
-  int upkeep;
-  int sabotage;
-  struct impr_effect *effect;		/* list; .type==EFT_LAST terminated */
-  int variant;			/* FIXME: remove when gen-impr obsoletes */
-  char *helptext;
-};
-
-
 enum specialist_type {
   SP_ELVIS, SP_SCIENTIST, SP_TAXMAN
 };
@@ -213,22 +60,29 @@ enum city_options {
 /* for new city: default auto-attack options all on, others off: */
 #define CITYOPT_DEFAULT (CITYOPT_AUTOATTACK_BITS)
 
+/* Diameter of the workable city area. Must be unequal.
+   Some places in the code hardcodes this number (yet). */
 #define CITY_MAP_SIZE 5
 
 
 /* Iterate a city map */
 
 #define city_map_iterate(x, y) \
+{ \
+  int x, y; \
   for (y=0;y<CITY_MAP_SIZE;y++) \
     for (x=0;x<CITY_MAP_SIZE;x++) \
       if (! ((x == 0 || x == (CITY_MAP_SIZE-1)) && \
 	     (y == 0 || y == (CITY_MAP_SIZE-1))) )
 
+#define city_map_iterate_end \
+}
 /* Iterate a city map, from the center (the city) outwards */
 
 extern int city_map_iterate_outwards_indices[(CITY_MAP_SIZE*CITY_MAP_SIZE)-4][2];
 
 #define city_map_iterate_outwards(x, y) { \
+  int x, y; \
   int city_map_iterate_outwards_index; \
   for \
   ( \
@@ -245,8 +99,8 @@ extern int city_map_iterate_outwards_indices[(CITY_MAP_SIZE*CITY_MAP_SIZE)-4][2]
 /* Iterate a city radius: (dx,dy) centered on (0,0) */
 
 #define city_radius_iterate(dx, dy) \
-  for (dy=-(int)(CITY_MAP_SIZE/2);dy<(int)(CITY_MAP_SIZE/2);dy++) \
-    for (dx=-(int)(CITY_MAP_SIZE/2);dx<(int)(CITY_MAP_SIZE/2);dx++) \
+  for (dy = -(int)(CITY_MAP_SIZE/2); dy<(int)(CITY_MAP_SIZE/2); dy++) \
+    for (dx = -(int)(CITY_MAP_SIZE/2); dx<(int)(CITY_MAP_SIZE/2); dx++) \
       if (! ((dx == -(int)(CITY_MAP_SIZE/2) || dx == (int)(CITY_MAP_SIZE/2)) && \
 	     (dy == -(int)(CITY_MAP_SIZE/2) || dy == (int)(CITY_MAP_SIZE/2))) )
 
@@ -254,6 +108,7 @@ extern int city_map_iterate_outwards_indices[(CITY_MAP_SIZE*CITY_MAP_SIZE)-4][2]
 
 #define map_city_radius_iterate(city_x, city_y, x_itr, y_itr) \
 { \
+  int x_itr, y_itr; \
   int MCMI_x, MCMI_y; \
   for (MCMI_x=0; MCMI_x<CITY_MAP_SIZE; MCMI_x++) { \
     for (MCMI_y=0; MCMI_y<CITY_MAP_SIZE; MCMI_y++) { \
@@ -271,6 +126,22 @@ extern int city_map_iterate_outwards_indices[(CITY_MAP_SIZE*CITY_MAP_SIZE)-4][2]
 }
 
 
+enum choice_type { CT_NONE = 0, CT_BUILDING = 0, CT_NONMIL, CT_ATTACKER,
+                   CT_DEFENDER, CT_LAST };
+
+/* FIXME:
+
+   This should detect also cases where type is just initialized with
+   CT_NONE (probably in order to silence compiler warnings), but no real value
+   is given. You have to change value of CT_BUILDING into 1 before you
+   can add this check. It's left this way for now, is case hardcoded
+   value 0 is still used somewhere instead of CT_BUILDING.
+
+   -- Caz
+*/
+#define ASSERT_REAL_CHOICE_TYPE(type)                                    \
+        assert(type >= 0 && type < CT_LAST /* && type != CT_NONE */ );
+
 
 struct ai_choice {
   int choice;            /* what the advisor wants */
@@ -293,7 +164,9 @@ struct ai_city {
   struct ai_choice choice; /* to spend gold in the right place only */
   int downtown; /* distance from neighbours, for locating wonders wisely */
   int distance_to_wonder_city; /* wondercity will set this for us, avoiding paradox */
-  /* caching these so that CPU usage is O(cities) instead of O(cities^2) -- Syela */
+
+  /* Used for caching when settlers evalueate which tile to improve,
+     and when we place workers. */
   signed short int detox[5][5];
   signed short int derad[5][5];
   signed short int mine[5][5];
@@ -301,7 +174,8 @@ struct ai_city {
   signed short int road[5][5];
   signed short int railroad[5][5];
   signed short int transform[5][5];
-  signed short int tile_value[5][5]; /* caching these will help too. */
+  signed short int tile_value[5][5];
+
   /* so we can contemplate with warmap fresh and decide later */
   int settler_want, founder_want; /* for builder (F_SETTLERS) and founder (F_CITIES) */
   int a, f, invasion; /* who's coming to kill us, for attack co-ordination */
@@ -316,7 +190,15 @@ struct city {
   /* the people */
   int size;
 
+  /* How the citizens feel:
+     ppl_*[0] is distribution before any of the modifiers below.
+     ppl_*[1] is distribution after luxury.
+     ppl_*[2] is distribution after after building effects.
+     ppl_*[3] is distribution after units enfored martial order.
+     ppl_*[4] is distribution after wonders. (final result.) */
   int ppl_happy[5], ppl_content[5], ppl_unhappy[5];
+
+  /* Specialists */
   int ppl_elvis, ppl_scientist, ppl_taxman;
 
   /* trade routes */
@@ -354,6 +236,8 @@ struct city {
   int turn_changed_target;    /* Suffer shield loss at most once per turn */
   int changed_from_id;	      /* If changed this turn, what changed from (id) */
   int changed_from_is_unit;   /* If changed this turn, what changed from (unit?) */
+  int disbanded_shields;      /* If you disband unit in a city. Count them */
+  int caravan_shields;        /* If caravan has helped city to build wonder. */
   int before_change_shields;  /* If changed this turn, shields before penalty */
   int anarchy;		      /* anarchy rounds count */ 
   int rapture;                /* rapture rounds count */ 
@@ -362,6 +246,9 @@ struct city {
   int original;			/* original owner */
   int city_options;		/* bitfield; positions as enum city_options */
 
+  /* server variable. indicates if the city map is synced with the client. */
+  int synced;
+
   /* info for dipl/spy investigation -- used only in client */
   struct unit_list info_units_supported;
   struct unit_list info_units_present;
@@ -396,10 +283,6 @@ extern struct citystyle *city_styles;
     TYPED_LIST_ITERATE(struct city, citylist, pcity)
 #define city_list_iterate_end  LIST_ITERATE_END
 
-
-extern struct impr_type improvement_types[B_LAST];
-extern char **misc_city_names;
-
 /* properties */
 
 struct player *city_owner(struct city *pcity);
@@ -411,34 +294,6 @@ int city_unhappy(struct city *pcity);                /* anarchy??? */
 int city_celebrating(struct city *pcity);            /* love the king ??? */
 int city_rapture_grow(struct city *pcity);
 
-/* improvement effect functions */
-
-Eff_Range_id effect_range_from_str(char *str);
-char *effect_range_name(Eff_Range_id id);
-Eff_Type_id effect_type_from_str(char *str);
-char *effect_type_name(Eff_Type_id id);
-
-/* improvement functions */
-
-int improvement_value(Impr_Type_id id);
-int improvement_obsolete(struct player *pplayer, Impr_Type_id id);
-struct impr_type *get_improvement_type(Impr_Type_id id);
-int wonder_obsolete(Impr_Type_id id);
-int is_wonder_useful(Impr_Type_id id);
-int is_wonder(Impr_Type_id id);
-int improvement_exists(Impr_Type_id id);
-Impr_Type_id find_improvement_by_name(char *s);
-int improvement_variant(Impr_Type_id id);	/* FIXME: remove when gen-impr obsoletes */
-
-/* player related improvement and unit functions */
-
-int could_player_eventually_build_improvement(struct player *p, Impr_Type_id id);
-int could_player_build_improvement(struct player *p, Impr_Type_id id);
-int can_player_build_improvement(struct player *p, Impr_Type_id id);
-int can_player_build_unit_direct(struct player *p, Unit_Type_id id);
-int can_player_build_unit(struct player *p, Unit_Type_id id);
-int can_player_eventually_build_unit(struct player *p, Unit_Type_id id);
-
 /* city related improvement and unit functions */
 
 int improvement_upkeep(struct city *pcity, int i); 
@@ -459,17 +314,26 @@ int city_turns_to_build(struct city *pcity, int id, int id_is_unit);
 
 /* textual representation of buildings */
 
-char *get_improvement_name(Impr_Type_id id);
 char *get_impr_name_ex(struct city *pcity, Impr_Type_id id);
 
+/* tile production functions */
+
+int get_shields_tile(int x, int y); /* shield on spot */
+int get_trade_tile(int x, int y);   /* trade  on spot */
+int get_food_tile(int x, int y);    /* food   on spot */
+
 /* city map functions */
 
-int get_shields_tile(int x, int y, struct city *pcity); /* shield on spot */
-int get_trade_tile(int x, int y, struct city *pcity);   /* trade  on spot */
-int get_food_tile(int x, int y, struct city *pcity);    /* food   on spot */
-void set_worker_city(struct city *pcity, int x, int y, enum city_tile_type type); 
-enum city_tile_type get_worker_city(struct city *pcity, int x, int y);
-int is_worker_here(struct city *pcity, int x, int y);
+int is_valid_city_coords(const int city_x, const int city_y);
+int get_citymap_xy(const struct city *pcity, const int x, const int y,
+		   int *city_x, int *city_y);
+int city_get_shields_tile(int x, int y, struct city *pcity); /* shield on spot */
+int city_get_trade_tile(int x, int y, struct city *pcity);   /* trade  on spot */
+int city_get_food_tile(int x, int y, struct city *pcity);    /* food   on spot */
+void set_worker_city(struct city *pcity, int city_x, int city_y,
+		     enum city_tile_type type); 
+enum city_tile_type get_worker_city(struct city *pcity, int city_x, int city_y);
+int is_worker_here(struct city *pcity, int city_x, int city_y);
 int map_to_city_x(struct city *pcity, int x);
 int map_to_city_y(struct city *pcity, int y);
 
@@ -494,6 +358,19 @@ int citygov_free_gold(struct city *pcity, struct government *gov);
 
 /* city style functions */
 int get_city_style(struct city *pcity);
+int get_player_city_style(struct player *plr);
 int get_style_by_name(char *);
 
+struct city *is_enemy_city_tile(struct tile *ptile, int playerid);
+struct city *is_allied_city_tile(struct tile *ptile, int playerid);
+struct city *is_non_attack_city_tile(struct tile *ptile, int playerid);
+struct city *is_non_allied_city_tile(struct tile *ptile, int playerid);
+
+int is_unit_near_a_friendly_city(struct unit *punit);
+int is_friendly_city_near(int owner, int x, int y);
+int city_exists_within_city_radius(int x, int y, int may_be_on_center);
+
+/* granary size as a function of city size */
+int city_granary_size(int city_size);
+
 #endif  /* FC__CITY_H */
