@@ -19,6 +19,7 @@
 #include <string.h>
 
 #include "city.h"
+#include "connection.h"
 #include "fcintl.h"
 #include "government.h"
 #include "idex.h"
@@ -102,14 +103,16 @@ static char when_char (int when)
   return (((when >= 0) && (when < sizeof (list))) ? list[when] : '?');
 }
 
-static void print_landarea_map (struct claim_map *pcmap, int turn)
+/**************************************************************************
+...
+**************************************************************************/
+static void print_landarea_map(struct claim_map *pcmap, int turn)
 {
   int x, y, p;
 
-  if (turn == 0)
-    {
-      putchar ('\n');
-    }
+  if (turn == 0) {
+    putchar ('\n');
+  }
 
   if (turn == 0)
     {
@@ -201,52 +204,43 @@ Call free_landarea_map(&cmap) to free allocated memory.
 
 static int no_owner = MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS;
 
-/* allocate and clear claim map; determine city radii */
-
-static void build_landarea_map_new (struct claim_map *pcmap)
+/**************************************************************************
+allocate and clear claim map; determine city radii
+**************************************************************************/
+static void build_landarea_map_new(struct claim_map *pcmap)
 {
   int nbytes;
-  int n, i;
-  int cx, cy, mx, my;
-  int halfcity = (CITY_MAP_SIZE / 2);
 
-  nbytes = map.xsize * map.ysize * sizeof (struct claim_cell);
-  pcmap->claims = fc_malloc (nbytes);
+  nbytes = map.xsize * map.ysize * sizeof(struct claim_cell);
+  pcmap->claims = fc_malloc(nbytes);
   memset (pcmap->claims, 0, nbytes);
 
-  nbytes  = game.nplayers * sizeof (int);
-  pcmap->player_landarea = fc_malloc (nbytes);
+  nbytes  = game.nplayers * sizeof(int);
+  pcmap->player_landarea = fc_malloc(nbytes);
   memset (pcmap->player_landarea, 0, nbytes);
 
-  nbytes  = game.nplayers * sizeof (int);
-  pcmap->player_owndarea = fc_malloc (nbytes);
+  nbytes  = game.nplayers * sizeof(int);
+  pcmap->player_owndarea = fc_malloc(nbytes);
   memset (pcmap->player_owndarea, 0, nbytes);
 
-  nbytes = 2 * map.xsize * map.ysize * sizeof (struct map_position);
-  pcmap->edges = fc_malloc (nbytes);
+  nbytes = 2 * map.xsize * map.ysize * sizeof(struct map_position);
+  pcmap->edges = fc_malloc(nbytes);
 
-  for (n = 0; n < game.nplayers; n++)
-    {
-      city_list_iterate (game.players[n].cities, pcity) {
-	city_map_iterate (cx, cy) {
-	  my = pcity->y + cy - halfcity;
-	  if ((my >= 0) && (my < map.ysize))
-	    {
-	      mx = map_adjust_x (pcity->x + cx - halfcity);
-	      i = map_inx (mx, my);
-	      pcmap->claims[i].cities |= (1u << pcity->owner);
-	    }
-	}
-      }
-      city_list_iterate_end;
-    }
+  players_iterate(pplayer) {
+    city_list_iterate(pplayer->cities, pcity) {
+      map_city_radius_iterate(pcity->x, pcity->y, x, y) {
+	int i = map_inx(x, y);
+	pcmap->claims[i].cities |= (1u << pcity->owner);
+      } map_city_radius_iterate_end;
+    } city_list_iterate_end;
+  } players_iterate_end;
 }
 
-/* 0th turn: install starting points, counting their tiles */
-
-static void build_landarea_map_turn_0 (struct claim_map *pcmap)
+/**************************************************************************
+0th turn: install starting points, counting their tiles
+**************************************************************************/
+static void build_landarea_map_turn_0(struct claim_map *pcmap)
 {
-  int x, y, i;
   int turn;
   struct map_position *nextedge;
   struct claim_cell *pclaim;
@@ -256,66 +250,53 @@ static void build_landarea_map_turn_0 (struct claim_map *pcmap)
   turn = 0;
   nextedge = pcmap->edges;
 
-  for (y = 0; y < map.ysize; y++)
-    {
-      for (x = 0, i = y * map.xsize; x < map.xsize; x++, i++)
-	{
-	  pclaim = &(pcmap->claims[i]);
-	  ptile = &(map.tiles[i]);
+  whole_map_iterate(x, y) {
+    int i = map_inx(x, y);
+    pclaim = &(pcmap->claims[i]);
+    ptile = &(map.tiles[i]);
 
-	  if (ptile->terrain == T_OCEAN)
-	    {
-	      /* pclaim->when = 0; */
-	      pclaim->whom = no_owner;
-	      /* pclaim->know = 0; */
-	    }
-	  else if (ptile->city)
-	    {
-	      owner = ptile->city->owner;
-	      pclaim->when = turn + 1;
-	      pclaim->whom = owner;
-	      nextedge->x = x;
-	      nextedge->y = y;
-	      nextedge++;
-	      (pcmap->player_landarea[owner])++;
-	      (pcmap->player_owndarea[owner])++;
-	      pclaim->know = ptile->known;
-	    }
-	  else if (ptile->worked)
-	    {
-	      owner = ptile->worked->owner;
-	      pclaim->when = turn + 1;
-	      pclaim->whom = owner;
-	      nextedge->x = x;
-	      nextedge->y = y;
-	      nextedge++;
-	      (pcmap->player_landarea[owner])++;
-	      (pcmap->player_owndarea[owner])++;
-	      pclaim->know = ptile->known;
-	    }
-	  else if (unit_list_size (&(ptile->units)) > 0)
-	    {
-	      owner = (unit_list_get (&(ptile->units), 0))->owner;
-	      pclaim->when = turn + 1;
-	      pclaim->whom = owner;
-	      nextedge->x = x;
-	      nextedge->y = y;
-	      nextedge++;
-	      (pcmap->player_landarea[owner])++;
-	      if (pclaim->cities & (1u << owner))
-		{
-		  (pcmap->player_owndarea[owner])++;
-		}
-	      pclaim->know = ptile->known;
-	    }
-	  else
-	    {
-	      /* pclaim->when = 0; */
-	      pclaim->whom = no_owner;
-	      pclaim->know = ptile->known;
-	    }
-	}
+    if (ptile->terrain == T_OCEAN) {
+      /* pclaim->when = 0; */
+      pclaim->whom = no_owner;
+      /* pclaim->know = 0; */
+    } else if (ptile->city) {
+      owner = ptile->city->owner;
+      pclaim->when = turn + 1;
+      pclaim->whom = owner;
+      nextedge->x = x;
+      nextedge->y = y;
+      nextedge++;
+      (pcmap->player_landarea[owner])++;
+      (pcmap->player_owndarea[owner])++;
+      pclaim->know = ptile->known;
+    } else if (ptile->worked) {
+      owner = ptile->worked->owner;
+      pclaim->when = turn + 1;
+      pclaim->whom = owner;
+      nextedge->x = x;
+      nextedge->y = y;
+      nextedge++;
+      (pcmap->player_landarea[owner])++;
+      (pcmap->player_owndarea[owner])++;
+      pclaim->know = ptile->known;
+    } else if (unit_list_size (&(ptile->units)) > 0) {
+      owner = (unit_list_get (&(ptile->units), 0))->owner;
+      pclaim->when = turn + 1;
+      pclaim->whom = owner;
+      nextedge->x = x;
+      nextedge->y = y;
+      nextedge++;
+      (pcmap->player_landarea[owner])++;
+      if (pclaim->cities & (1u << owner)) {
+	(pcmap->player_owndarea[owner])++;
+      }
+      pclaim->know = ptile->known;
+    } else {
+      /* pclaim->when = 0; */
+      pclaim->whom = no_owner;
+      pclaim->know = ptile->known;
     }
+  } whole_map_iterate_end;
 
   nextedge->x = -1;
 
@@ -324,11 +305,13 @@ static void build_landarea_map_turn_0 (struct claim_map *pcmap)
 #endif
 }
 
-/* expand outwards evenly from each starting point, counting tiles */
 
-static void build_landarea_map_expand (struct claim_map *pcmap)
+/**************************************************************************
+expand outwards evenly from each starting point, counting tiles
+**************************************************************************/
+static void build_landarea_map_expand(struct claim_map *pcmap)
 {
-  int x, y, n, i, j, mx, my;
+  int x, y, i, j;
   struct map_position *midedge;
   int turn;
   struct map_position *thisedge;
@@ -336,81 +319,64 @@ static void build_landarea_map_expand (struct claim_map *pcmap)
   int accum;
   struct claim_cell *pclaim;
   int owner, other;
-  static int nx[8] = { -1,  0,  1, -1,  1, -1,  0,  1 };
-  static int ny[8] = { -1, -1, -1,  0,  0,  1,  1,  1 };
 
   midedge = &(pcmap->edges[map.xsize * map.ysize]);
 
-  for (accum = 1, turn = 1; accum > 0; turn++)
-    {
-      thisedge = (turn & 0x1) ? pcmap->edges : midedge;
-      nextedge = (turn & 0x1) ? midedge : pcmap->edges;
+  for (accum = 1, turn = 1; accum > 0; turn++) {
+    thisedge = (turn & 0x1) ? pcmap->edges : midedge;
+    nextedge = (turn & 0x1) ? midedge : pcmap->edges;
 
-      for (accum = 0; thisedge->x >= 0; thisedge++)
-	{
-	  x = thisedge->x;
-	  y = thisedge->y;
-	  i = map_inx (x, y);
-	  owner = pcmap->claims[i].whom;
+    for (accum = 0; thisedge->x >= 0; thisedge++) {
+      x = thisedge->x;
+      y = thisedge->y;
+      i = map_inx (x, y);
+      owner = pcmap->claims[i].whom;
 
-	  if (owner != no_owner)
-	    {
-	      for (n = 0; n < 8; n++)
-		{
-		  my = y + ny[n];
-		  if ((my < 0) || (my >= map.ysize))
-		    {
-		      continue;
-		    }
-		  mx = map_adjust_x (x + nx[n]);
-		  j = map_inx (mx, my);
-		  pclaim = &(pcmap->claims[j]);
+      if (owner != no_owner) {
+	adjc_iterate(x, y, mx, my) {
+	  j = map_inx(mx, my);
+	  pclaim = &(pcmap->claims[j]);
 
-		  if (pclaim->know & (1u << owner))
-		    {
-		      if (pclaim->when == 0)
-			{
-			  pclaim->when = turn + 1;
-			  pclaim->whom = owner;
-			  nextedge->x = mx;
-			  nextedge->y = my;
-			  nextedge++;
-			  (pcmap->player_landarea[owner])++;
-			  if (pclaim->cities & (1u << owner))
-			    {
-			      (pcmap->player_owndarea[owner])++;
-			    }
-			  accum++;
-			}
-		      else if ((pclaim->when == (turn + 1)) &&
-			       (pclaim->whom != no_owner) &&
-			       (pclaim->whom != owner))
-			{
-			  other = pclaim->whom;
-			  if (pclaim->cities & (1u << other))
-			    {
-			      (pcmap->player_owndarea[other])--;
-			    }
-			  (pcmap->player_landarea[other])--;
-			  pclaim->whom = no_owner;
-			  accum--;
-			}
-		    }
-		}
+	  if (pclaim->know & (1u << owner)) {
+	    if (pclaim->when == 0) {
+	      pclaim->when = turn + 1;
+	      pclaim->whom = owner;
+	      nextedge->x = mx;
+	      nextedge->y = my;
+	      nextedge++;
+	      (pcmap->player_landarea[owner])++;
+	      if (pclaim->cities & (1u << owner)) {
+		(pcmap->player_owndarea[owner])++;
+	      }
+	      accum++;
+	    } else if ((pclaim->when == (turn + 1)) &&
+		       (pclaim->whom != no_owner) &&
+		       (pclaim->whom != owner)) {
+	      other = pclaim->whom;
+	      if (pclaim->cities & (1u << other)) {
+		(pcmap->player_owndarea[other])--;
+	      }
+	      (pcmap->player_landarea[other])--;
+	      pclaim->whom = no_owner;
+	      accum--;
 	    }
-	}
+	  }
+	} adjc_iterate_end;
+      }
+    }
 
-      nextedge->x = -1;
+    nextedge->x = -1;
 
 #if LAND_AREA_DEBUG >= 2
-      print_landarea_map (pcmap, turn);
+    print_landarea_map (pcmap, turn);
 #endif
-    }
+  }
 }
 
-/* this just calls the three worker routines */
-
-static void build_landarea_map (struct claim_map *pcmap)
+/**************************************************************************
+this just calls the three worker routines
+**************************************************************************/
+static void build_landarea_map(struct claim_map *pcmap)
 {
   build_landarea_map_new (pcmap);
   build_landarea_map_turn_0 (pcmap);
@@ -420,82 +386,70 @@ static void build_landarea_map (struct claim_map *pcmap)
 /**************************************************************************
 Frees and NULLs an allocated claim map.
 **************************************************************************/
-
-static void free_landarea_map (struct claim_map *pcmap)
+static void free_landarea_map(struct claim_map *pcmap)
 {
-  if (pcmap)
-    {
-      if (pcmap->claims)
-	{
-	  free (pcmap->claims);
-	  pcmap->claims = NULL;
-	}
-      if (pcmap->player_landarea)
-	{
-	  free (pcmap->player_landarea);
-	  pcmap->player_landarea = NULL;
-	}
-      if (pcmap->player_owndarea)
-	{
-	  free (pcmap->player_owndarea);
-	  pcmap->player_owndarea = NULL;
-	}
-      if (pcmap->edges)
-	{
-	  free (pcmap->edges);
-	  pcmap->edges = NULL;
-	}
+  if (pcmap) {
+    if (pcmap->claims) {
+      free (pcmap->claims);
+      pcmap->claims = NULL;
     }
+    if (pcmap->player_landarea) {
+      free (pcmap->player_landarea);
+      pcmap->player_landarea = NULL;
+    }
+    if (pcmap->player_owndarea) {
+      free (pcmap->player_owndarea);
+      pcmap->player_owndarea = NULL;
+    }
+    if (pcmap->edges) {
+      free (pcmap->edges);
+      pcmap->edges = NULL;
+    }
+  }
 }
 
 /**************************************************************************
 Returns the given player's land and settled areas from a claim map.
 **************************************************************************/
-
-static void get_player_landarea (struct claim_map *pcmap, struct player *pplayer,
+static void get_player_landarea(struct claim_map *pcmap, struct player *pplayer,
 				 int *return_landarea, int *return_settledarea)
 {
-  if (pcmap && pplayer)
-    {
+  if (pcmap && pplayer) {
 #if LAND_AREA_DEBUG >= 1
-      printf ("%-14s", pplayer->name);
+    printf ("%-14s", pplayer->name);
 #endif
-      if (return_landarea && pcmap->player_landarea)
-	{
-	  *return_landarea =
-	    USER_AREA_MULT * pcmap->player_landarea[pplayer->player_no];
+    if (return_landarea && pcmap->player_landarea) {
+      *return_landarea =
+	USER_AREA_MULT * pcmap->player_landarea[pplayer->player_no];
 #if LAND_AREA_DEBUG >= 1
-	  printf (" l=%d", *return_landarea / USER_AREA_MULT);
-#endif
-	}
-      if (return_settledarea && pcmap->player_owndarea)
-	{
-	  *return_settledarea =
-	    USER_AREA_MULT * pcmap->player_owndarea[pplayer->player_no];
-#if LAND_AREA_DEBUG >= 1
-	  printf (" s=%d", *return_settledarea / USER_AREA_MULT);
-#endif
-	}
-#if LAND_AREA_DEBUG >= 1
-      printf ("\n");
+      printf (" l=%d", *return_landarea / USER_AREA_MULT);
 #endif
     }
+    if (return_settledarea && pcmap->player_owndarea) {
+      *return_settledarea =
+	USER_AREA_MULT * pcmap->player_owndarea[pplayer->player_no];
+#if LAND_AREA_DEBUG >= 1
+      printf (" s=%d", *return_settledarea / USER_AREA_MULT);
+#endif
+    }
+#if LAND_AREA_DEBUG >= 1
+    printf ("\n");
+#endif
+  }
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-
 int research_time(struct player *pplayer)
 {
   int timemod=(game.year>0) ? 2:1;
-  return timemod*pplayer->research.researchpoints*game.techlevel;
+  return timemod*pplayer->research.researchpoints*game.researchcost;
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-
 int total_player_citizens(struct player *pplayer)
 {
   return (pplayer->score.happy
@@ -586,7 +540,7 @@ int civ_score(struct player *pplayer)
     if (is_military_unit(punit)) pplayer->score.units++;
   unit_list_iterate_end;
   
-  for (i=0;i<B_LAST;i++) {
+  for (i=0;i<game.num_impr_types;i++) {
     if (is_wonder(i) && (pcity=find_city_by_id(game.global_wonders[i])) && 
 	player_owns_city(pplayer, pcity))
       pplayer->score.wonders++;
@@ -698,15 +652,13 @@ void game_remove_unit(int unit_id)
 **************************************************************************/
 void game_remove_city(struct city *pcity)
 {
-  int x,y;
-  
   freelog(LOG_DEBUG, "game_remove_city %d", pcity->id);
   freelog(LOG_DEBUG, "removing city %s, %s, (%d %d)", pcity->name,
 	   get_nation_name(city_owner(pcity)->nation), pcity->x, pcity->y);
   
   city_map_iterate(x,y) {
     set_worker_city(pcity, x, y, C_TILE_EMPTY);
-  }
+  } city_map_iterate_end;
   city_list_unlink(&game.players[pcity->owner].cities, pcity);
   map_set_city(pcity->x, pcity->y, NULL);
   idex_unregister_city(pcity);
@@ -720,22 +672,26 @@ void game_remove_city(struct city *pcity)
 void game_init(void)
 {
   int i;
-  game.is_new_game = 1;
-  game.globalwarming=0;
-  game.warminglevel=8;
-  game.nuclearwinter=0;
-  game.coolinglevel=8;
-  game.gold        = GAME_DEFAULT_GOLD;
-  game.tech        = GAME_DEFAULT_TECHLEVEL;
-  game.skill_level = GAME_DEFAULT_SKILL_LEVEL;
-  game.timeout     = GAME_DEFAULT_TIMEOUT;
-  game.end_year    = GAME_DEFAULT_END_YEAR;
-  game.year        = GAME_START_YEAR;
-  game.min_players = GAME_DEFAULT_MIN_PLAYERS;
-  game.max_players = GAME_DEFAULT_MAX_PLAYERS;
+  game.is_new_game   = 1;
+  game.globalwarming = 0;
+  game.warminglevel  = 8;
+  game.nuclearwinter = 0;
+  game.coolinglevel  = 8;
+  game.gold          = GAME_DEFAULT_GOLD;
+  game.tech          = GAME_DEFAULT_TECHLEVEL;
+  game.skill_level   = GAME_DEFAULT_SKILL_LEVEL;
+  game.timeout       = GAME_DEFAULT_TIMEOUT;
+  game.tcptimeout    = GAME_DEFAULT_TCPTIMEOUT;
+  game.netwait       = GAME_DEFAULT_NETWAIT;
+  game.last_ping     = 0;
+  game.pingtimeout   = GAME_DEFAULT_PINGTIMEOUT;
+  game.end_year      = GAME_DEFAULT_END_YEAR;
+  game.year          = GAME_START_YEAR;
+  game.min_players   = GAME_DEFAULT_MIN_PLAYERS;
+  game.max_players  = GAME_DEFAULT_MAX_PLAYERS;
   game.aifill      = GAME_DEFAULT_AIFILL;
   game.nplayers=0;
-  game.techlevel   = GAME_DEFAULT_RESEARCHLEVEL;
+  game.researchcost = GAME_DEFAULT_RESEARCHCOST;
   game.diplcost    = GAME_DEFAULT_DIPLCOST;
   game.diplchance  = GAME_DEFAULT_DIPLCHANCE;
   game.freecost    = GAME_DEFAULT_FREECOST;
@@ -763,7 +719,7 @@ void game_init(void)
   game.occupychance= GAME_DEFAULT_OCCUPYCHANCE;
   game.heating     = 0;
   game.cooling     = 0;
-  sz_strlcpy(game.save_name, "civgame");
+  sz_strlcpy(game.save_name, GAME_DEFAULT_SAVE_NAME);
   game.save_nturns=10;
 #ifdef HAVE_LIBZ
   game.save_compress_level = GAME_DEFAULT_COMPRESS_LEVEL;
@@ -782,6 +738,7 @@ void game_init(void)
   sz_strlcpy(game.ruleset.game,        GAME_DEFAULT_RULESET);
   game.firepower_factor = 1;
   game.num_unit_types = 0;
+  game.num_impr_types = 0;
   game.num_tech_types = 0;
 
   game.government_count = 0;
@@ -790,35 +747,55 @@ void game_init(void)
   game.ai_goal_government = G_MAGIC;        /* flag */
 
   sz_strlcpy(game.demography, GAME_DEFAULT_DEMOGRAPHY);
+  sz_strlcpy(game.allow_connect, GAME_DEFAULT_ALLOW_CONNECT);
 
+  game.save_options.save_random = 1;
+  game.save_options.save_players = 1;
+  game.save_options.save_known = 1;
+  game.save_options.save_starts = 1;
+  game.save_options.save_private_map = 1;
+
+  game.load_options.load_random = 1;
+  game.load_options.load_players = 1;
+  game.load_options.load_known = 1;
+  game.load_options.load_starts = 1;
+  game.load_options.load_private_map = 1;
+  game.load_options.load_settings = 1;
+  
   map_init();
   idex_init();
   
+  conn_list_init(&game.all_connections);
+  conn_list_init(&game.est_connections);
+  conn_list_init(&game.game_connections);
+  
   for(i=0; i<MAX_NUM_PLAYERS+MAX_NUM_BARBARIANS; i++)
     player_init(&game.players[i]);
   for (i=0; i<A_LAST; i++)      /* game.num_tech_types = 0 here */
     game.global_advances[i]=0;
-  for (i=0; i<B_LAST; i++)
+  for (i=0; i<B_LAST; i++)      /* game.num_impr_types = 0 here */
     game.global_wonders[i]=0;
+  game.conn_id = 0;
   game.player_idx=0;
   game.player_ptr=&game.players[0];
   terrain_control.river_help_text = NULL;
 }
 
+/***************************************************************
+...
+***************************************************************/
 void initialize_globals(void)
 {
-  int i,j;
-  struct player *plr;
-  for (j=0;j<game.nplayers;j++) {
-    plr=&game.players[j];
+  int i;
+
+  players_iterate(plr) {
     city_list_iterate(plr->cities, pcity) {
-      for (i=0;i<B_LAST;i++) {
+      for (i=0;i<game.num_impr_types;i++) {
 	if (city_got_building(pcity, i) && is_wonder(i))
-	  game.global_wonders[i]=pcity->id;
+	  game.global_wonders[i] = pcity->id;
       }
-    }
-    city_list_iterate_end;
-  }
+    } city_list_iterate_end;
+  } players_iterate_end;
 }
 
 /***************************************************************
@@ -924,6 +901,9 @@ void game_renumber_players(int plrno)
   for(i=plrno; i<game.nplayers-1; ++i) {
     game.players[i]=game.players[i+1];
     game.players[i].player_no=i;
+    conn_list_iterate(game.players[i].connections, pconn)
+      pconn->player = &game.players[i];
+    conn_list_iterate_end;
   }
 
   if(game.player_idx>plrno) {
@@ -961,54 +941,68 @@ int get_num_human_and_ai_players(void)
 ***************************************************************/
 void translate_data_names(void)
 {
-  int i, j;
+  int i;
+  static const char too_long_msg[]
+    = "Translated name is too long, truncating: %s";
+
+#define name_strlcpy(dst, src) sz_loud_strlcpy(dst, src, too_long_msg)
   
   for (i=0; i<game.num_tech_types; i++) {
     struct advance *tthis = &advances[i];
     sz_strlcpy(tthis->name_orig, tthis->name);
-    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    name_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
   for (i=0; i<game.num_unit_types; i++) {
     struct unit_type *tthis = &unit_types[i];
     sz_strlcpy(tthis->name_orig, tthis->name);
-    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    name_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
-  for (i=0; i<B_LAST; i++) {
+  for (i=0; i<game.num_impr_types; i++) {
     struct impr_type *tthis = &improvement_types[i];
     sz_strlcpy(tthis->name_orig, tthis->name);
-    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    name_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
   for (i=T_FIRST; i<T_COUNT; i++) {
     struct tile_type *tthis = &tile_types[i];
     sz_strlcpy(tthis->terrain_name_orig, tthis->terrain_name);
-    sz_strlcpy(tthis->terrain_name, Q_(tthis->terrain_name_orig));
+    name_strlcpy(tthis->terrain_name,
+		 strcmp(tthis->terrain_name_orig, "") ?
+			Q_(tthis->terrain_name_orig) : "");
     sz_strlcpy(tthis->special_1_name_orig, tthis->special_1_name);
-    sz_strlcpy(tthis->special_1_name, Q_(tthis->special_1_name_orig));
+    name_strlcpy(tthis->special_1_name,
+		 strcmp(tthis->special_1_name_orig, "") ?
+			Q_(tthis->special_1_name_orig) : "");
     sz_strlcpy(tthis->special_2_name_orig, tthis->special_2_name);
-    sz_strlcpy(tthis->special_2_name, Q_(tthis->special_2_name_orig));
+    name_strlcpy(tthis->special_2_name,
+		 strcmp(tthis->special_2_name_orig, "") ?
+			Q_(tthis->special_2_name_orig) : "");
   }
   for (i=0; i<game.government_count; i++) {
+    int j;
     struct government *tthis = &governments[i];
     sz_strlcpy(tthis->name_orig, tthis->name);
-    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    name_strlcpy(tthis->name, Q_(tthis->name_orig));
     for(j=0; j<tthis->num_ruler_titles; j++) {
       struct ruler_title *that = &tthis->ruler_titles[j];
       sz_strlcpy(that->male_title_orig, that->male_title);
-      sz_strlcpy(that->male_title, Q_(that->male_title_orig));
+      name_strlcpy(that->male_title, Q_(that->male_title_orig));
       sz_strlcpy(that->female_title_orig, that->female_title);
-      sz_strlcpy(that->female_title, Q_(that->female_title_orig));
+      name_strlcpy(that->female_title, Q_(that->female_title_orig));
     }
   }
   for (i=0; i<game.nation_count; i++) {
     struct nation_type *tthis = get_nation_by_idx(i);
     sz_strlcpy(tthis->name_orig, tthis->name);
-    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    name_strlcpy(tthis->name, Q_(tthis->name_orig));
     sz_strlcpy(tthis->name_plural_orig, tthis->name_plural);
-    sz_strlcpy(tthis->name_plural, Q_(tthis->name_plural_orig));
+    name_strlcpy(tthis->name_plural, Q_(tthis->name_plural_orig));
   }
   for (i=0; i<game.styles_count; i++) {
     struct citystyle *tthis = &city_styles[i];
     sz_strlcpy(tthis->name_orig, tthis->name);
-    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    name_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
+
+#undef name_strlcpy
+
 }
