@@ -18,10 +18,12 @@
 #include <sys/time.h>
 #endif
 
+#include "connection.h"		/* struct conn_list */
 #include "shared.h"
 #include "player.h"
 
 #define MAX_LEN_DEMOGRAPHY  16
+#define MAX_LEN_ALLOW_CONNECT 16
 
 enum server_states { 
   PRE_GAME_STATE, 
@@ -57,10 +59,14 @@ struct civ_game {
   int tech;
   int skill_level;
   int timeout;
+  int tcptimeout;
+  int netwait;
+  time_t last_ping;
+  int pingtimeout;
   time_t turn_start;
   int end_year;
   int year;
-  int techlevel;
+  int researchcost; /* Multiplier on cost of new research */
   int diplcost, freecost, conquercost;
   int diplchance;
   int cityfactor;
@@ -73,20 +79,28 @@ struct civ_game {
   int occupychance;
   int unhappysize;
   char *startmessage;
+  int conn_id;			/* client-only: id client known to server as */
   int player_idx;
   struct player *player_ptr;
   struct player players[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
+  struct conn_list all_connections;        /* including not yet established */
+  struct conn_list est_connections;        /* all established client conns */
+  struct conn_list game_connections;       /* involved in game; send map etc */
   int global_advances[A_LAST];             /* a counter */
   int global_wonders[B_LAST];              /* contains city id's */
          /* global_wonders[] may also be (-1), or the id of a city
 	    which no longer exists, if the wonder has been destroyed */
-  int globalwarming;                       /* counter of how disturbed 
-					      mother nature is */
-  int heating;
-  int warminglevel;
-  int nuclearwinter;
-  int cooling;
-  int coolinglevel;
+
+  int heating; /* Number of polluted squares. */
+  int globalwarming; /* Total damage done. (counts towards a warming event.) */
+  int warminglevel; /* If globalwarming is higher than this number there is
+		       a chance of a warming event. */
+
+  int cooling; /* Number of irradiated squares. */
+  int nuclearwinter; /* Total damage done. (counts towards a cooling event.) */
+  int coolinglevel; /* If nuclearwinter is higher than this number there is
+		       a chance of a cooling event. */
+
   char save_name[MAX_LEN_NAME];
   int save_nturns;
   int save_compress_level;
@@ -109,6 +123,7 @@ struct civ_game {
 			   the server we need to remember the old setting */
 
   int num_unit_types;
+  int num_impr_types;
   int num_tech_types;  /* including A_NONE */
 
   int government_count;
@@ -159,11 +174,33 @@ struct civ_game {
     int hut_overflight;
     int pillage_select;
     int nuke_contamination;
+    int granary_food_ini;
+    int granary_food_inc;
   } rgame;
 
   char demography[MAX_LEN_DEMOGRAPHY];
+  char allow_connect[MAX_LEN_ALLOW_CONNECT];
+
+  /* used by the map editor to control game_save; could be used by the server too */
+  struct {
+    int save_random;
+    int save_players;
+    int save_known; /* loading will just reveal the squares around cities and units */
+    int save_starts; /* start positions will be auto generated */
+    int save_private_map; /* FoW map; will be created if not saved */
+  } save_options;
+  /* Used by mapeditor to load only the map */
+  struct {
+    int load_random;
+    int load_players;
+    int load_known; /* Only makes sense if the players are loaded. */
+    int load_starts; /* Should be done if the players are loaded. */
+    int load_private_map; /* Only makes sense if the players are loaded. */
+    int load_settings;
+  } load_options;
 };
 
+/* Unused? */
 struct lvldat {
   int advspeed;
 };
@@ -240,9 +277,9 @@ extern struct civ_game game;
 #define GAME_MIN_AIFILL              0
 #define GAME_MAX_AIFILL              GAME_MAX_MAX_PLAYERS
 
-#define GAME_DEFAULT_RESEARCHLEVEL   10
-#define GAME_MIN_RESEARCHLEVEL       4
-#define GAME_MAX_RESEARCHLEVEL       100
+#define GAME_DEFAULT_RESEARCHCOST	   10
+#define GAME_MIN_RESEARCHCOST	       4
+#define GAME_MAX_RESEARCHCOST        100
 
 #define GAME_DEFAULT_DIPLCOST        0
 #define GAME_MIN_DIPLCOST            0
@@ -312,6 +349,18 @@ extern struct civ_game game;
 #define GAME_MIN_TIMEOUT             0
 #define GAME_MAX_TIMEOUT             8639999
 
+#define GAME_DEFAULT_TCPTIMEOUT      10
+#define GAME_MIN_TCPTIMEOUT          0
+#define GAME_MAX_TCPTIMEOUT          120
+
+#define GAME_DEFAULT_NETWAIT         4
+#define GAME_MIN_NETWAIT             0
+#define GAME_MAX_NETWAIT             20
+
+#define GAME_DEFAULT_PINGTIMEOUT     60
+#define GAME_MIN_PINGTIMEOUT         60
+#define GAME_MAX_PINGTIMEOUT         1800
+
 #define GAME_DEFAULT_BARBARIANRATE   2
 #define GAME_MIN_BARBARIANRATE       0
 #define GAME_MAX_BARBARIANRATE       4
@@ -325,12 +374,14 @@ extern struct civ_game game;
 #define GAME_MIN_OCCUPYCHANCE        0
 #define GAME_MAX_OCCUPYCHANCE        100
 
+#define GAME_DEFAULT_SAVE_NAME       "civgame"
 #define GAME_DEFAULT_RULESET         "default"
 
 #define GAME_DEFAULT_SKILL_LEVEL 3      /* easy */
 #define GAME_OLD_DEFAULT_SKILL_LEVEL 5  /* normal; for old save games */
 
 #define GAME_DEFAULT_DEMOGRAPHY      "NASRLPEMOqrb"
+#define GAME_DEFAULT_ALLOW_CONNECT   "NHAhad"
 
 #define GAME_DEFAULT_COMPRESS_LEVEL 6    /* if we have compression */
 #define GAME_MIN_COMPRESS_LEVEL     0
