@@ -62,11 +62,11 @@ struct government *governments;
 
 struct ai_gov_tech_hint ai_gov_tech_hints[MAX_NUM_TECH_LIST];
 
-static char *flag_names[] = {
+static const char *flag_names[] = {
   "Build_Veteran_Diplomats", "Revolution_When_Unhappy", "Has_Senate",
   "Unbribable", "Inspires_Partisans", "Rapture_City_Growth"
 };
-static char *hint_names[] = {
+static const char *hint_names[] = {
   "Is_Nice", "Favors_Growth"
 };
 
@@ -200,7 +200,7 @@ char *get_ruler_title(int gov, int male, int nation)
   if (best_match) {
     return male ? best_match->male_title : best_match->female_title;
   } else {
-    freelog(LOG_NORMAL,
+    freelog(LOG_ERROR,
 	    "get_ruler_title: found no title for government %d (%s) nation %d",
 	    gov, g->name, nation);
     return male ? "Mr." : "Ms.";
