@@ -39,8 +39,8 @@ static int logd_num_files;
 static struct logd_fileinfo *logd_files;
 
 /**************************************************************************
-level_str should be either "0", "1", "2", "3" or
-"3:filename" or "3:file1:file2" or "3:filename,100,200" etc
+level_str should be either "0", "1", "2", "3", "4" or
+"4:filename" or "4:file1:file2" or "4:filename,100,200" etc
 
 If everything goes ok, returns the level.
 If there was a parsing problem, prints to stderr, and returns -1.
@@ -90,8 +90,13 @@ int log_parse_level_str(char *level_str)
   }
 
   c = level_str;
-  if (c[0] == '3' && c[1] == ':') {
-    level = 3;
+  if (c[0] == ('0' + LOG_DEBUG) && c[1] == ':') {
+    level = LOG_DEBUG;
+    if (max_level < LOG_DEBUG) {
+      fprintf(stderr, _("Freeciv must be compiled with the DEBUG flag"
+			" to use debug level %d.\n"), LOG_DEBUG);
+      return -1;
+    }
   } else {
     fprintf(stderr, _("Badly formed log level argument \"%s\".\n"), level_str);
     return -1;
@@ -217,7 +222,7 @@ For repeat message, may wait and print instead
 Calls log_callback if non-null, else prints to stderr.
 **************************************************************************/
 #define MAX_LEN_LOG_LINE 512
-void vreal_freelog(int level, char *message, va_list ap)
+void vreal_freelog(int level, const char *message, va_list ap)
 {
   static char bufbuf[2][MAX_LEN_LOG_LINE];
   char buf[MAX_LEN_LOG_LINE];
@@ -283,7 +288,7 @@ void vreal_freelog(int level, char *message, va_list ap)
       fclose(fs);
   }
 }
-void real_freelog(int level, char *message, ...)
+void real_freelog(int level, const char *message, ...)
 {
   va_list ap;
   va_start(ap, message);
