@@ -18,9 +18,10 @@
 #include "attribute.h"
 
 #define LOG_FATAL   0
-#define LOG_NORMAL  1
-#define LOG_VERBOSE 2		/* not shown by default */
-#define LOG_DEBUG   3		/* suppressed unless DEBUG defined;
+#define LOG_ERROR   1		/* non-fatal errors */
+#define LOG_NORMAL  2
+#define LOG_VERBOSE 3		/* not shown by default */
+#define LOG_DEBUG   4		/* suppressed unless DEBUG defined;
 				   may be enabled on file/line basis */
 
 /* Some variables local to each file which includes log.h,
@@ -53,9 +54,9 @@ int log_parse_level_str(char *level_str);
 void log_init(char *filename, int initial_level, log_callback_fn callback);
 void log_set_level(int level);
 
-void real_freelog(int level, char *message, ...)
+void real_freelog(int level, const char *message, ...)
                   fc__attribute((format (printf, 2, 3)));
-void vreal_freelog(int level, char *message, va_list ap);
+void vreal_freelog(int level, const char *message, va_list ap);
 
 
 /* A static (per-file) function to use/update the above per-file vars.
@@ -90,18 +91,18 @@ static int logdebug_check(const char *file, int line)
 #ifdef DEBUG
 #define freelog(level, args...) do { \
   if ((level) != LOG_DEBUG || logdebug_check(__FILE__, __LINE__)) { \
-    real_freelog((level), ##args); \
+    real_freelog((level), args); \
   } \
 } while(0) 
 #else
 #define freelog(level, args...) do { \
   if ((level) != LOG_DEBUG) { \
-    real_freelog((level), ##args); } \
+    real_freelog((level), args); } \
 } while(0) 
 #endif  /* DEBUG */
 #else
 /* non-GCC: */
-static void freelog(int level, char *message, ...)
+static void freelog(int level, const char *message, ...)
 {
   int log_this; 
 #ifdef DEBUG
