@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -23,6 +24,7 @@
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
+#include "rand.h"
 #include "shared.h"
 #include "support.h"
 #include "unit.h"
@@ -43,10 +45,18 @@ struct tile_type tile_types[T_LAST];
 const int DIR_DX[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
 const int DIR_DY[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
 
+/* like DIR_DX[] and DIR_DY[], only cartesian */
+const int CAR_DIR_DX[4] = {1, 0, -1, 0};
+const int CAR_DIR_DY[4] = {0, 1, 0, -1};
+
+/* used to compute neighboring tiles */
+const int DIR_DX2[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
+const int DIR_DY2[8] = { -1, -1, 0, 1, 1, 1, 0, -1 };
+
 /* Names of specials.
  * (These must correspond to enum tile_special_type in terrain.h.)
  */
-static char *tile_special_type_names[] =
+static const char *tile_special_type_names[] =
 {
   N_("Special1"),
   N_("Road"),
@@ -63,6 +73,8 @@ static char *tile_special_type_names[] =
   N_("Fallout")
 };
 
+extern int is_server;
+
 /***************************************************************
   Return a (static) string with terrain name;
   eg: "Hills"
@@ -130,6 +142,20 @@ char *map_get_tile_info_text(int x, int y)
   return s;
 }
 
+/***************************************************************
+  Return a (static) string with a tile's food/prod/trade
+***************************************************************/
+char *map_get_tile_fpt_text(int x, int y)
+{
+  static char s[64];
+  
+  my_snprintf(s, sizeof(s), "%d/%d/%d",
+	      get_food_tile(x, y),
+	      get_shields_tile(x, y),
+	      get_trade_tile(x, y));
+  return s;
+}
+
 /***************************************************************
   Returns 1 if we are at a stage of the game where the map
   has not yet been generated/loaded.
@@ -167,6 +193,8 @@ void map_init(void)
   map.num_start_positions=0;
   map.fixed_start_positions=0;
   map.have_specials = 0;
+  map.have_rivers_overlay = 0;
+  map.have_huts = 0;
 
   tile_init(&void_tile);
 }
@@ -180,17 +208,13 @@ void map_init(void)
 **************************************************************************/
 void map_allocate(void)
 {
-  int x,y;
-
   freelog(LOG_DEBUG, "map_allocate (was %p) (%d,%d)",
 	  map.tiles, map.xsize, map.ysize);
   
   map.tiles=fc_realloc(map.tiles, map.xsize*map.ysize*sizeof(struct tile));
-  for(y=0; y<map.ysize; y++) {
-    for(x=0; x<map.xsize; x++) {
-      tile_init(map_get_tile(x, y));
-    }
-  }
+  whole_map_iterate(x, y) {
+    tile_init(map_get_tile(x, y));
+  } whole_map_iterate_end;
 }
 
 
@@ -251,7 +275,7 @@ enum tile_special_type get_special_by_name(char * name)
 /***************************************************************
 ...
 ***************************************************************/
-char *get_special_name(enum tile_special_type type)
+const char *get_special_name(enum tile_special_type type)
 {
   int inx;
 
@@ -267,16 +291,29 @@ char *get_special_name(enum tile_special_type type)
   return NULL;
 }
 
+/***************************************************************
+...
+***************************************************************/
+int xdist(int x0, int x1)
+{
+  int dist = (x0 > x1) ? x0 - x1 : x1 - x0;
+  return MIN(dist, map.xsize-dist);
+}
+
+/***************************************************************
+...
+***************************************************************/
+int ydist(int y0, int y1)
+{
+  return (y0 > y1) ? y0 - y1 : y1 - y0;
+}
+
 /***************************************************************
 ...
 ***************************************************************/
 int real_map_distance(int x0, int y0, int x1, int y1)
 {
-  int tmp;
-  if(y0>y1)
-    tmp=y0, y0=y1, y1=tmp;
-  tmp=map_adjust_x(x0-x1);
-  return MAX(y1 - y0, MIN(tmp, map.xsize-tmp));
+  return MAX(xdist(x0, x1), ydist(y0, y1));
 }
 
 /***************************************************************
@@ -284,10 +321,9 @@ int real_map_distance(int x0, int y0, int x1, int y1)
 ***************************************************************/
 int sq_map_distance(int x0, int y0, int x1, int y1)
 {
-  int tmp;
-  tmp=map_adjust_x(x0-x1);
-  tmp= MIN(tmp, map.xsize-tmp);
-  return (((y1 - y0) * (y1 - y0)) + tmp * tmp);
+  int xd = xdist(x0, x1);
+  int yd = ydist(y0, y1);
+  return (xd*xd + yd*yd);
 }
 
 /***************************************************************
@@ -295,47 +331,20 @@ int sq_map_distance(int x0, int y0, int x1, int y1)
 ***************************************************************/
 int map_distance(int x0, int y0, int x1, int y1)
 {
-  int tmp;
-
-  if(y0>y1)
-    tmp=y0, y0=y1, y1=tmp;
-  tmp = map_adjust_x(x0-x1);
-  return MIN(tmp, map.xsize-tmp)+y1-y0;
+  return xdist(x0, x1) + ydist(y0, y1);
 }
 
 /***************************************************************
- The following two function should always be used, such
- that hand optimizations need only be performed once.
- using isnt_ is faster than using is_ sometimes.
- is terrain close diagonally or gridwise ?
+...
 ***************************************************************/
 int is_terrain_near_tile(int x, int y, enum tile_terrain_type t)
 {
-  if (map_get_terrain(x-1, y+1)==t) return 1;
-  if (map_get_terrain(x+1, y-1)==t) return 1;
-  if (map_get_terrain(x-1, y-1)==t) return 1;
-  if (map_get_terrain(x+1, y+1)==t) return 1;
-  if (map_get_terrain(x, y+1)==t)   return 1;
-  if (map_get_terrain(x-1, y)==t)   return 1;
-  if (map_get_terrain(x+1, y)==t)   return 1;
-  if (map_get_terrain(x, y-1)==t)   return 1;
-  return 0;
-}
+  adjc_iterate(x, y, x1, y1) {
+    if (map_get_terrain(x1, y1) == t)
+      return 1;
+  } adjc_iterate_end;
 
-/***************************************************************
- is no terrain close diagonally or gridwise ?
-***************************************************************/
-int isnt_terrain_near_tile(int x, int y, enum tile_terrain_type t)
-{
-  if (map_get_terrain(x-1, y+1)==t) return 0;
-  if (map_get_terrain(x+1, y-1)==t) return 0;
-  if (map_get_terrain(x-1, y-1)==t) return 0;
-  if (map_get_terrain(x+1, y+1)==t) return 0;
-  if (map_get_terrain(x, y+1)==t)   return 0;
-  if (map_get_terrain(x-1, y)==t)   return 0;
-  if (map_get_terrain(x+1, y)==t)   return 0;
-  if (map_get_terrain(x, y-1)==t)   return 0;
-  return 1;
+  return 0;
 }
 
 /***************************************************************
@@ -343,16 +352,14 @@ int isnt_terrain_near_tile(int x, int y, enum tile_terrain_type t)
 ***************************************************************/
 int count_terrain_near_tile(int x, int y, enum tile_terrain_type t)
 {
-  int rval= 0;
-  if (map_get_terrain(x, y+1)==t)   rval++;
-  if (map_get_terrain(x-1, y-1)==t) rval++;
-  if (map_get_terrain(x-1, y)==t)   rval++;
-  if (map_get_terrain(x-1, y+1)==t) rval++;
-  if (map_get_terrain(x+1, y-1)==t) rval++;
-  if (map_get_terrain(x+1, y)==t)   rval++;
-  if (map_get_terrain(x+1, y+1)==t) rval++;
-  if (map_get_terrain(x, y-1)==t)   rval++;
-  return rval;
+  int count = 0;
+
+  adjc_iterate(x, y, x1, y1) {
+    if (map_get_terrain(x1, y1) == t)
+      count++;
+  } adjc_iterate_end;
+
+  return count;
 }
 
 /***************************************************************
@@ -360,14 +367,11 @@ int count_terrain_near_tile(int x, int y, enum tile_terrain_type t)
 ***************************************************************/
 int is_special_near_tile(int x, int y, enum tile_special_type spe)
 {
-  if (map_get_special(x-1, y+1)&spe) return 1;
-  if (map_get_special(x+1, y-1)&spe) return 1;
-  if (map_get_special(x-1, y-1)&spe) return 1;
-  if (map_get_special(x+1, y+1)&spe) return 1;
-  if (map_get_special(x, y+1)&spe)   return 1;
-  if (map_get_special(x-1, y)&spe)   return 1;
-  if (map_get_special(x+1, y)&spe)   return 1;
-  if (map_get_special(x, y-1)&spe)   return 1;
+  adjc_iterate(x, y, x1, y1) {
+    if (map_get_special(x1, y1) & spe)
+      return 1;
+  } adjc_iterate_end;
+
   return 0;
 }
 
@@ -376,16 +380,14 @@ int is_special_near_tile(int x, int y, enum tile_special_type spe)
 ***************************************************************/
 int count_special_near_tile(int x, int y, enum tile_special_type spe)
 {
-  int rval= 0;
-  if (map_get_special(x, y+1)&spe)   rval++;
-  if (map_get_special(x-1, y-1)&spe) rval++;
-  if (map_get_special(x-1, y)&spe)   rval++;
-  if (map_get_special(x-1, y+1)&spe) rval++;
-  if (map_get_special(x+1, y-1)&spe) rval++;
-  if (map_get_special(x+1, y)&spe)   rval++;
-  if (map_get_special(x+1, y+1)&spe) rval++;
-  if (map_get_special(x, y-1)&spe)   rval++;
-  return rval;
+  int count = 0;
+
+  adjc_iterate(x, y, x1, y1) {
+    if (map_get_special(x1, y1) & spe)
+      count++;
+  } adjc_iterate_end;
+
+  return count;
 }
 
 /***************************************************************
@@ -393,28 +395,26 @@ int count_special_near_tile(int x, int y, enum tile_special_type spe)
 ***************************************************************/
 int is_at_coast(int x, int y)
 {
-  if (map_get_terrain(x-1,y)==T_OCEAN)   return 1;
-  if (map_get_terrain(x,y-1)==T_OCEAN)   return 1;
-  if (map_get_terrain(x,y+1)==T_OCEAN)   return 1;
-  if (map_get_terrain(x+1,y)==T_OCEAN)   return 1;
+  cartesian_adjacent_iterate(x, y, x1, y1) {
+    if (map_get_terrain(x1, y1) == T_OCEAN)
+      return 1;
+  } cartesian_adjacent_iterate_end;
+
   return 0;
-  
 }
 
 /***************************************************************
 ...
 ***************************************************************/
-int is_coastline(int x,int y) 
+int is_coastline(int x, int y)
 {
-  /*if (map_get_terrain(x,y)!=T_OCEAN)   return 0;*/
-  if (map_get_terrain(x-1,y)!=T_OCEAN)   return 1;
-  if (map_get_terrain(x-1,y-1)!=T_OCEAN) return 1;
-  if (map_get_terrain(x-1,y+1)!=T_OCEAN) return 1;
-  if (map_get_terrain(x,y-1)!=T_OCEAN)   return 1;
-  if (map_get_terrain(x,y+1)!=T_OCEAN)   return 1;
-  if (map_get_terrain(x+1,y-1)!=T_OCEAN) return 1;
-  if (map_get_terrain(x+1,y)!=T_OCEAN)   return 1;
-  if (map_get_terrain(x+1,y+1)!=T_OCEAN) return 1;
+  adjc_iterate(x, y, x1, y1) {
+    enum tile_terrain_type ter = map_get_terrain(x1, y1);
+    if (ter != T_OCEAN
+	&& ter != T_UNKNOWN)
+      return 1;
+  } adjc_iterate_end;
+
   return 0;
 }
 
@@ -423,11 +423,12 @@ int is_coastline(int x,int y)
 ***************************************************************/
 int terrain_is_clean(int x, int y)
 {
-  int x1,y1;
-  for (x1=x-3;x1<x+3;x1++)
-    for (y1=y-3;y1<y+3;y1++) 
-      if (map_get_terrain(x1,y1)!=T_GRASSLAND
-	  && map_get_terrain(x1,y1)!=T_PLAINS) return 0;
+  square_iterate(x, y, 2, x1, y1) {
+    if (map_get_terrain(x1,y1) != T_GRASSLAND
+	&& map_get_terrain(x1,y1) != T_PLAINS)
+      return 0;
+  } square_iterate_end;
+
   return 1;
 }
 
@@ -464,7 +465,7 @@ int is_starter_close(int x, int y, int nr, int dist)
     return 1;
   
   /* don't want them starting on the poles: */
-  if (map_get_continent(x, y)<=2)
+  if (map_get_continent(x, y)<=2 && map.generator != 0)
     return 1;
 
   /* don't start too close to someone else: */
@@ -519,34 +520,16 @@ int is_good_tile(int x, int y)
   return 0;
 }
 
-
-/***************************************************************
-...
-***************************************************************/
-int is_water_adjacent(int x, int y)
-{
-  int x1,y1;
-  for (y1=y-1;y1<y+2;y1++) 
-    for (x1=x-1;x1<x+2;x1++) {
-      if (map_get_terrain(x1,y1)==T_OCEAN
-	  || map_get_terrain(x1, y1)==T_RIVER
-	  || map_get_special(x1, y1)&S_RIVER)
-	return 1;
-    } 
-  return 0;
-}
-
 /***************************************************************
 ...
 ***************************************************************/
 int is_hut_close(int x, int y)
 {
-  int x1,y1;
-  for (y1=y-3;y1<y+4;y1++) 
-    for (x1=x-3;x1<x+4;x1++) {
-      if (map_get_tile(x1,y1)->special&S_HUT)
-	return 1;
-    } 
+  square_iterate(x, y, 3, x1, y1) {
+    if (map_get_tile(x1, y1)->special & S_HUT)
+      return 1;
+  } square_iterate_end;
+
   return 0;
 }
 
@@ -556,38 +539,25 @@ int is_hut_close(int x, int y)
 ***************************************************************/
 int is_special_close(int x, int y)
 {
-  int x1,y1;
-  for (x1=x-1;x1<x+2;x1++)
-    for (y1=y-1;y1<=y+2;y1++) 
-      if(map_get_tile(x1,y1)->special&(S_SPECIAL_1 | S_SPECIAL_2))
-	return 1;
+  square_iterate(x, y, 1, x1, y1) {
+    if (map_get_tile(x1, y1)->special & (S_SPECIAL_1 | S_SPECIAL_2))
+      return 1;
+  } square_iterate_end;
+
   return 0;
 }
 
 /***************************************************************
-...
-***************************************************************/
-int is_special_type_close(int x, int y, enum tile_special_type spe)
-{
-  int x1,y1;
-  for (x1=x-1;x1<x+2;x1++)
-    for (y1=y-1;y1<=y+2;y1++) 
-      if((map_get_tile(x1,y1)->special)&spe)
-	return 1;
-  return 0;
-}
-
-/***************************************************************
-...
+Returns whether you can put a city on land near enough to use
+the tile.
 ***************************************************************/
 int is_sea_usable(int x, int y)
 {
-  int x1,y1;
-  for (x1=x-2;x1<x+3;x1++)
-    for (y1=y-2;y1<=y+3;y1++) 
-      if( !( (x==x1-2||x==x1+2) && (y==y1-2||y==y1+2) ) )
-	if(map_get_terrain(x1,y1)!=T_OCEAN)
-	  return 1;
+  map_city_radius_iterate(x, y, x1, y1) {
+    if (map_get_terrain(x1, y1) != T_OCEAN)
+      return 1;
+  } map_city_radius_iterate_end;
+
   return 0;
 }
 
@@ -643,7 +613,7 @@ int get_tile_infrastructure_set(struct tile * ptile)
 /***************************************************************
   Return a (static) string with special(s) name(s);
   eg: "Mine"
-  eg: "Road/Irrigation/Farmland"
+  eg: "Road/Farmland"
 ***************************************************************/
 char *map_get_infrastructure_text(int spe)
 {
@@ -651,14 +621,16 @@ char *map_get_infrastructure_text(int spe)
 
   *s = '\0';
 
-  if(spe&S_ROAD)
-    cat_snprintf(s, sizeof(s), "%s/", _("Road"));
+  /* Since railroad requires road, Road/Railroad is redundant */
   if(spe&S_RAILROAD)
     cat_snprintf(s, sizeof(s), "%s/", _("Railroad"));
-  if(spe&S_IRRIGATION)
-    cat_snprintf(s, sizeof(s), "%s/", _("Irrigation"));
+  else if(spe&S_ROAD)
+    cat_snprintf(s, sizeof(s), "%s/", _("Road"));
+  /* Likewise for farmland on irrigation */
   if(spe&S_FARMLAND)
     cat_snprintf(s, sizeof(s), "%s/", _("Farmland"));
+  else if(spe&S_IRRIGATION)
+    cat_snprintf(s, sizeof(s), "%s/", _("Irrigation"));
   if(spe&S_MINE)
     cat_snprintf(s, sizeof(s), "%s/", _("Mine"));
   if(spe&S_FORTRESS)
@@ -716,24 +688,25 @@ int get_preferred_pillage(int pset)
 ***************************************************************/
 int is_water_adjacent_to_tile(int x, int y)
 {
-  struct tile *ptile, *ptile_n, *ptile_e, *ptile_s, *ptile_w;
+  struct tile *ptile;
 
-  ptile=map_get_tile(x, y);
-  ptile_n=map_get_tile(x, y-1);
-  ptile_e=map_get_tile(x+1, y);
-  ptile_s=map_get_tile(x, y+1);
-  ptile_w=map_get_tile(x-1, y);
+  ptile = map_get_tile(x, y);
+  if (ptile->terrain == T_OCEAN
+      || ptile->terrain == T_RIVER
+      || ptile->special & S_RIVER
+      || ptile->special & S_IRRIGATION)
+    return 1;
 
-  return (ptile->terrain==T_OCEAN   || ptile->terrain==T_RIVER
-	  || ptile->special&S_RIVER || ptile->special&S_IRRIGATION ||
-	  ptile_n->terrain==T_OCEAN || ptile_n->terrain==T_RIVER
-	  || ptile_n->special&S_RIVER || ptile_n->special&S_IRRIGATION ||
-	  ptile_e->terrain==T_OCEAN || ptile_e->terrain==T_RIVER
-	  || ptile_e->special&S_RIVER || ptile_e->special&S_IRRIGATION ||
-	  ptile_s->terrain==T_OCEAN || ptile_s->terrain==T_RIVER
-	  || ptile_s->special&S_RIVER || ptile_s->special&S_IRRIGATION ||
-	  ptile_w->terrain==T_OCEAN || ptile_w->terrain==T_RIVER
-	  || ptile_w->special&S_RIVER || ptile_w->special&S_IRRIGATION);
+  cartesian_adjacent_iterate(x, y, x1, y1) {
+    ptile = map_get_tile(x1, y1);
+    if (ptile->terrain == T_OCEAN
+	|| ptile->terrain == T_RIVER
+	|| ptile->special & S_RIVER
+	|| ptile->special & S_IRRIGATION)
+      return 1;
+  } cartesian_adjacent_iterate_end;
+
+  return 0;
 }
 
 /***************************************************************
@@ -847,6 +820,33 @@ void map_mine_tile(int x, int y)
   map_clear_special(x,y, S_IRRIGATION);
 }
 
+/***************************************************************
+...
+***************************************************************/
+void change_terrain(int x, int y, enum tile_terrain_type type)
+{
+  map_set_terrain(x, y, type);
+  if (type==T_OCEAN) {
+    clear_infrastructure(x, y);
+    clear_dirtiness(x, y);
+    map_clear_special(x, y, S_RIVER);	/* FIXME: When rest of code can handle
+					   rivers in oceans, don't clear this! */
+  }
+
+  reset_move_costs(x, y);
+
+  /* Clear mining/irrigation if resulting terrain type cannot support
+     that feature.  (With current rules, this should only clear mines,
+     but I'm including both cases in the most general form for possible
+     future ruleset expansion. -GJW) */
+  if (tile_types[type].mining_result != type)
+    map_clear_special(x, y, S_MINE);
+  if (tile_types[type].irrigation_result != type) {
+    map_clear_special(x, y, S_FARMLAND);
+    map_clear_special(x, y, S_IRRIGATION);
+  }
+}
+
 /***************************************************************
 ...
 ***************************************************************/
@@ -857,27 +857,60 @@ void map_transform_tile(int x, int y)
   now = map_get_terrain(x, y);
   result = tile_types[now].transform_result;
   
-  if (result != T_LAST) {
-    map_set_terrain(x, y, result);
-    if (result==T_OCEAN) {
-      clear_infrastructure(x, y);
-      clear_dirtiness(x, y);
-      map_clear_special(x, y, S_RIVER);	/* FIXME: When rest of code can handle
-					   rivers in oceans, don't clear this! */
-    }
-    reset_move_costs(x, y);
-  }
+  if (result != T_LAST)
+    change_terrain(x, y, result);
+}
 
-  /* Clear mining/irrigation if resulting terrain type cannot support
-     that feature.  (With current rules, this should only clear mines,
-     but I'm including both cases in the most general form for possible
-     future ruleset expansion. -GJW) */
-  if (tile_types[result].mining_result != result)
-    map_clear_special(x, y, S_MINE);
-  if (tile_types[result].irrigation_result != result) {
-    map_clear_special(x, y, S_FARMLAND);
-    map_clear_special(x, y, S_IRRIGATION);
-  }
+/**************************************************************************
+This function returns true if the tile at the given location can be
+"reclaimed" from ocean into land.  This is the case only when there are
+a sufficient number of adjacent tiles that are not ocean.
+**************************************************************************/
+int can_reclaim_ocean(int x, int y)
+{
+  int landtiles;
+
+  if (terrain_control.ocean_reclaim_requirement >= 9)
+    return 0;
+  if (terrain_control.ocean_reclaim_requirement <= 0)
+    return 1;
+
+  landtiles = 0;
+  adjc_iterate(x, y, x1, y1) {
+    if (map_get_tile(x1, y1)->terrain != T_OCEAN) {
+      landtiles++;
+    }
+    if (landtiles >= terrain_control.ocean_reclaim_requirement)
+      return 1;
+  } adjc_iterate_end;
+
+  return 0;
+}
+
+/**************************************************************************
+This function returns true if the tile at the given location can be
+"channeled" from land into ocean.  This is the case only when there are
+a sufficient number of adjacent tiles that are ocean.
+**************************************************************************/
+int can_channel_land(int x, int y)
+{
+  int oceantiles;
+
+  if (terrain_control.land_channel_requirement >= 9)
+    return 0;
+  if (terrain_control.land_channel_requirement <= 0)
+    return 1;
+
+  oceantiles = 0;
+  adjc_iterate(x, y, x1, y1) {
+    if (map_get_tile(x1, y1)->terrain == T_OCEAN) {
+      oceantiles++;
+    }
+    if (oceantiles >= terrain_control.land_channel_requirement)
+      return 1;
+  } adjc_iterate_end;
+
+  return 0;
 }
 
 /***************************************************************
@@ -895,13 +928,14 @@ static int tile_move_cost_ptrs(struct unit *punit, struct tile *t1,
   int cardinal_move;
 
   if (punit && !is_ground_unit(punit))
-    return 3;
+    return SINGLE_MOVE;
   if( (t1->special&S_RAILROAD) && (t2->special&S_RAILROAD) )
-    return 0;
+    return MOVE_COST_RAIL;
+/* return (unit_move_rate(punit)/RAIL_MAX) */
   if (punit && unit_flag(punit->type, F_IGTER))
-    return 1;
+    return SINGLE_MOVE/3;
   if( (t1->special&S_ROAD) && (t2->special&S_ROAD) )
-    return 1;
+    return MOVE_COST_ROAD;
 
   if( ( (t1->terrain==T_RIVER) && (t2->terrain==T_RIVER) ) ||
       ( (t1->special&S_RIVER) && (t2->special&S_RIVER) ) ) {
@@ -911,21 +945,21 @@ static int tile_move_cost_ptrs(struct unit *punit, struct tile *t1,
       break;
     case RMV_FAST_STRICT:
       if (cardinal_move)
-	return 1;
+	return MOVE_COST_RIVER;
       break;
     case RMV_FAST_RELAXED:
       if (cardinal_move)
-	return 1;
+	return MOVE_COST_RIVER;
       else
-	return 2;
+	return 2 * MOVE_COST_RIVER;
     case RMV_FAST_ALWAYS:
-      return 1;
+      return MOVE_COST_RIVER;
     default:
       break;
     }
   }
 
-  return(get_tile_type(t2->terrain)->movement_cost*3);
+  return(get_tile_type(t2->terrain)->movement_cost*SINGLE_MOVE);
 }
 
 /***************************************************************
@@ -940,12 +974,14 @@ static int tile_move_cost_ptrs(struct unit *punit, struct tile *t1,
 static int tile_move_cost_ai(struct tile *tile0, struct tile *tile1,
 			     int x, int y, int x1, int y1, int maxcost)
 {
-  if (tile0->terrain == T_UNKNOWN || tile1->terrain == T_UNKNOWN) {
-    return maxcost;
-  } else if (tile0->terrain == T_OCEAN) {
-    return (tile1->city || tile1->terrain == T_OCEAN) ? -3 : maxcost;
+  assert(y >= 0 && y <= map.ysize);
+  assert(!is_server || (tile0->terrain != T_UNKNOWN && tile1->terrain != T_UNKNOWN));
+
+  if (tile0->terrain == T_OCEAN) {
+    return (tile1->city || tile1->terrain == T_OCEAN
+	    || tile1->terrain == T_UNKNOWN) ? -3 : maxcost;
   } else if (tile1->terrain == T_OCEAN) {
-    return (tile0->city) ? -3 : maxcost;
+    return (tile0->city || tile0->terrain == T_UNKNOWN) ? -3 : maxcost;
   } else {
     return tile_move_cost_ptrs(NULL, tile0, tile1, x, y, x1, y1);
   }
@@ -982,15 +1018,19 @@ void reset_move_costs(int x, int y)
   debug_log_move_costs("Resetting move costs for", x, y, tile0);
 
   for (dir = 0; dir < 8; dir++) {
-    x1 = map_adjust_x(x + DIR_DX[dir]);
+    x1 = x + DIR_DX[dir];
     y1 = y + DIR_DY[dir];
-    tile1 = map_get_tile(x1, y1);
-    tile0->move_cost[dir] = tile_move_cost_ai(tile0, tile1, x, y,
-					      x1, y1, maxcost);
-    /* reverse: not at all obfuscated now --dwp */
-    /* this might muck with void_tile, but who cares? */
-    tile1->move_cost[7 - dir] = tile_move_cost_ai(tile1, tile0, x1, y1,
-						x, y, maxcost);
+    if (normalize_map_pos(&x1, &y1)) {
+      tile1 = map_get_tile(x1, y1);
+      tile0->move_cost[dir] = tile_move_cost_ai(tile0, tile1, x, y,
+						x1, y1, maxcost);
+      /* reverse: not at all obfuscated now --dwp */
+      tile1->move_cost[7 - dir] = tile_move_cost_ai(tile1, tile0, x1, y1,
+						    x, y, maxcost);
+    } else {
+      /* trying to move off the screen. */
+      tile0->move_cost[dir] = maxcost;
+    }
   }
   debug_log_move_costs("Reset move costs for", x, y, tile0);
 }
@@ -1002,26 +1042,29 @@ void reset_move_costs(int x, int y)
 ***************************************************************/
 void initialize_move_costs(void)
 {
-  int x, y, x1, y1, dir;
   int maxcost = 72; /* should be big enough without being TOO big */
-  struct tile *tile0, *tile1;
+  int dir;
 
   for (dir = 0; dir < 8; dir++) {
     void_tile.move_cost[dir] = maxcost;
   }
 
-  for (x = 0; x < map.xsize; x++) {
-    for (y = 0; y < map.ysize; y++) {
-      tile0 = map_get_tile(x, y);
-      for (dir = 0; dir < 8; dir++) {
-        x1 = map_adjust_x(x + DIR_DX[dir]);
-        y1 = y + DIR_DY[dir];
-        tile1 = map_get_tile(x1, y1);
-        tile0->move_cost[dir] = tile_move_cost_ai(tile0, tile1, x, y,
-						x1, y1, maxcost);
+  whole_map_iterate(x, y) {
+    struct tile *tile0, *tile1;
+    int x1, y1;
+    tile0 = map_get_tile(x, y);
+    for (dir = 0; dir < 8; dir++) {
+      x1 = map_adjust_x(x + DIR_DX[dir]);
+      y1 = y + DIR_DY[dir];
+      if (normalize_map_pos(&x1, &y1)) {
+	tile1 = map_get_tile(x1, y1);
+	tile0->move_cost[dir] = tile_move_cost_ai(tile0, tile1, x, y,
+						  x1, y1, maxcost);
+      } else {
+	tile0->move_cost[dir] = maxcost;
       }
     }
-  }
+  } whole_map_iterate_end;
 }
 
 /***************************************************************
@@ -1039,12 +1082,12 @@ int map_move_cost(struct unit *punit, int x, int y)
 ***************************************************************/
 int is_tiles_adjacent(int x0, int y0, int x1, int y1)
 {
-  x0 = map_adjust_x(x0);
-  x1 = map_adjust_x(x1);
-  if (same_pos(x0, y0, x1, y1))
-    return 1;
-  return (((x0<=x1+1 && x0>=x1-1) || (x0==0 && x1==map.xsize-1) ||
-	   (x0==map.xsize-1 && x1==0)) && (y0<=y1+1 && y0>=y1-1));
+  adjc_iterate(x0, y0, x, y) {
+    if (x == x1 && y == y1)
+      return 1;
+  } adjc_iterate_end;
+
+  return 0;
 }
 
 
@@ -1092,7 +1135,8 @@ signed short map_get_continent(int x, int y)
 ***************************************************************/
 void map_set_continent(int x, int y, int val)
 {
-    (map.tiles + map_adjust_x(x) + y * map.xsize)->continent=val;
+  assert(is_real_tile(x, y));
+  (map.tiles + map_adjust_x(x) + y * map.xsize)->continent=val;
 }
 
 
@@ -1171,8 +1215,8 @@ Only for use on the client side
 ***************************************************************/
 enum known_type tile_is_known(int x, int y)
 {
-  return ((map.tiles+map_adjust_x(x)+
-	   map_adjust_y(y)*map.xsize)->known);
+  return (enum known_type) (((map.tiles+map_adjust_x(x)+
+	   map_adjust_y(y)*map.xsize)->known));
 }
 
 /***************************************************************
@@ -1185,6 +1229,26 @@ int same_pos(int x1, int y1, int x2, int y2)
 	  && map_adjust_y(y1) == map_adjust_y(y2)); 
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+int check_coords(int *x, int *y)
+{
+  int save_x = *x, save_y = *y;
+
+  if (!normalize_map_pos(x, y)) {
+    freelog(LOG_ERROR, "%d, %d is not a real tile!", *x, *y);
+    *x = map_adjust_x(*x);
+    *y = map_adjust_y(*y);
+    return 0;
+  }
+
+  if (*x != save_x || *y != save_y)
+    freelog(LOG_ERROR, "Had to adjust coords %d, %d", *x, *y);
+
+  return 1;
+}
+
 /***************************************************************
 If we just adjust we might light a tile twice; That would be a
 Mayor problem with fog of war
@@ -1201,10 +1265,32 @@ int normalize_map_pos(int *x, int *y) {
   if (*y < 0 || *y >= map.ysize)
     return FALSE;
 
-  if (*x < 0)
+  while (*x < 0)
     *x += map.xsize;
-  else if (*x >= map.xsize)
+  while (*x > map.xsize-1)
     *x -= map.xsize;
 
   return TRUE;
 }
+
+/**************************************************************************
+Random neighbouring square.
+**************************************************************************/
+void rand_neighbour(int x0, int y0, int *x, int *y)
+{
+  int choice;
+
+  if (y0 == 0) {
+    choice = 3 + myrand(5);
+  } else if(y0 == map.ysize-1){
+    choice = myrand(5);
+  } else {
+    choice = myrand(8);
+  }
+
+  *x = x0 + DIR_DX[choice];
+  *y = y0 + DIR_DY[choice];
+
+  assert(is_real_tile(*x, *y));
+  normalize_map_pos(x, y);
+}
