@@ -26,6 +26,17 @@ struct map_position {
   int x,y;
 };
 
+struct goto_route {
+  int first_index; /* first valid tile pos */
+  int last_index; /* point to the first non_legal pos. Note that the pos
+		   is always alloced in the pos array (for coding reasons) */
+  int length; /* length of pos array (use this as modulus when iterating)
+		 Note that this is always at least 1 greater than the number
+		 of valid positions, to make comparing first_index with
+		 last_index during wrapped iteration easier. */
+  struct map_position *pos;
+};
+
 struct tile {
   enum tile_terrain_type terrain;
   enum tile_special_type special;
@@ -150,6 +161,8 @@ struct civ_map {
   int num_start_positions;
   int fixed_start_positions;
   int have_specials;
+  int have_huts;
+  int have_rivers_overlay;	/* only applies if !have_specials */
   int num_continents;
   struct tile *tiles;
   struct map_position start_positions[MAX_NUM_NATIONS];
@@ -160,8 +173,11 @@ void map_init(void);
 void map_allocate(void);
 
 char *map_get_tile_info_text(int x, int y);
+char *map_get_tile_fpt_text(int x, int y);
 struct tile *map_get_tile(int x, int y);
 
+int xdist(int x0, int x1);
+int ydist(int y0, int y1);
 int map_distance(int x0, int y0, int x1, int y1);
 int real_map_distance(int x0, int y0, int x1, int y1);
 int sq_map_distance(int x0, int y0, int x1, int y1);
@@ -174,8 +190,13 @@ int map_same_continent(int x1, int y1, int x2, int y2);
 void initialize_move_costs(void);
 void reset_move_costs(int x, int y);
 
-#define map_adjust_x(X) \
- (((X)<0) ?  (X)+map.xsize : (((X)>=map.xsize) ? (X)-map.xsize : (X)))
+#define map_adjust_x(X)            \
+  ((X) < 0                         \
+   ? ((X) % map.xsize != 0 ? (X) % map.xsize + map.xsize : 0) \
+   : ((X) >= map.xsize             \
+      ? (X) % map.xsize            \
+      : (X)))
+
 #define map_adjust_y(Y) \
   (((Y)<0) ? 0 : (((Y)>=map.ysize) ? map.ysize-1 : (Y)))
 
@@ -191,8 +212,10 @@ void map_set_special(int x, int y, enum tile_special_type spe);
 void map_clear_special(int x, int y, enum tile_special_type spe);
 void tile_init(struct tile *ptile);
 enum known_type tile_is_known(int x, int y);
+int check_coords(int *x, int *y);
 int is_real_tile(int x, int y);
 int normalize_map_pos(int *x, int *y);
+void rand_neighbour(int x0, int y0, int *x, int *y);
 
 int is_water_adjacent_to_tile(int x, int y);
 int is_tiles_adjacent(int x0, int y0, int x1, int y1);
@@ -201,21 +224,18 @@ struct tile_type *get_tile_type(enum tile_terrain_type type);
 enum tile_terrain_type get_terrain_by_name(char * name);
 char *get_terrain_name(enum tile_terrain_type type);
 enum tile_special_type get_special_by_name(char * name);
-char *get_special_name(enum tile_special_type type);
+const char *get_special_name(enum tile_special_type type);
 int is_terrain_near_tile(int x, int y, enum tile_terrain_type t);
-int isnt_terrain_near_tile(int x, int y, enum tile_terrain_type t);
 int count_terrain_near_tile(int x, int y, enum tile_terrain_type t);
 int is_special_near_tile(int x, int y, enum tile_special_type spe);
 int count_special_near_tile(int x, int y, enum tile_special_type spe);
 int is_coastline(int x,int y);
 int terrain_is_clean(int x, int y);
 int is_at_coast(int x, int y);
-int is_water_adjacent(int x, int y);
 int is_hut_close(int x, int y);
 int is_starter_close(int x, int y, int nr, int dist); 
 int is_good_tile(int x, int y);
 int is_special_close(int x, int y);
-int is_special_type_close(int x, int y, enum tile_special_type spe);
 int is_sea_usable(int x, int y);
 int get_tile_food_base(struct tile * ptile);
 int get_tile_shield_base(struct tile * ptile);
@@ -227,7 +247,10 @@ int get_preferred_pillage(int pset);
 
 void map_irrigate_tile(int x, int y);
 void map_mine_tile(int x, int y);
+void change_terrain(int x, int y, enum tile_terrain_type type);
 void map_transform_tile(int x, int y);
+int can_reclaim_ocean(int x, int y);
+int can_channel_land(int x, int y);
 
 int map_build_road_time(int x, int y);
 int map_build_irrigation_time(int x, int y);
@@ -245,8 +268,6 @@ extern struct tile_type tile_types[T_LAST];
    The values given are adjusted.
    You should make sure that the arguments passed to the macro are adjusted,
    or you could have some very nasty intermediate errors.
-   The results are returned through ARG_x_itr and ARG_y_itr,
-   which must already be defined.
 
    Internally it works by for a given distance
    1) assume y positive and iterate over x
@@ -261,36 +282,37 @@ extern struct tile_type tile_types[T_LAST];
 */
 #define iterate_outward(ARG_start_x, ARG_start_y, ARG_max_dist, ARG_x_itr, ARG_y_itr) \
 {                                                                             \
+  int ARG_x_itr, ARG_y_itr;                                                   \
   int MACRO_max_dx = map.xsize/2;                                             \
   int MACRO_min_dx = -(MACRO_max_dx - (map.xsize%2 ? 0 : 1));                 \
   int MACRO_xcycle = 1;                                                       \
   int MACRO_positive = 0;                                                     \
   int MACRO_dxy = 0, MACRO_do_xy;                                             \
-  while(MACRO_dxy <= ARG_max_dist) {                                          \
+  while(MACRO_dxy <= (ARG_max_dist)) {                                        \
     for (MACRO_do_xy = -MACRO_dxy; MACRO_do_xy <= MACRO_dxy; MACRO_do_xy++) { \
       if (MACRO_xcycle) {                                                     \
-	ARG_x_itr = ARG_start_x + MACRO_do_xy;                                \
+	ARG_x_itr = (ARG_start_x) + MACRO_do_xy;                              \
 	if (MACRO_positive)                                                   \
-	  ARG_y_itr = ARG_start_y + MACRO_dxy;                                \
+	  ARG_y_itr = (ARG_start_y) + MACRO_dxy;                              \
 	else                                                                  \
-	  ARG_y_itr = ARG_start_y - MACRO_dxy;                                \
+	  ARG_y_itr = (ARG_start_y) - MACRO_dxy;                              \
       } else { /* ! MACRO_xcycle */                                           \
         if (MACRO_dxy == MACRO_do_xy || MACRO_dxy == -MACRO_do_xy)            \
           continue;                                                           \
-	ARG_y_itr = ARG_start_y + MACRO_do_xy;                                \
+	ARG_y_itr = (ARG_start_y) + MACRO_do_xy;                              \
 	if (MACRO_positive)                                                   \
-	  ARG_x_itr = ARG_start_x + MACRO_dxy;                                \
+	  ARG_x_itr = (ARG_start_x) + MACRO_dxy;                              \
 	else                                                                  \
-	  ARG_x_itr = ARG_start_x - MACRO_dxy;                                \
+	  ARG_x_itr = (ARG_start_x) - MACRO_dxy;                              \
       }                                                                       \
       if (ARG_y_itr<0 || ARG_y_itr >= map.ysize)                              \
 	continue;                                                             \
       {                                                                       \
-	int MACRO_dx = ARG_start_x - ARG_x_itr;                               \
+	int MACRO_dx = (ARG_start_x) - ARG_x_itr;                             \
 	if (MACRO_dx > MACRO_max_dx || MACRO_dx < MACRO_min_dx)               \
 	  continue;                                                           \
       }                                                                       \
-      if (ARG_x_itr > map.xsize)                                              \
+      if (ARG_x_itr >= map.xsize)                                             \
 	ARG_x_itr -= map.xsize;                                               \
       else if (ARG_x_itr < 0)                                                 \
 	ARG_x_itr += map.xsize;
@@ -312,13 +334,14 @@ extern struct tile_type tile_types[T_LAST];
  */
 #define square_iterate(SI_center_x, SI_center_y, radius, SI_x_itr, SI_y_itr)  \
 {                                                                             \
+  int SI_x_itr, SI_y_itr;                                                     \
   int SI_x_itr1;                                                              \
-  for (SI_y_itr = SI_center_y - radius;                                       \
-       SI_y_itr <= SI_center_y + radius; SI_y_itr++) {                        \
+  for (SI_y_itr = (SI_center_y) - (radius);                                   \
+       SI_y_itr <= (SI_center_y) + (radius); SI_y_itr++) {                    \
     if (SI_y_itr < 0 || SI_y_itr >= map.ysize)                                \
       continue;                                                               \
-    for (SI_x_itr1 = SI_center_x - radius;                                    \
-	 SI_x_itr1 <= SI_center_x + radius; SI_x_itr1++) {                    \
+    for (SI_x_itr1 = (SI_center_x) - (radius);                                \
+	 SI_x_itr1 <= (SI_center_x) + (radius); SI_x_itr1++) {                \
       SI_x_itr = map_adjust_x(SI_x_itr1);
 
 #define square_iterate_end                                                    \
@@ -326,6 +349,66 @@ extern struct tile_type tile_types[T_LAST];
   }                                                                           \
 }
 
+/* Iterate through all tiles adjacent to a tile */
+#define adjc_iterate(RI_center_x, RI_center_y, RI_x_itr, RI_y_itr)            \
+{                                                                             \
+  int RI_x_itr, RI_y_itr;                                                     \
+  int RI_x_itr1;                                                              \
+  for (RI_y_itr = RI_center_y - 1;                                            \
+       RI_y_itr <= RI_center_y + 1; RI_y_itr++) {                             \
+    if (RI_y_itr < 0 || RI_y_itr >= map.ysize)                                \
+      continue;                                                               \
+    for (RI_x_itr1 = RI_center_x - 1;                                         \
+	 RI_x_itr1 <= RI_center_x + 1; RI_x_itr1++) {                         \
+      RI_x_itr = map_adjust_x(RI_x_itr1);                                     \
+      if (RI_x_itr == RI_center_x && RI_y_itr == RI_center_y)                 \
+        continue; 
+
+#define adjc_iterate_end                                                      \
+    }                                                                         \
+  }                                                                           \
+}
+
+/* Iterate through all tiles adjacent to a tile.  dir_itr is the
+   directional value (see DIR_D[XY]).  This assumes that center_x and
+   center_y are normalized. --JDS */
+#define adjc_dir_iterate(center_x, center_y, x_itr, y_itr, dir_itr)           \
+{                                                                             \
+  int x_itr, y_itr, dir_itr, MACRO_border;                                    \
+  int MACRO_center_x = (center_x);                                            \
+  int MACRO_center_y = (center_y);                                            \
+  assert(0 <= MACRO_center_y && MACRO_center_y < map.ysize                    \
+         && 0 <= MACRO_center_x && MACRO_center_x < map.xsize);               \
+  MACRO_border = (MACRO_center_y == 0                                         \
+                  || MACRO_center_x == 0                                      \
+                  || MACRO_center_y == map.ysize-1                            \
+                  || MACRO_center_x == map.xsize-1);                          \
+  for (dir_itr = 0; dir_itr < 8; dir_itr++) {                                 \
+    y_itr = MACRO_center_y + DIR_DY[dir_itr];                                 \
+    x_itr = MACRO_center_x + DIR_DX[dir_itr];                                 \
+    if (MACRO_border) {                                                       \
+      if (y_itr < 0 || y_itr >= map.ysize)                                    \
+        continue;                                                             \
+      if (x_itr < 0)                                                          \
+        x_itr += map.xsize;                                                   \
+      else if (x_itr >= map.xsize)                                            \
+        x_itr -= map.xsize;                                                   \
+    }
+
+#define adjc_iterate_dir_end                                                  \
+  }                                                                           \
+}
+
+/* iterating y, x for cache efficiency */
+#define whole_map_iterate(WMI_x_itr, WMI_y_itr)                               \
+{                                                                             \
+  int WMI_x_itr, WMI_y_itr;                                                   \
+  for (WMI_y_itr = 0; WMI_y_itr < map.ysize; WMI_y_itr++)                     \
+    for (WMI_x_itr = 0; WMI_x_itr < map.xsize; WMI_x_itr++)
+
+#define whole_map_iterate_end                                                 \
+}
+
 /*
 used to compute neighboring tiles:
 using
@@ -343,6 +426,66 @@ will give you the tile as shown below.
 extern const int DIR_DX[8];
 extern const int DIR_DY[8];
 
+/* like DIR_DX[] and DIR_DY[], only cartesian */
+extern const int CAR_DIR_DX[4];
+extern const int CAR_DIR_DY[4];
+
+#define cartesian_adjacent_iterate(x, y, IAC_x, IAC_y) \
+{                                                      \
+  int IAC_i;                                           \
+  int IAC_x, IAC_y;                                    \
+  for (IAC_i = 0; IAC_i < 4; IAC_i++) {                \
+    switch (IAC_i) {                                   \
+    case 0:                                            \
+      IAC_x = x + 1;                                   \
+      IAC_y = y;                                       \
+      if (!normalize_map_pos(&IAC_x, &IAC_y))          \
+	continue;                                      \
+      break;                                           \
+    case 1:                                            \
+      IAC_x = x;                                       \
+      IAC_y = y + 1;                                   \
+      if (!normalize_map_pos(&IAC_x, &IAC_y))          \
+	continue;                                      \
+      break;                                           \
+    case 2:                                            \
+      IAC_x = x - 1;                                   \
+      IAC_y = y;                                       \
+      if (!normalize_map_pos(&IAC_x, &IAC_y))          \
+	continue;                                      \
+      break;                                           \
+    case 3:                                            \
+      IAC_x = x;                                       \
+      IAC_y = y - 1;                                   \
+      if (!normalize_map_pos(&IAC_x, &IAC_y))          \
+	continue;                                      \
+      break;                                           \
+    default:                                           \
+      abort();                                         \
+    }
+
+#define cartesian_adjacent_iterate_end                 \
+  }                                                    \
+}
+
+
+/*
+used to compute neighboring tiles:
+using
+x1 = x + DIR_DX2[dir];
+y1 = y + DIR_DY2[dir];
+will give you the tile as shown below.
+-------
+|7|0|1|
+|-+-+-|
+|6| |2|
+|-+-+-|
+|5|4|3|
+-------
+ */
+extern const int DIR_DX2[8];
+extern const int DIR_DY2[8];
+
 #define MAP_DEFAULT_HUTS         50
 #define MAP_MIN_HUTS             0
 #define MAP_MAX_HUTS             500
