@@ -13,16 +13,15 @@
 #ifndef FC__PACKETS_H
 #define FC__PACKETS_H
 
+#include "connection.h"		/* struct connection, MAX_LEN_* */
 #include "map.h"
 #include "player.h"
 #include "shared.h"		/* MAX_LEN_NAME, MAX_LEN_ADDR */
 #include "spaceship.h"
 #include "worklist.h"
 
-#define MAX_LEN_PACKET    4096
 #define MAX_LEN_USERNAME    10	     /* see below */
 #define MAX_LEN_MSG       1536
-#define MAX_LEN_CAPSTR     512
 
 /* Note that MAX_LEN_USERNAME cannot be expanded, because it
    is used for the name in the first packet sent by the client,
@@ -111,6 +110,14 @@ enum packet_type {
   PACKET_SABOTAGE_LIST,
   PACKET_ADVANCE_FOCUS,
   PACKET_RULESET_GAME,
+  PACKET_CONN_INFO,
+  PACKET_SHORT_CITY,
+  PACKET_PLAYER_REMOVE_VISION,
+  PACKET_GOTO_ROUTE,
+  PACKET_PATROL_ROUTE,
+  PACKET_CONN_PING,
+  PACKET_CONN_PONG,
+  PACKET_UNIT_AIRLIFT,
   PACKET_LAST  /* leave this last */
 };
 
@@ -259,9 +266,6 @@ send to each client whenever the turn has ended.
 struct packet_new_year {
   int year;
 };
-struct packet_before_new_year {
-  int dummy;
-};
 
 
 /*********************************************************
@@ -321,7 +325,7 @@ struct packet_city_info {
   char name[MAX_LEN_NAME];
 
   int size;
-  int ppl_happy, ppl_content, ppl_unhappy;
+  int ppl_happy[5], ppl_content[5], ppl_unhappy[5];
   int ppl_elvis, ppl_scientist, ppl_taxman;
   int food_prod, food_surplus;
   int shield_prod, shield_surplus;
@@ -342,6 +346,8 @@ struct packet_city_info {
   int changed_from_id;
   int changed_from_is_unit;
   int before_change_shields;
+  int disbanded_shields;
+  int caravan_shields;
 
   struct worklist worklist;
 
@@ -356,6 +362,17 @@ struct packet_city_info {
 };
 
 
+struct packet_short_city {
+  int id;			/* uint16 */
+  int owner;			/* uint8 */
+  int x, y;			/* uint8 */
+  char name[MAX_LEN_NAME];
+  int size;			/* uint8 */
+  int happy;			/* boolean */
+  int capital;			/* boolean */
+  int walls;			/* boolean */
+};
+
 
 /*********************************************************
  this packet is the very first packet send by the client.
@@ -382,6 +399,7 @@ struct packet_join_game_reply {
   int you_can_join;             /* true/false */
   char message[MAX_LEN_MSG];
   char capability[MAX_LEN_CAPSTR];
+  int conn_id;			/* clients conn id as known in server */
 };
 
 
@@ -416,6 +434,14 @@ struct packet_generic_integer {
 };
 
 
+/*********************************************************
+  like the packet above. 
+*********************************************************/
+struct packet_generic_empty {
+  int dummy;
+};
+
+
 /*********************************************************
 ...
 *********************************************************/
@@ -439,12 +465,31 @@ struct packet_player_info {
   int tech_goal;
   unsigned char inventions[A_LAST+1];
   int is_connected;
-  char addr[MAX_LEN_ADDR];
   int revolution;
   int ai;
   int is_barbarian;
-  char capability[MAX_LEN_CAPSTR];
   struct worklist worklists[MAX_NUM_WORKLISTS];
+  unsigned int gives_shared_vision;
+};
+
+/**************************************************************************
+  For telling clients information about other connections to server.
+  Clients may not use all info, but supply now to avoid unnecessary
+  protocol changes later.
+**************************************************************************/
+struct packet_conn_info {
+  int id;
+  int used;			/* boolean; 0 means client should forget its
+				   info about this connection */
+  int established;		/* boolean */
+  int player_num;		/* range uchar; index in game.players, or 255 */
+  int observer;			/* boolean (?) */
+  
+  enum cmdlevel_id access_level;   /* range uchar */
+  
+  char name[MAX_LEN_NAME];
+  char addr[MAX_LEN_ADDR];
+  char capability[MAX_LEN_CAPSTR];
 };
 
 /*********************************************************
@@ -501,6 +546,7 @@ struct packet_ruleset_control {
   int sewer_size;
   int add_to_size_limit;
   int num_unit_types;
+  int num_impr_types;
   int num_tech_types;
   struct {
     int get_bonus_tech;
@@ -727,6 +773,8 @@ struct packet_ruleset_game {
   int hut_overflight;
   int pillage_select;
   int nuke_contamination;
+  int granary_food_ini;
+  int granary_food_inc;
 };
 
 /*********************************************************
@@ -736,7 +784,7 @@ struct packet_game_info {
   int gold;
   int civstyle;
   int tech;
-  int techlevel;
+  int researchcost;
   int skill_level;
   int timeout;
   int end_year;
@@ -791,16 +839,6 @@ struct packet_city_name_suggestion {
   char name[MAX_LEN_NAME];
 };
 
-/*********************************************************
-this is where the data is first collected, whenever it
-arrives to the client/server.
-*********************************************************/
-struct socket_packet_buffer {
-  int ndata;
-  int do_buffer_sends;
-  unsigned char data[10*MAX_LEN_PACKET];
-};
-
 struct packet_sabotage_list
 {
   int diplomat_id;
@@ -808,132 +846,124 @@ struct packet_sabotage_list
   char improvements[B_LAST+1];
 };
 
-
-typedef	void	CLOSE_FUN	(struct connection *pc);
-
-
-struct connection {
-  int sock, used;
-  int first_packet;		/* check byte order on first packet */
-  int byte_swap;		/* connection uses non-network byte order */
-  char *player; 
-  struct socket_packet_buffer buffer;
-  struct socket_packet_buffer send_buffer;
-  char addr[MAX_LEN_ADDR];
-  char capability[MAX_LEN_CAPSTR];
-  /* "capability" gives the capability string of the executable (be it
-   * a client or server) at the other end of the connection.
-   */
-  enum cmdlevel_id access_level;
-  /* Used in the server, "access_level" stores the access granted
-   * to the client corresponding to this connection.
-   */
+struct packet_goto_route
+{
+  int length;
+  int first_index;
+  int last_index;
+  struct map_position *pos;
+  int unit_id;
 };
 
-
-int read_socket_data(int sock, struct socket_packet_buffer *buffer);
-int send_connection_data(struct connection *pc, unsigned char *data, int len);
-
 /* These two are non-static for meta.c; others are now static --dwp */
 unsigned char *put_uint16(unsigned char *buffer, int val);
-unsigned char *put_string(unsigned char *buffer, char *mystring);
+unsigned char *put_string(unsigned char *buffer, const char *mystring);
 
 int send_packet_diplomacy_info(struct connection *pc, enum packet_type pt,
-			       struct packet_diplomacy_info *packet);
+			       const struct packet_diplomacy_info *packet);
 struct packet_diplomacy_info *
 receive_packet_diplomacy_info(struct connection *pc);
 
 int send_packet_diplomat_action(struct connection *pc, 
-				struct packet_diplomat_action *packet);
+				const struct packet_diplomat_action *packet);
 struct packet_diplomat_action *
 receive_packet_diplomat_action(struct connection *pc);
 
 int send_packet_nuke_tile(struct connection *pc, 
-			  struct packet_nuke_tile *packet);
+			  const struct packet_nuke_tile *packet);
 struct packet_nuke_tile *
 receive_packet_nuke_tile(struct connection *pc);
 
 
 int send_packet_unit_combat(struct connection *pc, 
-			    struct packet_unit_combat *packet);
+			    const struct packet_unit_combat *packet);
 struct packet_unit_combat *
 receive_packet_unit_combat(struct connection *pc);
 
 
 int send_packet_unit_connect(struct connection *pc, 
-			    struct packet_unit_connect *packet);
+			     const struct packet_unit_connect *packet);
 struct packet_unit_connect *
 receive_packet_unit_connect(struct connection *pc);
 
 
 int send_packet_tile_info(struct connection *pc, 
-			 struct packet_tile_info *pinfo);
+			  const struct packet_tile_info *pinfo);
 struct packet_tile_info *receive_packet_tile_info(struct connection *pc);
 
 int send_packet_map_info(struct connection *pc, 
-			  struct packet_map_info *pinfo);
+			 const struct packet_map_info *pinfo);
 struct packet_map_info *receive_packet_map_info(struct connection *pc);
 
 int send_packet_game_info(struct connection *pc, 
-			  struct packet_game_info *pinfo);
+			  const struct packet_game_info *pinfo);
 struct packet_game_info *receive_packet_game_info(struct connection *pc);
 
 
 struct packet_player_info *receive_packet_player_info(struct connection *pc);
 int send_packet_player_info(struct connection *pc, 
-			    struct packet_player_info *pinfo);
+			    const struct packet_player_info *pinfo);
 
+struct packet_conn_info *receive_packet_conn_info(struct connection *pc);
+int send_packet_conn_info(struct connection *pc,
+			  const struct packet_conn_info *pinfo);
 
 int send_packet_new_year(struct connection *pc, 
-			 struct packet_new_year *request);
+			 const struct packet_new_year *request);
 struct packet_new_year *receive_packet_new_year(struct connection *pc);
 
 int send_packet_move_unit(struct connection *pc, 
-			  struct packet_move_unit *request);
+			  const struct packet_move_unit *request);
 struct packet_move_unit *receive_packet_move_unit(struct connection *pc);
 
 
 int send_packet_unit_info(struct connection *pc,
-			  struct packet_unit_info *req);
+			  const struct packet_unit_info *req);
 struct packet_unit_info *receive_packet_unit_info(struct connection *pc);
 
 int send_packet_req_join_game(struct connection *pc, 
-			      struct packet_req_join_game *request);
+			      const struct packet_req_join_game *request);
 struct packet_req_join_game *receive_packet_req_join_game(struct 
 							  connection *pc);
 
 int send_packet_join_game_reply(struct connection *pc, 
-			       struct packet_join_game_reply *reply);
+			        const struct packet_join_game_reply *reply);
 struct packet_join_game_reply *receive_packet_join_game_reply(struct 
 							      connection *pc);
 
 int send_packet_alloc_nation(struct connection *pc, 
-			   struct packet_alloc_nation *packet);
+			     const struct packet_alloc_nation *packet);
 struct packet_alloc_nation *receive_packet_alloc_nation(struct connection *pc);
 
 
 int send_packet_generic_message(struct connection *pc, int type,
-				struct packet_generic_message *message);
+				const struct packet_generic_message *message);
 struct packet_generic_message *receive_packet_generic_message(struct 
 							      connection *pc);
 
 int send_packet_generic_integer(struct connection *pc, int type,
-				struct packet_generic_integer *packet);
+				const struct packet_generic_integer *packet);
 struct packet_generic_integer *receive_packet_generic_integer(struct 
 							      connection *pc);
 
 
-int send_packet_city_info(struct connection *pc,struct packet_city_info *req);
+int send_packet_city_info(struct connection *pc,
+                          const struct packet_city_info *req);
 struct packet_city_info *receive_packet_city_info(struct connection *pc);
 
+int send_packet_short_city(struct connection *pc,
+                           const struct packet_short_city *req);
+struct packet_short_city *receive_packet_short_city(struct connection *pc);
+
 int send_packet_city_request(struct connection *pc, 
-			     struct packet_city_request *packet, enum packet_type);
+			     const struct packet_city_request *packet,
+			     enum packet_type req_type);
 struct packet_city_request *
 receive_packet_city_request(struct connection *pc);
 
 
 int send_packet_player_request(struct connection *pc, 
-			       struct packet_player_request *packet,
+			       const struct packet_player_request *packet,
 			       enum packet_type req_type);
 struct packet_player_request *
 receive_packet_player_request(struct connection *pc);
@@ -941,100 +971,102 @@ receive_packet_player_request(struct connection *pc);
 struct packet_unit_request *
 receive_packet_unit_request(struct connection *pc);
 int send_packet_unit_request(struct connection *pc, 
-			     struct packet_unit_request *packet,
+			     const struct packet_unit_request *packet,
 			     enum packet_type req_type);
 
-int send_packet_before_new_year(struct connection *pc);
-struct packet_before_new_year *receive_packet_before_new_year(struct connection *pc);
-
 int send_packet_unittype_info(struct connection *pc, int type, int action);
 struct packet_unittype_info *receive_packet_unittype_info(struct connection *pc);
 
 int send_packet_ruleset_control(struct connection *pc, 
-				struct packet_ruleset_control *pinfo);
+				const struct packet_ruleset_control *pinfo);
 struct packet_ruleset_control *
 receive_packet_ruleset_control(struct connection *pc);
 
 int send_packet_ruleset_unit(struct connection *pc,
-			     struct packet_ruleset_unit *packet);
+			     const struct packet_ruleset_unit *packet);
 struct packet_ruleset_unit *
 receive_packet_ruleset_unit(struct connection *pc);
 
 int send_packet_ruleset_tech(struct connection *pc,
-			     struct packet_ruleset_tech *packet);
+			     const struct packet_ruleset_tech *packet);
 struct packet_ruleset_tech *
 receive_packet_ruleset_tech(struct connection *pc);
 
 int send_packet_ruleset_building(struct connection *pc,
-			     struct packet_ruleset_building *packet);
+			     const struct packet_ruleset_building *packet);
 struct packet_ruleset_building *
 receive_packet_ruleset_building(struct connection *pc);
 
 int send_packet_ruleset_terrain(struct connection *pc,
-			     struct packet_ruleset_terrain *packet);
+			     const struct packet_ruleset_terrain *packet);
 struct packet_ruleset_terrain *
 receive_packet_ruleset_terrain(struct connection *pc);
 int send_packet_ruleset_terrain_control(struct connection *pc,
-					struct terrain_misc *packet);
+					const struct terrain_misc *packet);
 struct terrain_misc *
 receive_packet_ruleset_terrain_control(struct connection *pc);
 
 int send_packet_ruleset_government(struct connection *pc,
-				struct packet_ruleset_government *packet);
+			       const struct packet_ruleset_government *packet);
 struct packet_ruleset_government *
 receive_packet_ruleset_government(struct connection *pc);
 int send_packet_ruleset_government_ruler_title(struct connection *pc,
-				struct packet_ruleset_government_ruler_title *packet);
+		   const struct packet_ruleset_government_ruler_title *packet);
 struct packet_ruleset_government_ruler_title *
 receive_packet_ruleset_government_ruler_title(struct connection *pc);
 
 int send_packet_ruleset_nation(struct connection *pc,
-			       struct packet_ruleset_nation *packet);
+			       const struct packet_ruleset_nation *packet);
 struct packet_ruleset_nation *
 receive_packet_ruleset_nation(struct connection *pc);
 
 int send_packet_ruleset_city(struct connection *pc,
-			     struct packet_ruleset_city *packet);
+			     const struct packet_ruleset_city *packet);
 struct packet_ruleset_city *
 receive_packet_ruleset_city(struct connection *pc);
 
 int send_packet_ruleset_game(struct connection *pc,
-                             struct packet_ruleset_game *packet);
+                             const struct packet_ruleset_game *packet);
 struct packet_ruleset_game *
 receive_packet_ruleset_game(struct connection *pc);
 
-int send_packet_before_end_year(struct connection *pc);
-
 int send_packet_generic_values(struct connection *pc, int type,
-			       struct packet_generic_values *req);
+			       const struct packet_generic_values *req);
 struct packet_generic_values *
 receive_packet_generic_values(struct connection *pc);
 
 int send_packet_spaceship_info(struct connection *pc,
-			       struct packet_spaceship_info *packet);
+			       const struct packet_spaceship_info *packet);
 struct packet_spaceship_info *
 receive_packet_spaceship_info(struct connection *pc);
 
 int send_packet_spaceship_action(struct connection *pc,
-				 struct packet_spaceship_action *packet);
+				 const struct packet_spaceship_action *packet);
 struct packet_spaceship_action *
 receive_packet_spaceship_action(struct connection *pc);
 
 int send_packet_city_name_suggestion(struct connection *pc,
-				     struct packet_city_name_suggestion *packet);
+				const struct packet_city_name_suggestion *packet);
 struct packet_city_name_suggestion *
 receive_packet_city_name_suggestion(struct connection *pc);
 
 int send_packet_sabotage_list(struct connection *pc,
-			      struct packet_sabotage_list *packet);
+			      const struct packet_sabotage_list *packet);
 struct packet_sabotage_list *
 receive_packet_sabotage_list(struct connection *pc);
 
-void *get_packet_from_connection(struct connection *pc, int *ptype);
+void *get_packet_from_connection(struct connection *pc, int *ptype, int *presult);
 void remove_packet_from_buffer(struct socket_packet_buffer *buffer);
-void close_socket_set_callback(CLOSE_FUN *fun);
-void flush_connection_send_buffer(struct connection *pc);
-void connection_do_buffer(struct connection *pc);
-void connection_do_unbuffer(struct connection *pc);
+
+int send_packet_goto_route(struct connection *pc,
+                           const struct packet_goto_route *packet,
+			   enum goto_route_type type);
+struct packet_goto_route *receive_packet_goto_route(struct connection *pc);
+
+int send_packet_generic_empty(struct connection *pc, int type);
+struct packet_generic_empty *
+receive_packet_generic_empty(struct connection *pc);
+
+#include "packets_lsend.h"		/* lsend_packet_* functions */
 
 #endif  /* FC__PACKETS_H */
