@@ -33,7 +33,6 @@
 
 #include "player.h"
 
-extern int is_server;
 
 /***************************************************************
 ...
@@ -72,7 +71,9 @@ void player_init(struct player *plr)
   plr->capital=0;
   unit_list_init(&plr->units);
   city_list_init(&plr->cities);
-  sz_strlcpy(plr->addr, "---.---.---.---");
+  conn_list_init(&plr->connections);
+  plr->current_conn = NULL;
+  plr->is_connected = 0;
   plr->is_alive=1;
   plr->embassy=0;
   plr->reputation=GAME_DEFAULT_REPUTATION;
@@ -99,6 +100,8 @@ void player_init(struct player *plr)
     init_worklist(&plr->worklists[i]);
     plr->worklists[i].is_valid = 0;
   }
+  plr->gives_shared_vision = 0;
+  plr->really_gives_vision = 0;
 }
 
 /***************************************************************
@@ -168,28 +171,25 @@ no map visibility check here!
 ***************************************************************/
 int player_can_see_unit(struct player *pplayer, struct unit *punit)
 {
-  if(punit->owner==pplayer->player_no)
+  if (punit->owner==pplayer->player_no)
     return 1;
-  if(is_hiding_unit(punit)) {
+  if (is_hiding_unit(punit)) {
     /* Search for units/cities that might be able to see the sub/missile */
-    int x, y;
     struct city *pcity;
-    struct unit *punit2;
-    for(y=punit->y-1; y<=punit->y+1; ++y) { 
-      if(y<0 || y>=map.ysize)
-	continue;
-      for(x=punit->x-1; x<=punit->x+1; ++x) { 
-	if((punit2=unit_list_get(&map_get_tile(x, y)->units, 0)) &&
-	   punit2->owner == pplayer->player_no ) return 1;
-
-	if((pcity=map_get_city(x, y)) && pcity->owner==pplayer->player_no)
+    square_iterate(punit->x, punit->y, 1, x, y) {
+      unit_list_iterate(map_get_tile(x, y)->units, punit2) {
+	if (punit2->owner == pplayer->player_no)
 	  return 1;
-      }
-    }
+      } unit_list_iterate_end;
+
+      pcity = map_get_city(x, y);
+      if (pcity && pcity->owner == pplayer->player_no)
+	return 1;
+    } square_iterate_end;
     return 0;
+  } else {
+    return 1;
   }
-  
-  return 1;
 }
 
 /***************************************************************
@@ -229,12 +229,12 @@ Return 1 if x,y is inside any of the player's city radii.
 **************************************************************************/
 int player_in_city_radius(struct player *pplayer, int x, int y)
 {
-  int i, j;
   struct city *pcity;
-  city_map_iterate(i, j) {
-    pcity = map_get_city(x+i-2, y+j-2);
-    if (pcity && (pcity->owner == pplayer->player_no)) return 1;
-  }
+  map_city_radius_iterate(x, y, x1, y1) {
+    pcity = map_get_city(x1, y1);
+    if (pcity && (pcity->owner == pplayer->player_no))
+      return 1;
+  } map_city_radius_iterate_end;
   return 0;
 }
 
@@ -296,35 +296,57 @@ void player_limit_to_government_rates(struct player *pplayer)
   }
 
   maxrate = get_government_max_rate(pplayer->government);
-
+  surplus = 0;
   if (pplayer->economic.luxury > maxrate) {
-    surplus = pplayer->economic.luxury - maxrate;
+    surplus += pplayer->economic.luxury - maxrate;
     pplayer->economic.luxury = maxrate;
-    if (pplayer->economic.science >= pplayer->economic.tax)
-      pplayer->economic.science += surplus;
-    else
-      pplayer->economic.tax += surplus;
   }
   if (pplayer->economic.tax > maxrate) {
-    surplus = pplayer->economic.tax - maxrate;
+    surplus += pplayer->economic.tax - maxrate;
     pplayer->economic.tax = maxrate;
-    if (pplayer->economic.science >= pplayer->economic.luxury)
-      pplayer->economic.science += surplus;
-    else
-      pplayer->economic.luxury += surplus;
   }
   if (pplayer->economic.science > maxrate) {
-    surplus = pplayer->economic.science - maxrate;
+    surplus += pplayer->economic.science - maxrate;
     pplayer->economic.science = maxrate;
-    if (pplayer->economic.tax >= pplayer->economic.luxury)
-      pplayer->economic.tax += surplus;
-    else
-      pplayer->economic.luxury += surplus;
+  }
+
+  assert(surplus % 10 == 0);
+  while (surplus > 0) {
+    if (pplayer->economic.science < maxrate) {
+      pplayer->economic.science += 10;
+    } else if (pplayer->economic.tax < maxrate) {
+      pplayer->economic.tax += 10;
+    } else if (pplayer->economic.luxury < maxrate) {
+      pplayer->economic.luxury += 10;
+    } else {
+      abort();
+    }
+    surplus -= 10;
   }
 
   return;
 }
 
+/**************************************************************************
+  Hack to support old code which expects player to have single address
+  string (possibly blank_addr_str) and not yet updated for multi-connects.
+  When all such code is converted, this function should be removed.
+  Returns "blank" address for no connections, single address for one,
+  or string "<multiple>" if there are multiple connections.
+  
+  Currently used by:
+      civserver.c: send_server_info_to_metaserver()
+      plrhand.c: send_player_info_c()
+      gui-{xaw,gtk,mui}/plrdlg.c: update_players_dialog()
+**************************************************************************/
+const char *player_addr_hack(struct player *pplayer)
+{
+  int n = conn_list_size(&pplayer->connections);
+  return ((n==0) ? blank_addr_str :
+	  (n==1) ? conn_list_get(&pplayer->connections, 0)->addr :
+	  "<multiple>");
+}
+
 /**************************************************************************
 Locate the city where the players palace is located, (NULL Otherwise) 
 **************************************************************************/
@@ -381,37 +403,6 @@ int ai_fuzzy(struct player *pplayer, int normal_decision)
   return !normal_decision;
 }
 
-/**************************************************************************
-Command access levels for client-side use; at present, they are 
-only used to control access to server commands typed at the client chatline.
-**************************************************************************/
-static const char *levelnames[] = {
-  "none",
-  "info",
-  "ctrl",
-  "hack"
-};
-
-const char *cmdlevel_name(enum cmdlevel_id lvl)
-{
-  assert (lvl >= 0 && lvl < ALLOW_NUM);
-  return levelnames[lvl];
-}
-
-enum cmdlevel_id cmdlevel_named(const char *token)
-{
-  enum cmdlevel_id i;
-  int len = strlen(token);
-
-  for (i = 0; i < ALLOW_NUM; ++i) {
-    if (strncmp(levelnames[i], token, len) == 0) {
-      return i;
-    }
-  }
-
-  return ALLOW_UNRECOGNIZED;
-}
-
 /**************************************************************************
 Return a reputation level as a human-readable string
 **************************************************************************/
@@ -442,18 +433,18 @@ Return a diplomatic state as a human-readable string
 **************************************************************************/
 const char *diplstate_text(const enum diplstate_type type)
 {
-  static char *ds_names[DS_LAST] = 
+  static const char *ds_names[DS_LAST] = 
   {
-    N_("Neutral"),
-    N_("War"), 
-    N_("Cease-fire"),
-    N_("Peace"),
-    N_("Alliance"),
-    N_("No Contact")
+    N_("?diplomatic_state:Neutral"),
+    N_("?diplomatic_state:War"), 
+    N_("?diplomatic_state:Cease-fire"),
+    N_("?diplomatic_state:Peace"),
+    N_("?diplomatic_state:Alliance"),
+    N_("?diplomatic_state:No Contact")
   };
 
   if (type < DS_LAST)
-    return _(ds_names[type]);
+    return Q_(ds_names[type]);
   freelog(LOG_FATAL, "Bad diplstate_type in diplstate_text: %d", type);
   abort();
 }
@@ -477,10 +468,7 @@ const struct player_diplstate *player_get_diplstate(const int player,
 }
 
 /***************************************************************
-returns true iff players can attack each other.  Note that this
-returns true if either player doesn't have the "dipl_states"
-capability, so anyone can beat on them; this is for backward
-compatibility with non-dipl_states-speaking clients.
+returns true iff players can attack each other.
 ***************************************************************/
 int pplayers_at_war(const struct player *pplayer,
 		    const struct player *pplayer2)
@@ -511,7 +499,7 @@ int pplayers_allied(const struct player *pplayer,
     return TRUE;
   if (is_barbarian(pplayer) || is_barbarian(pplayer2))
     return FALSE;
-  return ((pplayer == pplayer2) || (ds == DS_ALLIANCE));
+  return (ds == DS_ALLIANCE);
 }
 
 /***************************************************************
