@@ -14,6 +14,7 @@
 #define FC__PLAYER_H
 
 #include "city.h"
+#include "connection.h"		/* struct conn_list */
 #include "nation.h"
 #include "shared.h"
 #include "spaceship.h"
@@ -120,23 +121,6 @@ struct player_diplstate {
   int has_reason_to_cancel;	/* 0: no, 1: this turn, 2: this or next turn */
 };
 
-/**************************************************************************
-  command access levels for client-side use; at present, they are
-  only used for server commands typed at the client chatline
- **************************************************************************/
-enum cmdlevel_id {    /* access levels for users to issue commands        */
-  ALLOW_NONE = 0,     /* user may issue no commands at all                */
-  ALLOW_INFO,         /* user may issue informational commands            */
-  ALLOW_CTRL,         /* user may issue commands that affect game & users */
-  ALLOW_HACK,         /* user may issue *all* commands - dangerous!       */
-
-  ALLOW_NUM,          /* the number of levels                             */
-  ALLOW_UNRECOGNIZED  /* used as a failure return code                    */
-};
-/*  the set command is a special case:                                    */
-/*    - ALLOW_CTRL is required for SSET_TO_CLIENT options                 */
-/*    - ALLOW_HACK is required for SSET_TO_SERVER options                 */
-
 /***************************************************************************
   On the distinction between nations(formerly races), players, and users,
   see freeciv_hackers_guide.txt
@@ -167,10 +151,13 @@ struct player {
   struct player_spaceship spaceship;
   int future_tech;
   struct player_ai ai;
-  int is_connected;
-  struct connection *conn;
-  char addr[MAX_LEN_ADDR];
+  int is_connected;		       /* observers don't count */
+  struct connection *current_conn;     /* non-null while handling packet */
+  struct conn_list connections;	       /* will replace conn */
   struct worklist worklists[MAX_NUM_WORKLISTS];
+  struct player_tile *private_map;
+  unsigned int gives_shared_vision; /* bitvector those that give you shared vision */
+  unsigned int really_gives_vision; /* takes into account that p3 may see what p1 has via p2 */
 };
 
 void player_init(struct player *plr);
@@ -197,6 +184,8 @@ int player_knows_techs_with_flag(struct player *pplayer, int flag);
 
 void player_limit_to_government_rates(struct player *pplayer);
 
+const char *player_addr_hack(struct player *pplayer);
+
 struct city *find_palace(struct player *pplayer);
 
 int ai_handicap(struct player *pplayer, enum handicap_type htype);
@@ -220,9 +209,17 @@ int pplayers_non_attack(const struct player *pplayer,
 			const struct player *pplayer2);
 int players_non_attack(const int player, const int player2);
 
-const char *cmdlevel_name(enum cmdlevel_id lvl);
-enum cmdlevel_id cmdlevel_named(const char *token);
-
 int is_barbarian(const struct player *pplayer);
 
+#define players_iterate(PI_player)                                            \
+{                                                                             \
+  struct player *PI_player;                                                   \
+  int PI_p_itr;                                                               \
+  for (PI_p_itr = 0; PI_p_itr < game.nplayers; PI_p_itr++) {                  \
+    PI_player = get_player(PI_p_itr);
+
+#define players_iterate_end                                                   \
+  }                                                                           \
+}
+
 #endif  /* FC__PLAYER_H */
