@@ -232,9 +232,12 @@ section_file_insert_internal(struct section_file *my_section_file,
 			     char *fullpath);
 
 /**************************************************************************
-...
+  Return the filename the sectionfile was loaded as, or "(anonymous)"
+  if this sectionfile was created rather than loaded from file.
+  The memory is managed internally, and should not be altered,
+  nor used after section_file_free() called for the sectionfile.
 **************************************************************************/
-static const char *secfile_filename(const struct section_file *file)
+const char *secfile_filename(const struct section_file *file)
 {
   if (file->filename) {
     return file->filename;
@@ -365,7 +368,7 @@ static int section_file_load_dup(struct section_file *sf,
 
   inf = inf_open(filename, datafilename);
   if (!inf) {
-    freelog(LOG_NORMAL, "Could not open file \"%s\"", filename);
+    freelog(LOG_ERROR, "Could not open file \"%s\"", filename);
     return 0;
   }
   section_file_init(sf);
@@ -433,7 +436,8 @@ static int section_file_load_dup(struct section_file *sf,
 	  my_snprintf(entry_name.str, entry_name.n_alloc, "%s%d.%s",
 		      base_name.str, table_lineno, columns[i].str);
 	} else {
-	  astr_minsize(&entry_name, base_name.n + 20 + columns[i].n);
+	  astr_minsize(&entry_name,
+		       base_name.n + 20 +  columns[columns_tab.n-1].n);
 	  my_snprintf(entry_name.str, entry_name.n_alloc, "%s%d.%s,%d",
 		      base_name.str, table_lineno,
 		      columns[columns_tab.n-1].str, i-columns_tab.n+1);
@@ -644,7 +648,7 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 	  /* break out of tabular if doesn't match: */
 	  if((!pentry) || (strcmp(pentry->name, expect) != 0)) {
 	    if(icol != 0) {
-	      freelog(LOG_NORMAL, "unexpected exit from tabular at %s (%s)",
+	      freelog(LOG_ERROR, "unexpected exit from tabular at %s (%s)",
 		      pentry->name, filename);
 	      fz_fprintf(fs, "\n");
 	    }
@@ -685,13 +689,13 @@ int section_file_save(struct section_file *my_section_file, const char *filename
   moutstr(NULL);		/* free internal buffer */
 
   if (fz_ferror(fs)) {
-    freelog(LOG_NORMAL, "Error before closing %s: %s", filename,
+    freelog(LOG_ERROR, "Error before closing %s: %s", filename,
 	    fz_strerror(fs));
     fz_fclose(fs);
     return 0;
   }
   if (fz_fclose(fs) != 0) {
-    freelog(LOG_NORMAL, "Error closing %s", filename);
+    freelog(LOG_ERROR, "Error closing %s", filename);
     return 0;
   }
 
