@@ -14,11 +14,12 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
+#include <ctype.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-#include <assert.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -35,6 +36,7 @@
 
 #include "shared.h"
 
+
 /* If no path separator is defined use colon */
 #ifndef PATH_SEPARATOR
 #define PATH_SEPARATOR ":"
@@ -46,30 +48,37 @@
                           "~/.freeciv"
 #endif
 
-/**************************************************************************
-... 
-**************************************************************************/
-char *n_if_vowel(char ch)
-{
-	if (strchr("AEIOUaeiou",ch))
-		return "n";
-	else
-		return "";
-}
+/* Cached locale numeric formatting information.
+   Defaults are as appropriate for the US. */
+static char *grouping = "\3";
+static char *grouping_sep = ",";
+static size_t grouping_sep_len = 1;
 
 /***************************************************************
-...
+  Take a string containing multiple lines and create a copy where
+  each line is padded to the length of the longest line and centered.
+  We do not cope with tabs etc.  Note that we're assuming that the
+  last line does _not_ end with a newline.  The caller should
+  free() the result.
+
+  FIXME: This is only used in the Xaw client, and so probably does
+  not belong in common.
 ***************************************************************/
 char *create_centered_string(char *s)
 {
-  char *cp, *cp0, *r, *rn;
-  int i, maxlen, curlen, nlines;
+  char *cp;  /* Points to the part of the source that we're looking at. */
+  char *cp0; /* Points to the beginning of the line in the source that
+		we're looking at. */
+  char *r;   /* Points to the result. */
+  char *rn;  /* Points to the part of the result that we're filling in
+		right now. */
+
+  int i;
+
+  int maxlen = 0;
+  int curlen = 0;
+  int nlines = 1;
 
-  
-  maxlen=0;
-  curlen=0;
-  nlines=1;
-  
   for(cp=s; *cp; cp++) {
     if(*cp!='\n')
       curlen++;
@@ -99,19 +108,20 @@ char *create_centered_string(char *s)
       cp0=cp+1;
     }
   }
-  
   for(i=0; i<(maxlen-curlen)/2; i++)
     *rn++=' ';
   strcpy(rn, cp0);
+
   return r;
 }
 
 /**************************************************************************
   return a char * to the parameter of the option or NULL.
-  *i can be increased to get next string in the array argv[]. 
- **************************************************************************/
-
-char * get_option(const char *option_name,char **argv,int *i,int argc)
+  *i can be increased to get next string in the array argv[].
+  It is an error for the option to exist but be an empty string.
+  This doesn't use freelog() because it is used before logging is set up.
+**************************************************************************/
+char *get_option(const char *option_name, char **argv, int *i, int argc)
 {
   int len = strlen(option_name);
 
@@ -126,6 +136,10 @@ char * get_option(const char *option_name,char **argv,int *i,int argc)
       if (*i < argc - 1) {
 	(*i)++;
 	opt = argv[*i];
+	if (strlen(opt)==0) {
+	  fprintf(stderr, _("Empty argument for \"%s\".\n"), option_name);
+	  exit(1);
+	}
       }	else {
 	fprintf(stderr, _("Missing argument for \"%s\".\n"), option_name);
 	exit(1);
@@ -141,7 +155,6 @@ char * get_option(const char *option_name,char **argv,int *i,int argc)
 /***************************************************************
 ...
 ***************************************************************/
-
 int is_option(const char *option_name,char *option)
 {
   if (!strcmp(option_name,option) || 
@@ -150,48 +163,60 @@ int is_option(const char *option_name,char *option)
 }
 
 /***************************************************************
-...
+  Returns a statically allocated string containing a nicely-formatted
+  version of the given number according to the user's locale.
+  (Only works for numbers >= zero.)
 ***************************************************************/
 char *int_to_text(int nr)
 {
-  char tmpbuf[12];
-  static char buf[20];
-  char *to=buf;
-  char *from=tmpbuf;
-  my_snprintf(tmpbuf, sizeof(tmpbuf), "%d", nr);
-  while (*from) {
-    *to++=*from++;
-    if (strlen(from)%3==0 && *from)  
-      *to++=',';
+  static char buf[64]; /* Note that we'll be filling this in right to left. */
+  char *grp = grouping;
+  char *ptr;
+  int cnt;
+
+  assert(nr >= 0);
+
+  if (nr == 0) {
+    return "0";
   }
-  *to=0;
-  return buf;
+
+  ptr = &buf[sizeof(buf)];
+  *(--ptr) = '\0';
+
+  cnt = 0;
+  while (nr != 0) {
+    int dig = nr % 10;
+    assert(ptr > buf);
+    *(--ptr) = '0' + dig;
+    nr /= 10;
+
+    cnt++;
+    if (nr != 0 && cnt == *grp) {
+      /* Reached count of digits in group: insert separator and reset count. */
+      cnt = 0;
+      if (*grp == CHAR_MAX) {
+	/* This test is unlikely to be necessary since we would need at
+	   least 421-bit ints to break the 127 digit barrier, but why not. */
+	break;
+      }
+      ptr -= grouping_sep_len;
+      assert(ptr >= buf);
+      memcpy(ptr, grouping_sep, grouping_sep_len);
+      if (*(grp + 1) != 0) {
+	/* Zero means to repeat the present group-size indefinitely. */
+        grp++;
+      }
+    }
+  }
+
+  return ptr;
 }
 
-#if 0
 /***************************************************************
-pedantic name enforcer - at most one space between words
+  Check whether or not the given char is a valid,
+  printable ISO 8859-1 character.
 ***************************************************************/
-char *get_sane_name(char *name)
-{
-  static char str[MAX_LEN_NAME];
-  char *cp;
-  
-  if(!isalpha(*name))                      /* first char not a letter? */ 
-    return 0; 
-
-  for(cp=name; *cp && (isalnum(*cp) || (*cp==' ' && isalnum(*(cp+1)))); cp++);
-  if(*cp)
-    return 0; 
-
-  sz_strlcpy(str, name);
-  
-  *str=toupper(*str);
-  return str;
-}
-#endif
-
-static int iso_latin1(char ch)
+static int is_iso_latin1(char ch)
 {
    int i=ch;
    
@@ -200,26 +225,45 @@ static int iso_latin1(char ch)
     if (ch < ' ')  return 0;
     if (ch <= '~')  return 1;
   }
-  if (ch < '')  return 0;
+  if (ch < '')  return 0; /* FIXME: Is it really a good idea to
+				 use 8 bit characters in source code? */
   return 1;
 }
 
+/***************************************************************
+  This is used in sundry places to make sure that names of cities,
+  players etc. do not contain yucky characters of various sorts.
+  Returns the input argument if it points to an acceptable name,
+  otherwise returns NULL.
+  FIXME:  Not internationalised.
+***************************************************************/
 char *get_sane_name(char *name)
 {
-  static char str[MAX_LEN_NAME];
   char *cp;
-  
-  for(cp=name; iso_latin1(*cp); cp++);
-  if(*cp)
-    return 0; 
 
-  sz_strlcpy(str, name);
-  
-  return str;
+  /* must not be NULL or empty */
+  if (!name || !(*name)) {
+    return NULL; 
+  }
+
+  /* must begin and end with some non-space character */
+  if ((*name == ' ') || (*(strchr(name, '\0') - 1) == ' ')) {
+    return NULL; 
+  }
+
+  /* must be composed entirely of printable ISO 8859-1 characters */
+  for (cp = name; is_iso_latin1(*cp); cp++) ;
+  if (*cp) {
+    return NULL; 
+  }
+
+  /* otherwise, it's okay... */
+  return name;
 }
 
 /***************************************************************
-...
+  Produce a statically allocated textual representation of the given
+  year.
 ***************************************************************/
 char *textyear(int year)
 {
@@ -232,14 +276,31 @@ char *textyear(int year)
 }
 
 /**************************************************************************
-string_ptr_compare() - fiddles with pointers to do a simple string compare
-                       when passed pointers to two strings -- called from
-                       qsort().
+  Compares two strings, in the collating order of the current locale,
+  given pointers to the two strings (i.e., given "char *"s).
+  Case-sensitive.  Designed to be called from qsort().
 **************************************************************************/
-
-int string_ptr_compare(const void *first, const void *second)
+int compare_strings(const void *first, const void *second)
 {
-   return mystrcasecmp(*((const char **)first), *((const char **)second));
+#if defined(ENABLE_NLS) && defined(HAVE_STRCOLL)
+  return strcoll((const char *)first, (const char *)second);
+#else
+  return strcmp((const char *)first, (const char *)second);
+#endif
+}
+
+/**************************************************************************
+  Compares two strings, in the collating order of the current locale,
+  given pointers to the two string pointers (i.e., given "char **"s).
+  Case-sensitive.  Designed to be called from qsort().
+**************************************************************************/
+int compare_strings_ptrs(const void *first, const void *second)
+{
+#if defined(ENABLE_NLS) && defined(HAVE_STRCOLL)
+  return strcoll(*((const char **)first), *((const char **)second));
+#else
+  return strcmp(*((const char **)first), *((const char **)second));
+#endif
 }
 
 /***************************************************************************
@@ -254,6 +315,24 @@ char *skip_leading_spaces(char *s)
   return s;
 }
 
+/***************************************************************************
+  Removes leading spaces in string pointed to by 's'.
+  Note 's' must point to writeable memory!
+***************************************************************************/
+void remove_leading_spaces(char *s)
+{
+  char *t;
+  
+  assert(s!=NULL);
+  t = skip_leading_spaces(s);
+  if (t != s) {
+    while (*t) {
+      *s++ = *t++;
+    }
+    *s = '\0';
+  }
+}
+
 /***************************************************************************
   Terminates string pointed to by 's' to remove traling spaces;
   Note 's' must point to writeable memory!
@@ -261,15 +340,32 @@ char *skip_leading_spaces(char *s)
 void remove_trailing_spaces(char *s)
 {
   char *t;
+  size_t len;
   
   assert(s!=NULL);
-  t = s + strlen(s) -1;
-  while(t>=s && isspace(*t)) {
-    *t = '\0';
-    t--;
+  len = strlen(s);
+  if (len) {
+    t = s + len -1;
+    while(isspace(*t)) {
+      *t = '\0';
+      if (t == s) {
+	break;
+      }
+      t--;
+    }
   }
 }
 
+/***************************************************************************
+  Removes leading and trailing spaces in string pointed to by 's'.
+  Note 's' must point to writeable memory!
+***************************************************************************/
+void remove_leading_trailing_spaces(char *s)
+{
+  remove_leading_spaces(s);
+  remove_trailing_spaces(s);
+}
+
 /***************************************************************************
   As remove_trailing_spaces(), for specified char.
 ***************************************************************************/
@@ -353,6 +449,30 @@ char *end_of_strn(char *str, int *nleft)
   return str + len;
 }
 
+/********************************************************************** 
+  Check the length of the given string.  If the string is too long,
+  log errmsg, which should be a string in printf-format taking up to
+  two arguments: the string and the length.
+**********************************************************************/ 
+int check_strlen(const char *str, size_t len, const char *errmsg)
+{
+  if (strlen(str) >= len) {
+    freelog(LOG_ERROR, errmsg, str, len);
+    return 1;
+  }
+  return 0;
+}
+
+/********************************************************************** 
+  Call check_strlen() on str and then strlcpy() it into buffer.
+**********************************************************************/
+size_t loud_strlcpy(char *buffer, const char *str, size_t len,
+		   const char *errmsg)
+{
+  check_strlen(str, len, errmsg);
+  return mystrlcpy(buffer, str, len);
+}
+
 /********************************************************************** 
  cat_snprintf is like a combination of my_snprintf and mystrlcat;
  it does snprintf to the end of an existing string.
@@ -380,7 +500,7 @@ int cat_snprintf(char *str, size_t n, const char *format, ...)
   va_start(ap, format);
   ret = my_vsnprintf(str+len, n-len, format, ap);
   va_end(ap);
-  return ret + len;
+  return (int) (ret + len);
 }
 
 /***************************************************************************
@@ -400,7 +520,7 @@ char *user_home_dir(void)
       home_dir = mystrdup(env);	        /* never free()d */
       freelog(LOG_VERBOSE, "HOME is %s", home_dir);
     } else {
-      freelog(LOG_NORMAL, "Could not find home directory (HOME is not set)");
+      freelog(LOG_ERROR, "Could not find home directory (HOME is not set)");
       home_dir = NULL;
     }
     init = 1;
@@ -441,7 +561,12 @@ char *user_username(void)
   }
 #endif
   username = fc_malloc(MAX_LEN_NAME);
+
+#ifdef ALWAYS_ROOT
+  my_snprintf(username, MAX_LEN_NAME, "name");
+#else
   my_snprintf(username, MAX_LEN_NAME, "name%d", (int)getuid());
+#endif
   freelog(LOG_VERBOSE, "fake username is %s", username);
   return username;
 }
@@ -493,7 +618,7 @@ char *datafilename(const char *filename)
       i = strlen(tok);
       if (tok[0] == '~') {
 	if (i>1 && tok[1] != '/') {
-	  freelog(LOG_NORMAL, "For \"%s\" in data path cannot expand '~'"
+	  freelog(LOG_ERROR, "For \"%s\" in data path cannot expand '~'"
 		              " except as '~/'; ignoring", tok);
 	  i = 0;   /* skip this one */
 	} else {
@@ -585,20 +710,48 @@ char *datafilename_required(const char *filename)
   }
 }
 
-
 /***************************************************************************
   Setup for Native Language Support, if configured to use it.
+  (Call this only once, or it may leak memory.)
 ***************************************************************************/
 void init_nls(void)
 {
 #ifdef ENABLE_NLS
-  setlocale (LC_ALL, "");
-  bindtextdomain (PACKAGE, LOCALEDIR);
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
+
+  /* Don't touch the defaults when LC_NUMERIC == "C".
+     This is intended to cater to the common case where:
+       1) The user is from North America. ;-)
+       2) The user has not set the proper environment variables.
+	  (Most applications are (unfortunately) US-centric
+	  by default, so why bother?)
+     This would result in the "C" locale being used, with grouping ""
+     and thousands_sep "", where we really want "\3" and ",". */
+
+  if (strcmp(setlocale(LC_NUMERIC, NULL), "C")) {
+    struct lconv *lc = localeconv();
+
+    if (lc->grouping[0] == '\0') {
+      /* This actually indicates no grouping at all. */
+      static char m = CHAR_MAX;
+      grouping = &m;
+    } else {
+      size_t len;
+      for (len = 0; lc->grouping[len] && lc->grouping[len] != CHAR_MAX; len++)
+	;
+      len++;
+      grouping = fc_malloc(len);
+      memcpy(grouping, lc->grouping, len);
+    }
+
+    grouping_sep = mystrdup(lc->thousands_sep);
+    grouping_sep_len = strlen(grouping_sep);
+  }
 #endif
 }
 
-
 /***************************************************************************
   If we have root privileges, die with an error.
   (Eg, for security reasons.)
@@ -623,7 +776,6 @@ void dont_run_as_root(const char *argv0, const char *fallback)
 #endif
 }
 
-
 /***************************************************************************
   Return a description string of the result.
   In English, form of description is suitable to substitute in, eg:
@@ -696,6 +848,9 @@ enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
 
 /***************************************************************************
   Return the Freeciv motto.
+  (The motto is common code:
+   only one instance of the string in the source;
+   only one time gettext needs to translate it. --jjm)
 ***************************************************************************/
 char *freeciv_motto(void)
 {
