@@ -48,8 +48,8 @@
 
 #define BOOL_VAL(x) ((x)!=0)
 
-/* Used to update bitvectors when a player is removed.
-   Deletes bit no in val, moves all bits larger than no one down,
+/* Deletes bit no in val,
+   moves all bits larger than no one down,
    and inserts a zero at the top. */
 #define WIPEBIT(val, no) ( ((~(-1<<(no)))&(val)) \
                            | (( (-1<<((no)+1)) & (val)) >>1) )
@@ -57,7 +57,6 @@
 /* This is duplicated in rand.h to avoid extra includes: */
 #define MAX_UINT32 0xFFFFFFFF
 
-char *n_if_vowel(char ch);
 char *create_centered_string(char *s);
 
 char * get_option(const char *option_name,char **argv,int *i,int argc);
@@ -65,13 +64,23 @@ int is_option(const char *option_name,char *option);
 char *int_to_text(int nr);
 char *get_sane_name(char *name);
 char *textyear(int year);
-int string_ptr_compare(const void *first, const void *second);
+int compare_strings(const void *first, const void *second);
+int compare_strings_ptrs(const void *first, const void *second);
 
 char *skip_leading_spaces(char *s);
+void remove_leading_spaces(char *s);
 void remove_trailing_spaces(char *s);
+void remove_leading_trailing_spaces(char *s);
 void remove_trailing_char(char *s, char trailing);
 int wordwrap_string(char *s, int len);
 
+int check_strlen(const char *str, size_t len, const char *errmsg);
+size_t loud_strlcpy(char *buffer, const char *str, size_t len,
+		   const char *errmsg);
+/* Convenience macro. */
+#define sz_loud_strlcpy(buffer, str, errmsg) \
+    loud_strlcpy(buffer, str, sizeof(buffer), errmsg)
+
 char *end_of_strn(char *str, int *nleft);
 int cat_snprintf(char *str, size_t n, const char *format, ...)
      fc__attribute((format (printf, 3, 4)));
@@ -113,10 +122,4 @@ enum m_pre_result match_prefix(m_pre_accessor_fn_t accessor_fn,
 
 char *freeciv_motto(void);
 
-/*Mac constants-resource IDs*/
-enum DITL_ids{
-  kChatDITL=133,
-  kCityDITL=140
-};
-
 #endif  /* FC__SHARED_H */
