@@ -50,7 +50,7 @@
 #include <errno.h>
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>		/* usleep, fcntl */
+#include <unistd.h>		/* usleep, fcntl, gethostname */
 #endif
 
 #ifdef HAVE_SYS_TIME_H
@@ -70,12 +70,21 @@
 #include <sys/select.h>
 #endif
 
+#ifdef HAVE_WINSOCK
+#include <winsock.h>
+#endif
+#ifdef WIN32_NATIVE
+#include <process.h>
+#include <windows.h>
+#endif
+
 #ifdef GENERATING_MAC
 #include <events.h>		/* for WaitNextEvent() */
 #endif
 
 #include "fcintl.h"
 #include "mem.h"
+#include "netintf.h"
 
 #include "support.h"
 
@@ -84,11 +93,16 @@
 ***************************************************************/
 int mystrcasecmp(const char *str0, const char *str1)
 {
+#ifdef HAVE_STRCASECMP
+  return strcasecmp (str0, str1);
+#else
   for(; tolower(*str0)==tolower(*str1); str0++, str1++)
     if(*str0=='\0')
       return 0;
 
-  return tolower(*str0)-tolower(*str1);
+  return ((int) (unsigned char) tolower(*str0))
+    - ((int) (unsigned char) tolower(*str1));
+#endif
 }
 
 /***************************************************************
@@ -97,18 +111,26 @@ int mystrcasecmp(const char *str0, const char *str1)
 ***************************************************************/
 int mystrncasecmp(const char *str0, const char *str1, size_t n)
 {
+#ifdef HAVE_STRNCASECMP
+  return strncasecmp (str0, str1, n);
+#else
   size_t i;
   
   for(i=0; i<n && tolower(*str0)==tolower(*str1); i++, str0++, str1++)
     if(*str0=='\0')
       return 0;
 
-  return (i==n) ? 0 : (tolower(*str0)-tolower(*str1));
+  if (i == n)
+    return 0;
+  else
+    return ((int) (unsigned char) tolower(*str0))
+      - ((int) (unsigned char) tolower(*str1));
+#endif
 }
 
 
 /***************************************************************
-...
+  Return a string which describes a given error (errno-style.)
 ***************************************************************/
 char *mystrerror(int errnum)
 {
@@ -124,7 +146,7 @@ char *mystrerror(int errnum)
 
 
 /***************************************************************
-...
+  Suspend execution for the specified number of milliseconds.
 ***************************************************************/
 void myusleep(unsigned long usec)
 {
@@ -139,7 +161,7 @@ void myusleep(unsigned long usec)
   usec /= 16666;		/* microseconds to 1/60th seconds */
   if (usec < 1) usec = 1;
   /* suposed to give other application processor time for the mac */
-  WaitNextEvent(0, &the_event, usec, nil);
+  WaitNextEvent(0, &the_event, usec, 0L);
 #else
   struct timeval tv;
   tv.tv_sec=0;
@@ -155,7 +177,7 @@ void myusleep(unsigned long usec)
  mystrlcpy() and mystrlcat() provide (non-standard) functions
  strlcpy() and strlcat(), with semantics following OpenBSD (and
  maybe others).  They are intended as more user-friendly
- versions of stncpy and strncat, in particular easier to
+ versions of strncpy and strncat, in particular easier to
  use safely and correctly, and ensuring nul-terminated results
  while being able to detect truncation.
 
@@ -229,65 +251,6 @@ size_t mystrlcat(char *dest, const char *src, size_t n)
 #endif
 }
 
-
-#ifdef HAVE_VSNPRINTF
-/**********************************************************************
- Convenience function used by check_native_vsnprintf() below.
- (Can test check_native_vsnprintf() by replacing vsnprintf call
- below with vsprintf(), or by changing return value to eg -1.)
-***********************************************************************/
-static int test_snprintf(char *str, size_t n, const char *format, ...)
-{
-  int ret;
-  va_list ap;
-
-  va_start(ap, format);
-  ret = vsnprintf(str, n, format, ap);
-  va_end(ap);
-  return ret;
-}
-
-/**********************************************************************
- This function checks, at runtime, whether a native vsnprintf() meets
- our requirements; specifically:
- - Should actually obey the parameter n, rather than, eg, just
-   calling sprintf and ignoring n.
- - On truncation, should return the number of chars (not counting
-   trailing null) which would have been printed, rather than, eg, -1.
- Returns 1 if both ok.
- Also checks whether null-terminates on truncation, but we don't
- base return value on this since it is easy to force this behaviour.
- Reports to stderr if DEBUG set (cannot use freelog since that
- calls my_vsnprintf).
- Could do these checks at configure time, but seems to me easier to
- do at runtime on first call...
-***********************************************************************/
-static int check_native_vsnprintf(void)
-{
-  char buf[20]   = "abcdefghijkl";   /* 12 + null */
-  char *test_str = "0123456789";
-  const int ntrunc = 5;
-  const char one_past = buf[ntrunc];
-  int test_len = strlen(test_str);
-  int ret;
-
-  ret = test_snprintf(buf, ntrunc, "%s", test_str);
-#if DEBUG
-  if (buf[ntrunc] != one_past) {
-    fprintf(stderr, "debug: test_snprintf wrote past n\n");
-  } else if (buf[ntrunc-1] != '\0') {
-    fprintf(stderr, "debug: test_snprintf did not null-terminate\n");
-  }
-  if (ret != test_len) {
-    fprintf(stderr, "debug: test_snprintf returned %d,"
-	    " not untruncated length %d\n", ret, test_len);
-  }
-#endif
-
-  return (buf[ntrunc]==one_past && ret==test_len);
-}
-#endif
-
 /**********************************************************************
  vsnprintf() replacement using a big malloc()ed internal buffer,
  originally by David Pfitzner <dwp@mso.anu.edu.au>
@@ -306,6 +269,15 @@ static int check_native_vsnprintf(void)
  (Linux man page says returns -1 on truncation, but glibc seems to
  do as above nevertheless; check_native_vsnprintf() above tests this.)
 
+ [glibc is correct.  Viz.
+
+ PRINTF(3)           Linux Programmer's Manual           PRINTF(3)
+ 
+ (Thus until glibc 2.0.6.  Since glibc 2.1 these functions follow the
+ C99 standard and return the number of characters (excluding the
+ trailing '\0') which would have been written to the final string if
+ enough space had been available.)]
+ 
  The method is simply to malloc (first time called) a big internal
  buffer, longer than any result is likely to be (for non-malicious
  usage), then vsprintf to that buffer, and copy the appropriate
@@ -329,84 +301,68 @@ static int check_native_vsnprintf(void)
 
 ***********************************************************************/
 
-/* buffer size: "64k should be big enough for anyone" ;-) */
+/* "64k should be big enough for anyone" ;-) */
 #define VSNP_BUF_SIZE (64*1024)
-
 int my_vsnprintf(char *str, size_t n, const char *format, va_list ap)
 {
-  /* This may be overzealous, but I suspect any triggering of these
-   * to be bugs.  (Do this even if call native function.)
-   */
+  int r;
+
+  /* This may be overzealous, but I suspect any triggering of these to
+   * be bugs.  */
+
   assert(str);
   assert(n>0);
   assert(format);
 
-#ifdef HAVE_VSNPRINTF
-  {
-    static int native_is_ok = -1; /* set to 0 or 1 on first call */
+#ifdef HAVE_WORKING_VSNPRINTF
+  r = vsnprintf(str, n, format, ap);
+  str[n - 1] = 0;
 
-    if (native_is_ok == -1) {
-      native_is_ok = check_native_vsnprintf();
-    }
+  /* Convert C99 return value to C89.  */
+  if (r >= n)
+    return -1;
 
-    if (native_is_ok) {
-      int ret = vsnprintf(str, n, format, ap);
-      /* Guarantee null-terminated: (native_is_ok means can use ret like this) */
-      if (ret >= n) {
-	str[n-1] = '\0';
-      }
-      return ret;
-    }
-  }
-#endif
-  /* Following is used if don't have native, or if fall through
-   * from above if native doesn't pass checks.
-   */
-  {
-    static char *buf = NULL;
-    int len;
-
-    if (buf==NULL) {
-      buf = fc_malloc(VSNP_BUF_SIZE);
-    }
-
-#ifdef HAVE_VSNPRINTF
-    /* This occurs if have native, but didn't pass check:
-     * may just be that native doesn't give the right return,
-     * in which case may be slightly safer to use it here:
-     */
-    vsnprintf(buf, VSNP_BUF_SIZE, format, ap);
+  return r;
 #else
-    vsprintf(buf, format, ap);
-#endif
+  {
+    /* Don't use fc_malloc() or freelog() here, since they may call
+       my_vsnprintf() if it fails.  */
+ 
+    static char *buf;
+    size_t len;
 
-    /* Check strlen of buf straight away: could be more efficient
-       not to do this and step through instead (eg if n small and
-       len long), but useful anyway to get the return value, and
-       importantly want to abort if vsprintf clobbered the heap!
-       (Don't just use return from vsprintf since not sure if
-       that gives the right thing on all platforms?)
-       Will maintain last char of buffer as null, and use SIZE-2
-       as longest string which we can detect as untruncated.
-       (Don't use freelog() for report since that uses vsnprintf...)
-    */
-    buf[VSNP_BUF_SIZE-1] = '\0';
+    if (buf == NULL) {
+      buf = malloc(VSNP_BUF_SIZE);
+
+      if (buf == NULL) {
+	fprintf(stderr, "Could not allocate %i bytes for vsnprintf() "
+		"replacement.", VSNP_BUF_SIZE);
+	exit(1);
+      }
+    }
+#ifdef HAVE_VSNPRINTF
+    r = vsnprintf(buf, n, format, ap);
+#else
+    r = vsprintf(buf, format, ap);
+#endif
+    buf[VSNP_BUF_SIZE - 1] = '\0';
     len = strlen(buf);
-    if (len >= VSNP_BUF_SIZE-1) {
+
+    if (len >= VSNP_BUF_SIZE - 1) {
       fprintf(stderr, "Overflow in vsnprintf replacement!"
-	      " (buffer size %d) aborting...\n", VSNP_BUF_SIZE);
+              " (buffer size %d) aborting...\n", VSNP_BUF_SIZE);
       abort();
     }
-
-    if (n > len) {
-      memcpy(str, buf, len+1);	/* +1 for terminating null */
+    if (n >= len + 1) {
+      memcpy(str, buf, len+1);
       return len;
     } else {
       memcpy(str, buf, n-1);
-      str[n-1] = '\0';
-      return len;		/* truncated */
+      str[n - 1] = '\0';
+      return -1;
     }
   }
+#endif  
 }
 
 int my_snprintf(char *str, size_t n, const char *format, ...)
@@ -421,3 +377,99 @@ int my_snprintf(char *str, size_t n, const char *format, ...)
   va_end(ap);
   return ret;
 }
+
+/**********************************************************************
+  Call gethostname() if supported, else just returns -1.
+***********************************************************************/
+int my_gethostname(char *buf, size_t len)
+{
+#ifdef HAVE_GETHOSTNAME
+  return gethostname(buf, len);
+#else
+  return -1;
+#endif
+}
+
+#ifdef SOCKET_ZERO_ISNT_STDIN
+/**********************************************************************
+  Support for console I/O in case SOCKET_ZERO_ISNT_STDIN.
+***********************************************************************/
+
+#define MYBUFSIZE 100
+static char mybuf[MYBUFSIZE+1];
+
+/**********************************************************************/
+
+#ifdef WIN32_NATIVE
+static HANDLE mythread = INVALID_HANDLE_VALUE;
+
+static DWORD WINAPI thread_proc(LPVOID data)
+{
+  if (fgets(mybuf, MYBUFSIZE, stdin)) {
+    char *s;
+    if ((s = strchr(mybuf, '\n')))
+      *s = '\0';
+  }
+  return 0;
+}
+#endif
+
+/**********************************************************************
+  Initialize console I/O in case SOCKET_ZERO_ISNT_STDIN.
+***********************************************************************/
+void my_init_console(void)
+{
+#ifdef WIN32_NATIVE
+  unsigned int threadid;
+
+  if (mythread != INVALID_HANDLE_VALUE)
+    return;
+
+  mybuf[0] = '\0';
+  mythread = (HANDLE)_beginthreadex(NULL, 0, thread_proc, NULL, 0, &threadid);
+#else
+  static int initialized = 0;
+  if (!initialized) {
+    initialized = 1;
+#ifdef HAVE_FILENO
+    my_nonblock(fileno(stdin));
+#endif
+  }
+#endif
+}
+
+/**********************************************************************
+  Read a line from console I/O in case SOCKET_ZERO_ISNT_STDIN.
+
+  This returns a pointer to a statically allocated buffer.
+  Subsequent calls to my_read_console() or my_init_console() will
+  overwrite it.
+***********************************************************************/
+char *my_read_console(void)
+{
+#ifdef WIN32_NATIVE
+  if (WaitForSingleObject(mythread, 0) == WAIT_OBJECT_0) {
+    CloseHandle(mythread);
+    mythread = INVALID_HANDLE_VALUE;
+    return mybuf;
+  }
+  return NULL;
+#else
+  if (!feof(stdin)) {    /* input from server operator */
+    static char *bufptr = mybuf;
+    /* fetch chars until \n, or run out of space in buffer */
+    /* blocks if my_nonblock() in my_init_console() failed */
+    while ((*bufptr = fgetc(stdin)) != EOF) {
+      if (*bufptr == '\n') *bufptr = '\0';
+      if (*bufptr == '\0') {
+	bufptr = mybuf;
+	return mybuf;
+      }
+      if ((bufptr - mybuf) <= MYBUFSIZE) bufptr++; /* prevent overrun */
+    }
+  }
+  return NULL;
+#endif
+}
+
+#endif
