@@ -26,9 +26,10 @@ struct advance advances[A_LAST];
    server/ruleset.c (for the server)
    client/packhand.c (for the client) */
 
-static char *flag_names[] = {
+static const char *flag_names[] = {
   "Bonus_Tech","Boat_Fast","Bridge","Railroad","Fortress",
-  "Population_Pollution_Inc","Trade_Revenue_Reduce","Airbase","Farmland"
+  "Population_Pollution_Inc","Trade_Revenue_Reduce","Airbase","Farmland",
+  "Reduce_Trireme_Loss1", "Reduce_Trireme_Loss2"
 };
 /* Note that these strings must correspond with the enums in tech_flag_id,
    in common/tech.h */
