@@ -243,12 +243,12 @@ void start_timer(struct timer *t)
     return;
   }
   if (t->state == TIMER_STARTED) {
-    freelog(LOG_NORMAL, "tried to start already started timer");
+    freelog(LOG_ERROR, "tried to start already started timer");
     return;
   }
   if (t->type == TIMER_CPU) {
     t->start.c = clock();
-    if (t->start.c == -1) {
+    if (t->start.c == (clock_t) -1) {
       report_clock_failed(t);
       return;
     }
@@ -261,7 +261,7 @@ void start_timer(struct timer *t)
     }
 #else
     t->start.t = time(NULL);
-    if (t->start.t == -1) {
+    if (t->start.t == (time_t) -1) {
       report_time_failed(t);
       return;
     }
@@ -293,12 +293,12 @@ void stop_timer(struct timer *t)
     return;
   }
   if (t->state == TIMER_STOPPED) {
-    freelog(LOG_NORMAL, "tried to stop already stopped timer");
+    freelog(LOG_ERROR, "tried to stop already stopped timer");
     return;
   }
   if (t->type == TIMER_CPU) {
     clock_t now = clock();
-    if (now == -1) {
+    if (now == (clock_t) -1) {
       report_clock_failed(t);
       return;
     }
@@ -325,7 +325,7 @@ void stop_timer(struct timer *t)
     t->start.tv = now;
 #else
     time_t now = time(NULL);
-    if (now == -1) {
+    if (now == (time_t) -1) {
       report_time_failed(t);
       return;
     }
