@@ -15,17 +15,11 @@
 #endif
 
 #include <assert.h>
-#include <stdio.h>
-#include <stdlib.h>		/* free */
-#include <string.h>
 
-#include "astring.h"
 #include "fcintl.h"
 #include "game.h"
-#include "government.h"
 #include "log.h"
 #include "map.h"
-#include "mem.h"
 #include "player.h"
 #include "shared.h"
 #include "support.h"
@@ -39,58 +33,6 @@
 #include "speclist_c.h"
 
 
-struct unit_type unit_types[U_LAST];
-/* the unit_types array is now setup in:
-   server/ruleset.c (for the server)
-   client/packhand.c (for the client) */
-
-static char *move_type_names[] = {
-  "Land", "Sea", "Heli", "Air"
-};
-static char *flag_names[] = {
-  "Caravan", "Missile", "IgZOC", "NonMil", "IgTer", "Carrier",
-  "OneAttack", "Pikemen", "Horse", "IgWall", "FieldUnit", "AEGIS",
-  "Fighter", "Marines", "Partial_Invis", "Settlers", "Diplomat",
-  "Trireme", "Nuclear", "Spy", "Transform", "Paratroopers",
-  "Airbase", "Cities", "IgTired", "Missile_Carrier", "No_Land_Attack"
-};
-static char *role_names[] = {
-  "FirstBuild", "Explorer", "Hut", "HutTech", "Partisan",
-  "DefendOk", "DefendGood", "AttackFast", "AttackStrong",
-  "Ferryboat", "Barbarian", "BarbarianTech", "BarbarianBoat",
-  "BarbarianBuild", "BarbarianBuildTech", "BarbarianLeader",
-  "BarbarianSea", "BarbarianSeaTech"
-};
-static char *unit_class_names[] = {
-  "Air",
-  "Helicopter",
-  "Land",
-  "Missile",
-  "Nuclear",
-  "Sea"
-};
-
-/***************************************************************
-...
-***************************************************************/
-char *get_unit_name(Unit_Type_id id)
-{
-  struct unit_type *ptype;
-  static char buffer[256];
-  ptype =get_unit_type(id);
-  if (ptype->fuel) {
-    my_snprintf(buffer, sizeof(buffer),
-		"%s [%d/%d/%d(%d)]", ptype->name, ptype->attack_strength,
-		ptype->defense_strength,
-		ptype->move_rate/3,(ptype->move_rate/3)*ptype->fuel);
-  } else {
-    my_snprintf(buffer, sizeof(buffer),
-		"%s [%d/%d/%d]", ptype->name, ptype->attack_strength,
-		ptype->defense_strength, ptype->move_rate/3);
-  }
-  return buffer;
-}
-
 /***************************************************************
 ...
 ***************************************************************/
@@ -104,14 +46,17 @@ int unit_move_rate(struct unit *punit)
     val = (val * punit->hp) / get_unit_type(punit->type)->hp;
   if(is_sailing_unit(punit)) {
     if(player_owns_active_wonder(pplayer, B_LIGHTHOUSE)) 
-      val+=3;
+      val+=SINGLE_MOVE;
     if(player_owns_active_wonder(pplayer, B_MAGELLAN)) 
-      val += (improvement_variant(B_MAGELLAN)==1) ? 3 : 6;
-    val += player_knows_techs_with_flag(pplayer,TF_BOAT_FAST)*3;
-    if (val < 6) 
-      val = 6;
+      val += (improvement_variant(B_MAGELLAN)==1) ? SINGLE_MOVE : 2 * SINGLE_MOVE;
+    val += player_knows_techs_with_flag(pplayer,TF_BOAT_FAST) * SINGLE_MOVE;
+    if (val < 2 * SINGLE_MOVE)
+      val = 2 * SINGLE_MOVE;
+  }
+  if (val < SINGLE_MOVE
+      && get_unit_type(punit->type)->move_rate > 0) {
+    val = SINGLE_MOVE;
   }
-  if (val < 3) val = 3;
   return val;
 }
 
@@ -135,7 +80,8 @@ int diplomat_can_do_action(struct unit *pdiplomat,
   if(!is_diplomat_action_available(pdiplomat, action, destx, desty))
     return 0;
 
-  if(!is_tiles_adjacent(pdiplomat->x, pdiplomat->y, destx, desty))
+  if(!is_tiles_adjacent(pdiplomat->x, pdiplomat->y, destx, desty)
+     && !same_pos(pdiplomat->x, pdiplomat->y, destx, desty))
     return 0;
 
   if(!pdiplomat->moves_left)
@@ -157,11 +103,16 @@ int is_diplomat_action_available(struct unit *pdiplomat,
   struct city *pcity=map_get_city(destx, desty);
   int playerid = pdiplomat->owner;
 
-  if(pcity) {  
+  if (action!=DIPLOMAT_MOVE && map_get_terrain(pdiplomat->x, pdiplomat->y)==T_OCEAN)
+    return 0;
+
+  if (pcity) {
     if(pcity->owner!=pdiplomat->owner &&
        real_map_distance(pdiplomat->x, pdiplomat->y, pcity->x, pcity->y) <= 1) {
       if(action==DIPLOMAT_SABOTAGE)
         return players_at_war(playerid, pcity->owner);
+      if(action==DIPLOMAT_MOVE)
+        return players_allied(playerid, pcity->owner);
       if(action==DIPLOMAT_EMBASSY &&
 	 !is_barbarian(&game.players[pcity->owner]) &&
 	 !player_has_embassy(&game.players[pdiplomat->owner], 
@@ -222,6 +173,8 @@ int unit_can_airlift_to(struct unit *punit, struct city *pcity)
     return 0;
   if (city1->airlift + pcity->airlift < 2) 
     return 0;
+  if (!is_ground_unit(punit))
+    return 0;
 
   return 1;
 }
@@ -231,11 +184,15 @@ int unit_can_airlift_to(struct unit *punit, struct city *pcity)
 **************************************************************************/
 int unit_can_help_build_wonder(struct unit *punit, struct city *pcity)
 {
-  return unit_flag(punit->type, F_CARAVAN) &&
-    is_tiles_adjacent(punit->x, punit->y, pcity->x, pcity->y) &&
-    punit->owner==pcity->owner && !pcity->is_building_unit  && 
-    is_wonder(pcity->currently_building) &&
-    (pcity->shield_stock < improvement_value(pcity->currently_building));
+  if (!is_tiles_adjacent(punit->x, punit->y, pcity->x, pcity->y)
+      && !same_pos(punit->x, punit->y, pcity->x, pcity->y))
+    return 0;
+
+  return unit_flag(punit->type, F_CARAVAN)
+    && punit->owner==pcity->owner
+    && !pcity->is_building_unit
+    && is_wonder(pcity->currently_building)
+    && pcity->shield_stock < improvement_value(pcity->currently_building);
 }
 
 
@@ -356,37 +313,6 @@ int is_ground_unit(struct unit *punit)
 {
   return (unit_types[punit->type].move_type == LAND_MOVING);
 }
-/**************************************************************************
-...
-**************************************************************************/
-int is_ground_unittype(Unit_Type_id id)
-{
-  return (unit_types[id].move_type == LAND_MOVING);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int is_air_unittype(Unit_Type_id id)
-{
-  return (unit_types[id].move_type == AIR_MOVING);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int is_heli_unittype(Unit_Type_id id)
-{
-  return (unit_types[id].move_type == HELI_MOVING);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int is_water_unit(Unit_Type_id id)
-{
-  return (unit_types[id].move_type == SEA_MOVING);
-}
 
 /**************************************************************************
 ...
@@ -404,38 +330,6 @@ int is_diplomat_unit(struct unit *punit)
   return (unit_flag(punit->type, F_DIPLOMAT));
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-int utype_shield_cost(struct unit_type *ut, struct government *g)
-{
-  return ut->shield_cost * g->unit_shield_cost_factor;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int utype_food_cost(struct unit_type *ut, struct government *g)
-{
-  return ut->food_cost * g->unit_food_cost_factor;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int utype_happy_cost(struct unit_type *ut, struct government *g)
-{
-  return ut->happy_cost * g->unit_happy_cost_factor;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int utype_gold_cost(struct unit_type *ut, struct government *g)
-{
-  return ut->gold_cost * g->unit_gold_cost_factor;
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -452,17 +346,14 @@ int is_ground_threat(struct player *pplayer, struct unit *punit)
 **************************************************************************/
 int is_square_threatened(struct player *pplayer, int x, int y)
 {
-  int i,j;
   int threat=0;
 
-  for(i=x-2;i<=x+2;i++) {
-    for(j=y-2;j<=y+2;j++) {
-      unit_list_iterate(map_get_tile(i,j)->units, punit) {
-	threat += is_ground_threat(pplayer, punit);
-      }
-      unit_list_iterate_end;
-    }
-  }
+  square_iterate(x, y, 2, x1, y1) {
+    unit_list_iterate(map_get_tile(x1, y1)->units, punit) {
+      threat += is_ground_threat(pplayer, punit);
+    } unit_list_iterate_end;
+  } square_iterate_end;
+
   return threat;
 }
 
@@ -472,7 +363,6 @@ int is_square_threatened(struct player *pplayer, int x, int y)
 int is_field_unit(struct unit *punit)
 {
   return ((unit_flag(punit->type, F_FIELDUNIT) > 0));
-
 }
 
 
@@ -499,108 +389,6 @@ int is_hiding_unit(struct unit *punit)
 }
 
 
-/**************************************************************************
-...
-**************************************************************************/
-int unit_flag(Unit_Type_id id, int flag)
-{
-  assert(flag>=0 && flag<F_LAST);
-  return BOOL_VAL(unit_types[id].flags & (1<<flag));
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int unit_has_role(Unit_Type_id id, int role)
-{
-  assert(role>=L_FIRST && role<L_LAST);
-  return BOOL_VAL(unit_types[id].roles & (1<<(role-L_FIRST)));
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int unit_value(Unit_Type_id id)
-{
-  return (unit_types[id].build_cost);
-}
-
-
-/**************************************************************************
-...
-**************************************************************************/
-char *unit_name(Unit_Type_id id)
-{
-  return (unit_types[id].name);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-char *unit_class_name(Unit_Class_id id)
-{
-  if ((id >= 0) && (id < UCL_LAST)) {
-    return unit_class_names[id];
-  } else {
-    return "";
-  }
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-struct unit_type *get_unit_type(Unit_Type_id id)
-{
-  return &unit_types[id];
-}
-
-/**************************************************************************
- Return a string with all the names of units with this flag
- Return NULL if no unit with this flag exists.
- The string must be free'd
-
- TODO: if there are more than 4 units with this flag return
-       a fallback string (e.g. first unit name + "and similar units"
-**************************************************************************/
-char *get_units_with_flag_string(int flag)
-{
-  int count=num_role_units(flag);
-
-  if(count==1)
-    return mystrdup(unit_name(get_role_unit(flag,0)));
-
-  if(count) {
-    struct astring astr;
-
-    astr_init(&astr);
-    astr_minsize(&astr,1);
-    astr.str[0] = 0;
-
-    while(count--) {
-      int u = get_role_unit(flag,count);
-      char *unitname = unit_name(u);
-
-      /* there should be something like astr_append() */
-      astr_minsize(&astr,astr.n+strlen(unitname));
-      strcat(astr.str,unitname);
-
-      if(count==1) {
-	char *and_str = _(" and ");
-        astr_minsize(&astr,astr.n+strlen(and_str));
-        strcat(astr.str, and_str);
-      }
-      else {
-        if(count != 0) {
-          astr_minsize(&astr,astr.n+strlen(", "));
-          strcat(astr.str,", ");
-        }
-        else return astr.str;
-      }
-    }
-  }
-  return NULL;
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -638,7 +426,7 @@ int can_unit_add_to_city(struct unit *punit)
 
   if(!pcity)
     return 0;
-  if(pcity->size > game.add_to_size_limit)
+  if(pcity->size >= game.add_to_size_limit)
     return 0;
   if(pcity->owner != punit->owner)
     return 0;
@@ -665,35 +453,6 @@ int can_unit_change_homecity(struct unit *punit)
   return pcity && pcity->owner==punit->owner;
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-int can_upgrade_unittype(struct player *pplayer, Unit_Type_id id)
-{
-  Unit_Type_id best_upgrade = -1;
-
-  if (!can_player_build_unit_direct(pplayer, id))
-    return -1;
-  while (unit_type_exists(id = unit_types[id].obsoleted_by))
-    if (can_player_build_unit_direct(pplayer, id))
-      best_upgrade = id;
-
-  return best_upgrade;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-int unit_upgrade_price(struct player *pplayer, Unit_Type_id from, Unit_Type_id to)
-{
-  int total, build;
-  build = unit_value(from)/2;
-  total = unit_value(to);
-  if (build>=total)
-    return 0;
-  return (total-build)*2+(total-build)*(total-build)/20; 
-}
-
 /**************************************************************************
 Return whether the unit can be put in auto-mode.
 (Auto-settler for settlers, auto-attack for military units.)
@@ -756,6 +515,7 @@ char* get_activity_text (int activity)
   case ACTIVITY_TRANSFORM:	text = _("Transform"); break;
   case ACTIVITY_AIRBASE:	text = _("Airbase"); break;
   case ACTIVITY_FALLOUT:	text = _("Fallout"); break;
+  case ACTIVITY_PATROL:  	text = _("Patrol"); break;
   default:			text = _("Unknown"); break;
   }
 
@@ -796,63 +556,27 @@ int can_unit_paradrop(struct unit *punit)
 }
 
 /**************************************************************************
-This function returns true if the tile at the given location can be
-"reclaimed" from ocean into land.  This is the case only when there are
-a sufficient number of adjacent tiles that are not ocean.
+Check if the unit's current activity is actually legal.
 **************************************************************************/
-static int can_reclaim_ocean(int x, int y)
+int can_unit_continue_current_activity(struct unit *punit)
 {
-  int i, j, landtiles;
+  enum unit_activity current = punit->activity;
+  int target = punit->activity_target;
+  int current2 = current == ACTIVITY_FORTIFIED ? ACTIVITY_FORTIFYING : current;
+  int result;
 
-  if (terrain_control.ocean_reclaim_requirement >= 9)
-    return 0;
-  if (terrain_control.ocean_reclaim_requirement <= 0)
-    return 1;
+  if (punit->connecting)
+    return can_unit_do_connect(punit, current);
 
-  landtiles = 0;
-  for (i = -1; i <= 1; i++) {
-    for (j = -1; j <= 1; j++) {
-      if (i || j) {
-	if (map_get_tile(x+i, y+j)->terrain != T_OCEAN) {
-	  landtiles++;
-	}
-	if (landtiles >= terrain_control.ocean_reclaim_requirement)
-	  return 1;
-      }
-    }
-  }
+  punit->activity = ACTIVITY_IDLE;
+  punit->activity_target = 0;
 
-  return 0;
-}
+  result = can_unit_do_activity_targeted(punit, current2, target);
 
-/**************************************************************************
-This function returns true if the tile at the given location can be
-"channeled" from land into ocean.  This is the case only when there are
-a sufficient number of adjacent tiles that are ocean.
-**************************************************************************/
-static int can_channel_land(int x, int y)
-{
-  int i, j, oceantiles;
+  punit->activity = current;
+  punit->activity_target = target;
 
-  if (terrain_control.land_channel_requirement >= 9)
-    return 0;
-  if (terrain_control.land_channel_requirement <= 0)
-    return 1;
-
-  oceantiles = 0;
-  for (i = -1; i <= 1; i++) {
-    for (j = -1; j <= 1; j++) {
-      if (i || j) {
-	if (map_get_tile(x+i, y+j)->terrain == T_OCEAN) {
-	  oceantiles++;
-	}
-	if (oceantiles >= terrain_control.land_channel_requirement)
-	  return 1;
-      }
-    }
-  }
-
-  return 0;
+  return result;
 }
 
 /**************************************************************************
@@ -882,6 +606,7 @@ int can_unit_do_activity_targeted(struct unit *punit,
   switch(activity) {
   case ACTIVITY_IDLE:
   case ACTIVITY_GOTO:
+  case ACTIVITY_PATROL:
     return 1;
 
   case ACTIVITY_POLLUTION:
@@ -947,13 +672,15 @@ int can_unit_do_activity_targeted(struct unit *punit,
   case ACTIVITY_FORTIFYING:
     return (is_ground_unit(punit) &&
 	    (punit->activity != ACTIVITY_FORTIFIED) &&
-	    !unit_flag(punit->type, F_SETTLERS));
+	    !unit_flag(punit->type, F_SETTLERS) &&
+	    ptile->terrain != T_OCEAN);
 
   case ACTIVITY_FORTIFIED:
     return 0;
 
   case ACTIVITY_FORTRESS:
     return (unit_flag(punit->type, F_SETTLERS) &&
+	    !map_get_city(punit->x, punit->y) &&
 	    player_knows_techs_with_flag(pplayer, TF_FORTRESS) &&
 	    !(ptile->special&S_FORTRESS) && ptile->terrain!=T_OCEAN);
 
@@ -984,8 +711,14 @@ int can_unit_do_activity_targeted(struct unit *punit,
       pspresent = get_tile_infrastructure_set(ptile);
       if (pspresent && is_ground_unit(punit)) {
 	psworking = get_unit_tile_pillage_set(punit->x, punit->y);
-	if (target == S_NO_SPECIAL)
-	  return ((pspresent & (~psworking)) != 0);
+	if (ptile->city && (target & (S_ROAD | S_RAILROAD)))
+	    return 0;
+	if (target == S_NO_SPECIAL) {
+	  if (ptile->city)
+	    return ((pspresent & (~(psworking | S_ROAD |S_RAILROAD))) != 0);
+	  else
+	    return ((pspresent & (~psworking)) != 0);
+	}
 	else if ((!game.rgame.pillage_select) &&
 		 (target != get_preferred_pillage(pspresent)))
 	  return 0;
@@ -1012,7 +745,8 @@ int can_unit_do_activity_targeted(struct unit *punit,
 	    unit_flag(punit->type, F_TRANSFORM));
 
   default:
-    freelog(LOG_NORMAL,"Unknown activity %d\n",activity);
+    freelog(LOG_ERROR, "Unknown activity %d in can_unit_do_activity_targeted()",
+	    activity);
     return 0;
   }
 }
@@ -1097,35 +831,38 @@ char *unit_activity_text(struct unit *punit)
      moves_str = _("Moves");
      if(is_air_unit(punit)) {
        int rate,f;
-       rate=get_unit_type(punit->type)->move_rate/3;
+       rate=get_unit_type(punit->type)->move_rate/SINGLE_MOVE;
        f=((punit->fuel)-1);
-       if(punit->moves_left%3) {
-	 if(punit->moves_left/3>0) {
-	   my_snprintf(text, sizeof(text), "%s: (%d)%d %d/3", moves_str,
-		       ((rate*f)+(punit->moves_left/3)),
-		       punit->moves_left/3, punit->moves_left%3);
+       if(punit->moves_left%SINGLE_MOVE) {
+	 if(punit->moves_left/SINGLE_MOVE>0) {
+	   my_snprintf(text, sizeof(text), "%s: (%d)%d %d/%d", moves_str,
+		       ((rate*f)+(punit->moves_left/SINGLE_MOVE)),
+		       punit->moves_left/SINGLE_MOVE, punit->moves_left%SINGLE_MOVE,
+		       SINGLE_MOVE);
 	 } else {
-	   my_snprintf(text, sizeof(text), "%s: (%d)%d/3", moves_str,
-		       ((rate*f)+(punit->moves_left/3)),
-		       punit->moves_left%3);
+	   my_snprintf(text, sizeof(text), "%s: (%d)%d/%d", moves_str,
+		       ((rate*f)+(punit->moves_left/SINGLE_MOVE)),
+		       punit->moves_left%SINGLE_MOVE, SINGLE_MOVE);
 	 }
        } else {
 	 my_snprintf(text, sizeof(text), "%s: (%d)%d", moves_str,
-		     rate*f+punit->moves_left/3,
-		     punit->moves_left/3);
+		     rate*f+punit->moves_left/SINGLE_MOVE,
+		     punit->moves_left/SINGLE_MOVE);
        }
      } else {
-       if(punit->moves_left%3) {
-	 if(punit->moves_left/3>0) {
-	   my_snprintf(text, sizeof(text), "%s: %d %d/3", moves_str,
-		       punit->moves_left/3, punit->moves_left%3);
+       if(punit->moves_left%SINGLE_MOVE) {
+	 if(punit->moves_left/SINGLE_MOVE>0) {
+	   my_snprintf(text, sizeof(text), "%s: %d %d/%d", moves_str,
+		       punit->moves_left/SINGLE_MOVE, punit->moves_left%SINGLE_MOVE,
+		       SINGLE_MOVE);
 	 } else {
 	   my_snprintf(text, sizeof(text),
-		       "%s: %d/3", moves_str, punit->moves_left%3);
+		       "%s: %d/%d", moves_str, punit->moves_left%SINGLE_MOVE,
+		       SINGLE_MOVE);
 	 }
        } else {
 	 my_snprintf(text, sizeof(text),
-		     "%s: %d", moves_str, punit->moves_left/3);
+		     "%s: %d", moves_str, punit->moves_left/SINGLE_MOVE);
        }
      }
      return text;
@@ -1143,6 +880,7 @@ char *unit_activity_text(struct unit *punit)
    case ACTIVITY_SENTRY:
    case ACTIVITY_GOTO:
    case ACTIVITY_EXPLORE:
+   case ACTIVITY_PATROL:
      return get_activity_text (punit->activity);
    case ACTIVITY_PILLAGE:
      if(punit->activity_target == 0) {
@@ -1154,8 +892,9 @@ char *unit_activity_text(struct unit *punit)
        return (text);
      }
    default:
-    freelog(LOG_FATAL, "Unknown unit activity:%d in unit_activity_text()", punit->activity);
-    exit(0);
+    freelog(LOG_FATAL, "Unknown unit activity %d in unit_activity_text()",
+	    punit->activity);
+    exit(1);
   }
   return 0;
 }
@@ -1223,216 +962,6 @@ void unit_list_sort_ord_city(struct unit_list *This)
   }
 }
 
-/**************************************************************************
-Returns 1 if the unit_type "exists" in this game, 0 otherwise.
-A unit_type doesn't exist if one of:
-- id is out of range
-- the unit_type has been flagged as removed by setting its
-  tech_requirement to A_LAST.
-**************************************************************************/
-int unit_type_exists(Unit_Type_id id)
-{
-  if (id<0 || id>=U_LAST || id>=game.num_unit_types)
-    return 0;
-  else 
-    return unit_types[id].tech_requirement!=A_LAST;
-}
-
-/**************************************************************************
-Does a linear search of unit_types[].name
-Returns U_LAST if none match.
-**************************************************************************/
-Unit_Type_id find_unit_type_by_name(char *s)
-{
-  int i;
-
-  for( i=0; i<game.num_unit_types; i++ ) {
-    if (strcmp(unit_types[i].name, s)==0)
-      return i;
-  }
-  return U_LAST;
-}
-
-
-/**************************************************************************
-The following functions use static variables so we can quickly look up
-which unit types have given flag or role.
-For these functions flags and roles are considered to be in the same "space",
-and any "role" argument can also be a "flag".
-Only units which pass unit_type_exists are counted.
-Unit order is in terms of the order in the units ruleset.
-**************************************************************************/
-static int first_init = 1;
-static int n_with_role[L_LAST];
-static Unit_Type_id *with_role[L_LAST];
-
-/**************************************************************************
-Do the real work for role_unit_precalcs, for one role (or flag), given by i.
-**************************************************************************/
-static void precalc_one(int i, int (*func_has)(Unit_Type_id, int))
-{
-  Unit_Type_id u;
-  int j;
-
-  /* Count: */
-  for(u=0; u<game.num_unit_types; u++) {
-    if(unit_type_exists(u) && func_has(u, i)) {
-      n_with_role[i]++;
-    }
-  }
-  if(n_with_role[i] > 0) {
-    with_role[i] = fc_malloc(n_with_role[i]*sizeof(Unit_Type_id));
-    for(j=0, u=0; u<game.num_unit_types; u++) {
-      if(unit_type_exists(u) && func_has(u, i)) {
-	with_role[i][j++] = u;
-      }
-    }
-    assert(j==n_with_role[i]);
-  }
-}
-
-/**************************************************************************
-Initialize; it is safe to call this multiple times (eg, if units have
-changed due to rulesets in client).
-**************************************************************************/
-void role_unit_precalcs(void)
-{
-  int i;
-  
-  if(!first_init) {
-    for(i=0; i<L_LAST; i++) {
-      free(with_role[i]);
-    }
-  }
-  for(i=0; i<L_LAST; i++) {
-    with_role[i] = NULL;
-    n_with_role[i] = 0;
-  }
-
-  for(i=0; i<F_LAST; i++) {
-    precalc_one(i, unit_flag);
-  }
-  for(i=L_FIRST; i<L_LAST; i++) {
-    precalc_one(i, unit_has_role);
-  }
-  first_init = 0;
-}
-
-/**************************************************************************
-How many unit types have specified role/flag.
-**************************************************************************/
-int num_role_units(int role)
-{
-  assert((role>=0 && role<F_LAST) || (role>=L_FIRST && role<L_LAST));
-  return n_with_role[role];
-}
-
-/**************************************************************************
-Return index-th unit with specified role/flag.
-Index -1 means (n-1), ie last one.
-**************************************************************************/
-Unit_Type_id get_role_unit(int role, int index)
-{
-  assert((role>=0 && role<F_LAST) || (role>=L_FIRST && role<L_LAST));
-  if (index==-1) index = n_with_role[role]-1;
-  assert(index>=0 && index<n_with_role[role]);
-  return with_role[role][index];
-}
-
-/**************************************************************************
-Return "best" unit this city can build, with given role/flag.
-Returns U_LAST if none match.
-**************************************************************************/
-Unit_Type_id best_role_unit(struct city *pcity, int role)
-{
-  Unit_Type_id u;
-  int j;
-  
-  assert((role>=0 && role<F_LAST) || (role>=L_FIRST && role<L_LAST));
-
-  for(j=n_with_role[role]-1; j>=0; j--) {
-    u = with_role[role][j];
-    if (can_build_unit(pcity, u)) {
-      return u;
-    }
-  }
-  return U_LAST;
-}
-
-/**************************************************************************
-  Convert unit_move_type names to enum; case insensitive;
-  returns 0 if can't match.
-**************************************************************************/
-enum unit_move_type unit_move_type_from_str(char *s)
-{
-  enum unit_move_type i;
-
-  /* a compile-time check would be nicer, but this will do: */
-  assert(sizeof(move_type_names)/sizeof(char*)==AIR_MOVING-LAND_MOVING+1);
-
-  for(i=LAND_MOVING; i<=AIR_MOVING; i++) {
-    if (mystrcasecmp(move_type_names[i-LAND_MOVING], s)==0) {
-      return i;
-    }
-  }
-  return 0;
-}
-
-/**************************************************************************
-  Convert Unit_Class_id names to enum; case insensitive;
-  returns UCL_LAST if can't match.
-**************************************************************************/
-Unit_Class_id unit_class_from_str(char *s)
-{
-  Unit_Class_id i;
-
-  assert(sizeof(unit_class_names)/sizeof(unit_class_names[0])==UCL_LAST);
-
-  for (i = 0; i < UCL_LAST; i++) {
-    if (mystrcasecmp(unit_class_names[i], s)==0) {
-      return i;
-    }
-  }
-
-  return UCL_LAST;
-}
-
-/**************************************************************************
-  Convert flag names to enum; case insensitive;
-  returns F_LAST if can't match.
-**************************************************************************/
-enum unit_flag_id unit_flag_from_str(char *s)
-{
-  enum unit_flag_id i;
-
-  assert(sizeof(flag_names)/sizeof(char*)==F_LAST);
-  
-  for(i=0; i<F_LAST; i++) {
-    if (mystrcasecmp(flag_names[i], s)==0) {
-      return i;
-    }
-  }
-  return F_LAST;
-}
-
-/**************************************************************************
-  Convert role names to enum; case insensitive;
-  returns L_LAST if can't match.
-**************************************************************************/
-enum unit_role_id unit_role_from_str(char *s)
-{
-  enum unit_role_id i;
-
-  assert(sizeof(role_names)/sizeof(char*)==L_LAST-L_FIRST);
-  
-  for(i=L_FIRST; i<L_LAST; i++) {
-    if (mystrcasecmp(role_names[i-L_FIRST], s)==0) {
-      return i;
-    }
-  }
-  return L_LAST;
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1444,7 +973,8 @@ struct player *unit_owner(struct unit *punit)
 /**************************************************************************
 Returns the number of free spaces for missiles. Can be 0 or negative.
 **************************************************************************/
-int missile_carrier_capacity(int x, int y, int playerid)
+int missile_carrier_capacity(int x, int y, int playerid,
+			     int count_units_with_extra_fuel)
 {
   struct tile *ptile = map_get_tile(x, y);
   int misonly = 0;
@@ -1463,7 +993,9 @@ int missile_carrier_capacity(int x, int y, int playerid)
 	misonly += get_transporter_capacity(punit);
 	continue;
       }
-      if (is_air_unit(punit)) {
+      /* Don't count units which have enough fuel (>1) */
+      if (is_air_unit(punit)
+	  && (count_units_with_extra_fuel || punit->fuel <= 1)) {
 	if (unit_flag(punit->type, F_MISSILE))
 	  misonly--;
 	else
@@ -1485,7 +1017,8 @@ int missile_carrier_capacity(int x, int y, int playerid)
 Returns the number of free spaces for airunits (includes missiles).
 Can be 0 or negative.
 **************************************************************************/
-int airunit_carrier_capacity(int x, int y, int playerid)
+int airunit_carrier_capacity(int x, int y, int playerid,
+			     int count_units_with_extra_fuel)
 {
   struct tile *ptile = map_get_tile(x, y);
   int misonly = 0;
@@ -1503,7 +1036,9 @@ int airunit_carrier_capacity(int x, int y, int playerid)
 	misonly += get_transporter_capacity(punit);
 	continue;
       }
-      if (is_air_unit(punit)) {
+      /* Don't count units which have enough fuel (>1) */
+      if (is_air_unit(punit)
+	  && (count_units_with_extra_fuel || punit->fuel <= 1)) {
 	if (unit_flag(punit->type, F_MISSILE))
 	  misonly--;
 	else
@@ -1518,3 +1053,109 @@ int airunit_carrier_capacity(int x, int y, int playerid)
 
   return airall;
 }
+
+/**************************************************************************
+Returns true if the tile contains an allied unit and only allied units.
+(ie, if your nation A is allied with B, and B is allied with C, a tile
+containing units from B and C will return false)
+**************************************************************************/
+struct unit *is_allied_unit_tile(struct tile *ptile, int playerid)
+{
+  struct unit *punit = NULL;
+
+  unit_list_iterate(ptile->units, cunit)
+    if (players_allied(playerid, cunit->owner))
+      punit = cunit;
+    else
+      return NULL;
+  unit_list_iterate_end;
+
+  return punit;
+}
+
+/**************************************************************************
+ is there an enemy unit on this tile?
+**************************************************************************/
+struct unit *is_enemy_unit_tile(struct tile *ptile, int playerid)
+{
+  unit_list_iterate(ptile->units, punit)
+    if (players_at_war(punit->owner, playerid))
+      return punit;
+  unit_list_iterate_end;
+
+  return NULL;
+}
+
+/**************************************************************************
+ is there an non-allied unit on this tile?
+**************************************************************************/
+struct unit *is_non_allied_unit_tile(struct tile *ptile, int playerid)
+{
+  unit_list_iterate(ptile->units, punit)
+    if (!players_allied(punit->owner, playerid))
+      return punit;
+  unit_list_iterate_end;
+
+  return NULL;
+}
+
+/**************************************************************************
+ is there an unit we have peace or ceasefire with on this tile?
+**************************************************************************/
+struct unit *is_non_attack_unit_tile(struct tile *ptile, int playerid)
+{
+  unit_list_iterate(ptile->units, punit)
+    if (players_non_attack(punit->owner, playerid))
+      return punit;
+  unit_list_iterate_end;
+
+  return NULL;
+}
+
+/**************************************************************************
+  Is this square controlled by the unit's owner?
+
+  Here "is_my_zoc" means essentially a square which is
+  *not* adjacent to an enemy unit on a land tile.
+  (Or, currently, an enemy city even if empty.)
+
+  Note this function only makes sense for ground units.
+**************************************************************************/
+int is_my_zoc(struct unit *myunit, int x0, int y0)
+{
+  int owner=myunit->owner;
+
+  assert(is_ground_unit(myunit));
+     
+  square_iterate(x0, y0, 1, x1, y1) {
+    if ((map_get_terrain(x1, y1)!=T_OCEAN)
+	&& is_non_allied_unit_tile(map_get_tile(x1, y1), owner))
+      return 0;
+  } square_iterate_end;
+
+  return 1;
+}
+
+/*
+ * Triremes have a varying loss percentage. based on tech.
+ * Seafaring reduces this to 25%, Navigation to 12.5%.  The Lighthouse
+ * wonder reduces this to 0.  AJS 20010301
+ */
+int trireme_loss_pct(struct player *pplayer, int x, int y) {
+  int losspct = 50;
+
+  /*
+   * If we are in a city or next to land, we have no chance of losing
+   * the ship.  To make this really useful for ai planning purposes, we'd
+   * need to confirm that we can exist/move at the x,y location we are given.
+   */
+  if ((map_get_terrain(x, y) != T_OCEAN) || is_coastline(x, y) ||
+      (player_owns_active_wonder(pplayer, B_LIGHTHOUSE)))
+	losspct = 0;
+  else if (player_knows_techs_with_flag(pplayer,TF_REDUCE_TRIREME_LOSS2))
+	losspct /= 4;
+  else if (player_knows_techs_with_flag(pplayer,TF_REDUCE_TRIREME_LOSS1))
+	losspct /= 2;
+
+  return losspct;
+}
