@@ -25,10 +25,10 @@
 #define MAJOR_VERSION		1
 #endif
 #ifndef MINOR_VERSION
-#define MINOR_VERSION		11
+#define MINOR_VERSION		12
 #endif
 #ifndef PATCH_VERSION
-#define PATCH_VERSION		4
+#define PATCH_VERSION		0
 #endif
 #ifndef VERSION_LABEL
 #define VERSION_LABEL		""
@@ -42,12 +42,12 @@
 
 /* This is only used if IS_BETA_VERSION is true. */
 #ifndef NEXT_STABLE_VERSION
-#define NEXT_STABLE_VERSION	"1.11.4"
+#define NEXT_STABLE_VERSION	"1.12.0"
 #endif
 /* This is only used in version.c, and only if IS_BETA_VERSION is true.
    The month[] array is defined in version.c (index: 1==Jan, 2==Feb, ...). */
 #ifndef NEXT_RELEASE_MONTH
-#define NEXT_RELEASE_MONTH	(month[7])
+#define NEXT_RELEASE_MONTH	(month[8])
 #endif
 
 #ifndef VERSION_STRING
@@ -58,16 +58,11 @@
                        VER_STRINGIFY(PATCH_VERSION) VERSION_LABEL
 #endif
 
-/* below, FREECIV_NAME_VERSION uses pre-processor string concatenation --dwp */
-#if IS_BETA_VERSION
-#define FREECIV_NAME_VERSION "Freeciv version " VERSION_STRING " (beta version)"
-#define WORD_VERSION "betatest version"
-#else
-#define FREECIV_NAME_VERSION "Freeciv version " VERSION_STRING
-#define WORD_VERSION "version"
-#endif
+/* version informational strings */
+char *freeciv_name_version(void);
+char *word_version(void);
 
 /* If returns NULL, not a beta version. */
-char *beta_message (void);
+char *beta_message(void);
 
 #endif  /* FC__VERSION_H */
