@@ -33,14 +33,14 @@
 #include "map.h"
 #include "nation.h"
 #include "rand.h"
+#include "support.h"
 #include "tech.h"
 
-#include "civserver.h"
 #include "gamehand.h"
 #include "maphand.h"
 #include "plrhand.h"
+#include "srv_main.h"
 #include "stdinhand.h"
-#include "unitfunc.h"
 #include "unithand.h"
 #include "unittools.h"
 
@@ -70,37 +70,6 @@ int is_sea_barbarian(struct player *pplayer)
   return (pplayer->ai.is_barbarian == SEA_BARBARIAN);
 }
 
-/**************************************************************************
-Random neighbouring cell - it maybe a stupid idea. Can anyone say how to
-move barbarians out of the hut?
-**************************************************************************/
-
-static void rand_neighbour( int x0, int y0, int *x, int *y)
-{
-  int choice;
-  int xoff[] = { -1,  0,  1, -1,  1, -1,  0,  1 };
-  int yoff[] = { -1, -1, -1,  0,  0,  1,  1,  1 };
-
-  if (y0 == 0) {
-    choice = 3 + myrand(5);
-  } else if(y0 == map.ysize-1){
-    choice = myrand(5);
-  } else {
-    choice = myrand(8);
-  }
-  *x = x0 + xoff[choice];
-  *y = y0 + yoff[choice];
-  
-#if 0
-  /* shouldn't need this now: */
-  *y = map_adjust_y(*y);
-  if( *x == x0 && *y == y0 ) /* might happen after y adjustments */
-    *x += 1;                 /* stupid but safe, rare anyway     */
-#endif
-  
-  *x = map_adjust_x(*x);
-}
-
 /**************************************************************************
 Creates the land/sea barbarian player and inits some stuff.
 If barbarian player already exists, return player pointer.
@@ -122,6 +91,7 @@ static struct player *create_barbarian_player(int land)
         barbarians->economic.gold = 0;
         barbarians->is_alive = 1;
         pick_ai_player_name( game.nation_count-1, barbarians->name);
+	sz_strlcpy(barbarians->username, barbarians->name);
         /* I need to make them to forget the map, I think */
         for( j = 0; j < map.xsize; j++ )
           for( k = 0; k < map.ysize; k++ )
@@ -146,8 +116,8 @@ static struct player *create_barbarian_player(int land)
   barbarians->nation = game.nation_count-1;
   pick_ai_player_name( game.nation_count-1, barbarians->name);
 
+  sz_strlcpy(barbarians->username, barbarians->name);
   barbarians->is_connected = 0;
-  barbarians->conn = NULL;
   barbarians->government = game.government_when_anarchy; 
   barbarians->capital = 0;
   barbarians->economic.gold = 100;
@@ -218,8 +188,6 @@ int unleash_barbarians(struct player* victim, int x, int y)
   int unit, unit_cnt, land_cnt=0, sea_cnt=0;
   int boat;
   int i, xu, yu, me;
-  int xx[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
-  int yy[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
   int alive = 1;     /* explorer survived */
 
   if(!game.barbarianrate || (game.year < game.onsetbarbarian)) {
@@ -240,10 +208,10 @@ int unleash_barbarians(struct player* victim, int x, int y)
     freelog(LOG_DEBUG, "Created barbarian unit %s",unit_types[unit].name);
   }
 
-  for( i = 0; i < 8; i++ ) {
-    land_cnt += is_free_land(x+xx[i], y+yy[i], me);
-    sea_cnt += is_free_sea(x+xx[i], y+yy[i], me);
-  }
+  adjc_iterate(x, y, x1, y1) {
+    land_cnt += is_free_land(x1, y1, me);
+    sea_cnt += is_free_sea(x1, y1, me);
+  } adjc_iterate_end;
 
   if( land_cnt >= 3 ) {           /* enough land, scatter guys around */
     unit_list_iterate(map_get_tile(x, y)->units, punit2) {
@@ -252,7 +220,7 @@ int unleash_barbarians(struct player* victim, int x, int y)
         do { 
           do rand_neighbour(x, y, &xu, &yu); 
           while( !is_free_land(xu, yu, me) );
-        } while( !handle_unit_move_request(barbarians, punit2, xu, yu, TRUE) );
+        } while( !handle_unit_move_request(punit2, xu, yu, TRUE, FALSE) );
         freelog(LOG_DEBUG, "Moved barbarian unit from %d %d to %d, %d", x,y,xu,yu);
       }
     }
@@ -278,7 +246,7 @@ int unleash_barbarians(struct player* victim, int x, int y)
 	      break;
 	    }
           }
-          handle_unit_move_request(barbarians, punit2, xu, yu, TRUE);
+          handle_unit_move_request(punit2, xu, yu, TRUE, FALSE);
         }
       unit_list_iterate_end;
     }
@@ -303,38 +271,29 @@ int unleash_barbarians(struct player* victim, int x, int y)
 /**************************************************************************
 Is sea not further than a couple of tiles away from land
 **************************************************************************/
-
-static int is_near_land( int x0, int y0)
+static int is_near_land(int x0, int y0)
 {
-  int x, y;
+  square_iterate(x0, y0, 4, x, y) {
+    if (map_get_terrain(x,y) != T_OCEAN)
+      return 1;
+  } square_iterate_end;
 
-  for( x = x0-4; x < x0+5; x++) {
-    for(y = y0-4; y < y0+5; y++) {
-      if( y < 0 || y >= map.ysize )
-        continue;
-      if( map_get_terrain(x,y) != T_OCEAN )
-        return 1;
-    }
-  }
   return 0;
 }
 
 /**************************************************************************
 return this or a neighbouring tile that is free of any units
 **************************************************************************/
-
 static int find_empty_tile_nearby(int x0, int y0, int *x, int *y)
 {
-  int xx[9] = { 0, -1, 0, 1, -1, 1, -1, 0, 1 };
-  int yy[9] = { 0, -1, -1, -1, 0, 0, 1, 1, 1 };
-  int i;
-
-  for( i = 0; i < 9; i++) {
-    *x = x0 + xx[i];
-    *y = map_adjust_y(y0 + yy[i]);
-    if( unit_list_size(&map_get_tile(*x, *y)->units) == 0 )
+  square_iterate(x0, y0, 1, x1, y1) {
+    if (unit_list_size(&map_get_tile(x1, y1)->units) == 0) {
+      *x = x1;
+      *y = y1;
       return 1;
-  }
+    }
+  } square_iterate_end;
+
   return 0;
 }
 
