@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -31,15 +32,13 @@
 #include "tech.h"
 #include "unit.h"
 
-#include "cityhand.h"
 #include "citytools.h"
-#include "civserver.h"
 #include "gamelog.h"
 #include "maphand.h"
 #include "plrhand.h"
 #include "settlers.h"
 #include "spacerace.h"
-#include "unitfunc.h"
+#include "srv_main.h"
 #include "unittools.h"
 
 #include "advdomestic.h"
@@ -48,8 +47,6 @@
 
 #include "cityturn.h"
 
-extern signed int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
-
 static void set_tax_income(struct city *pcity);
 static void set_food_trade_shields(struct city *pcity);
 static void add_buildings_effect(struct city *pcity);
@@ -82,14 +79,17 @@ static void pay_for_buildings(struct player *pplayer, struct city *pcity);
 
 static void sanity_check_city(struct city *pcity);
 
-static void disband_city(struct city *pcity);
+static int disband_city(struct city *pcity);
 
-static void begin_city_turn(struct player *pplayer, struct city *pcity);
+static void define_orig_production_values(struct city *pcity);
 static int update_city_activity(struct player *pplayer, struct city *pcity);
+static void nullify_caravan_and_disband_plus(struct city *pcity);
 
 static void worker_loop(struct city *pcity, int *foodneed,
 			int *prodneed, int *workers);
 
+static int advisor_choose_build(struct player *pplayer, struct city *pcity);
+
 /**************************************************************************
 calculate the incomes according to the taxrates and # of specialists.
 **************************************************************************/
@@ -163,14 +163,13 @@ static void happy_copy(struct city *pcity, int i)
 **************************************************************************/
 static void citizen_happy_size(struct city *pcity)
 {
-  int citizens, tmp;
+  int workers, tmp;
+
+  workers = pcity->size - city_specialists(pcity);
   tmp = content_citizens(&game.players[pcity->owner]);
-  citizens = MIN(pcity->size, tmp); 
-  tmp = (citizens - city_specialists(pcity));
-  pcity->ppl_content[0] = MAX(0, tmp);
-  tmp = (pcity->size - (pcity->ppl_content[0] + city_specialists(pcity)));
-  pcity->ppl_unhappy[0] = MAX(0, tmp);
-  pcity->ppl_happy[0]=0;
+  pcity->ppl_content[0] = MAX(0, MIN(workers, tmp));
+  pcity->ppl_unhappy[0] = workers - pcity->ppl_content[0];
+  pcity->ppl_happy[0] = 0; /* no one is born happy */
 }
 
 /**************************************************************************
@@ -180,7 +179,8 @@ static void citizen_happy_luxury(struct city *pcity)
 {
   int x=pcity->luxury_total;
   happy_copy(pcity, 0);
-  /* make people happy, content are made happy first, then unhappy content etc.   each conversions costs 2 luxuries. */
+  /* make people happy, content are made happy first, then unhappy content,
+     etc.  each conversions costs 2 luxuries. */
   while (x>=2 && (pcity->ppl_content[1])) {
     pcity->ppl_content[1]--;
     pcity->ppl_happy[1]++;
@@ -345,7 +345,6 @@ static void set_pollution(struct city *pcity)
 static void set_food_trade_shields(struct city *pcity)
 {
   int i;
-  int x,y;
   pcity->food_prod=0;
   pcity->shield_prod=0;
   pcity->trade_prod=0;
@@ -356,11 +355,11 @@ static void set_food_trade_shields(struct city *pcity)
 
   city_map_iterate(x, y) {
     if(get_worker_city(pcity, x, y)==C_TILE_WORKER) {
-      pcity->food_prod+=get_food_tile(x, y, pcity);
-      pcity->shield_prod+=get_shields_tile(x, y, pcity);
-      pcity->trade_prod+=get_trade_tile(x, y, pcity);
+      pcity->food_prod += city_get_food_tile(x, y, pcity);
+      pcity->shield_prod += city_get_shields_tile(x, y, pcity);
+      pcity->trade_prod += city_get_trade_tile(x, y, pcity);
     }
-  }
+  } city_map_iterate_end;
   pcity->tile_trade=pcity->trade_prod;
   
   pcity->food_surplus=pcity->food_prod-pcity->size*2;
@@ -393,14 +392,18 @@ int city_refresh(struct city *pcity)
   return (city_happy(pcity) && pcity->was_happy);
 }
 
+/**************************************************************************
+...
+called on government change or wonder completion or stuff like that -- Syela
+**************************************************************************/
 void global_city_refresh(struct player *pplayer)
-{ /* called on government change or wonder completion or stuff like that -- Syela */
-  connection_do_buffer(pplayer->conn);
+{
+  conn_list_do_buffer(&pplayer->connections);
   city_list_iterate(pplayer->cities, pcity)
     city_refresh(pcity);
     send_city_info(pplayer, pcity);
   city_list_iterate_end;
-  connection_do_unbuffer(pplayer->conn);
+  conn_list_do_unbuffer(&pplayer->connections);
 }
 
 /**************************************************************************
@@ -418,13 +421,9 @@ int unit_being_aggressive(struct unit *punit)
   if (map_get_city(punit->x,punit->y))
     return 0;
   if (is_ground_unit(punit) &&
-      map_get_special(punit->x,punit->y)&S_FORTRESS) {
-    city_list_iterate(get_player(punit->owner)->cities, pcity) {
-      if (real_map_distance(punit->x, punit->y, pcity->x, pcity->y)<=3)
-	return 0;
-    }
-    city_list_iterate_end;
-  }
+      map_get_special(punit->x,punit->y)&S_FORTRESS) 
+    return !is_unit_near_a_friendly_city (punit);
+  
   return 1;
 }
 
@@ -554,36 +553,51 @@ static void city_support(struct city *pcity)
   } unit_list_iterate_end;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void remove_obsolete_buildings_city(struct city *pcity, int refresh)
+{
+  struct player *pplayer = city_owner(pcity);
+  int i, sold = 0;
+  for (i=0;i<game.num_impr_types;i++) {
+    if (city_got_building(pcity, i) 
+	&& !is_wonder(i) 
+	&& improvement_obsolete(pplayer, i)) {
+      do_sell_building(pplayer, pcity, i);
+      notify_player_ex(pplayer, pcity->x, pcity->y, E_IMP_SOLD, 
+		       _("Game: %s is selling %s (obsolete) for %d."),
+		       pcity->name, get_improvement_name(i), 
+		       improvement_value(i));
+      sold = 1;
+    }
+  }
+
+  if (sold && refresh) {
+    city_refresh(pcity);
+    send_city_info(pplayer, pcity);
+    send_player_info(pplayer, 0); /* Send updated gold to all */
+  }
+}
 
 /**************************************************************************
 ...
 **************************************************************************/
 void remove_obsolete_buildings(struct player *pplayer)
 {
-  int i;
   city_list_iterate(pplayer->cities, pcity) {
-    for (i=0;i<B_LAST;i++) {
-      if(city_got_building(pcity, i) 
-	 && !is_wonder(i) 
-	 && improvement_obsolete(pplayer, i)) {
-	do_sell_building(pplayer, pcity, i);
-	notify_player_ex(pplayer, pcity->x, pcity->y, E_IMP_SOLD, 
-			 _("Game: %s is selling %s (obsolete) for %d."),
-			 pcity->name, get_improvement_name(i), 
-			 improvement_value(i)/2);
-      }
-    }
-  }
-  city_list_iterate_end;
+    remove_obsolete_buildings_city(pcity, 0);
+  } city_list_iterate_end;
 }
 
 /**************************************************************************
-...
+You need to call sync_cities for teh affected cities to be synced with the
+client.
 **************************************************************************/
 static void worker_loop(struct city *pcity, int *foodneed,
 			int *prodneed, int *workers)
 {
-  int x, y, bx, by, best, cur;
+  int bx, by, best, cur;
   int conflict[5][5];
   int e, pwr, luxneed = 0; /* I should have thought of this earlier, it is so simple */
 
@@ -613,16 +627,24 @@ static void worker_loop(struct city *pcity, int *foodneed,
   freelog(LOG_DEBUG, "%s, foodneed %d prodneed %d",
 	  pcity->name, *foodneed, *prodneed);
 
+  /* better than nothing, not as good as a global worker allocation -- Syela */
   city_map_iterate(x, y) {
-    conflict[x][y] = -1 - minimap[map_adjust_x(pcity->x+x-2)][map_adjust_y(pcity->y+y-2)];
-  } /* better than nothing, not as good as a global worker allocation -- Syela */
+    int map_x = pcity->x + x - CITY_MAP_SIZE/2;
+    int map_y = pcity->y + y - CITY_MAP_SIZE/2;
+
+    if (normalize_map_pos(&map_x, &map_y)) {
+      conflict[x][y] = -1 - minimap[map_x][map_y];
+    } else {
+      conflict[x][y] = 0;
+    }
+  } city_map_iterate_end;
 
   do {
     bx=0;
     by=0;
     best = 0;
     /* try to work near the city */
-    city_map_iterate_outwards(x, y)
+    city_map_iterate_outwards(x, y) {
       if(can_place_worker_here(pcity, x, y)) {
         cur = city_tile_value(pcity, x, y, *foodneed, *prodneed) - conflict[x][y];
         if (cur > best) {
@@ -631,12 +653,12 @@ static void worker_loop(struct city *pcity, int *foodneed,
           best = cur;
 	}
       }
-    city_map_iterate_outwards_end;
+    } city_map_iterate_outwards_end;
     if(bx || by) {
-      set_worker_city(pcity, bx, by, C_TILE_WORKER);
+      server_set_worker_city(pcity, bx, by);
       (*workers)--; /* amazing what this did with no parens! -- Syela */
-      *foodneed -= get_food_tile(bx,by,pcity) - 2;
-      *prodneed -= get_shields_tile(bx,by,pcity) - 1;
+      *foodneed -= city_get_food_tile(bx,by,pcity) - 2;
+      *prodneed -= city_get_shields_tile(bx,by,pcity) - 1;
     }
   } while(*workers && (bx || by));
   *foodneed += 2 * (*workers - 1 - e);
@@ -648,26 +670,33 @@ static void worker_loop(struct city *pcity, int *foodneed,
 }
 
 /**************************************************************************
-...
+You need to call sync_cities for teh affected cities to be synced with the
+client.
 **************************************************************************/
-int  add_adjust_workers(struct city *pcity)
+int add_adjust_workers(struct city *pcity)
 {
   int workers=pcity->size;
   int iswork=0;
   int toplace;
   int foodneed;
   int prodneed = 0;
-  int x,y;
-  city_map_iterate(x, y) 
+
+  city_map_iterate(x, y) {
     if (get_worker_city(pcity, x, y)==C_TILE_WORKER) 
       iswork++;
-  
-  iswork--;
-  if (iswork+city_specialists(pcity)>workers)
+  } city_map_iterate_end;
+  iswork--; /* City center */
+
+  if (iswork+city_specialists(pcity)>workers) {
+    freelog(LOG_ERROR, "Encountered an inconsistency in "
+	    "add_adjust_workers() for city %s", pcity->name);
     return 0;
+  }
+
   if (iswork+city_specialists(pcity)==workers)
     return 1;
-  toplace=workers-(iswork+city_specialists(pcity));
+
+  toplace = workers-(iswork+city_specialists(pcity));
   foodneed = -pcity->food_surplus;
   prodneed = -pcity->shield_surplus;
 
@@ -678,25 +707,25 @@ int  add_adjust_workers(struct city *pcity)
 }
 
 /**************************************************************************
-...
+You need to call sync_cities for teh affected cities to be synced with the
+client.
 **************************************************************************/
-
 void auto_arrange_workers(struct city *pcity)
 {
   struct government *g = get_gov_pcity(pcity);
   int workers = pcity->size;
   int taxwanted,sciwanted;
-  int x,y;
   int foodneed, prodneed;
 
-  city_map_iterate(x, y)
-    if (get_worker_city(pcity, x, y)==C_TILE_WORKER) 
-      set_worker_city(pcity, x, y, C_TILE_EMPTY);
+  city_map_iterate(x, y) {
+    if (get_worker_city(pcity, x, y)==C_TILE_WORKER
+	&& (x != CITY_MAP_SIZE/2 || y != CITY_MAP_SIZE/2))
+      server_remove_worker_city(pcity, x, y);
+  } city_map_iterate_end;
   
-  set_worker_city(pcity, 2, 2, C_TILE_WORKER); 
-  foodneed=(pcity->size *2 -get_food_tile(2,2, pcity)) + settler_eats(pcity);
+  foodneed=(pcity->size *2 -city_get_food_tile(2,2, pcity)) + settler_eats(pcity);
   prodneed = 0;
-  prodneed -= get_shields_tile(2,2,pcity);
+  prodneed -= city_get_shields_tile(2,2,pcity);
   prodneed -= citygov_free_shield(pcity, g);
 
   unit_list_iterate(pcity->units_supported, this_unit) {
@@ -730,18 +759,45 @@ void auto_arrange_workers(struct city *pcity)
   pcity->ppl_elvis=workers;
 
   city_refresh(pcity);
-  send_city_info(city_owner(pcity), pcity);
 }
 
 /**************************************************************************
-...
+Notices about cities that should be sent to all players.
 **************************************************************************/
-void send_city_turn_notifications(struct player *pplayer, struct city *pcity)
+void send_global_city_turn_notifications(struct conn_list *dest)
+{
+  if (!dest)
+    dest = &game.all_connections;
+
+  players_iterate(pplayer) {
+    city_list_iterate(pplayer->cities, pcity) {
+      /* can_player_build_improvement() checks whether wonder is build
+	 elsewhere (or destroyed) */
+      if (!pcity->is_building_unit && is_wonder(pcity->currently_building)
+	  && (city_turns_to_build(pcity, pcity->currently_building, 0) <= 1)
+	  && can_player_build_improvement(city_owner(pcity), pcity->currently_building)) {
+	notify_conn_ex(dest, pcity->x, pcity->y,
+		       E_CITY_WONDER_WILL_BE_BUILT,
+		       _("Game: Notice: Wonder %s in %s will be finished"
+			 " next turn."), 
+		       get_improvement_name(pcity->currently_building),
+		       pcity->name);
+      }
+    } city_list_iterate_end;
+  } players_iterate_end;
+}
+
+/**************************************************************************
+  Send turn notifications for specified city to specified connections.
+  Neither dest nor pcity may be NULL.
+**************************************************************************/
+void send_city_turn_notifications(struct conn_list *dest, struct city *pcity)
 {
   int turns_growth, turns_granary;
-
+  int can_grow;
+ 
   if (pcity->food_surplus > 0) {
-    turns_growth = (((pcity->size+1) * game.foodbox) - pcity->food_stock - 1)
+    turns_growth = (city_granary_size(pcity->size) - pcity->food_stock - 1)
 		   / pcity->food_surplus;
 
     if (!city_got_effect(pcity,B_GRANARY) && !pcity->is_building_unit &&
@@ -752,7 +808,7 @@ void send_city_turn_notifications(struct player *pplayer, struct city *pcity)
 	 preserves food.  -AJS */
       if ((turns_growth < 5) && (turns_granary < 5) &&
 	  (turns_growth < turns_granary)) {
-	notify_player_ex(city_owner(pcity), pcity->x, pcity->y,
+	notify_conn_ex(dest, pcity->x, pcity->y,
 			 E_CITY_GRAN_THROTTLE,
 			 _("Game: Suggest throttling growth in %s to use %s "
 			   "(being built) more effectively."), pcity->name,
@@ -760,13 +816,26 @@ void send_city_turn_notifications(struct player *pplayer, struct city *pcity)
       }
     }
 
-    if ((turns_growth <= 0) && !city_celebrating(pcity)) {
-      notify_player_ex(city_owner(pcity), pcity->x, pcity->y,
+    can_grow = (city_got_building(pcity, B_AQUEDUCT)
+		|| pcity->size < game.aqueduct_size)
+      && (city_got_building(pcity, B_SEWER)
+	  || pcity->size < game.sewer_size);
+
+    if ((turns_growth <= 0) && !city_celebrating(pcity) && can_grow) {
+      notify_conn_ex(dest, pcity->x, pcity->y,
 		       E_CITY_MAY_SOON_GROW,
 		       _("Game: %s may soon grow to size %i."),
 		       pcity->name, pcity->size + 1);
     }
+  } else {
+    if (pcity->food_stock + pcity->food_surplus <= 0 && pcity->food_surplus < 0) {
+      notify_conn_ex(dest, pcity->x, pcity->y,
+		     E_CITY_FAMINE_FEARED,
+		     _("Game: Warning: Famine feared in %s."),
+		     pcity->name);
+    }
   }
+  
 }
 
 /**************************************************************************
@@ -775,7 +844,7 @@ void send_city_turn_notifications(struct player *pplayer, struct city *pcity)
 void begin_cities_turn(struct player *pplayer)
 {
   city_list_iterate(pplayer->cities, pcity)
-     begin_city_turn(pplayer, pcity);
+     define_orig_production_values(pcity);
   city_list_iterate_end;
 }
 
@@ -792,7 +861,7 @@ void update_city_activities(struct player *pplayer)
   city_list_iterate_end;
   pplayer->ai.prev_gold = gold;
   if (gold-(gold-pplayer->economic.gold)*3<0) {
-    notify_player_ex(pplayer, 0, 0, E_LOW_ON_FUNDS,
+    notify_player_ex(pplayer, -1, -1, E_LOW_ON_FUNDS,
 		     _("Game: WARNING, we're LOW on FUNDS sire."));  
   }
     /* uncomment to unbalance the game, like in civ1 (CLG)
@@ -814,18 +883,18 @@ void city_auto_remove_worker(struct city *pcity)
   if(city_specialists(pcity)) {
     if(pcity->ppl_taxman) {
       pcity->ppl_taxman--;
-      return;
     } else if(pcity->ppl_scientist) {
       pcity->ppl_scientist--;
-      return;
-    } else if(pcity->ppl_elvis) {
+    } else {
+      assert(pcity->ppl_elvis);
       pcity->ppl_elvis--; 
-      return;
     }
+    city_refresh(pcity);
+    send_city_info(city_owner(pcity), pcity);
+    return;
   } 
   auto_arrange_workers(pcity);
-  city_refresh(pcity);
-  send_city_info(city_owner(pcity), pcity);
+  sync_cities();
 }
 
 /**************************************************************************
@@ -833,25 +902,26 @@ Note: We do not send info about the city to the clients as part of this function
 **************************************************************************/
 static void city_increase_size(struct city *pcity)
 {
-  int have_square, x, y;
+  struct player *powner = city_owner(pcity);
+  int have_square;
   int has_granary = city_got_effect(pcity, B_GRANARY);
   int rapture_grow = city_rapture_grow(pcity); /* check before size increase! */
   int new_food;
-  
+
   if (!city_got_building(pcity, B_AQUEDUCT)
       && pcity->size>=game.aqueduct_size) {/* need aqueduct */
     if (!pcity->is_building_unit && pcity->currently_building == B_AQUEDUCT) {
-      notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_AQ_BUILDING,
+      notify_player_ex(powner, pcity->x, pcity->y, E_CITY_AQ_BUILDING,
 		       _("Game: %s needs %s (being built) "
 			 "to grow any further."), pcity->name,
 		       improvement_types[B_AQUEDUCT].name);
     } else {
-      notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_AQUEDUCT,
+      notify_player_ex(powner, pcity->x, pcity->y, E_CITY_AQUEDUCT,
 		       _("Game: %s needs %s to grow any further."),
 		       pcity->name, improvement_types[B_AQUEDUCT].name);
     }
     /* Granary can only hold so much */
-    new_food = ((pcity->size+1) * game.foodbox *
+    new_food = (city_granary_size(pcity->size) *
 		(100 - game.aqueductloss/(1+has_granary))) / 100;
     pcity->food_stock = MIN(pcity->food_stock, new_food);
     return;
@@ -860,17 +930,17 @@ static void city_increase_size(struct city *pcity)
   if (!city_got_building(pcity, B_SEWER)
       && pcity->size>=game.sewer_size) {/* need sewer */
     if (!pcity->is_building_unit && pcity->currently_building == B_SEWER) {
-      notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_AQ_BUILDING,
+      notify_player_ex(powner, pcity->x, pcity->y, E_CITY_AQ_BUILDING,
 		       _("Game: %s needs %s (being built) "
 			 "to grow any further."), pcity->name,
 		       improvement_types[B_SEWER].name);
     } else {
-      notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_AQUEDUCT,
+      notify_player_ex(powner, pcity->x, pcity->y, E_CITY_AQUEDUCT,
 		       _("Game: %s needs %s to grow any further."),
 		       pcity->name, improvement_types[B_SEWER].name);
     }
     /* Granary can only hold so much */
-    new_food = ((pcity->size+1) * game.foodbox *
+    new_food = (city_granary_size(pcity->size) *
 		(100 - game.aqueductloss/(1+has_granary))) / 100;
     pcity->food_stock = MIN(pcity->food_stock, new_food);
     return;
@@ -879,10 +949,10 @@ static void city_increase_size(struct city *pcity)
   pcity->size++;
   /* Do not empty food stock if city is growing by celebrating */
   if (rapture_grow) {
-    new_food = (pcity->size+1) * game.foodbox;
+    new_food = city_granary_size(pcity->size);
   } else {
     if (has_granary)
-      new_food = ((pcity->size+1) * game.foodbox) / 2;
+      new_food = city_granary_size(pcity->size) / 2;
     else
       new_food = 0;
   }
@@ -896,7 +966,7 @@ static void city_increase_size(struct city *pcity)
     if (can_place_worker_here(pcity, x, y)) {
       have_square = TRUE;
     }
-  }
+  } city_map_iterate_end;
   if (((pcity->food_surplus >= 2) || !have_square)  &&  pcity->size >= 5  &&
       (pcity->city_options & ((1<<CITYO_NEW_EINSTEIN) | (1<<CITYO_NEW_TAXMAN)))){
 
@@ -913,14 +983,14 @@ static void city_increase_size(struct city *pcity)
 
   city_refresh(pcity);
 
-  if (game.players[pcity->owner].ai.control) /* don't know if we need this -- Syela */
+  if (powner->ai.control) /* don't know if we need this -- Syela */
     if (ai_fix_unhappy(pcity))
       ai_scientists_taxmen(pcity);
 
-  connection_do_buffer(city_owner(pcity)->conn);
-  notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_GROWTH,
+  notify_player_ex(powner, pcity->x, pcity->y, E_CITY_GROWTH,
                    _("Game: %s grows to size %d."), pcity->name, pcity->size);
-  connection_do_unbuffer(city_owner(pcity)->conn);
+
+  sync_cities();
 }
 
 /**************************************************************************
@@ -929,9 +999,9 @@ static void city_increase_size(struct city *pcity)
 static void city_reduce_size(struct city *pcity)
 {
   notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_FAMINE,
-		   _("Game: Famine feared in %s."), pcity->name);
+		   _("Game: Famine causes population loss in %s."), pcity->name);
   if (city_got_effect(pcity, B_GRANARY))
-    pcity->food_stock=(pcity->size*game.foodbox)/2;
+    pcity->food_stock=city_granary_size(pcity->size-1)/2;
   else
     pcity->food_stock=0;
   pcity->size--;
@@ -946,8 +1016,10 @@ static void city_reduce_size(struct city *pcity)
 static void city_populate(struct city *pcity)
 {
   pcity->food_stock+=pcity->food_surplus;
-  if(pcity->food_stock >= (pcity->size+1)*game.foodbox || city_rapture_grow(pcity))
+  if(pcity->food_stock >= city_granary_size(pcity->size) 
+     || city_rapture_grow(pcity)) {
     city_increase_size(pcity);
+  }
   else if(pcity->food_stock<0) {
     /* FIXME: should this depend on units with ability to build
      * cities or on units that require food in uppkeep?
@@ -967,7 +1039,7 @@ static void city_populate(struct city *pcity)
 		get_nation_name_plural(game.players[pcity->owner].nation),
 		utname);
 	if (city_got_effect(pcity, B_GRANARY))
-	  pcity->food_stock=((pcity->size+1)*game.foodbox)/2;
+	  pcity->food_stock=city_granary_size(pcity->size)/2;
 	else
 	  pcity->food_stock=0;
 	return;
@@ -981,8 +1053,8 @@ static void city_populate(struct city *pcity)
 /**************************************************************************
 ...
 **************************************************************************/
-int advisor_choose_build(struct player *pplayer, struct city *pcity)
-{ /* Old stuff that I obsoleted deleted. -- Syela */
+static int advisor_choose_build(struct player *pplayer, struct city *pcity)
+{
   struct ai_choice choice;
   int i;
   int id=-1;
@@ -1002,7 +1074,7 @@ int advisor_choose_build(struct player *pplayer, struct city *pcity)
     return 1; /* making something.  return value = 1 */
   }
 
-  for (i=0;i<B_LAST;i++)
+  for (i=0;i<game.num_impr_types;i++)
     if(can_build_improvement(pcity, i) && i != B_PALACE) { /* build something random, undecided */
       pcity->currently_building=i;
       pcity->is_building_unit=0;
@@ -1104,11 +1176,28 @@ static int worklist_change_build_target(struct player *pplayer, struct city *pci
 	 we can build. */
       if (new_target == target) {
 	/* Nope, no use.  *sigh*  */
-	notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
-			 _("Game: %s can't build %s from the worklist; "
-			   "tech not yet available.  Postponing..."),
-			 pcity->name,
-			 get_impr_name_ex(pcity, target));
+	if (!player_knows_improvement_tech(pplayer, target)) {
+	  notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			   _("Game: %s can't build %s from the worklist; "
+			     "tech not yet available.  Postponing..."),
+			   pcity->name,
+			   get_impr_name_ex(pcity, target));
+	} else if (improvement_types[target].bldg_req != B_LAST) {
+	  notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			   _("Game: %s can't build %s from the worklist; "
+			     "need to have %s first.  Postponing..."),
+			   pcity->name,
+			   get_impr_name_ex(pcity, target),
+			   get_impr_name_ex(pcity, improvement_types[target].bldg_req));
+	} else {
+	  /* This shouldn't happen...
+	     FIXME: make can_build_improvement() return a reason enum. */
+	  notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
+			   _("Game: %s can't build %s from the worklist; "
+			     "Reason unknown!  Postponing..."),
+			   pcity->name,
+			   get_impr_name_ex(pcity, target));
+	}
 	continue;
       } else {
 	/* Hey, we can upgrade the improvement!  */
@@ -1231,7 +1320,6 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 {
   struct government *g = get_gov_pplayer(pplayer);
   int space_part;
-  int city_built_city_builder;
 
   while (pcity->shield_surplus<0) {
     unit_list_iterate(pcity->units_supported, punit) {
@@ -1246,10 +1334,15 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
     unit_list_iterate_end;
   }
 
+  /* Now we confirm changes made last turn. */
   pcity->shield_stock+=pcity->shield_surplus;
+  pcity->before_change_shields=pcity->shield_stock;
+  nullify_caravan_and_disband_plus(pcity);
+
   if (!pcity->is_building_unit) {
     if (pcity->currently_building==B_CAPITAL) {
       pplayer->economic.gold+=pcity->shield_surplus;
+      pcity->before_change_shields=0;
       pcity->shield_stock=0;
     }    
     upgrade_building_prod(pcity);
@@ -1280,7 +1373,9 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	space_part = 0;
 	pcity->improvements[pcity->currently_building]=1;
       }
-      pcity->shield_stock-=improvement_value(pcity->currently_building); 
+      pcity->before_change_shields-=
+			  improvement_value(pcity->currently_building); 
+      pcity->shield_stock-=improvement_value(pcity->currently_building);
       pcity->turn_last_built = game.year;
       /* to eliminate micromanagement */
       if(is_wonder(pcity->currently_building)) {
@@ -1324,40 +1419,40 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	send_spaceship_info(pplayer, 0);
       } else {
 	city_refresh(pcity);
-	/* If there's something in the worklist, change the build target. */
-	if (!worklist_change_build_target(pplayer, pcity)) {
-	  /* Fall back to the good old ways. */
-	  freelog(LOG_DEBUG, "Trying advisor_choose_build.");
-	  advisor_choose_build(pplayer, pcity);
-	  freelog(LOG_DEBUG, "Advisor_choose_build didn't kill us.");
-	}
+      }
+      /* If there's something in the worklist, change the build target.
+       * Else if just built a spaceship part, keep building the same part.
+       * (Fixme? - doesn't check whether spaceship part is still sensible.)
+       * Else co-opt AI routines as "city advisor".
+       */
+      if (!worklist_change_build_target(pplayer, pcity) && !space_part) {
+	/* Fall back to the good old ways. */
+	freelog(LOG_DEBUG, "Trying advisor_choose_build.");
+	advisor_choose_build(pplayer, pcity);
+	freelog(LOG_DEBUG, "Advisor_choose_build didn't kill us.");
       }
     } 
-  } else {
-    city_built_city_builder = 0;
-
+  } else { /* is_building_unit */
     upgrade_unit_prod(pcity);
 
     /* FIXME: F_CITIES should be changed to any unit
      * that 'contains' 1 (or more) pop -- sjolie
      */
-    if(pcity->shield_stock>=unit_value(pcity->currently_building)) {
+    if (pcity->shield_stock>=unit_value(pcity->currently_building)) {
       if (unit_flag(pcity->currently_building, F_CITIES)) {
 	if (pcity->size==1) {
 
 	  /* Should we disband the city? -- Massimo */
 	  if (pcity->city_options & ((1<<CITYO_DISBAND))) {
-	    disband_city(pcity);
-	    return 0;
+	    return !disband_city(pcity);
 	  } else {
 	    notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
 			     _("Game: %s can't build %s yet."),
 			     pcity->name, unit_name(pcity->currently_building));
+	    return 1;
 	  }
-	  return 1;
 
 	}
-	city_built_city_builder = 1;
       }
       
       pcity->turn_last_built = game.year;
@@ -1366,34 +1461,35 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
       create_unit(pplayer, pcity->x, pcity->y, pcity->currently_building,
 		  do_make_unit_veteran(pcity, pcity->currently_building), 
 		  pcity->id, -1);
-      /* to eliminate micromanagement, we only subtract the unit's cost */
-      pcity->shield_stock-=unit_value(pcity->currently_building); 
-
-      if (city_built_city_builder) {
+      /* After we created the unit, so that removing the worker will take
+	 into account the extra resources (food) needed. */
+      if (unit_flag(pcity->currently_building, F_CITIES)) {
 	pcity->size--;
 	city_auto_remove_worker(pcity);
       }
 
+      /* to eliminate micromanagement, we only subtract the unit's cost */
+      pcity->before_change_shields-=unit_value(pcity->currently_building); 
+      pcity->shield_stock-=unit_value(pcity->currently_building);
+
       notify_player_ex(pplayer, pcity->x, pcity->y, E_UNIT_BUILD,
 		       _("Game: %s is finished building %s."), 
 		       pcity->name, 
 		       unit_types[pcity->currently_building].name);
 
+      gamelog(GAMELOG_UNIT, "%s build %s in %s (%i,%i)",
+	      get_nation_name_plural(pplayer->nation), 
+	      unit_types[pcity->currently_building].name,
+	      pcity->name, pcity->x, pcity->y);
+
       /* If there's something in the worklist, change the build target. 
 	 If there's nothing there, worklist_change_build_target won't
 	 do anything. */
       worklist_change_build_target(pplayer, pcity);
-
-
-    gamelog(GAMELOG_UNIT, "%s build %s in %s (%i,%i)",
-	    get_nation_name_plural(pplayer->nation), 
-	    unit_types[pcity->currently_building].name,
-	    pcity->name, pcity->x, pcity->y);
-
-
     }
   }
-return 1;
+
+  return 1;
 }
 
 /**************************************************************************
@@ -1402,7 +1498,7 @@ return 1;
 static void pay_for_buildings(struct player *pplayer, struct city *pcity)
 {
   int i;
-  for (i=0;i<B_LAST;i++) 
+  for (i=0;i<game.num_impr_types;i++) 
     if (city_got_building(pcity, i)) {
       if (is_wonder(i)) {
 	if (wonder_obsolete(i)) {
@@ -1429,39 +1525,6 @@ static void pay_for_buildings(struct player *pplayer, struct city *pcity)
     }
 }
 
-/**************************************************************************
-1) check for enemy units on citymap tiles
-2) withdraw workers from such tiles
-3) mark citymap tiles accordingly empty/unavailable  
-**************************************************************************/
-void city_check_workers(struct player *pplayer, struct city *pcity)
-{
-  int x, y;
-  
-  city_map_iterate(x, y) {
-    struct tile *ptile=map_get_tile(pcity->x+x-2, pcity->y+y-2);
-    
-    if(unit_list_size(&ptile->units)>0) {
-      struct unit *punit=unit_list_get(&ptile->units, 0);
-      if (players_at_war(pplayer->player_no, punit->owner)) {
-	if (get_worker_city(pcity, x, y)==C_TILE_WORKER) {
-/*	  pcity->ppl_elvis++; -- this is really not polite -- Syela */
-  	  set_worker_city(pcity, x, y, C_TILE_UNAVAILABLE);
-          add_adjust_workers(pcity); /* will place the displaced */
-          city_refresh(pcity); /* may be unnecessary; can't hurt */
-        } else
-	  set_worker_city(pcity, x, y, C_TILE_UNAVAILABLE);
-	continue;
-      }
-    }
-    if(get_worker_city(pcity, x, y)==C_TILE_UNAVAILABLE)
-      set_worker_city(pcity, x, y, C_TILE_EMPTY);
-    if(get_worker_city(pcity, x, y)!=C_TILE_WORKER &&
-       !can_place_worker_here(pcity, x, y))
-      set_worker_city(pcity, x, y, C_TILE_UNAVAILABLE);
-  }
-}
-
 /**************************************************************************
  Add some Pollution if we have waste
 **************************************************************************/
@@ -1497,19 +1560,19 @@ static void check_pollution(struct city *pcity)
 static void sanity_check_city(struct city *pcity)
 {
   int size=pcity->size;
-  int x,y;
   int iswork=0;
   city_map_iterate(x, y) {
     if (get_worker_city(pcity, x, y)==C_TILE_WORKER) 
 	iswork++;
-  }
+  } city_map_iterate_end;
   iswork--;
   if (iswork+city_specialists(pcity)!=size) {
-    freelog(LOG_NORMAL,
+    freelog(LOG_ERROR,
 	    "%s is bugged: size:%d workers:%d elvis: %d tax:%d sci:%d",
 	    pcity->name, size, iswork, pcity->ppl_elvis,
 	    pcity->ppl_taxman, pcity->ppl_scientist); 
     auto_arrange_workers(pcity);
+    sync_cities();
   }
 }
 
@@ -1549,13 +1612,13 @@ void city_incite_cost(struct city *pcity)
 /**************************************************************************
  Called every turn, at beginning of turn, for every city.
 **************************************************************************/
-static void begin_city_turn(struct player *pplayer, struct city *pcity)
+static void define_orig_production_values(struct city *pcity)
 {
   /* remember what this city is building at start of turn,
      so user can switch production back without penalty */
   pcity->changed_from_id = pcity->currently_building;
   pcity->changed_from_is_unit = pcity->is_building_unit;
-  pcity->before_change_shields = pcity->shield_stock;
+
   freelog(LOG_DEBUG,
 	  "In %s, building %s.  Beg of Turn shields = %d",
 	  pcity->name,
@@ -1566,6 +1629,24 @@ static void begin_city_turn(struct player *pplayer, struct city *pcity)
 	  );
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static void nullify_caravan_and_disband_plus(struct city *pcity)
+{
+  pcity->disbanded_shields=0;
+  pcity->caravan_shields=0;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void nullify_prechange_production(struct city *pcity)
+{
+  nullify_caravan_and_disband_plus(pcity);
+  pcity->before_change_shields=0;
+}
+
 /**************************************************************************
  Called every turn, at end of turn, for every city.
 **************************************************************************/
@@ -1574,7 +1655,6 @@ static int update_city_activity(struct player *pplayer, struct city *pcity)
   struct government *g = get_gov_pcity(pcity);
   int got_tech = 0;
 
-  city_check_workers(pplayer, pcity);
   city_refresh(pcity);
 
   /* the AI often has widespread disorder when the Gardens or Oracle
@@ -1655,14 +1735,14 @@ static int update_city_activity(struct player *pplayer, struct city *pcity)
       handle_player_revolution(pplayer);
     }
     sanity_check_city(pcity);
-    }
+  }
   return got_tech;
 }
 
 /**************************************************************************
  disband a city into a settler, supported by the closest city -- Massimo
 **************************************************************************/
-static void disband_city(struct city *pcity)
+static int disband_city(struct city *pcity)
 {
   struct player *pplayer = get_player(pcity->owner);
   int x = pcity->x, y = pcity->y;
@@ -1677,15 +1757,17 @@ static void disband_city(struct city *pcity)
 		     _("Game: %s can't build %s yet, "
 		     "and we can't disband our only city."),
 		     pcity->name, unit_name(pcity->currently_building));
-    return;
+    return 0;
   }
 
   create_unit(pplayer, x, y, pcity->currently_building,
 	      do_make_unit_veteran(pcity, pcity->currently_building), 
 	      pcity->id, -1);
 
-  /* shift all the units supported by pcity (including the new settler) to rcity */
-  transfer_city_units(pplayer, pplayer, rcity, pcity, -1, 1);
+  /* shift all the units supported by pcity (including the new settler) to rcity.
+     transfer_city_units does not make sure no units are left floating without a
+     transport, but since all units are transfered this is not a problem. */
+  transfer_city_units(pplayer, pplayer, &pcity->units_supported, rcity, pcity, -1, 1);
 
   notify_player_ex(pplayer, x, y, E_UNIT_BUILD,
 		   _("Game: %s is disbanded into %s."), 
@@ -1695,4 +1777,5 @@ static void disband_city(struct city *pcity)
 	  get_nation_name_plural(pplayer->nation));
 
   remove_city(pcity);
+  return 1;
 }
