@@ -17,25 +17,24 @@
 struct city;
 struct player;
 struct unit;
+struct conn_list;
 
 int unit_being_aggressive(struct unit *punit);
 
-int advisor_choose_build(struct player *pplayer, 
-			 struct city *pcity);  /* used by the AI */
 int city_refresh(struct city *pcity);          /* call if city has changed */
 void global_city_refresh(struct player *pplayer); /* tax/govt changed */
 
 void auto_arrange_workers(struct city *pcity); /* will arrange the workers */
-int  add_adjust_workers(struct city *pcity);   /* will add workers */
-void city_check_workers(struct player *pplayer, struct city *pcity);
-
-/**** This part isn't meant to be used ****/  /* huh? which part?  --dwp */
+int add_adjust_workers(struct city *pcity);   /* will add workers */
 
 void city_auto_remove_worker(struct city *pcity); 
-void send_city_turn_notifications(struct player *pplayer, struct city *pcity);
+void send_global_city_turn_notifications(struct conn_list *dest);
+void send_city_turn_notifications(struct conn_list *dest, struct city *pcity);
 void begin_cities_turn(struct player *pplayer);
 void update_city_activities(struct player *pplayer);
 void city_incite_cost(struct city *pcity);
+void remove_obsolete_buildings_city(struct city *pcity, int refresh);
 void remove_obsolete_buildings(struct player *plr);
 
+void nullify_prechange_production(struct city *pcity);
 #endif  /* FC__CITYTURN_H */
