@@ -15,34 +15,32 @@
 
 #include "map.h"		/* MAP_MAX_ */
 
-enum goto_move_restriction {
-  GOTO_MOVE_ANY, GOTO_MOVE_CARDINAL_ONLY, GOTO_MOVE_STRAIGHTEST
-};
-
-void do_unit_goto(struct player *pplayer, struct unit *punit,
-		  enum goto_move_restriction restriction);
+void do_unit_goto(struct unit *punit, enum goto_move_restriction restriction,
+		  int trigger_special_ability);
 void generate_warmap(struct city *pcity, struct unit *punit);
 void really_generate_warmap(struct city *pcity, struct unit *punit,
 			    enum unit_move_type which);
-int calculate_move_cost(struct player *pplayer, struct unit *punit,
-			int dest_x, int dest_y);
-int naive_air_can_move_between(int moves, int src_x, int src_y,
-				      int dest_x, int dest_y, int playerid);
+int calculate_move_cost(struct unit *punit, int dest_x, int dest_y);
+int air_can_move_between(int moves, int src_x, int src_y,
+			 int dest_x, int dest_y, int playerid);
 
 /* all other functions are internal */
 
 #define THRESHOLD 12
 
 
-int goto_is_sane(struct player *pplayer, struct unit *punit, int x, int y, int omni);
+int goto_is_sane(struct unit *punit, int x, int y, int omni);
 
 struct move_cost_map {
   unsigned char *cost[MAP_MAX_WIDTH];
   unsigned char *seacost[MAP_MAX_WIDTH];
   unsigned char *vector[MAP_MAX_WIDTH];
+  unsigned char *returned[MAP_MAX_WIDTH];
   struct city *warcity; /* so we know what we're dealing with here */
   struct unit *warunit; /* so we know what we're dealing with here */
   int orig_x, orig_y;
 };
 
+extern struct move_cost_map warmap;
+
 #endif  /* FC__GOTOHAND_H */
