@@ -14,9 +14,11 @@
 #define FC__MAPHAND_H
 
 #include "map.h"
+#include "packets.h"
 
 struct player;
 struct section_file;
+struct conn_list;
 
 struct dumb_city{
   int id;
@@ -30,6 +32,12 @@ struct player_tile{
   enum tile_terrain_type terrain;
   enum tile_special_type special;
   unsigned short seen;
+  unsigned short own_seen;
+  /* If you build a city with an unknown square within city radius
+     the square stays unknown. However, we still have to keep count
+     of the seen points, so they are kept in here. When the tile
+     then becomes known they are moved to seen. */
+  unsigned short pending_seen;
   struct dumb_city* city;
   short last_updated;
 };
@@ -40,42 +48,39 @@ void give_map_from_player_to_player(struct player *pfrom, struct player *pdest);
 void give_seamap_from_player_to_player(struct player *pfrom, struct player *pdest);
 void give_citymap_from_player_to_player(struct city *pcity,
 					struct player *pfrom, struct player *pdest);
-void send_all_known_tiles(struct player *dest);
-void send_tile_info(struct player *dest, int x, int y);
+void send_all_known_tiles(struct conn_list *dest);
+void send_tile_info(struct conn_list *dest, int x, int y);
 void unfog_area(struct player *pplayer, int x, int y, int len);
-void send_NODRAW_tiles(struct player *pplayer, int x, int y, int len);
 void fog_area(struct player *pplayer, int x, int y, int len);
 void upgrade_city_rails(struct player *pplayer, int discovery);
-void send_map_info(struct player *dest);
-void map_save(struct section_file *file);
-void map_startpos_load(struct section_file *file);
-void map_tiles_load(struct section_file *file);
-void map_load(struct section_file *file);
-void map_rivers_overlay_load(struct section_file *file);
+void send_map_info(struct conn_list *dest);
 void map_fog_city_area(struct city *pcity);
 void map_unfog_city_area(struct city *pcity);
 void remove_unit_sight_points(struct unit *punit);
 void show_area(struct player *pplayer,int x, int y, int len);
 void map_unfog_pseudo_city_area(struct player *pplayer, int x,int y);
 void map_fog_pseudo_city_area(struct player *pplayer, int x,int y);
-void unfog_map_for_ai_players(void);
-void unfog_map_for_player(struct player *pplayer);
 
 int map_get_known_and_seen(int x, int y, int playerid);
-int map_get_seen(int x, int y, int playerid);
 void map_change_seen(int x, int y, int playerid, int change);
+int map_get_own_seen(int x, int y, int playerid);
+void map_change_own_seen(int x, int y, int playerid, int change);
 int map_get_known(int x, int y, struct player *pplayer);
 void map_set_known(int x, int y, struct player *pplayer);
 void map_clear_known(int x, int y, struct player *pplayer);
-int map_get_sent(int x, int y, struct player *pplayer);
-void map_set_sent(int x, int y, struct player *pplayer);
-void map_clear_sent(int x, int y, struct player *pplayer);
 void map_know_all(struct player *pplayer);
 void map_know_and_see_all(struct player *pplayer);
-void set_unknown_tiles_to_unsent(struct player *pplayer);
+void show_map_to_all(void);
 void player_map_allocate(struct player *pplayer);
 struct player_tile *map_get_player_tile(int x, int y, int playerid);
 void update_tile_knowledge(struct player *pplayer,int x, int y);
 void update_player_tile_last_seen(struct player *pplayer, int x, int y);
 
+void give_shared_vision(struct player *pfrom, struct player *pto);
+void remove_shared_vision(struct player *pfrom, struct player *pto);
+void handle_player_remove_vision(struct player *pplayer,
+				 struct packet_generic_integer *packet);
+
+void enable_fog_of_war(void);
+void disable_fog_of_war(void);
 #endif  /* FC__MAPHAND_H */
