@@ -17,36 +17,23 @@
  * Definitions for UDP.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#define DEFAULT_META_SERVER_NO_SEND	1
+#define DEFAULT_META_SERVER_PORT	12245
+#define DEFAULT_META_SERVER_ADDR	"meta.freeciv.org"
+#define METASERVER_UPDATE_INTERVAL	(3*60)
 
-#include "version.h"
+#define PACKET_UDP_PCKT 2
 
-#define	DEFAULT_META_SERVER_PORT	12245
-#define	DEFAULT_META_SERVER_ADDR	"meta.freeciv.org"
-#define METASERVER_UPDATE_INTERVAL 3*60
-#if IS_BETA_VERSION
-#  define DEFAULT_META_SERVER_INFO_STRING "unstable pre-" NEXT_STABLE_VERSION ": beware"
-#else
-#  if IS_DEVEL_VERSION
-#    define DEFAULT_META_SERVER_INFO_STRING "development version: beware"
-#  else
-#    define DEFAULT_META_SERVER_INFO_STRING "(default)"
-#  endif
-#endif
-
-extern int server_is_open;
+char *default_meta_server_info_string(void);
 
 void meta_addr_split(void);
 char *meta_addr_port(void);
 
-/* Returns true if able to send */
-int send_to_metaserver(char *desc, char *info);
-
 void server_close_udp(void);
 void server_open_udp(void);
 
-#define PACKET_UDP_PCKT 2
+int send_server_info_to_metaserver(int do_send, int reset_timer);
+
+extern int server_is_open;
 
 #endif /* FC__META_H */
