@@ -18,14 +18,17 @@
 struct player;
 struct packet_player_request;
 struct section_file;
+struct connection;
+struct conn_list;
 
+enum plr_info_level { INFO_MINIMUM, INFO_MEETING, INFO_EMBASSY, INFO_FULL };
+
+void server_player_init(struct player *pplayer, int initmap);
 void server_remove_player(struct player *pplayer);
 void begin_player_turn(struct player *pplayer);
 void update_player_activities(struct player *pplayer);
 
 void handle_player_revolution(struct player *pplayer);
-void handle_player_ai_options(struct player *pplayer,
-			      struct packet_player_request *preq);
 void handle_player_rates(struct player *pplayer, 
 			 struct packet_player_request *preq);
 void check_player_government_rates(struct player *pplayer);
@@ -35,11 +38,15 @@ void maybe_make_first_contact(int x, int y, int playerid);
 void neutralize_ai_player(struct player *pplayer);
 
 void send_player_info(struct player *src, struct player *dest);
+void send_player_info_c(struct player *src, struct conn_list *dest);
+void send_conn_info(struct conn_list *src, struct conn_list *dest);
+void send_conn_info_remove(struct conn_list *src, struct conn_list *dest);
 
-void page_player(struct player *pplayer, char *caption, char *headline,
-		 char *lines);
-void page_player_generic(struct player *pplayer, char *caption, char *headline,
-			 char *lines, int event);
+void notify_conn_ex(struct conn_list *dest, int x, int y, int event,
+		    const char *format, ...) 
+                    fc__attribute((format (printf, 5, 6)));
+void notify_conn(struct conn_list *dest, const char *format, ...) 
+                 fc__attribute((format (printf, 2, 3)));
 void notify_player_ex(const struct player *pplayer, int x, int y,
 		      int event, const char *format, ...)
                       fc__attribute((format (printf, 5, 6)));
@@ -48,6 +55,9 @@ void notify_player(const struct player *pplayer, const char *format, ...)
 void notify_embassies(struct player *pplayer, struct player *exclude,
 		      const char *format, ...)
 		      fc__attribute((format (printf, 3, 4)));
+
+struct conn_list *player_reply_dest(struct player *pplayer);
+
 void handle_player_government(struct player *pplayer,
 			     struct packet_player_request *preq);
 void handle_player_research(struct player *pplayer,
@@ -56,7 +66,6 @@ void handle_player_tech_goal(struct player *pplayer,
                             struct packet_player_request *preq);
 void handle_player_worklist(struct player *pplayer,
                             struct packet_player_request *preq);
-void researchprogress(int plr);
 void found_new_tech(struct player *plr, int tech_found, char was_discovery, 
 		    char saving_bulbs);
 void tech_researched(struct player* plr);
@@ -66,18 +75,25 @@ void choose_random_tech(struct player *plr);
 void choose_tech(struct player *plr, int tech);
 void choose_tech_goal(struct player *plr, int tech);
 int choose_goal_tech(struct player *plr);
-void wonders_of_the_world(struct player *pplayer);
-void demographics_report(struct player *pplayer);
-void top_five_cities(struct player *pplayer);
-void send_player_turn_notifications(struct player *pplayer);
-void make_history_report(void);
+void get_a_tech(struct player *pplayer, struct player *target);
+
+void send_player_turn_notifications(struct conn_list *dest);
+
 void do_dipl_cost(struct player *pplayer);
 void do_free_cost(struct player *pplayer);
 void do_conquer_cost(struct player *pplayer);
-void show_ending(void);
-void show_map_to_all(void);
-void player_load(struct player *plr, int plrno, struct section_file *file);
-void player_map_load(struct player *plr, int plrno, struct section_file *file);
-void player_save(struct player *plr, int plrno, struct section_file *file);
+
+void associate_player_connection(struct player *plr, struct connection *pconn);
+void unassociate_player_connection(struct player *plr, struct connection *pconn);
+
+void shuffle_players(void);
+struct player *shuffled_player(int i);
+
+int civil_war_triggered(struct player *pplayer);
+void civil_war(struct player *pplayer);
+struct player *split_player(struct player *pplayer);
+
+enum plr_info_level player_info_level(struct player *plr,
+                                      struct player *receiver);
 
 #endif  /* FC__PLRHAND_H */
