@@ -56,21 +56,27 @@
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
+#ifdef HAVE_WINSOCK
+#include <winsock.h>
+#endif
 
 #ifdef HAVE_LIBREADLINE
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
 
+#include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
 #include "packets.h"
+#include "plrhand.h"
 #include "shared.h"
 #include "support.h"
 
-#include "civserver.h"
 #include "console.h"
+#include "meta.h"
+#include "srv_main.h"
 #include "stdinhand.h"
 
 #include "sernet.h"
@@ -82,23 +88,40 @@ TEndpointInfo serv_info;
 EndpointRef serv_ep;
 #else
 static int sock;
-extern int port;
-#endif
-extern int force_end_of_sniff;
-
-#ifdef __VMS
-#include <descrip.h>
-#include <iodef.h>
-#include <stsdef.h>
-#include <starlet.h>
-#include <lib$routines.h>
-#include <efndef.h>
-static unsigned long int tt_chan;
-static char input_char = 0;
-static char got_input = 0;
-void user_interrupt_callback();
 #endif
 
+#if defined(__VMS)
+#  if defined(_VAX_)
+#    define lib$stop LIB$STOP
+#    define sys$qiow SYS$QIOW
+#    define sys$assign SYS$ASSIGN
+#  endif
+#  include <descrip.h>
+#  include <iodef.h>
+#  include <stsdef.h>
+#  include <starlet.h>
+#  include <lib$routines.h>
+#  include <efndef.h>
+   static unsigned long int tt_chan;
+   static char input_char = 0;
+   static char got_input = 0;
+   void user_interrupt_callback();
+#endif
+
+
+static int server_accept_connection(int sockfd);
+
+
+/*****************************************************************************
+  This happens if you type an EOF character with nothing on the current line.
+*****************************************************************************/
+static void handle_stdin_close(void)
+{
+  notify_player(0, _("Server quitting because it cannot read standard input."));
+
+  printf("quit");
+  quit_game(NULL);
+}
 
 #ifdef HAVE_LIBREADLINE
 /****************************************************************************/
@@ -115,24 +138,39 @@ static int readline_handled_input = 0;
 *****************************************************************************/
 static void handle_readline_input_callback(char *line)
 {
-  if (line) {
-    if (*line)
-      add_history(line);
-    con_prompt_enter();		/* just got an 'Enter' hit */
-    handle_stdin_input((struct player *)NULL, line);
+  if (!line) {
+    handle_stdin_close();	/* maybe print an 'are you sure?' message? */
   }
+
+  if (*line)
+    add_history(line);
+
+  con_prompt_enter();		/* just got an 'Enter' hit */
+  handle_stdin_input((struct connection*)NULL, line);
+
   readline_handled_input = 1;
 }
 
-#endif
+#endif /* HAVE_LIBREADLINE */
 /*****************************************************************************
 ...
 *****************************************************************************/
 void close_connection(struct connection *pconn)
 {
-  close(pconn->sock);
-  pconn->used=0;
-  pconn->access_level=ALLOW_NONE;
+  /* safe to do these even if not in lists: */
+  conn_list_unlink(&game.all_connections, pconn);
+  conn_list_unlink(&game.est_connections, pconn);
+  conn_list_unlink(&game.game_connections, pconn);
+
+  my_closesocket(pconn->sock);
+  pconn->used = 0;
+  pconn->established = 0;
+  pconn->player = NULL;
+  pconn->access_level = ALLOW_NONE;
+  free_socket_packet_buffer(pconn->buffer);
+  free_socket_packet_buffer(pconn->send_buffer);
+  pconn->buffer = NULL;
+  pconn->send_buffer = NULL;
 }
 
 /*****************************************************************************
@@ -147,7 +185,7 @@ void close_connections_and_socket(void)
       close_connection(&connections[i]);
     }
   }
-  close(sock);
+  my_closesocket(sock);
 
 #ifdef HAVE_LIBREADLINE
   if (history_file) {
@@ -158,14 +196,85 @@ void close_connections_and_socket(void)
 }
 
 /*****************************************************************************
-...
+  Used for errors and other strangeness.  As well as some direct uses, is
+  passed to packet routines as callback for when packet sending has a write
+  error.  Closes the connection cleanly, calling lost_connection_to_client()
+  to clean up server variables, notify other clients, etc.
 *****************************************************************************/
 static void close_socket_callback(struct connection *pc)
 {
-  lost_connection_to_player(pc);
+  lost_connection_to_client(pc);
   close_connection(pc);
 }
 
+/****************************************************************************
+  Attempt to flush all information in the send buffers for upto 'netwait'
+  seconds.
+*****************************************************************************/
+void flush_packets(void)
+{
+  int i;
+  int max_desc;
+  fd_set writefs, exceptfs;
+  struct timeval tv;
+  time_t start;
+
+  time(&start);
+
+  for(;;) {
+    tv.tv_sec=(game.netwait - (time(NULL) - start));
+    tv.tv_usec=0;
+
+    if (tv.tv_sec < 0)
+      return;
+
+    MY_FD_ZERO(&writefs);
+    MY_FD_ZERO(&exceptfs);
+    max_desc=-1;
+
+    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
+      struct connection *pconn = &connections[i];
+      if(pconn->used && pconn->send_buffer->ndata) {
+	FD_SET(pconn->sock, &writefs);
+	FD_SET(pconn->sock, &exceptfs);
+	max_desc=MAX(pconn->sock, max_desc);
+      }
+    }
+
+    if (max_desc == -1) {
+      return;
+    }
+
+    if(select(max_desc+1, NULL, &writefs, &exceptfs, &tv)<=0) {
+      return;
+    }
+
+    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {   /* check for freaky players */
+      struct connection *pconn = &connections[i];
+      if(pconn->used) {
+        if(FD_ISSET(pconn->sock, &exceptfs)) {
+	  freelog(LOG_NORMAL, "cut connection %s due to exception data",
+		  conn_description(pconn));
+	  close_socket_callback(pconn);
+        } else {
+	  if(pconn->send_buffer && pconn->send_buffer->ndata) {
+	    if(FD_ISSET(pconn->sock, &writefs)) {
+	      flush_connection_send_buffer_all(pconn);
+	    } else {
+	      if(game.tcptimeout && pconn->last_write
+		 && (time(NULL)>pconn->last_write + game.tcptimeout)) {
+	        freelog(LOG_NORMAL, "cut connection %s due to lagging player",
+			conn_description(pconn));
+		close_socket_callback(pconn);
+	      }
+	    }
+	  }
+        }
+      }
+    }
+  }
+}
+
 /*****************************************************************************
 Get and handle:
 - new connections,
@@ -182,12 +291,11 @@ int sniff_packets(void)
 {
   int i;
   int max_desc;
-  fd_set readfs, exceptfs;
+  fd_set readfs, writefs, exceptfs;
   struct timeval tv;
   static int year;
 #ifdef SOCKET_ZERO_ISNT_STDIN
-  char buf[BUF_SIZE+1];
-  char *bufptr = buf;
+  char *bufptr;    
 #endif
 
   con_prompt_init();
@@ -212,11 +320,12 @@ int sniff_packets(void)
 
       rl_initialize();
       rl_callback_handler_install("> ", handle_readline_input_callback);
+      rl_attempted_completion_function = freeciv_completion;
 
       readline_initialized = 1;
     }
   }
-#endif
+#endif /* HAVE_LIBREADLINE */
 
   if(year!=game.year) {
     if (server_state == RUN_GAME_STATE) year=game.year;
@@ -232,29 +341,86 @@ int sniff_packets(void)
       con_prompt_off();
       return 2;
     }
-    
+
+
+    /* quit server if no players for 'srvarg.quitidle' seconds */
+    if (srvarg.quitidle && server_state != PRE_GAME_STATE) {
+      static time_t last_noplayers;
+      if(conn_list_size(&game.est_connections) == 0) {
+	if (last_noplayers) {
+	  if (time(NULL)>last_noplayers + srvarg.quitidle) {
+	    sz_strlcpy(srvarg.metaserver_info_line,
+		       "restarting for lack of players");
+	    freelog(LOG_NORMAL, srvarg.metaserver_info_line);
+	    send_server_info_to_metaserver(1,0);
+
+	    quit_game(NULL);
+	  }
+	} else {
+	  last_noplayers = time(NULL);
+	  
+	  my_snprintf(srvarg.metaserver_info_line,
+		      sizeof(srvarg.metaserver_info_line),
+		      "restarting in %d seconds for lack of players",
+		      srvarg.quitidle);
+	  freelog(LOG_NORMAL, srvarg.metaserver_info_line);
+	  send_server_info_to_metaserver(1,0);
+	}
+      } else {
+        last_noplayers = 0;
+      }
+    }
+
+    /* send PACKET_CONN_PING & cut mute players */
+    if ((time(NULL)>game.last_ping + game.pingtimeout)) {
+      for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
+	struct connection *pconn = &connections[i];
+	if (pconn->used) {
+	  send_packet_generic_empty(pconn, PACKET_CONN_PING);
+
+	  if (pconn->ponged) {
+	    pconn->ponged = 0;
+	  } else {
+	    freelog(LOG_NORMAL, "cut connection %s due to ping timeout",
+		    conn_description(pconn));
+	    close_socket_callback(pconn);
+	  }
+	}
+      }
+      game.last_ping = time(NULL);
+    }
+
+
     tv.tv_sec=1;
     tv.tv_usec=0;
-    
+
     MY_FD_ZERO(&readfs);
+    MY_FD_ZERO(&writefs);
     MY_FD_ZERO(&exceptfs);
-#ifndef SOCKET_ZERO_ISNT_STDIN
-    FD_SET(0, &readfs);	
+#ifdef SOCKET_ZERO_ISNT_STDIN
+    my_init_console();
+#else
+#   if !defined(__VMS)
+      FD_SET(0, &readfs);
+#   endif	
 #endif
     FD_SET(sock, &readfs);
     FD_SET(sock, &exceptfs);
     max_desc=sock;
-    
+
     for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
       if(connections[i].used) {
 	FD_SET(connections[i].sock, &readfs);
+	if(connections[i].send_buffer->ndata) {
+	  FD_SET(connections[i].sock, &writefs);
+	}
 	FD_SET(connections[i].sock, &exceptfs);
         max_desc=MAX(connections[i].sock, max_desc);
       }
     }
     con_prompt_off();		/* output doesn't generate a new prompt */
-    
-    if(select(max_desc+1, &readfs, NULL, &exceptfs, &tv)==0) { /* timeout */
+
+    if(select(max_desc+1, &readfs, &writefs, &exceptfs, &tv)==0) { /* timeout */
       send_server_info_to_metaserver(0,0);
       if((game.timeout) 
 	&& (time(NULL)>game.turn_start + game.timeout)
@@ -262,7 +428,7 @@ int sniff_packets(void)
 	con_prompt_off();
 	return 0;
       }
-#ifdef __VMS
+#if defined(__VMS)
       {
 	struct { short numchars; char firstchar; char reserved; int reserved2; } ttchar;
 	unsigned long status;
@@ -274,12 +440,11 @@ int sniff_packets(void)
 	else
 	  continue;
       }
-#else
-#ifdef SOCKET_ZERO_ISNT_STDIN
-    if (feof(stdin))
-#endif
+#else  /* !__VMS */
+#ifndef SOCKET_ZERO_ISNT_STDIN
       continue;
-#endif
+#endif /* SOCKET_ZERO_ISNT_STDIN */
+#endif /* !__VMS */
     }
     if (!game.timeout)
       game.turn_start = time(NULL);
@@ -289,16 +454,24 @@ int sniff_packets(void)
     }
     if(FD_ISSET(sock, &readfs)) {	     /* new players connects */
       freelog(LOG_VERBOSE, "got new connection");
-      if(server_accept_connection(sock)==-1)
-	freelog(LOG_NORMAL, "failed accepting connection");
-    }
-    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {   /* check for freaky players */
-      if(connections[i].used && FD_ISSET(connections[i].sock, &exceptfs)) {
-	freelog(LOG_VERBOSE, "cut freaky player");
-	close_socket_callback(&connections[i]);
+      if(server_accept_connection(sock)==-1) {
+	freelog(LOG_ERROR, "failed accepting connection");
       }
     }
-#ifndef SOCKET_ZERO_ISNT_STDIN
+    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {   /* check for freaky players */
+      struct connection *pconn = &connections[i];
+      if(pconn->used && FD_ISSET(pconn->sock, &exceptfs)) {
+ 	freelog(LOG_ERROR, "cut connection %s due to exception data",
+		conn_description(pconn));
+	close_socket_callback(pconn);
+      }
+    }
+#ifdef SOCKET_ZERO_ISNT_STDIN
+    if ((bufptr = my_read_console())) {
+      con_prompt_enter();	/* will need a new prompt, regardless */
+      handle_stdin_input((struct connection *)NULL, bufptr);
+    }
+#else  /* !SOCKET_ZERO_ISNT_STDIN */
     if(FD_ISSET(0, &readfs)) {    /* input from server operator */
 #ifdef HAVE_LIBREADLINE
       rl_callback_read_char();
@@ -307,7 +480,7 @@ int sniff_packets(void)
 	con_prompt_enter_clear();
       }
       continue;
-#else
+#else  /* !HAVE_LIBREADLINE */
       int didget;
       char buf[BUF_SIZE+1];
       
@@ -315,41 +488,55 @@ int sniff_packets(void)
 	freelog(LOG_FATAL, "read from stdin failed");
 	exit(1);
       }
+
+      if(didget==0) {
+        handle_stdin_close();
+      }
+
       *(buf+didget)='\0';
       con_prompt_enter();	/* will need a new prompt, regardless */
-      handle_stdin_input((struct player *)NULL, buf);
-#endif
+      handle_stdin_input((struct connection *)NULL, buf);
+#endif /* !HAVE_LIBREADLINE */
     }
-#else
-    if(!feof(stdin)) {    /* input from server operator */
-      /* fetch chars until \n or run out of space in buffer */
-      while ((*bufptr=fgetc(stdin)) != EOF) {
-          if (*bufptr == '\n') *bufptr = '\0';
-          if (*bufptr == '\0') {
-              bufptr = buf;
-              con_prompt_enter(); /* will need a new prompt, regardless */
-              handle_stdin_input((struct player *)NULL, buf);
-              break;
-          }
-          if ((bufptr-buf) <= BUF_SIZE) bufptr++; /* prevent overrun */
-      }
-    }
-#endif
-    else {                             /* input from a player */
+    else
+#endif /* !SOCKET_ZERO_ISNT_STDIN */
+     
+    {                             /* input from a player */
       for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
-	if(connections[i].used && FD_ISSET(connections[i].sock, &readfs)) {
-	  if(read_socket_data(connections[i].sock, 
-			      &connections[i].buffer)>=0) {
+  	struct connection *pconn = &connections[i];
+	if(pconn->used && FD_ISSET(pconn->sock, &readfs)) {
+	  if(read_socket_data(pconn->sock, pconn->buffer)>=0) {
 	    char *packet;
-	    int type;
-	    while((packet=get_packet_from_connection(&connections[i], &type)))
-	      handle_packet_input(&connections[i], packet, type);
-	  }
-	  else {
-	    close_socket_callback(&connections[i]);
+	    int type, result;
+	    while (1) {
+	      packet = get_packet_from_connection(pconn, &type, &result);
+	      if (result) {
+		handle_packet_input(pconn, packet, type);
+	      } else {
+		break;
+	      }
+	    }
+	  } else {
+	    close_socket_callback(pconn);
 	  }
 	}
       }
+
+      for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
+        struct connection *pconn = &connections[i];
+        if(pconn->used && pconn->send_buffer && pconn->send_buffer->ndata) {
+	  if(FD_ISSET(pconn->sock, &writefs)) {
+	    flush_connection_send_buffer_all(pconn);
+	  } else {
+	    if(game.tcptimeout && pconn->last_write
+	       && (time(NULL)>pconn->last_write + game.tcptimeout)) {
+	      freelog(LOG_NORMAL, "cut connection %s due to lagging player",
+		      conn_description(pconn));
+	      close_socket_callback(pconn);
+	    }
+	  }
+        }
+      }
     }
     break;
   }
@@ -360,14 +547,48 @@ int sniff_packets(void)
     return 0;
   return 1;
 }
-  
+
+/********************************************************************
+  Make up a name for the connection, before we get any data from
+  it to use as a sensible name.  Name will be 'c' + integer,
+  guaranteed not to be the same as any other connection name,
+  nor player name nor user name, nor connection id (avoid possible
+  confusions).   Returns pointer to static buffer, and fills in
+  (*id) with chosen value.
+********************************************************************/
+static const char *makeup_connection_name(int *id)
+{
+  static unsigned short i = 0;
+  static char name[MAX_LEN_NAME];
+
+  for(;;) {
+    if (i==(unsigned short)-1) i++;              /* don't use 0 */
+    my_snprintf(name, sizeof(name), "c%u", (unsigned int)++i);
+    if (!find_player_by_name(name)
+	&& !find_player_by_user(name)
+	&& !find_conn_by_id(i)
+	&& !find_conn_by_name(name)) {
+      *id = i;
+      return name;
+    }
+  }
+}
   
 /********************************************************************
- server accepts connections from client
+  Server accepts connection from client:
+  Low level socket stuff, and basic-initialize the connection struct.
+  Returns 0 on success, -1 on failure (bad accept(), or too many
+  connections).
 ********************************************************************/
-int server_accept_connection(int sockfd)
+static int server_accept_connection(int sockfd)
 {
-  int fromlen;
+
+# if defined(__VMS) && !defined(_DECC_V4_SOURCE)
+    size_t fromlen;
+# else
+    int fromlen;
+# endif
+
   int new_sock;
   struct sockaddr_in fromend;
   struct hostent *from;
@@ -376,7 +597,7 @@ int server_accept_connection(int sockfd)
   fromlen = sizeof(fromend);
 
   if ((new_sock=accept(sockfd, (struct sockaddr *) &fromend, &fromlen)) == -1) {
-    freelog(LOG_VERBOSE, "accept failed: %s", mystrerror(errno));
+    freelog(LOG_ERROR, "accept failed: %s", mystrerror(errno));
     return -1;
   }
 
@@ -386,31 +607,38 @@ int server_accept_connection(int sockfd)
 		     AF_INET);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
-    if(!connections[i].used) {
-      connections[i].used=1;
-      connections[i].sock=new_sock;
-      connections[i].player=NULL;
-      connections[i].buffer.ndata=0;
-      connections[i].first_packet=1;
-      connections[i].byte_swap=0;
-      connections[i].access_level=(i?default_access_level:first_access_level);
+    struct connection *pconn = &connections[i];
+    if (!pconn->used) {
+      pconn->used = 1;
+      pconn->sock = new_sock;
+      pconn->established = 0;
+      pconn->observer = 0;
+      pconn->player = NULL;
+      pconn->buffer = new_socket_packet_buffer();
+      pconn->send_buffer = new_socket_packet_buffer();
+      pconn->last_write = 0;
+      pconn->ponged = 1;
+      pconn->first_packet = 1;
+      pconn->byte_swap = 0;
+      pconn->capability[0] = '\0';
+      pconn->access_level = access_level_for_next_connection();
+      pconn->delayed_disconnect = 0;
 
-      if (from)
-	sz_strlcpy(connections[i].addr, from->h_name);
-      else
-	sz_strlcpy(connections[i].addr, inet_ntoa(fromend.sin_addr));
+      sz_strlcpy(pconn->name, makeup_connection_name(&pconn->id));
+      sz_strlcpy(pconn->addr,
+		 (from ? from->h_name : inet_ntoa(fromend.sin_addr)));
 
-      freelog(LOG_VERBOSE, "connection from %s", connections[i].addr);
+      conn_list_insert_back(&game.all_connections, pconn);
+  
+      freelog(LOG_VERBOSE, "connection (%s) from %s", pconn->name, pconn->addr);
       return 0;
     }
   }
 
-  freelog(LOG_FATAL, "maximum number of connections reached");
+  freelog(LOG_ERROR, "maximum number of connections reached");
   return -1;
 }
 
-
-
 /********************************************************************
  open server socket to be used to accept client connections
 ********************************************************************/
@@ -433,14 +661,13 @@ int server_open_socket(void)
   opt=1; 
   if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
 		(char*)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_FATAL, "SO_REUSEADDR failed: %s", mystrerror(errno));
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror(errno));
   }
 
   memset(&src, 0, sizeof(src));
   src.sin_family = AF_INET;
   src.sin_addr.s_addr = htonl(INADDR_ANY);
-  src.sin_port = htons(port);
-
+  src.sin_port = htons(srvarg.port);
 
   if(bind(sock, (struct sockaddr *) &src, sizeof (src)) == -1) {
     freelog(LOG_FATAL, "bind failed: %s", mystrerror(errno));
@@ -464,10 +691,13 @@ void init_connections(void)
 {
   int i;
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) { 
-    connections[i].used=0;
-    connections[i].buffer.ndata=0;
+    struct connection *pconn = &connections[i];
+    pconn->used = 0;
+    conn_list_init(&pconn->self);
+    conn_list_insert(&pconn->self, pconn);
+    pconn->route = NULL;
   }
-#ifdef __VMS
+#if defined(__VMS)
   {
     unsigned long status;
     $DESCRIPTOR (tt_desc, "SYS$INPUT");
