@@ -19,8 +19,8 @@ struct connection;
 #define DEFAULT_SOCK_PORT 5555
 #define BUF_SIZE 512
 
-int server_accept_connection(int sockfd);
 int server_open_socket(void);
+void flush_packets(void);
 int sniff_packets(void);
 void close_connections_and_socket(void);
 void init_connections(void);
