@@ -25,13 +25,11 @@
 #include "shared.h"
 #include "spaceship.h"
 
-#include "civserver.h"
 #include "plrhand.h"
+#include "srv_main.h"
 
 #include "spacerace.h"
 
-/* use shuffled order from civserver.c: */
-extern struct player *shuffled[];
 
 /**********************************************************************
 Calculate and fill in the derived quantities about the spaceship.
@@ -107,46 +105,45 @@ void spaceship_calc_derived(struct player_spaceship *ship)
 }
 
 /**************************************************************************
-both src and dest can be NULL
-NULL means all players
+  Send details of src's spaceship (or spaceships of all players
+  if src is NULL) to specified destinations.  If dest is NULL then
+  game.game_connections is used.
 **************************************************************************/
-void send_spaceship_info(struct player *src, struct player *dest)
+void send_spaceship_info(struct player *src, struct conn_list *dest)
 {
-  int o, i, j;
+  int i, j;
+
+  if (dest==NULL) dest = &game.game_connections;
   
-  for(o=0; o<game.nplayers; o++) {          /* dests */
-    if(!dest || &game.players[o]==dest) {
-      for(i=0; i<game.nplayers; i++) {      /* srcs  */
-	if(!src || &game.players[i]==src) {
-	  struct packet_spaceship_info info;
-	  struct player_spaceship *ship = &game.players[i].spaceship;
+  for(i=0; i<game.nplayers; i++) {      /* srcs  */
+    if(!src || &game.players[i]==src) {
+      struct packet_spaceship_info info;
+      struct player_spaceship *ship = &game.players[i].spaceship;
 	  
-	  info.player_num = i;
-	  info.sship_state = ship->state;
-	  info.structurals = ship->structurals;
-	  info.components = ship->components;
-	  info.modules = ship->modules;
-	  info.fuel = ship->fuel;
-	  info.propulsion = ship->propulsion;
-	  info.habitation = ship->habitation;
-	  info.life_support = ship->life_support;
-	  info.solar_panels = ship->solar_panels;
-	  info.launch_year = ship->launch_year;
-	  info.population = ship->population;
-	  info.mass = ship->mass;
-	  info.support_rate = ship->support_rate;
-	  info.energy_rate = ship->energy_rate;
-	  info.success_rate = ship->success_rate;
-	  info.travel_time = ship->travel_time;
-	  
-	  for(j=0; j<NUM_SS_STRUCTURALS; j++) {
-	    info.structure[j] = ship->structure[j] + '0';
-	  }
-	  info.structure[j] = '\0';
-	  
-	  send_packet_spaceship_info(game.players[o].conn, &info);
-	}
+      info.player_num = i;
+      info.sship_state = ship->state;
+      info.structurals = ship->structurals;
+      info.components = ship->components;
+      info.modules = ship->modules;
+      info.fuel = ship->fuel;
+      info.propulsion = ship->propulsion;
+      info.habitation = ship->habitation;
+      info.life_support = ship->life_support;
+      info.solar_panels = ship->solar_panels;
+      info.launch_year = ship->launch_year;
+      info.population = ship->population;
+      info.mass = ship->mass;
+      info.support_rate = ship->support_rate;
+      info.energy_rate = ship->energy_rate;
+      info.success_rate = ship->success_rate;
+      info.travel_time = ship->travel_time;
+      
+      for(j=0; j<NUM_SS_STRUCTURALS; j++) {
+	info.structure[j] = ship->structure[j] + '0';
       }
+      info.structure[j] = '\0';
+	  
+      lsend_packet_spaceship_info(dest, &info);
     }
   }
 }
@@ -158,7 +155,13 @@ void handle_spaceship_launch(struct player *pplayer)
 {
   struct player_spaceship *ship = &pplayer->spaceship;
   int arrival;
-  
+
+  if (!find_palace(pplayer)) {
+    notify_player(pplayer,
+                  _("Game: You need to have a capital in order to launch "
+		    "your spaceship."));
+    return;
+  }
   if (ship->state >= SSHIP_LAUNCHED) {
     notify_player(pplayer, _("Game: Your spaceship is already launched!"));
     return;
@@ -168,15 +171,15 @@ void handle_spaceship_launch(struct player *pplayer)
     notify_player(pplayer, _("Game: Your spaceship can't be launched yet!"));
     return;
   }
-  
+
   ship->state = SSHIP_LAUNCHED;
   ship->launch_year = game.year;
   arrival = ship->launch_year + (int) ship->travel_time;
-  
-  notify_player_ex(0, 0, 0, E_SPACESHIP,
+
+  notify_player_ex(0, -1, -1, E_SPACESHIP,
 		_("Game: The %s have launched a spaceship!  "
 		  "It is estimated to arrive on Alpha Centauri in %s."),
-		get_nation_name(pplayer->nation),
+		get_nation_name_plural(pplayer->nation),
 		textyear(arrival));
 
   send_spaceship_info(pplayer, 0);
@@ -323,7 +326,7 @@ void handle_spaceship_action(struct player *pplayer,
     send_spaceship_info(pplayer, 0);
     return;
   }
-  freelog(LOG_NORMAL, "Received unknown spaceship action %d from %s",
+  freelog(LOG_ERROR, "Received unknown spaceship action %d from %s",
        action, pplayer->name);
 }
 
@@ -332,16 +335,14 @@ void handle_spaceship_action(struct player *pplayer,
 **************************************************************************/
 void spaceship_lost(struct player *pplayer)
 {
-  notify_player_ex(0, 0, 0, E_SPACESHIP,
-		_("Game: With the capture of %s's capital, the %s"
-		" spaceship is lost!"), pplayer->name,
-		get_nation_name(pplayer->nation));
+  notify_player_ex(0, -1, -1, E_SPACESHIP,
+		   _("Game: Without guidance from the capital, the %s "
+                     "spaceship is lost!"), get_nation_name(pplayer->nation));
   spaceship_init(&pplayer->spaceship);
   send_spaceship_info(pplayer, 0);
 }
 
 /**************************************************************************
-...
 Use shuffled order to randomly resolve ties.
 **************************************************************************/
 void check_spaceship_arrivals(void)
@@ -353,9 +354,7 @@ void check_spaceship_arrivals(void)
   struct player_spaceship *ship;
 
   for(i=0; i<game.nplayers; i++) {
-    if (!shuffled[i]) continue;
-
-    pplayer = shuffled[i];
+    pplayer = shuffled_player(i);
     ship = &pplayer->spaceship;
     
     if (ship->state == SSHIP_LAUNCHED) {
@@ -370,9 +369,8 @@ void check_spaceship_arrivals(void)
   if (best_pplayer) {
     best_pplayer->spaceship.state = SSHIP_ARRIVED;
     server_state = GAME_OVER_STATE;
-    notify_player_ex(0, 0, 0, E_SPACESHIP,
+    notify_player_ex(0, -1, -1, E_SPACESHIP,
 		     _("Game: The %s spaceship has arrived at Alpha Centauri."),
 		     get_nation_name(best_pplayer->nation));
   }
 }
-
