@@ -13,20 +13,31 @@
 #ifndef FC__STDINHAND_H
 #define FC__STDINHAND_H
 
-#include "player.h"		/* struct player; enum cmdlevel_id */
+#include "connection.h"		/* enum cmdlevel_id */
+
+struct player;
 
 #define SERVER_COMMAND_PREFIX '/'
   /* the character to mark chatlines as server commands */
 
-extern void handle_stdin_input(struct player *caller, char *str);
-extern void report_server_options(struct player *pplayer, int which);
-extern void set_ai_level_direct(struct player *pplayer, int level);
-extern void set_ai_level_directer(struct player *pplayer, int level);
-extern void read_init_script(char *script_filename);
+void handle_stdin_input(struct connection *caller, char *str);
+void report_server_options(struct conn_list *dest, int which);
+void set_ai_level_direct(struct player *pplayer, int level);
+void set_ai_level_directer(struct player *pplayer, int level);
+void read_init_script(char *script_filename);
+void show_players(struct connection *caller);
 
-void toggle_ai_player_direct(struct player *caller, struct player *subject);
+void quit_game(struct connection *caller);
 
-extern enum cmdlevel_id default_access_level;  /* for sernet.c in */
-extern enum cmdlevel_id   first_access_level;  /* initing a new connection */
+void toggle_ai_player_direct(struct connection *caller, struct player *subject);
+
+/* for sernet.c in initing a new connection */
+enum cmdlevel_id access_level_for_next_connection(void);
+
+void notify_if_first_access_level_is_available(void);
+
+#ifdef HAVE_LIBREADLINE
+char **freeciv_completion(const char *text, int start, int end);
+#endif
 
 #endif /* FC__STDINHAND_H */
