@@ -18,6 +18,8 @@
 
 void handle_unit_goto_tile(struct player *pplayer, 
 			   struct packet_unit_request *req);
+void handle_unit_airlift(struct player *pplayer,
+			 struct packet_unit_request *req);
 void handle_unit_connect(struct player *pplayer, 
 		          struct packet_unit_connect *req);
 void handle_upgrade_unittype_request(struct player *pplayer, 
@@ -33,19 +35,17 @@ void handle_unit_disband(struct player *pplayer,
 void handle_unit_disband_safe(struct player *pplayer, 
 			      struct packet_unit_request *req,
 			      struct genlist_iterator *iter);
+void do_unit_disband_safe(struct city *pcity, struct unit *punit,
+			  struct genlist_iterator *iter); /* AI uses this directly */
 void handle_unit_build_city(struct player *pplayer, 
 			    struct packet_unit_request *req);
 void handle_unit_info(struct player *pplayer, struct packet_unit_info *pinfo);
-void handle_unit_attack_request(struct player *pplayer, struct unit *punit,
-				struct unit *pdefender);
-int handle_unit_enter_hut(struct unit *punit);
-int handle_unit_move_request(struct player *pplayer, struct unit *punit,
-			      int dest_x, int dest_y, int igzoc);
+int handle_unit_move_request(struct unit *punit, int dest_x, int dest_y,
+			     int igzoc, int move_diplomat_city);
 void handle_unit_help_build_wonder(struct player *pplayer, 
 				   struct packet_unit_request *req);
 int handle_unit_establish_trade(struct player *pplayer, 
 				struct packet_unit_request *req);
-void handle_unit_enter_city(struct unit *punit, struct city *pcity);
 void handle_unit_auto_request(struct player *pplayer, 
 			      struct packet_unit_request *req);
 void handle_unit_activity_request(struct unit *punit, 
@@ -60,11 +60,10 @@ void handle_unit_nuke(struct player *pplayer,
 void handle_unit_paradrop_to(struct player *pplayer, 
                      struct packet_unit_request *req);
 void handle_move_unit(struct player *pplayer, struct packet_move_unit *pmove);
-void handle_incite_inq(struct player *pplayer,
+void handle_incite_inq(struct connection *pconn,
 		       struct packet_generic_integer *packet);
-void package_unit(struct unit *punit, struct packet_unit_info *packet,
-		  int x, int y, int carried, int select_it,
-		  enum unit_info_use packet_use, int info_city_id,
-		  int new_serial_num);
+
+void handle_goto_route(struct player *pplayer, struct packet_goto_route *packet);
+void handle_patrol_route(struct player *pplayer, struct packet_goto_route *packet);
 
 #endif  /* FC__UNITHAND_H */
